BEGIN;

--
-- PostgreSQL database dump
--

-- Dumped from database version 9.2.23
-- Dumped by pg_dump version 9.5.12

-- SET statement_timeout = 0;
-- SET lock_timeout = 0;
SET client_encoding = 'UTF8';
SET standard_conforming_strings = off;
SET check_function_bodies = false;
SET client_min_messages = warning;
SET escape_string_warning = off;
SET search_path = public, pg_catalog;
SET default_tablespace = '';

--
-- Name: plpgsql; Type: EXTENSION; Schema: -; Owner:
--

CREATE EXTENSION IF NOT EXISTS plpgsql WITH SCHEMA pg_catalog;


--
-- Name: EXTENSION plpgsql; Type: COMMENT; Schema: -; Owner:
--

COMMENT ON EXTENSION plpgsql IS 'PL/pgSQL procedural language';


--
-- Name: fuzzystrmatch; Type: EXTENSION; Schema: -; Owner:
--

CREATE EXTENSION IF NOT EXISTS fuzzystrmatch WITH SCHEMA public;


--
-- Name: EXTENSION fuzzystrmatch; Type: COMMENT; Schema: -; Owner:
--

COMMENT ON EXTENSION fuzzystrmatch IS 'determine similarities and distance between strings';


--
-- Name: unaccent; Type: EXTENSION; Schema: -; Owner:
--

CREATE EXTENSION IF NOT EXISTS unaccent WITH SCHEMA public;


--
-- Name: EXTENSION unaccent; Type: COMMENT; Schema: -; Owner:
--

COMMENT ON EXTENSION unaccent IS 'Permet de remplacer tous les accents par les valeurs non accentuées';

--
-- Name: add_missing_constraint(text, text, text, text, text, boolean); Type: FUNCTION; Schema: public; Owner: postgres
--

CREATE OR REPLACE FUNCTION public.add_missing_constraint(text, text, text, text, text, boolean) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$
DECLARE
p_namespace   alias for $1;
p_table       alias for $2;
p_constraintname alias for $3;
p_foreigntable  alias for $4;
p_foreignkeyname alias for $5;
p_deletecascade  alias for $6;
v_row         record;
v_query       text;
BEGIN
SELECT 1 INTO v_row
FROM information_schema.table_constraints tc
LEFT JOIN information_schema.key_column_usage kcu ON (
tc.constraint_catalog = kcu.constraint_catalog
AND tc.constraint_schema = kcu.constraint_schema
AND tc.constraint_name = kcu.constraint_name
)
LEFT JOIN information_schema.referential_constraints rc ON (
tc.constraint_catalog = rc.constraint_catalog
AND tc.constraint_schema = rc.constraint_schema
AND tc.constraint_name = rc.constraint_name
)
LEFT JOIN information_schema.constraint_column_usage ccu ON (
rc.unique_constraint_catalog = ccu.constraint_catalog
AND rc.unique_constraint_schema = ccu.constraint_schema
AND rc.unique_constraint_name = ccu.constraint_name
)
WHERE
tc.table_schema = p_namespace
AND tc.table_name = p_table
AND tc.constraint_type = 'FOREIGN KEY'
AND tc.constraint_name = substring( p_constraintname from 1 for 63 ) -- INFO: les noms sont juste tronqués, pas de chiffre à la fin -> ça ne devrait pas poser de problème
AND kcu.column_name = p_foreignkeyname
AND ccu.table_name = p_foreigntable
AND ccu.column_name = 'id';

IF NOT FOUND THEN
RAISE NOTICE 'Upgrade table %.% - add constraint %', p_namespace, p_table, p_constraintname;
v_query := 'alter table ' || p_namespace || '.' || p_table || ' add constraint ';
v_query := v_query || p_constraintname || ' FOREIGN KEY (' || p_foreignkeyname || ') REFERENCES ' || p_foreigntable || '(id)';

IF p_deletecascade THEN
v_query := v_query || ' ON DELETE CASCADE ON UPDATE CASCADE;';
ELSE
v_query := v_query || ' ON DELETE SET NULL ON UPDATE CASCADE;';
END IF;

EXECUTE v_query;
RETURN 't';
ELSE
RETURN 'f';
END IF;
END;
$_$;


ALTER FUNCTION public.add_missing_constraint(text, text, text, text, text, boolean) OWNER TO webgfc;

--
-- Name: FUNCTION add_missing_constraint(text, text, text, text, text, boolean); Type: COMMENT; Schema: public; Owner: postgres
--

COMMENT ON FUNCTION public.add_missing_constraint(text, text, text, text, text, boolean) IS 'Add a constraint to a table if it is missing';


--
-- Name: add_missing_table_field(text, text, text, text); Type: FUNCTION; Schema: public; Owner: webgfc
--

CREATE OR REPLACE FUNCTION public.add_missing_table_field(text, text, text, text) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$
DECLARE
  p_namespace alias for $1;
  p_table     alias for $2;
  p_field     alias for $3;
  p_type      alias for $4;
  v_row       record;
  v_query     text;
BEGIN
  select 1 into v_row from pg_namespace n, pg_class c, pg_attribute a
     where
         --public.slon_quote_brute(n.nspname) = p_namespace and
         n.nspname = p_namespace and
         c.relnamespace = n.oid and
         --public.slon_quote_brute(c.relname) = p_table and
         c.relname = p_table and
         a.attrelid = c.oid and
         --public.slon_quote_brute(a.attname) = p_field;
         a.attname = p_field;
  if not found then
    raise notice 'Upgrade table %.% - add field %', p_namespace, p_table, p_field;
    v_query := 'alter table ' || p_namespace || '.' || p_table || ' add column ';
    v_query := v_query || p_field || ' ' || p_type || ';';
    execute v_query;
    return 't';
  else
    return 'f';
  end if;
END;$_$;


ALTER FUNCTION public.add_missing_table_field(text, text, text, text) OWNER TO webgfc;

--
-- Name: FUNCTION add_missing_table_field(text, text, text, text); Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON FUNCTION public.add_missing_table_field(text, text, text, text) IS 'Drops a column from a table if it exists.';


--
-- Name: alter_table_drop_column_if_exists(text, text, text); Type: FUNCTION; Schema: public; Owner: webgfc
--

CREATE OR REPLACE FUNCTION public.alter_table_drop_column_if_exists(text, text, text) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$
	DECLARE
		p_namespace alias for $1;
		p_table     alias for $2;
		p_field     alias for $3;
		v_row       record;
		v_query     text;
	BEGIN
		SELECT 1 INTO v_row FROM pg_namespace n, pg_class c, pg_attribute a
			WHERE
				n.nspname = p_namespace
				AND c.relnamespace = n.oid
				AND c.relname = p_table
				AND a.attrelid = c.oid
				AND a.attname = p_field;
		IF FOUND THEN
			RAISE NOTICE 'Upgrade table %.% - drop field %', p_namespace, p_table, p_field;
			v_query := 'ALTER TABLE ' || p_namespace || '.' || p_table || ' DROP column ' || p_field || ';';
			EXECUTE v_query;
			RETURN 't';
		ELSE
			RETURN 'f';
		END IF;
	END;
$_$;


ALTER FUNCTION public.alter_table_drop_column_if_exists(text, text, text) OWNER TO webgfc;

--
-- Name: alter_table_drop_constraint_if_exists(text, text, text); Type: FUNCTION; Schema: public; Owner: webgfc
--

CREATE OR REPLACE FUNCTION public.alter_table_drop_constraint_if_exists(text, text, text) RETURNS boolean
    LANGUAGE plpgsql
    AS $_$
	DECLARE
		p_namespace			alias for $1;
		p_table				alias for $2;
		p_constraintname    alias for $3;
		v_row				record;
		v_query				text;
	BEGIN
		SELECT
				1 INTO v_row
			FROM pg_constraint
				INNER JOIN pg_namespace ON ( pg_constraint.connamespace = pg_namespace.oid )
				INNER JOIN pg_class ON ( pg_constraint.conrelid = pg_class.oid )
			WHERE
				pg_namespace.nspname = p_namespace
				AND pg_class.relname = p_table
				AND pg_constraint.conname = p_constraintname;
		IF FOUND THEN
			RAISE NOTICE 'Alter table %.% - drop constraint %', p_namespace, p_table, p_constraintname;
			v_query := 'ALTER TABLE ' || p_namespace || '.' || p_table || ' DROP constraint ' || p_constraintname || ';';
			EXECUTE v_query;
			RETURN 't';
		ELSE
			RETURN 'f';
		END IF;
	END;
$_$;


ALTER FUNCTION public.alter_table_drop_constraint_if_exists(text, text, text) OWNER TO webgfc;

--
-- Name: FUNCTION alter_table_drop_constraint_if_exists(text, text, text); Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON FUNCTION public.alter_table_drop_constraint_if_exists(text, text, text) IS 'Équivalent de la fonctionnalité ALTER TABLE <table> DROP CONSTRAINT <name> disponible à partir de PostgreSQl 9.1';


--
-- Name: cakephp_validate_in_list(text, text[]); Type: FUNCTION; Schema: public; Owner: webgfc
--

CREATE OR REPLACE FUNCTION public.cakephp_validate_in_list(text, text[]) RETURNS boolean
    LANGUAGE sql IMMUTABLE
    AS $_$
	SELECT $1 IS NULL OR ( ARRAY[CAST($1 AS TEXT)] <@ CAST($2 AS TEXT[]) );
$_$;


ALTER FUNCTION public.cakephp_validate_in_list(text, text[]) OWNER TO webgfc;

--
-- Name: FUNCTION cakephp_validate_in_list(text, text[]); Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON FUNCTION public.cakephp_validate_in_list(text, text[]) IS '@see http://api.cakephp.org/class/validation#method-ValidationinList';


--
-- Name: noaccents_upper(text); Type: FUNCTION; Schema: public; Owner: webgfc
--

CREATE OR REPLACE FUNCTION public.noaccents_upper(text) RETURNS text
    LANGUAGE plpgsql IMMUTABLE STRICT
    AS $_$
    DECLARE
        st text;

    BEGIN
        -- On transforme les caractèes accentués et on passe en majuscule
        st:=translate($1,'aàäâeéèêëiïîoôöuùûücçñAÀÄÂEÉÈÊËIÏÎOÔÖUÙÛÜCÇÑ','AAAAEEEEEIIIOOOUUUUCCNAAAAEEEEEIIIOOOUUUUCCN');
        st:=upper(st);

        return st;
    END;
$_$;


ALTER FUNCTION public.noaccents_upper(text) OWNER TO webgfc;

--
-- Name: rebuilt_sequences(); Type: FUNCTION; Schema: public; Owner: webgfc
--

CREATE OR REPLACE FUNCTION public.rebuilt_sequences() RETURNS integer
    LANGUAGE plpgsql
    AS $$
  DECLARE sequencedefs RECORD; c integer ;
  BEGIN
    FOR sequencedefs IN Select
      constraint_column_usage.table_name as tablename,
      constraint_column_usage.table_name as tablename,
      constraint_column_usage.column_name as columnname,
      replace(replace(columns.column_default,'''::regclass)',''),'nextval(''','') as sequencename
      from information_schema.constraint_column_usage, information_schema.columns
      where constraint_column_usage.table_schema ='public' AND
      columns.table_schema = 'public' AND columns.table_name=constraint_column_usage.table_name
      AND constraint_column_usage.column_name = columns.column_name
      AND columns.column_default is not null
   LOOP
      EXECUTE 'select max('||sequencedefs.columnname||') from ' || sequencedefs.tablename INTO c;
      IF c is null THEN c = 0; END IF;
      IF c is not null THEN c = c+ 1; END IF;
      EXECUTE 'alter sequence ' || sequencedefs.sequencename ||' restart  with ' || c;
   END LOOP;

   RETURN 1; END;
$$;


ALTER FUNCTION public.rebuilt_sequences() OWNER TO webgfc;

SET default_tablespace = '';


--
-- Name: acos; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.acos (
    id integer NOT NULL,
    parent_id integer,
    model character varying(255) DEFAULT NULL::character varying,
    foreign_key integer,
    alias character varying(255) DEFAULT NULL::character varying,
    lft integer,
    rght integer
);


ALTER TABLE public.acos OWNER TO webgfc;

--
-- Name: acos_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.acos_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.acos_id_seq OWNER TO webgfc;

--
-- Name: acos_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.acos_id_seq OWNED BY public.acos.id;


--
-- Name: activites; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.activites (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    description character varying(255),
    active boolean DEFAULT true NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.activites OWNER TO webgfc;

--
-- Name: TABLE activites; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.activites IS 'Table permettant de stocker les activités liées aux contacts (SAERP)';


--
-- Name: activites_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.activites_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.activites_id_seq OWNER TO webgfc;

--
-- Name: activites_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.activites_id_seq OWNED BY public.activites.id;


--
-- Name: activites_organismes; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.activites_organismes (
    id integer NOT NULL,
    activite_id integer NOT NULL,
    organisme_id integer NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.activites_organismes OWNER TO webgfc;

--
-- Name: TABLE activites_organismes; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.activites_organismes IS 'Table de liaison entre les activités et les organismes (SAERP)';


--
-- Name: activites_organismes_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.activites_organismes_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.activites_organismes_id_seq OWNER TO webgfc;

--
-- Name: activites_organismes_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.activites_organismes_id_seq OWNED BY public.activites_organismes.id;


--
-- Name: addressbooks; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.addressbooks (
    id integer NOT NULL,
    name character varying(255),
    description bytea,
    created timestamp without time zone DEFAULT now(),
    modified timestamp without time zone DEFAULT now(),
    foreign_key integer,
    active boolean DEFAULT true NOT NULL,
    private boolean DEFAULT false
);


ALTER TABLE public.addressbooks OWNER TO webgfc;

--
-- Name: addressbooks_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.addressbooks_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.addressbooks_id_seq OWNER TO webgfc;

--
-- Name: addressbooks_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.addressbooks_id_seq OWNED BY public.addressbooks.id;


--
-- Name: affaires; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.affaires (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    comment bytea,
    dossier_id integer,
    created timestamp without time zone,
    modified timestamp without time zone
);


ALTER TABLE public.affaires OWNER TO webgfc;

--
-- Name: affaires_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.affaires_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.affaires_id_seq OWNER TO webgfc;

--
-- Name: affaires_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.affaires_id_seq OWNED BY public.affaires.id;


--
-- Name: armodels; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.armodels (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    gedpath text,
    content bytea,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL,
    soustype_id integer NOT NULL,
    sms_content character varying(160),
    email_content bytea,
    email_title character varying(255),
    format character varying(50),
    mime character varying(255),
    ext character varying(10),
    size integer,
    preview bytea
);


ALTER TABLE public.armodels OWNER TO webgfc;

--
-- Name: armodels_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.armodels_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.armodels_id_seq OWNER TO webgfc;

--
-- Name: armodels_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.armodels_id_seq OWNED BY public.armodels.id;


--
-- Name: aros; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.aros (
    id integer NOT NULL,
    parent_id integer,
    model character varying(255) DEFAULT NULL::character varying,
    foreign_key integer,
    alias character varying(255) DEFAULT NULL::character varying,
    lft integer,
    rght integer
);


ALTER TABLE public.aros OWNER TO webgfc;

--
-- Name: aros_acos; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.aros_acos (
    id integer NOT NULL,
    aro_id integer NOT NULL,
    aco_id integer NOT NULL,
    _create character varying(2) DEFAULT '0'::character varying(255) NOT NULL,
    _read character varying(2) DEFAULT '0'::character varying(255) NOT NULL,
    _update character varying(2) DEFAULT '0'::character varying(255) NOT NULL,
    _delete character varying(2) DEFAULT '0'::character varying(255) NOT NULL
);


ALTER TABLE public.aros_acos OWNER TO webgfc;

--
-- Name: aros_acos_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.aros_acos_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.aros_acos_id_seq OWNER TO webgfc;

--
-- Name: aros_acos_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.aros_acos_id_seq OWNED BY public.aros_acos.id;


--
-- Name: aros_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.aros_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.aros_id_seq OWNER TO webgfc;

--
-- Name: aros_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.aros_id_seq OWNED BY public.aros.id;


--
-- Name: ars; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.ars (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    gedpath text,
    content bytea,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL,
    courrier_id integer NOT NULL,
    format character varying(50),
    sms_dest character varying(20),
    sms_content character varying(160),
    email_title character varying(255),
    email_content bytea,
    ext character varying(10),
    mime character varying(255),
    used boolean DEFAULT false NOT NULL,
    size integer,
    email_dest character varying(255),
    preview bytea
);


ALTER TABLE public.ars OWNER TO webgfc;

--
-- Name: ars_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.ars_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.ars_id_seq OWNER TO webgfc;

--
-- Name: ars_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.ars_id_seq OWNED BY public.ars.id;


--
-- Name: bancontenus; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.bancontenus (
    id integer NOT NULL,
    bannette_id integer,
    courrier_id integer,
    created timestamp without time zone,
    modified timestamp without time zone,
    etat integer DEFAULT 1 NOT NULL,
    desktop_id integer,
    read boolean DEFAULT false NOT NULL,
    desktopmanager_id integer,
    user_id integer,
    pastell_id integer
);


ALTER TABLE public.bancontenus OWNER TO webgfc;

--
-- Name: bancontenus_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.bancontenus_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.bancontenus_id_seq OWNER TO webgfc;

--
-- Name: bancontenus_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.bancontenus_id_seq OWNED BY public.bancontenus.id;


--
-- Name: bannettes; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.bannettes (
    id integer NOT NULL,
    name character varying(255)
);


ALTER TABLE public.bannettes OWNER TO webgfc;

--
-- Name: bannettes_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.bannettes_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.bannettes_id_seq OWNER TO webgfc;

--
-- Name: bannettes_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.bannettes_id_seq OWNED BY public.bannettes.id;


--
-- Name: bans; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.bans (
    id integer NOT NULL,
    name character varying(100) NOT NULL,
    active boolean DEFAULT true,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.bans OWNER TO webgfc;

--
-- Name: TABLE bans; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.bans IS 'Table permettant le stockage des Bases Adresse Nationale';


--
-- Name: bans_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.bans_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.bans_id_seq OWNER TO webgfc;

--
-- Name: bans_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.bans_id_seq OWNED BY public.bans.id;


--
-- Name: bansadresses; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.bansadresses (
    id integer NOT NULL,
    bancommune_id integer NOT NULL,
    name character varying(255) NOT NULL,
    ident character varying(255) NOT NULL,
    nom_voie character varying(255),
    id_fantoir character varying(255),
    numero character varying(255),
    rep character varying(255),
    code_insee character varying(255),
    code_post character varying(255),
    alias character varying(255),
    nom_ld character varying(255),
    nom_afnor character varying(255),
    x character varying(255),
    y character varying(255),
    lon character varying(255),
    lat character varying(255),
    nom_commune character varying(255),
    active boolean DEFAULT true,
    canton character varying(255),
    code_insee_ancienne_commune character varying(255),
    libelle_acheminement character varying(255),
    nom_ancienne_commune character varying(255),
    source_position character varying(255),
    source_nom_voie character varying(255),
    date_der_maj timestamp without time zone,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.bansadresses OWNER TO webgfc;

--
-- Name: TABLE bansadresses; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.bansadresses IS 'Table permettant le stockage des adresses par commune de la Base Adresse Nationale';


--
-- Name: bansadresses_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.bansadresses_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.bansadresses_id_seq OWNER TO webgfc;

--
-- Name: bansadresses_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.bansadresses_id_seq OWNED BY public.bansadresses.id;


--
-- Name: banscommunes; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.banscommunes (
    id integer NOT NULL,
    ban_id integer NOT NULL,
    name character varying(255) NOT NULL,
    active boolean DEFAULT true,
    date_der_maj timestamp without time zone,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.banscommunes OWNER TO webgfc;

--
-- Name: TABLE banscommunes; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.banscommunes IS 'Table permettant le stockage des Communes par Base Adresse Nationale';


--
-- Name: banscommunes_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.banscommunes_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.banscommunes_id_seq OWNER TO webgfc;

--
-- Name: banscommunes_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.banscommunes_id_seq OWNED BY public.banscommunes.id;


--
-- Name: comments; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.comments (
    id integer NOT NULL,
    slug character varying(255),
    content bytea,
    private boolean DEFAULT false,
    owner_id integer NOT NULL,
    target_id integer NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL,
    objet text,
    read boolean DEFAULT false
);


ALTER TABLE public.comments OWNER TO webgfc;

--
-- Name: comments_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.comments_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.comments_id_seq OWNER TO webgfc;

--
-- Name: comments_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.comments_id_seq OWNED BY public.comments.id;


--
-- Name: comments_readers; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.comments_readers (
    id integer NOT NULL,
    reader_id integer NOT NULL,
    comment_id integer NOT NULL,
    read boolean DEFAULT false,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.comments_readers OWNER TO webgfc;

--
-- Name: comments_readers_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.comments_readers_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.comments_readers_id_seq OWNER TO webgfc;

--
-- Name: comments_readers_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.comments_readers_id_seq OWNED BY public.comments_readers.id;


--
-- Name: compteurs; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.compteurs (
    id integer NOT NULL,
    nom character varying(255) NOT NULL,
    commentaire character varying(255) NOT NULL,
    def_compteur character varying(255) NOT NULL,
    sequence_id integer NOT NULL,
    def_reinit character varying(255) NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone,
    val_reinit character varying(255)
);


ALTER TABLE public.compteurs OWNER TO webgfc;

--
-- Name: compteurs_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.compteurs_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.compteurs_id_seq OWNER TO webgfc;

--
-- Name: compteurs_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.compteurs_id_seq OWNED BY public.compteurs.id;


--
-- Name: connecteurs; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.connecteurs (
    id integer NOT NULL,
    name character varying(250) DEFAULT NULL::character varying,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL,
    use_signature boolean,
    signature_protocol character varying(30),
    host character varying(255),
    login character varying(255),
    pwd character varying(255),
    type character varying(255),
    visibility character varying(255),
    wsdl character varying(255),
    cacert bytea,
    clientcert bytea,
    cert bytea,
    certpwd bytea,
    nom_cert text,
    use_ged boolean,
    ged_url character varying(255),
    ged_login character varying(255),
    ged_passwd character varying(255),
    ged_repo character varying(255),
    ged_xml_version character varying(1),
    ged_cmis_version character varying(10),
    use_ldap boolean DEFAULT false,
    ldap_active boolean DEFAULT false,
    ldap_type character varying(50) DEFAULT 'OpenLDAP'::character varying,
    ldap_host character varying(255) DEFAULT 'localhost'::character varying,
    ldap_port character varying(5) DEFAULT '389'::character varying,
    ldap_uniqueid character varying(255) DEFAULT 'UID'::character varying,
    ldap_base_dn character varying(255),
    ldap_account_suffix character varying(255),
    ldap_dn character varying(255) DEFAULT 'dn'::character varying,
    ldap_has_fall_over text,
    ldap_login text,
    ldap_password character varying(255),
    ldap_tls boolean,
    ldap_version text,
    ldap_use_ad boolean,
    ldap_username character varying(100),
    ldap_note character varying(100),
    ldap_nom character varying(100),
    ldap_prenom character varying(100),
    ldap_mail character varying(100),
    ldap_numtel character varying(100),
    ldap_portable character varying(100),
    use_grc boolean DEFAULT false,
    grc_active boolean DEFAULT false,
    grc_apikey character varying(255),
    grc_appname character varying(255),
    grc_host character varying(255),
    grc_cityid character varying(255),
    grc_canalid character varying(255),
    grc_servid character varying(255),
    grc_sqrtid character varying(255),
    grc_qrtid character varying(255),
    use_pastell boolean,
    pastell_parapheur_type character varying(255),
    id_entity integer,
    json_pastell text,
    type_doc_pastell character varying,
    use_mailsec boolean,
    use_ds boolean,
    ds_active boolean,
    ds_apikey character varying(255),
    ds_procedure_id integer,
    ds_dossier_id integer,
    ds_desktopmanager_id integer,
    ldaps_cert bytea,
    ldaps_nom_cert character varying(255),
    use_sms boolean,
    sms_active boolean,
    sms_apikey character varying(255),
    sms_expediteur character varying(11),
    use_dm boolean,
    dm_active boolean,
    dm_apikey character varying(255),
    dm_usermail character varying(255),
    dm_userpassword character varying(255),
    dm_desktopmanager_id integer
);


ALTER TABLE public.connecteurs OWNER TO webgfc;

--
-- Name: TABLE connecteurs; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.connecteurs IS 'Table permettant la définition des différents connecteurs présents';


--
-- Name: connecteurs_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.connecteurs_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.connecteurs_id_seq OWNER TO webgfc;

--
-- Name: connecteurs_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.connecteurs_id_seq OWNED BY public.connecteurs.id;


--
-- Name: consultations; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.consultations (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    numero character varying(10) NOT NULL,
    objet character varying(255),
    datelimite date NOT NULL,
    heurelimite time without time zone NOT NULL,
    operation_id integer NOT NULL,
    active boolean DEFAULT true NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.consultations OWNER TO webgfc;

--
-- Name: TABLE consultations; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.consultations IS 'Table permettant de stocker les informations en lien avec les consultations (SAERP)';


--
-- Name: consultations_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.consultations_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.consultations_id_seq OWNER TO webgfc;

--
-- Name: consultations_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.consultations_id_seq OWNED BY public.consultations.id;


--
-- Name: contactinfos; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.contactinfos (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    civilite character varying(255),
    nom character varying(255) NOT NULL,
    prenom character varying(255),
    email character varying(255),
    adresse character varying(255),
    compl character varying(255),
    cp character varying(5),
    ville character varying(255),
    region character varying(255),
    pays character varying(255),
    tel character varying(20),
    role character varying(255),
    organisation character varying(255),
    active boolean DEFAULT true NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL,
    contact_id integer NOT NULL,
    slug character varying,
    numvoie character varying(20),
    typevoie character varying(4),
    nomvoie character varying(25),
    adressecomplete character varying(255),
    portable character varying(20),
    ban_id integer,
    bancommune_id integer,
    banadresse character varying(250)
);


ALTER TABLE public.contactinfos OWNER TO webgfc;

--
-- Name: contactinfos_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.contactinfos_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.contactinfos_id_seq OWNER TO webgfc;

--
-- Name: contactinfos_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.contactinfos_id_seq OWNED BY public.contactinfos.id;


--
-- Name: contacts; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.contacts (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    civilite character varying(255),
    nom character varying(255) NOT NULL,
    prenom character varying(255),
    created timestamp without time zone DEFAULT now(),
    modified timestamp without time zone DEFAULT now(),
    addressbook_id integer,
    foreign_key integer,
    active boolean DEFAULT true NOT NULL,
    citoyen boolean DEFAULT true NOT NULL,
    slug character varying,
    organisme_id integer NOT NULL,
    email character varying(255),
    adresse text,
    tel character varying(20),
    portable character varying(20),
    role character varying(255),
    ville character varying(255),
    ban_id integer,
    bancommune_id integer,
    banadresse character varying(255),
    numvoie character varying(255),
    compl character varying(255),
    cp character varying(5),
    nomvoie character varying(255),
    adressecomplete character varying(255),
    actif boolean DEFAULT true NOT NULL,
    pays character varying(50) DEFAULT 'FRANCE'::character varying,
    lignedirecte character varying(20),
    observations text,
    fonction_id integer,
    contactgrc_id INTEGER,
    titre character varying(255),
    canton character varying(255),
    titre_id INTEGER
);


ALTER TABLE public.contacts OWNER TO webgfc;

--
-- Name: contacts_events; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.contacts_events (
    id integer NOT NULL,
    contact_id integer NOT NULL,
    event_id integer NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.contacts_events OWNER TO webgfc;

--
-- Name: TABLE contacts_events; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.contacts_events IS 'Table de liaison entre les contacts et les événements(SAERP)';


--
-- Name: contacts_events_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.contacts_events_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.contacts_events_id_seq OWNER TO webgfc;

--
-- Name: contacts_events_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.contacts_events_id_seq OWNED BY public.contacts_events.id;


--
-- Name: contacts_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.contacts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.contacts_id_seq OWNER TO webgfc;

--
-- Name: contacts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.contacts_id_seq OWNED BY public.contacts.id;


--
-- Name: contacts_operations; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.contacts_operations (
    id integer NOT NULL,
    contact_id integer NOT NULL,
    operation_id integer NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.contacts_operations OWNER TO webgfc;

--
-- Name: TABLE contacts_operations; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.contacts_operations IS 'Table de liaison entre les contacts et les opérations (SAERP)';


--
-- Name: contacts_operations_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.contacts_operations_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.contacts_operations_id_seq OWNER TO webgfc;

--
-- Name: contacts_operations_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.contacts_operations_id_seq OWNED BY public.contacts_operations.id;


--
-- Name: contractants; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.contractants (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    description character varying(255),
    active boolean DEFAULT true NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.contractants OWNER TO webgfc;

--
-- Name: TABLE contractants; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.contractants IS 'Table permettant de définir les types de contractant pour les marchés (SAERP)';


--
-- Name: contractants_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.contractants_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.contractants_id_seq OWNER TO webgfc;

--
-- Name: contractants_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.contractants_id_seq OWNED BY public.contractants.id;



--
-- Name: courriers; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.courriers (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    fichier_associe character varying(255),
    objet text,
    ref_ancien integer,
    soustype_id integer,
    lft integer,
    rght integer,
    parent_id integer,
    affaire_id integer,
    adressemail character varying(255),
    wav_associe character varying(255),
    video_associe character varying(255),
    numtelsms character varying(15),
    messagesms character varying(160),
    reference character varying(255),
    fichier_genere character varying(255),
    priorite integer DEFAULT 0 NOT NULL,
    user_creator_id integer,
    desktop_creator_id integer,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL,
    date date DEFAULT now() NOT NULL,
    datereception date DEFAULT now() NOT NULL,
    direction integer,
    contactinfo_id integer,
    service_creator_id integer,
    delai_nb smallint,
    delai_unite smallint,
    affairesuiviepar_id integer,
    parapheur_etat integer,
    parapheur_id character varying(50),
    parapheur_commentaire text,
    parapheur_cible character varying(250),
    signee boolean DEFAULT false,
    motifrefus text,
    dateenvoiparapheur timestamp without time zone,
    mail_retard_envoye boolean DEFAULT false,
    numero_depot integer,
    origineflux_id integer,
    typecontact character varying(20),
    contact_id integer,
    contact integer,
    organisme_id integer,
    marche_id integer,
    typedocument character varying(50),
    consultation_id integer,
    marcheconsult character varying(20),
    mail_clos_envoye boolean DEFAULT false,
    contactgrc_id INTEGER,
    courriergrc_id INTEGER,
    statutgrc character varying(10),
    pastell_id character varying(7),
    ds_procedure_id integer,
    ds_dossier_id integer,
    dm_issue_id integer,
    mail_bloque_parapheur boolean DEFAULT false,
    dateenvoipastell timestamp without time zone,
	deletedpastell_id character varying(7),
	pastell_etat integer  DEFAULT 0,
	pastell_typedoc character varying(255)
);


ALTER TABLE public.courriers OWNER TO webgfc;

--
-- Name: courriers_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.courriers_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.courriers_id_seq OWNER TO webgfc;

--
-- Name: courriers_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.courriers_id_seq OWNED BY public.courriers.id;



--
-- Name: courriers_metadonnees; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.courriers_metadonnees (
    id integer NOT NULL,
    valeur character varying(255),
    created timestamp without time zone,
    modified timestamp without time zone,
    courrier_id integer NOT NULL,
    metadonnee_id integer NOT NULL
);


ALTER TABLE public.courriers_metadonnees OWNER TO webgfc;

--
-- Name: courriers_metadonnees_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.courriers_metadonnees_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.courriers_metadonnees_id_seq OWNER TO webgfc;

--
-- Name: courriers_metadonnees_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.courriers_metadonnees_id_seq OWNED BY public.courriers_metadonnees.id;


--
-- Name: courriers_reference_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.courriers_reference_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.courriers_reference_seq OWNER TO webgfc;

--
-- Name: courriers_repertoires; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.courriers_repertoires (
    id integer NOT NULL,
    courrier_id integer,
    repertoire_id integer
);


ALTER TABLE public.courriers_repertoires OWNER TO webgfc;

--
-- Name: courriers_repertoires_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.courriers_repertoires_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.courriers_repertoires_id_seq OWNER TO webgfc;

--
-- Name: courriers_repertoires_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.courriers_repertoires_id_seq OWNED BY public.courriers_repertoires.id;


--
-- Name: dacos; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.dacos (
    id integer NOT NULL,
    parent_id integer,
    model character varying(255),
    foreign_key integer,
    alias character varying(255),
    lft integer,
    rght integer
);


ALTER TABLE public.dacos OWNER TO webgfc;

--
-- Name: dacos_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.dacos_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.dacos_id_seq OWNER TO webgfc;

--
-- Name: dacos_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.dacos_id_seq OWNED BY public.dacos.id;


--
-- Name: daros; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.daros (
    id integer NOT NULL,
    parent_id integer,
    model character varying(255),
    foreign_key integer,
    alias character varying(255),
    lft integer,
    rght integer
);


ALTER TABLE public.daros OWNER TO webgfc;

--
-- Name: daros_dacos; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.daros_dacos (
    id integer NOT NULL,
    daro_id integer NOT NULL,
    daco_id integer NOT NULL,
    _create character(2) DEFAULT 0 NOT NULL,
    _read character(2) DEFAULT 0 NOT NULL,
    _update character(2) DEFAULT 0 NOT NULL,
    _delete character(2) DEFAULT 0 NOT NULL
);


ALTER TABLE public.daros_dacos OWNER TO webgfc;

--
-- Name: daros_dacos_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.daros_dacos_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.daros_dacos_id_seq OWNER TO webgfc;

--
-- Name: daros_dacos_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.daros_dacos_id_seq OWNED BY public.daros_dacos.id;


--
-- Name: daros_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.daros_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.daros_id_seq OWNER TO webgfc;

--
-- Name: daros_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.daros_id_seq OWNED BY public.daros.id;


--
-- Name: desktops; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.desktops (
    name character varying(255) NOT NULL,
    profil_id integer NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL,
    id integer NOT NULL,
    active boolean DEFAULT true NOT NULL
);


ALTER TABLE public.desktops OWNER TO webgfc;

--
-- Name: desktops_desktopsmanagers; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.desktops_desktopsmanagers (
    id integer NOT NULL,
    desktop_id integer,
    desktopmanager_id integer,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.desktops_desktopsmanagers OWNER TO webgfc;

--
-- Name: desktops_desktopsmanagers_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.desktops_desktopsmanagers_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.desktops_desktopsmanagers_id_seq OWNER TO webgfc;

--
-- Name: desktops_desktopsmanagers_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.desktops_desktopsmanagers_id_seq OWNED BY public.desktops_desktopsmanagers.id;


--
-- Name: desktops_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.desktops_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.desktops_id_seq OWNER TO webgfc;

--
-- Name: desktops_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.desktops_id_seq OWNED BY public.desktops.id;


--
-- Name: desktops_services; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.desktops_services (
    id integer NOT NULL,
    service_id integer NOT NULL,
    desktop_id integer NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.desktops_services OWNER TO webgfc;

--
-- Name: desktops_services_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.desktops_services_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.desktops_services_id_seq OWNER TO webgfc;

--
-- Name: desktops_services_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.desktops_services_id_seq OWNED BY public.desktops_services.id;


--
-- Name: desktops_taches; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.desktops_taches (
    id integer NOT NULL,
    desktop_id integer NOT NULL,
    tache_id integer NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.desktops_taches OWNER TO webgfc;

--
-- Name: desktops_taches_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.desktops_taches_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.desktops_taches_id_seq OWNER TO webgfc;

--
-- Name: desktops_taches_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.desktops_taches_id_seq OWNED BY public.desktops_taches.id;


--
-- Name: desktops_users; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.desktops_users (
    id integer NOT NULL,
    user_id integer NOT NULL,
    desktop_id integer NOT NULL,
    delegation boolean DEFAULT false,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.desktops_users OWNER TO webgfc;

--
-- Name: desktops_users_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.desktops_users_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.desktops_users_id_seq OWNER TO webgfc;

--
-- Name: desktops_users_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.desktops_users_id_seq OWNED BY public.desktops_users.id;


--
-- Name: desktopsmanagers; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.desktopsmanagers (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    active boolean DEFAULT true NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL,
    isdispatch boolean,
    intituleagent_id integer,
    parent_id integer,
    profil_id integer,
	isautocreated boolean DEFAULT false NOT NULL
);


ALTER TABLE public.desktopsmanagers OWNER TO webgfc;

--
-- Name: desktopsmanagers_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.desktopsmanagers_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.desktopsmanagers_id_seq OWNER TO webgfc;

--
-- Name: desktopsmanagers_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.desktopsmanagers_id_seq OWNED BY public.desktopsmanagers.id;


--
-- Name: desktopsmanagers_sousoperations; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE public.desktopsmanagers_sousoperations (
    id integer NOT NULL,
    desktopmanager_id integer,
    sousoperation_id integer NOT NULL,
    ordre character varying(10) NOT NULL,
    intituleagent_id integer NOT NULL,
    active boolean DEFAULT true NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.desktopsmanagers_sousoperations OWNER TO webgfc;

--
-- Name: TABLE desktopsmanagers_sousoperations; Type: COMMENT; Schema: public; Owner: postgres
--

COMMENT ON TABLE public.desktopsmanagers_sousoperations IS 'Table de liaison entre les bureaux et les opérations (SAERP)';


--
-- Name: desktopsmanagers_sousoperations_id_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--

CREATE SEQUENCE public.desktopsmanagers_sousoperations_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.desktopsmanagers_sousoperations_id_seq OWNER TO webgfc;

--
-- Name: desktopsmanagers_sousoperations_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: postgres
--

ALTER SEQUENCE public.desktopsmanagers_sousoperations_id_seq OWNED BY public.desktopsmanagers_sousoperations.id;


--
-- Name: documents; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.documents (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    gedpath text,
    content bytea,
    size integer NOT NULL,
    mime character varying(255),
    preview bytea,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL,
    courrier_id integer,
    ext character varying(10),
    main_doc boolean DEFAULT false NOT NULL,
    parapheur_bordereau bytea,
    courrier_pdf bytea,
	signee boolean DEFAULT false,
    signature bytea,
    desktop_creator_id integer,
    path text,
    ishistorique BOOLEAN,
    ocr_data bytea,
    asigner boolean DEFAULT false
);


ALTER TABLE public.documents OWNER TO webgfc;

--
-- Name: documents_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.documents_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.documents_id_seq OWNER TO webgfc;

--
-- Name: documents_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.documents_id_seq OWNED BY public.documents.id;


--
-- Name: documents_name_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.documents_name_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.documents_name_seq OWNER TO webgfc;

--
-- Name: dossiers; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.dossiers (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    comment bytea,
    created timestamp without time zone,
    modified timestamp without time zone
);


ALTER TABLE public.dossiers OWNER TO webgfc;

--
-- Name: dossiers_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.dossiers_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.dossiers_id_seq OWNER TO webgfc;

--
-- Name: dossiers_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.dossiers_id_seq OWNED BY public.dossiers.id;


--
-- Name: events; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.events (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    description character varying(255),
    active boolean DEFAULT true NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.events OWNER TO webgfc;

--
-- Name: TABLE events; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.events IS 'Table permettant de stocker les événements liés aux contacts (SAERP)';


--
-- Name: events_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.events_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.events_id_seq OWNER TO webgfc;

--
-- Name: events_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.events_id_seq OWNED BY public.events.id;


--
-- Name: fonctions; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.fonctions (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    description character varying(255),
    active boolean DEFAULT true NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.fonctions OWNER TO webgfc;

--
-- Name: TABLE fonctions; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.fonctions IS 'Table permettant de stocker les fonctions liées aux contacts (SAERP)';


--
-- Name: fonctions_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.fonctions_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.fonctions_id_seq OWNER TO webgfc;

--
-- Name: fonctions_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.fonctions_id_seq OWNED BY public.fonctions.id;


--
-- Name: formatsreponse; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.formatsreponse (
    id integer NOT NULL,
    name character varying(255) NOT NULL
);


ALTER TABLE public.formatsreponse OWNER TO webgfc;

--
-- Name: TABLE formatsreponse; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.formatsreponse IS 'Table pour les formats de réponse';


--
-- Name: formatsreponse_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.formatsreponse_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.formatsreponse_id_seq OWNER TO webgfc;

--
-- Name: formatsreponse_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.formatsreponse_id_seq OWNED BY public.formatsreponse.id;


--
-- Name: formatsreponse_soustypes; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.formatsreponse_soustypes (
    id integer NOT NULL,
    formatreponse_id integer NOT NULL,
    soustype_id integer NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone
);


ALTER TABLE public.formatsreponse_soustypes OWNER TO webgfc;

--
-- Name: TABLE formatsreponse_soustypes; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.formatsreponse_soustypes IS 'Table pour lier les sous-types aux différents formats de réponse';


--
-- Name: formatsreponse_soustypes_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.formatsreponse_soustypes_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.formatsreponse_soustypes_id_seq OWNER TO webgfc;

--
-- Name: formatsreponse_soustypes_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.formatsreponse_soustypes_id_seq OWNED BY public.formatsreponse_soustypes.id;


--
-- Name: gabaritsdocuments; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE public.gabaritsdocuments (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    content bytea,
    format character varying(50),
    mime character varying(255),
    ext character varying(255),
    size character varying(255),
    preview bytea,
    active boolean DEFAULT true NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.gabaritsdocuments OWNER TO webgfc;

--
-- Name: TABLE gabaritsdocuments; Type: COMMENT; Schema: public; Owner: postgres
--

COMMENT ON TABLE public.gabaritsdocuments IS 'Table permettant de stocker les gabarits de documents (SAERP)';


--
-- Name: gabaritsdocuments_id_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--

CREATE SEQUENCE public.gabaritsdocuments_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.gabaritsdocuments_id_seq OWNER TO webgfc;

--
-- Name: gabaritsdocuments_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: postgres
--

ALTER SEQUENCE public.gabaritsdocuments_id_seq OWNED BY public.gabaritsdocuments.id;


--
-- Name: profils; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.profils (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone,
    readonly boolean DEFAULT false NOT NULL,
    active boolean DEFAULT true NOT NULL,
    profil_id integer
);


ALTER TABLE public.profils OWNER TO webgfc;

--
-- Name: profils_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.profils_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.profils_id_seq OWNER TO webgfc;

--
-- Name: profils_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.profils_id_seq OWNED BY public.profils.id;


--
-- Name: inits_services; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.inits_services (
    id integer NOT NULL,
    desktop_id integer NOT NULL,
    service_id integer NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.inits_services OWNER TO webgfc;

--
-- Name: inits_services_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.inits_services_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.inits_services_id_seq OWNER TO webgfc;

--
-- Name: inits_services_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.inits_services_id_seq OWNED BY public.inits_services.id;


--
-- Name: intituleagentbyoperations; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE public.intituleagentbyoperations (
    id integer NOT NULL,
    intituleagent_id integer NOT NULL,
    operation_id integer NOT NULL,
    desktopmanager_id integer NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.intituleagentbyoperations OWNER TO webgfc;

--
-- Name: TABLE intituleagentbyoperations; Type: COMMENT; Schema: public; Owner: postgres
--

COMMENT ON TABLE public.intituleagentbyoperations IS 'Table permettant de définir les bureaux pour un rôle donné dans une OP (SAERP)';


--
-- Name: intituleagentbyoperations_id_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--

CREATE SEQUENCE public.intituleagentbyoperations_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.intituleagentbyoperations_id_seq OWNER TO webgfc;

--
-- Name: intituleagentbyoperations_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: postgres
--

ALTER SEQUENCE public.intituleagentbyoperations_id_seq OWNED BY public.intituleagentbyoperations.id;


--
-- Name: intitulesagents; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.intitulesagents (
    id integer NOT NULL,
    name character varying(4) NOT NULL,
    description character varying(255),
    active boolean DEFAULT true NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.intitulesagents OWNER TO webgfc;

--
-- Name: TABLE intitulesagents; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.intitulesagents IS 'Table permettant de définir les intitulés d''agents (SAERP)';


--
-- Name: intitulesagents_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.intitulesagents_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.intitulesagents_id_seq OWNER TO webgfc;

--
-- Name: intitulesagents_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.intitulesagents_id_seq OWNED BY public.intitulesagents.id;


--
-- Name: journalevents; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.journalevents (
    id integer NOT NULL,
    user_id integer NOT NULL,
    username character varying(255) NOT NULL,
    date timestamp without time zone DEFAULT now() NOT NULL,
    desktop_id integer NOT NULL,
    courrier_id integer,
    reference character varying(255),
    action character varying(255) NOT NULL,
    level character varying(255) NOT NULL,
    message text NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.journalevents OWNER TO webgfc;

--
-- Name: TABLE journalevents; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.journalevents IS 'Table permettant de stocker les logs utilisateurs par actions';


--
-- Name: journalevents_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.journalevents_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.journalevents_id_seq OWNER TO webgfc;

--
-- Name: journalevents_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.journalevents_id_seq OWNED BY public.journalevents.id;


--
-- Name: keywordlists; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.keywordlists (
    id integer NOT NULL,
    nom character varying(255) NOT NULL,
    description bytea,
    version integer NOT NULL,
    active boolean DEFAULT true NOT NULL,
    keywordtype_id integer NOT NULL,
    scheme_id character varying(1024),
    scheme_name character varying(1024),
    scheme_agency_name character varying(1024),
    scheme_version_id character varying(1024),
    scheme_data_uri character varying(1024),
    scheme_uri character varying(1024),
    created_user_id integer,
    modified_user_id integer,
    created timestamp without time zone DEFAULT now(),
    modified timestamp without time zone DEFAULT now()
);


ALTER TABLE public.keywordlists OWNER TO webgfc;

--
-- Name: COLUMN keywordlists.scheme_id; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON COLUMN public.keywordlists.scheme_id IS 'Champ correpondant aux normes SEDA';


--
-- Name: COLUMN keywordlists.scheme_name; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON COLUMN public.keywordlists.scheme_name IS 'Champ correpondant aux normes SEDA';


--
-- Name: COLUMN keywordlists.scheme_agency_name; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON COLUMN public.keywordlists.scheme_agency_name IS 'Champ correpondant aux normes SEDA';


--
-- Name: COLUMN keywordlists.scheme_version_id; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON COLUMN public.keywordlists.scheme_version_id IS 'Champ correpondant aux normes SEDA';


--
-- Name: COLUMN keywordlists.scheme_data_uri; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON COLUMN public.keywordlists.scheme_data_uri IS 'Champ correpondant aux normes SEDA';


--
-- Name: COLUMN keywordlists.scheme_uri; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON COLUMN public.keywordlists.scheme_uri IS 'Champ correpondant aux normes SEDA';


--
-- Name: keywordlists_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.keywordlists_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.keywordlists_id_seq OWNER TO webgfc;

--
-- Name: keywordlists_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.keywordlists_id_seq OWNED BY public.keywordlists.id;


--
-- Name: keywords; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.keywords (
    id integer NOT NULL,
    version integer NOT NULL,
    code character varying(255) NOT NULL,
    libelle bytea,
    parent_id integer,
    lft integer,
    rght integer,
    keywordlist_id integer NOT NULL,
    created_user_id integer,
    modified_user_id integer,
    created timestamp without time zone DEFAULT now(),
    modified timestamp without time zone DEFAULT now()
);


ALTER TABLE public.keywords OWNER TO webgfc;

--
-- Name: keywords_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.keywords_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.keywords_id_seq OWNER TO webgfc;

--
-- Name: keywords_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.keywords_id_seq OWNED BY public.keywords.id;


--
-- Name: keywordtypes; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.keywordtypes (
    id integer NOT NULL,
    code character varying(10) NOT NULL,
    documentation character varying(250) NOT NULL,
    created timestamp without time zone DEFAULT now(),
    modified timestamp without time zone DEFAULT now()
);


ALTER TABLE public.keywordtypes OWNER TO webgfc;

--
-- Name: keywordtypes_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.keywordtypes_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.keywordtypes_id_seq OWNER TO webgfc;

--
-- Name: keywordtypes_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.keywordtypes_id_seq OWNED BY public.keywordtypes.id;


--
-- Name: ldapm_groups; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.ldapm_groups (
    id integer NOT NULL,
    parent_id integer,
    lft integer,
    rght integer,
    name character varying(500),
    readonly boolean,
    active boolean,
    dn text,
    cn text
);


ALTER TABLE public.ldapm_groups OWNER TO webgfc;

--
-- Name: ldapm_groups_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.ldapm_groups_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.ldapm_groups_id_seq OWNER TO webgfc;

--
-- Name: ldapm_groups_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.ldapm_groups_id_seq OWNED BY public.ldapm_groups.id;


--
-- Name: ldapm_models_groups; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.ldapm_models_groups (
    id integer NOT NULL,
    ldapm_groups_id integer,
    model character varying(255) DEFAULT NULL::character varying,
    foreign_key integer
);


ALTER TABLE public.ldapm_models_groups OWNER TO webgfc;

--
-- Name: ldapm_models_groups_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.ldapm_models_groups_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.ldapm_models_groups_id_seq OWNER TO webgfc;

--
-- Name: ldapm_models_groups_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.ldapm_models_groups_id_seq OWNED BY public.ldapm_models_groups.id;


--
-- Name: marches; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.marches (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    numero character varying(10) NOT NULL,
    objet character varying(255),
    datelimite date NOT NULL,
    heurelimite time without time zone NOT NULL,
    operation_id integer NOT NULL,
    chrono character varying(50),
    titulaire character varying(255),
    active boolean DEFAULT true NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL,
    typecontractant character varying(255),
    datenotification date,
    premiermois character varying(255),
    uniterif character varying(255),
    contractant_id integer,
    uniterif_id integer
);


ALTER TABLE public.marches OWNER TO webgfc;

--
-- Name: TABLE marches; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.marches IS 'Table permettant de stocker les informations en lien avec les marchés (SAERP)';


--
-- Name: marches_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.marches_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.marches_id_seq OWNER TO webgfc;

--
-- Name: marches_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.marches_id_seq OWNED BY public.marches.id;


--
-- Name: metadonnees; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.metadonnees (
    id integer NOT NULL,
    name character varying(255),
    typemetadonnee_id integer NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone,
    champfusion character varying(255),
    active boolean,
    isrepercutable boolean
);


ALTER TABLE public.metadonnees OWNER TO webgfc;

--
-- Name: metadonnees_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.metadonnees_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.metadonnees_id_seq OWNER TO webgfc;

--
-- Name: metadonnees_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.metadonnees_id_seq OWNED BY public.metadonnees.id;


--
-- Name: metadonnees_soustypes; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.metadonnees_soustypes (
    id integer NOT NULL,
    metadonnee_id integer NOT NULL,
    soustype_id integer NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL,
    obligatoire boolean
);


ALTER TABLE public.metadonnees_soustypes OWNER TO webgfc;

--
-- Name: TABLE metadonnees_soustypes; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.metadonnees_soustypes IS 'Table de liaison entre les métadonnées et les sous types';


--
-- Name: metadonnees_soustypes_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.metadonnees_soustypes_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.metadonnees_soustypes_id_seq OWNER TO webgfc;

--
-- Name: metadonnees_soustypes_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.metadonnees_soustypes_id_seq OWNED BY public.metadonnees_soustypes.id;


--
-- Name: notifications; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.notifications (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    description bytea,
    typenotif_id integer NOT NULL,
    desktop_id integer NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.notifications OWNER TO webgfc;

--
-- Name: notifications_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.notifications_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.notifications_id_seq OWNER TO webgfc;

--
-- Name: notifications_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.notifications_id_seq OWNED BY public.notifications.id;


--
-- Name: notifieddesktops; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.notifieddesktops (
    id integer NOT NULL,
    courrier_id integer,
    notification_id integer NOT NULL,
    desktop_id integer NOT NULL,
    read boolean DEFAULT false NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.notifieddesktops OWNER TO webgfc;

--
-- Name: notifieddesktops_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.notifieddesktops_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.notifieddesktops_id_seq OWNER TO webgfc;

--
-- Name: notifieddesktops_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.notifieddesktops_id_seq OWNED BY public.notifieddesktops.id;


--
-- Name: notifquotidiens; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.notifquotidiens (
    id integer NOT NULL,
    notification_id integer,
    courrier_id integer NOT NULL,
    desktop_id integer,
    user_id integer NOT NULL,
    name character varying(255) NOT NULL,
    description character varying(255),
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.notifquotidiens OWNER TO webgfc;

--
-- Name: TABLE notifquotidiens; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.notifquotidiens IS 'Table permettant de stocker les notifications quotidiennes';


--
-- Name: notifquotidiens_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.notifquotidiens_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.notifquotidiens_id_seq OWNER TO webgfc;

--
-- Name: notifquotidiens_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.notifquotidiens_id_seq OWNED BY public.notifquotidiens.id;


--
-- Name: operations; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.operations (
    id integer NOT NULL,
    name character varying(4) NOT NULL,
    nomoperation character varying(255) NOT NULL,
    ville character varying(255) NOT NULL,
    numvoie character varying(20),
    nomvoie character varying(255),
    compl character varying(255),
    codepostal character varying(5),
    commune character varying(255),
    numconvention character varying(255),
    nature character varying(255),
    lieu character varying(255),
    active boolean DEFAULT true NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL,
    type_id integer
);


ALTER TABLE public.operations OWNER TO webgfc;

--
-- Name: TABLE operations; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.operations IS 'Table permettant de définir les opérations (SAERP)';


--
-- Name: operations_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.operations_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.operations_id_seq OWNER TO webgfc;

--
-- Name: operations_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.operations_id_seq OWNED BY public.operations.id;


--
-- Name: ordresservices; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE public.ordresservices (
    id integer NOT NULL,
    name character varying(255),
    marche_id integer NOT NULL,
    numero integer,
    objet character varying(255),
    montant character varying(10),
    active boolean DEFAULT true NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL,
    courrier_id integer,
    organisme_id integer
);


ALTER TABLE public.ordresservices OWNER TO webgfc;

--
-- Name: TABLE ordresservices; Type: COMMENT; Schema: public; Owner: postgres
--

COMMENT ON TABLE public.ordresservices IS 'Table permettant de stocker les ordres de service (SAERP)';


--
-- Name: ordresservices_id_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--

CREATE SEQUENCE public.ordresservices_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.ordresservices_id_seq OWNER TO webgfc;

--
-- Name: ordresservices_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: postgres
--

ALTER SEQUENCE public.ordresservices_id_seq OWNED BY public.ordresservices.id;


--
-- Name: organismes; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.organismes (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    addressbook_id integer NOT NULL,
    ban_id integer,
    bancommune_id integer,
    banadresse character varying(255),
    numvoie character varying(255),
    adressecomplete character varying(255),
    compl character varying(255),
    cp character varying(5),
    email character varying(255),
    tel character varying(20),
    portable character varying(20),
    fax character varying(20),
    slug character varying(255),
    active boolean DEFAULT true NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL,
    infoscontact boolean,
    ville character varying(255),
    nomvoie character varying(255),
    pays character varying(50) DEFAULT 'FRANCE'::character varying,
    antenne character varying(255),
    siret character varying(255),
    contactgrc_id INTEGER,
    organismegrc_id INTEGER,
    canton character varying(255)
);


ALTER TABLE public.organismes OWNER TO webgfc;

--
-- Name: organismes_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.organismes_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.organismes_id_seq OWNER TO webgfc;

--
-- Name: organismes_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.organismes_id_seq OWNED BY public.organismes.id;


--
-- Name: organismes_operations; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.organismes_operations (
    id integer NOT NULL,
    organisme_id integer NOT NULL,
    operation_id integer NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.organismes_operations OWNER TO webgfc;

--
-- Name: TABLE organismes_operations; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.organismes_operations IS 'Table de liaison entre les organismes et les opérations (SAERP)';


--
-- Name: organismes_operations_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.organismes_operations_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.organismes_operations_id_seq OWNER TO webgfc;

--
-- Name: organismes_operations_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.organismes_operations_id_seq OWNED BY public.organismes_operations.id;


--
-- Name: originesflux; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.originesflux (
    id integer NOT NULL,
    name character varying(100) NOT NULL,
    active boolean DEFAULT true,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.originesflux OWNER TO webgfc;

--
-- Name: TABLE originesflux; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.originesflux IS 'Table permettant le stockage des origines de flux';


--
-- Name: originesflux_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.originesflux_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.originesflux_id_seq OWNER TO webgfc;

--
-- Name: originesflux_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.originesflux_id_seq OWNED BY public.originesflux.id;


--
-- Name: plandelegations; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.plandelegations (
    id integer NOT NULL,
    user_id integer NOT NULL,
    desktop_id integer NOT NULL,
    date_start timestamp without time zone NOT NULL,
    date_end timestamp without time zone NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.plandelegations OWNER TO webgfc;

--
-- Name: plandelegations_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.plandelegations_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.plandelegations_id_seq OWNER TO webgfc;

--
-- Name: plandelegations_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.plandelegations_id_seq OWNED BY public.plandelegations.id;


--
-- Name: plisconsultatifs; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.plisconsultatifs (
    id integer NOT NULL,
    name character varying(255),
    consultation_id integer NOT NULL,
    origineflux_id integer,
    courrier_id integer,
    numero character varying(10) NOT NULL,
    date date NOT NULL,
    heure time without time zone NOT NULL,
    objet character varying(255),
    active boolean DEFAULT true NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL,
    lot character varying(10)
);


ALTER TABLE public.plisconsultatifs OWNER TO webgfc;

--
-- Name: TABLE plisconsultatifs; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.plisconsultatifs IS 'Table permettant de stocker les plis consultatifs (SAERP)';


--
-- Name: plisconsultatifs_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.plisconsultatifs_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.plisconsultatifs_id_seq OWNER TO webgfc;

--
-- Name: plisconsultatifs_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.plisconsultatifs_id_seq OWNED BY public.plisconsultatifs.id;


--
-- Name: recheche_contactinfos; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.recheche_contactinfos (
    id integer NOT NULL,
    recherche_id integer NOT NULL,
    contactinfo_id integer NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.recheche_contactinfos OWNER TO webgfc;

--
-- Name: recheche_contactinfos_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.recheche_contactinfos_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.recheche_contactinfos_id_seq OWNER TO webgfc;

--
-- Name: recheche_contactinfos_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.recheche_contactinfos_id_seq OWNED BY public.recheche_contactinfos.id;


--
-- Name: recherches; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.recherches (
    id integer NOT NULL,
    name character varying(255),
    cname character varying(255),
    cdate date,
    cdatereception date,
    user_id integer,
    created timestamp without time zone,
    modified timestamp without time zone,
    cdatefin date,
    cdatereceptionfin date,
    contactname character varying(255),
    cobjet character varying(255),
    creference character varying(255),
    ccomment character varying(255),
    cretard character varying(255)
);


ALTER TABLE public.recherches OWNER TO webgfc;

--
-- Name: recherches_addressbooks; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.recherches_addressbooks (
    id integer NOT NULL,
    addressbook_id integer NOT NULL,
    recherche_id integer NOT NULL,
    created timestamp without time zone DEFAULT now(),
    modified timestamp without time zone DEFAULT now()
);


ALTER TABLE public.recherches_addressbooks OWNER TO webgfc;

--
-- Name: recherches_addressbooks_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.recherches_addressbooks_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.recherches_addressbooks_id_seq OWNER TO webgfc;

--
-- Name: recherches_addressbooks_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.recherches_addressbooks_id_seq OWNED BY public.recherches_addressbooks.id;


--
-- Name: recherches_affaires; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.recherches_affaires (
    id integer NOT NULL,
    affaire_id integer NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone,
    recherche_id integer NOT NULL
);


ALTER TABLE public.recherches_affaires OWNER TO webgfc;

--
-- Name: recherches_affaires_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.recherches_affaires_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.recherches_affaires_id_seq OWNER TO webgfc;

--
-- Name: recherches_affaires_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.recherches_affaires_id_seq OWNED BY public.recherches_affaires.id;


--
-- Name: recherches_circuits; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.recherches_circuits (
    id integer NOT NULL,
    recherche_id integer,
    circuit_id integer,
    created timestamp without time zone,
    modified timestamp without time zone
);


ALTER TABLE public.recherches_circuits OWNER TO webgfc;

--
-- Name: recherches_circuits_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.recherches_circuits_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.recherches_circuits_id_seq OWNER TO webgfc;

--
-- Name: recherches_circuits_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.recherches_circuits_id_seq OWNED BY public.recherches_circuits.id;


--
-- Name: recherches_contacts; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.recherches_contacts (
    id integer NOT NULL,
    contact_id integer NOT NULL,
    recherche_id integer NOT NULL,
    created timestamp without time zone DEFAULT now(),
    modified timestamp without time zone DEFAULT now()
);


ALTER TABLE public.recherches_contacts OWNER TO webgfc;

--
-- Name: recherches_contacts_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.recherches_contacts_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.recherches_contacts_id_seq OWNER TO webgfc;

--
-- Name: recherches_contacts_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.recherches_contacts_id_seq OWNED BY public.recherches_contacts.id;


--
-- Name: recherches_desktops; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.recherches_desktops (
    id integer NOT NULL,
    recherche_id integer NOT NULL,
    desktop_id integer NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.recherches_desktops OWNER TO webgfc;

--
-- Name: TABLE recherches_desktops; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.recherches_desktops IS 'Table de liaison entre les recherches et les desktops';


--
-- Name: recherches_desktops_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.recherches_desktops_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.recherches_desktops_id_seq OWNER TO webgfc;

--
-- Name: recherches_desktops_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.recherches_desktops_id_seq OWNED BY public.recherches_desktops.id;


--
-- Name: recherches_dossiers; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.recherches_dossiers (
    id integer NOT NULL,
    dossier_id integer NOT NULL,
    recherche_id integer NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone
);


ALTER TABLE public.recherches_dossiers OWNER TO webgfc;

--
-- Name: recherches_dossiers_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.recherches_dossiers_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.recherches_dossiers_id_seq OWNER TO webgfc;

--
-- Name: recherches_dossiers_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.recherches_dossiers_id_seq OWNED BY public.recherches_dossiers.id;


--
-- Name: recherches_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.recherches_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.recherches_id_seq OWNER TO webgfc;

--
-- Name: recherches_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.recherches_id_seq OWNED BY public.recherches.id;


--
-- Name: recherches_metadonnees; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.recherches_metadonnees (
    id integer NOT NULL,
    recherche_id integer,
    metadonnee_id integer,
    created timestamp without time zone,
    modified timestamp without time zone
);


ALTER TABLE public.recherches_metadonnees OWNER TO webgfc;

--
-- Name: recherches_metadonnees_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.recherches_metadonnees_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.recherches_metadonnees_id_seq OWNER TO webgfc;

--
-- Name: recherches_metadonnees_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.recherches_metadonnees_id_seq OWNED BY public.recherches_metadonnees.id;


--
-- Name: recherches_organismes; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.recherches_organismes (
    id integer NOT NULL,
    recherche_id integer NOT NULL,
    organisme_id integer NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.recherches_organismes OWNER TO webgfc;

--
-- Name: TABLE recherches_organismes; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.recherches_organismes IS 'Table de liaison entre les recherches et les organismes';


--
-- Name: recherches_organismes_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.recherches_organismes_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.recherches_organismes_id_seq OWNER TO webgfc;

--connecteurs
-- Name: recherches_organismes_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.recherches_organismes_id_seq OWNED BY public.recherches_organismes.id;


--
-- Name: recherches_selectvaluesmetadonnees; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.recherches_selectvaluesmetadonnees (
    id integer NOT NULL,
    recherche_id integer NOT NULL,
    selectvaluemetadonnee_id integer,
    metadonnee_id integer,
    valeur character varying(255),
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.recherches_selectvaluesmetadonnees OWNER TO webgfc;

--
-- Name: TABLE recherches_selectvaluesmetadonnees; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.recherches_selectvaluesmetadonnees IS 'Table de liaison entre les recherches et les selectvaluesmetadonnees';


--
-- Name: recherches_selectvaluesmetadonnees_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.recherches_selectvaluesmetadonnees_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.recherches_selectvaluesmetadonnees_id_seq OWNER TO webgfc;

--
-- Name: recherches_selectvaluesmetadonnees_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.recherches_selectvaluesmetadonnees_id_seq OWNED BY public.recherches_selectvaluesmetadonnees.id;


--
-- Name: recherches_services; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.recherches_services (
    id integer NOT NULL,
    recherche_id integer NOT NULL,
    service_id integer NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.recherches_services OWNER TO webgfc;

--
-- Name: TABLE recherches_services; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.recherches_services IS 'Table de liaison entre les recherches et les services';


--
-- Name: recherches_services_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.recherches_services_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.recherches_services_id_seq OWNER TO webgfc;

--
-- Name: recherches_services_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.recherches_services_id_seq OWNED BY public.recherches_services.id;


--
-- Name: recherches_soustypes; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.recherches_soustypes (
    id integer NOT NULL,
    soustype_id integer,
    recherche_id integer,
    created timestamp without time zone,
    modified timestamp without time zone
);


ALTER TABLE public.recherches_soustypes OWNER TO webgfc;

--
-- Name: recherches_soustypes_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.recherches_soustypes_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.recherches_soustypes_id_seq OWNER TO webgfc;

--
-- Name: recherches_soustypes_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.recherches_soustypes_id_seq OWNED BY public.recherches_soustypes.id;


--
-- Name: recherches_taches; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.recherches_taches (
    id integer NOT NULL,
    tache_id integer NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone,
    recherche_id integer NOT NULL
);


ALTER TABLE public.recherches_taches OWNER TO webgfc;

--
-- Name: recherches_taches_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.recherches_taches_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.recherches_taches_id_seq OWNER TO webgfc;

--
-- Name: recherches_taches_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.recherches_taches_id_seq OWNED BY public.recherches_taches.id;


--
-- Name: recherches_types; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.recherches_types (
    id integer NOT NULL,
    type_id integer,
    recherche_id integer,
    created timestamp without time zone,
    modified timestamp without time zone
);


ALTER TABLE public.recherches_types OWNER TO webgfc;

--
-- Name: recherches_types_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.recherches_types_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.recherches_types_id_seq OWNER TO webgfc;

--
-- Name: recherches_types_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.recherches_types_id_seq OWNED BY public.recherches_types.id;


--
-- Name: referentielsfantoir; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.referentielsfantoir (
    id integer NOT NULL,
    name character varying(50) NOT NULL,
    active boolean DEFAULT true,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.referentielsfantoir OWNER TO webgfc;

--
-- Name: TABLE referentielsfantoir; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.referentielsfantoir IS 'Table permettant le stockage des référentiels FANTOIR';


--
-- Name: referentielsfantoir_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.referentielsfantoir_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.referentielsfantoir_id_seq OWNER TO webgfc;

--
-- Name: referentielsfantoir_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.referentielsfantoir_id_seq OWNED BY public.referentielsfantoir.id;


--
-- Name: referentielsfantoircom; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.referentielsfantoircom (
    id integer NOT NULL,
    referentielfantoir_id integer NOT NULL,
    referentielfantoirdir_id integer NOT NULL,
    name character varying(255) NOT NULL,
    codecom character varying(3) NOT NULL,
    clerivoli character varying(1) NOT NULL,
    libellecom character varying(30) NOT NULL,
    typecom character varying(1) NOT NULL,
    caracrur character varying(1) DEFAULT NULL::character varying,
    caracpop character varying(1) DEFAULT NULL::character varying,
    popreelle character varying(7) DEFAULT NULL::character varying,
    popapart character varying(7) DEFAULT NULL::character varying,
    popfictive character varying(7) DEFAULT NULL::character varying,
    caracannul character varying(1) DEFAULT NULL::character varying,
    dateannul character varying(7) DEFAULT NULL::character varying,
    datecreation character varying(7) DEFAULT NULL::character varying,
    active boolean DEFAULT true,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL,
    CONSTRAINT referentielsfantoircom_typecom_in_list_chk CHECK (public.cakephp_validate_in_list((typecom)::text, ARRAY['N'::text, 'R'::text]))
);


ALTER TABLE public.referentielsfantoircom OWNER TO webgfc;

--
-- Name: TABLE referentielsfantoircom; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.referentielsfantoircom IS 'Table permettant le stockage des référentiels FANTOIR de commune';


--
-- Name: referentielsfantoircom_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.referentielsfantoircom_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.referentielsfantoircom_id_seq OWNER TO webgfc;

--
-- Name: referentielsfantoircom_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.referentielsfantoircom_id_seq OWNED BY public.referentielsfantoircom.id;


--
-- Name: referentielsfantoirdir; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.referentielsfantoirdir (
    id integer NOT NULL,
    referentielfantoir_id integer NOT NULL,
    name character varying(255) NOT NULL,
    codedpt character varying(2) NOT NULL,
    codedir character varying(1) NOT NULL,
    libelledir character varying(30) NOT NULL,
    active boolean DEFAULT true,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.referentielsfantoirdir OWNER TO webgfc;

--
-- Name: TABLE referentielsfantoirdir; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.referentielsfantoirdir IS 'Table permettant le stockage des référentiels FANTOIR de direction';


--
-- Name: referentielsfantoirdir_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.referentielsfantoirdir_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.referentielsfantoirdir_id_seq OWNER TO webgfc;

--
-- Name: referentielsfantoirdir_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.referentielsfantoirdir_id_seq OWNED BY public.referentielsfantoirdir.id;


--
-- Name: referentielsfantoirvoie; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.referentielsfantoirvoie (
    id integer NOT NULL,
    referentielfantoir_id integer NOT NULL,
    referentielfantoircom_id integer NOT NULL,
    name character varying(255) NOT NULL,
    identvoie character varying(4) NOT NULL,
    clerivoli character varying(1) NOT NULL,
    codevoie character varying(4) DEFAULT NULL::character varying,
    libellevoie character varying(26) NOT NULL,
    typecom character varying(1) NOT NULL,
    caracrur character varying(1) DEFAULT NULL::character varying,
    caracvoie character varying(1) NOT NULL,
    caracpop character varying(1) DEFAULT NULL::character varying,
    popapart character varying(7) DEFAULT NULL::character varying,
    popfictive character varying(7) DEFAULT NULL::character varying,
    caracannul character varying(1) DEFAULT NULL::character varying,
    dateannul character varying(7) DEFAULT NULL::character varying,
    datecreation character varying(7) DEFAULT NULL::character varying,
    codemajic character varying(5) NOT NULL,
    typevoie character varying(1) NOT NULL,
    caraclieudit character varying(1) DEFAULT NULL::character varying,
    libentiervoie character varying(8) NOT NULL,
    active boolean DEFAULT true,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL,
    CONSTRAINT referentielsfantoirvoie_caracvoie_in_list_chk CHECK (public.cakephp_validate_in_list((caracvoie)::text, ARRAY['1'::text, '0'::text])),
    CONSTRAINT referentielsfantoirvoie_typecom_in_list_chk CHECK (public.cakephp_validate_in_list((typecom)::text, ARRAY['N'::text, 'R'::text])),
    CONSTRAINT referentielsfantoirvoie_typevoie_in_list_chk CHECK (public.cakephp_validate_in_list((typevoie)::text, ARRAY['1'::text, '2'::text, '3'::text, '4'::text, '5'::text]))
);


ALTER TABLE public.referentielsfantoirvoie OWNER TO webgfc;

--
-- Name: TABLE referentielsfantoirvoie; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.referentielsfantoirvoie IS 'Table permettant le stockage des référentiels FANTOIR de voie';


--
-- Name: referentielsfantoirvoie_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.referentielsfantoirvoie_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.referentielsfantoirvoie_id_seq OWNER TO webgfc;

--
-- Name: referentielsfantoirvoie_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.referentielsfantoirvoie_id_seq OWNED BY public.referentielsfantoirvoie.id;


--
-- Name: relements; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.relements (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL,
    courrier_id integer NOT NULL,
    mime character varying(255),
    preview bytea,
    content bytea
);


ALTER TABLE public.relements OWNER TO webgfc;

--
-- Name: relements_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.relements_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.relements_id_seq OWNER TO webgfc;

--
-- Name: relements_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.relements_id_seq OWNED BY public.relements.id;


--
-- Name: repertoires; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.repertoires (
    id integer NOT NULL,
    name character varying(255),
    user_id integer,
    lft integer,
    rght integer,
    parent_id integer
);


ALTER TABLE public.repertoires OWNER TO webgfc;

--
-- Name: repertoires_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.repertoires_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.repertoires_id_seq OWNER TO webgfc;

--
-- Name: repertoires_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.repertoires_id_seq OWNED BY public.repertoires.id;


--
-- Name: rgpds; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.rgpds(
    id integer NOT NULL,
    collectivite_id INTEGER NOT NULL,
    conn VARCHAR(255) NOT NULL,
    nomhebergeur VARCHAR(255) NOT NULL,
    adressecompletehebergeur VARCHAR(255) NOT NULL,
    representanthebergeur VARCHAR(255),
    qualiterepresentanthebergeur VARCHAR(255),
    sirethebergeur VARCHAR(255),
    codeapehebergeur VARCHAR(255),
    numtelhebergeur VARCHAR(14),
    mailhebergeur VARCHAR(255),
    maildpo VARCHAR (255),
    active BOOLEAN NOT NULL DEFAULT 'true',
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);

ALTER TABLE public.rgpds OWNER TO webgfc;

--
-- Name: rights_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.rgpds_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.rgpds_id_seq OWNER TO webgfc;

--
-- Name: rgpds_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.rgpds_id_seq OWNED BY public.rgpds.id;

COMMENT ON TABLE rgpds IS 'Table permettant de stocker les politiques de confidentialité par collectivités (RGPD)';

--
-- Name: rights; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.rights (
    id integer NOT NULL,
    get_infos boolean DEFAULT false NOT NULL,
    get_meta boolean DEFAULT false NOT NULL,
    get_taches boolean DEFAULT false NOT NULL,
    get_affaire boolean DEFAULT false NOT NULL,
    create_infos boolean DEFAULT false NOT NULL,
    set_infos boolean DEFAULT false NOT NULL,
    set_meta boolean DEFAULT false NOT NULL,
    set_taches boolean DEFAULT false NOT NULL,
    set_affaire boolean DEFAULT false NOT NULL,
    administration boolean DEFAULT false NOT NULL,
    aiguillage boolean DEFAULT false NOT NULL,
    creation boolean DEFAULT false NOT NULL,
    validation boolean DEFAULT false NOT NULL,
    edition boolean DEFAULT false NOT NULL,
    documentation boolean DEFAULT false NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL,
    mode_classique boolean DEFAULT false NOT NULL,
    aro_id integer,
    envoi_cpy boolean DEFAULT false NOT NULL,
    envoi_wkf boolean DEFAULT false NOT NULL,
    envoi_ged boolean DEFAULT false NOT NULL,
    document_upload boolean DEFAULT false NOT NULL,
    document_suppr boolean DEFAULT false NOT NULL,
    document_setasdefault boolean DEFAULT false NOT NULL,
    relements boolean DEFAULT false NOT NULL,
    relements_upload boolean DEFAULT false NOT NULL,
    relements_suppr boolean DEFAULT false NOT NULL,
    ar_gen boolean DEFAULT false NOT NULL,
    mes_services boolean DEFAULT false NOT NULL,
    recherches boolean DEFAULT false NOT NULL,
    suppression_flux boolean DEFAULT false NOT NULL,
    model character varying(255),
    foreign_key integer,
    consult_scan boolean,
    carnet_adresse boolean DEFAULT false NOT NULL,
    statistiques boolean DEFAULT false NOT NULL,
    action_carnet_adresse_flux boolean DEFAULT false NOT NULL
);


ALTER TABLE public.rights OWNER TO webgfc;

--
-- Name: rights_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.rights_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.rights_id_seq OWNER TO webgfc;

--
-- Name: rights_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.rights_id_seq OWNED BY public.rights.id;


--
-- Name: rmodelgabarits; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.rmodelgabarits (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    gedpath text,
    content bytea,
    size integer NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL,
    soustype_id integer NOT NULL,
    mime character varying(255),
    preview bytea,
    ext character varying(10),
    path character varying(255)
);


ALTER TABLE public.rmodelgabarits OWNER TO webgfc;

--
-- Name: rmodelgabarits_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.rmodelgabarits_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.rmodelgabarits_id_seq OWNER TO webgfc;

--
-- Name: rmodelgabarits_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.rmodelgabarits_id_seq OWNED BY public.rmodelgabarits.id;


--
-- Name: rmodels; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.rmodels (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    gedpath text,
    content bytea,
    size integer NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL,
    soustype_id integer NOT NULL,
    mime character varying(255),
    preview bytea,
    ext character varying(10)
);


ALTER TABLE public.rmodels OWNER TO webgfc;

--
-- Name: rmodels_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.rmodels_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.rmodels_id_seq OWNER TO webgfc;

--
-- Name: rmodels_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.rmodels_id_seq OWNED BY public.rmodels.id;


--
-- Name: scanemails; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.scanemails (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    username character varying(255) NOT NULL,
    mail character varying(255),
    hostname character varying(255) NOT NULL,
    port character varying(24) DEFAULT '143'::character varying,
    password character varying(255) NOT NULL,
    desktop_id integer,
    type_id integer,
    soustype_id integer,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL,
    desktopmanager_id integer,
    scriptname character varying(255) NOT NULL,
    configuration character varying(255) DEFAULT '/novalidate-cert'::character varying
);


ALTER TABLE public.scanemails OWNER TO webgfc;

--
-- Name: TABLE scanemails; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.scanemails IS 'Table permettant de stocker les emails et destinataires pour chaque collectivité';


--
-- Name: scanemails_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.scanemails_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.scanemails_id_seq OWNER TO webgfc;

--
-- Name: scanemails_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.scanemails_id_seq OWNED BY public.scanemails.id;


--
-- Name: selectvaluesmetadonnees; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.selectvaluesmetadonnees (
    id integer NOT NULL,
    metadonnee_id integer NOT NULL,
    name character varying(250) DEFAULT NULL::character varying,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL,
    active boolean,
    sendgrc boolean
);


ALTER TABLE public.selectvaluesmetadonnees OWNER TO webgfc;

--
-- Name: selectvaluesmetadonnees_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.selectvaluesmetadonnees_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.selectvaluesmetadonnees_id_seq OWNER TO webgfc;

--
-- Name: selectvaluesmetadonnees_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.selectvaluesmetadonnees_id_seq OWNED BY public.selectvaluesmetadonnees.id;


--
-- Name: sendsmails; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.sendsmails (
    id integer NOT NULL,
    email character varying(255) NOT NULL,
    sujet character varying(255),
    message text,
    desktop_id integer,
    courrier_id integer,
    document_id integer,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL,
    ccmail character varying(255),
    templatemail_id integer
);


ALTER TABLE public.sendsmails OWNER TO webgfc;

--
-- Name: sendsmails_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.sendsmails_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.sendsmails_id_seq OWNER TO webgfc;

--
-- Name: sendsmails_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.sendsmails_id_seq OWNED BY public.sendsmails.id;


--
-- Name: sequences; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.sequences (
    id integer NOT NULL,
    nom character varying(255) NOT NULL,
    commentaire character varying(255) NOT NULL,
    num_sequence integer NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone
);


ALTER TABLE public.sequences OWNER TO webgfc;

--
-- Name: sequences_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.sequences_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.sequences_id_seq OWNER TO webgfc;

--
-- Name: sequences_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.sequences_id_seq OWNED BY public.sequences.id;


--
-- Name: services; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.services (
    name character varying(255) NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone,
    parent_id smallint,
    lft smallint,
    rght smallint,
    id integer NOT NULL,
    email character varying(255),
    active boolean DEFAULT true,
    scan_active boolean DEFAULT false,
    scan_local_path character varying(255),
    scan_service_name character varying(255),
    scan_desktopmanager_id INTEGER,
    scan_purge_delay INTEGER
);


ALTER TABLE public.services OWNER TO webgfc;

--
-- Name: services_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.services_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.services_id_seq OWNER TO webgfc;

--
-- Name: services_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.services_id_seq OWNED BY public.services.id;


--
-- Name: services; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.sms (
    id integer NOT NULL,
    courrier_id integer NOT NULL,
    numero VARCHAR(12) NOT NULL,
    message VARCHAR (160) NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);
COMMENT ON TABLE sms IS 'Table permettant de stocker les SMS envoyés';

ALTER TABLE public.sms OWNER TO webgfc;

--
-- Name: services_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.sms_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.sms_id_seq OWNER TO webgfc;

--
-- Name: sms_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.sms_id_seq OWNED BY public.sms.id;
--
-- Name: sousactivites; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.sousactivites (
    id integer NOT NULL,
    activite_id integer NOT NULL,
    name character varying(255) NOT NULL,
    description character varying(255),
    active boolean DEFAULT true NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.sousactivites OWNER TO webgfc;

--
-- Name: TABLE sousactivites; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.sousactivites IS 'Table permettant de stocker les sous-activités liées aux contacts (SAERP)';


--
-- Name: sousactivites_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.sousactivites_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.sousactivites_id_seq OWNER TO webgfc;

--
-- Name: sousactivites_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.sousactivites_id_seq OWNED BY public.sousactivites.id;


--
-- Name: sousoperations; Type: TABLE; Schema: public; Owner: postgres
--

CREATE TABLE public.sousoperations (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    operation_id integer NOT NULL,
    active boolean DEFAULT true NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL,
    soustype_id integer
);


ALTER TABLE public.sousoperations OWNER TO webgfc;

--
-- Name: TABLE sousoperations; Type: COMMENT; Schema: public; Owner: postgres
--

COMMENT ON TABLE public.sousoperations IS 'Table de liaison entre les bureaux et les opérations (SAERP)';


--
-- Name: sousoperations_id_seq; Type: SEQUENCE; Schema: public; Owner: postgres
--

CREATE SEQUENCE public.sousoperations_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.sousoperations_id_seq OWNER TO webgfc;

--
-- Name: sousoperations_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: postgres
--

ALTER SEQUENCE public.sousoperations_id_seq OWNED BY public.sousoperations.id;


--
-- Name: soustypes; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.soustypes (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone,
    type_id integer NOT NULL,
    circuit_id integer,
    entrant smallint DEFAULT 1 NOT NULL,
    delai_nb smallint,
    delai_unite smallint,
    lft integer,
    rght integer,
    parent_id integer,
    valeuremail bytea,
    compteur_id integer,
    information text,
    active boolean DEFAULT true,
	envoi_signature boolean DEFAULT false,
	envoi_mailsec boolean DEFAULT false,
	envoi_ged boolean DEFAULT false,
	envoi_sae boolean DEFAULT false,
	soustype_parapheur character varying(255)
);


ALTER TABLE public.soustypes OWNER TO webgfc;

--
-- Name: soustypes_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.soustypes_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.soustypes_id_seq OWNER TO webgfc;

--
-- Name: soustypes_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.soustypes_id_seq OWNED BY public.soustypes.id;


--
-- Name: soustypes_soustypescibles; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.soustypes_soustypescibles (
    id INTEGER NOT NULL,
    soustype_id integer not null,
    soustypecible_id integer not null,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.soustypes_soustypescibles OWNER TO webgfc;

--
-- Name: soustypes_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.soustypes_soustypescibles_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.soustypes_soustypescibles_id_seq OWNER TO webgfc;

--
-- Name: soustypes_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.soustypes_soustypescibles_id_seq OWNED BY public.soustypes_soustypescibles.id;

COMMENT ON TABLE soustypes_soustypescibles IS 'Table de liaison entre un sous-type et se(s) sous-type(s) de référence pour une réponse';



--
-- Name: taches; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.taches (
    id integer NOT NULL,
    courrier_id integer NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone,
    name character varying(255) NOT NULL,
    delai_nb integer,
    delai_unite integer,
    statut integer DEFAULT 0 NOT NULL,
    act_desktop_id integer DEFAULT 0 NOT NULL,
    creator_desktop_id integer DEFAULT 0 NOT NULL
);


ALTER TABLE public.taches OWNER TO webgfc;

--
-- Name: taches_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.taches_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.taches_id_seq OWNER TO webgfc;

--
-- Name: taches_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.taches_id_seq OWNED BY public.taches.id;



--
-- Name: templatemails; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.templatemails (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    subject character varying(255) NOT NULL,
    object text NOT NULL,
    active boolean DEFAULT true NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.templatemails OWNER TO webgfc;

--
-- Name: TABLE templatemails; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.templatemails IS 'Table permettant de définir des mails-type à envoyer';


--
-- Name: templatemails_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.templatemails_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.templatemails_id_seq OWNER TO webgfc;

--
-- Name: templatemails_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.templatemails_id_seq OWNED BY public.templatemails.id;



--
-- Name: templatenotifications; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.templatenotifications (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    subject character varying(255) NOT NULL,
    object text NOT NULL,
    active boolean DEFAULT true NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.templatenotifications OWNER TO webgfc;

--
-- Name: TABLE templatenotifications; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.templatenotifications IS 'Table permettant de définir les mails adressés par notifications';


--
-- Name: templatenotifications_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.templatenotifications_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.templatenotifications_id_seq OWNER TO webgfc;

--
-- Name: templatenotifications_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.templatenotifications_id_seq OWNED BY public.templatenotifications.id;

--
-- Name: titres; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.titres (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.titres OWNER TO webgfc;

--
-- Name: TABLE titres; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.titres IS 'Table permettant de définir les titres des contacts';


--
-- Name: titres_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.titres_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.titres_id_seq OWNER TO webgfc;

--
-- Name: titres_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.titres_id_seq OWNED BY public.titres.id;

--
-- Name: typemetadonnees; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.typemetadonnees (
    id integer NOT NULL,
    name character varying(255),
    created timestamp without time zone,
    modified timestamp without time zone
);


ALTER TABLE public.typemetadonnees OWNER TO webgfc;

--
-- Name: typemetadonnees_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.typemetadonnees_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.typemetadonnees_id_seq OWNER TO webgfc;

--
-- Name: typemetadonnees_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.typemetadonnees_id_seq OWNED BY public.typemetadonnees.id;


--
-- Name: typenotifs; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.typenotifs (
    id integer NOT NULL,
    name character varying(255) NOT NULL
);


ALTER TABLE public.typenotifs OWNER TO webgfc;

--
-- Name: typenotifs_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.typenotifs_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.typenotifs_id_seq OWNER TO webgfc;

--
-- Name: typenotifs_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.typenotifs_id_seq OWNED BY public.typenotifs.id;


--
-- Name: typenotifs_users; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.typenotifs_users (
    id integer NOT NULL,
    typenotif_id integer NOT NULL,
    user_id integer NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.typenotifs_users OWNER TO webgfc;

--
-- Name: typenotifs_users_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.typenotifs_users_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.typenotifs_users_id_seq OWNER TO webgfc;

--
-- Name: typenotifs_users_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.typenotifs_users_id_seq OWNED BY public.typenotifs_users.id;



--
-- Name: types; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.types (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone,
    active boolean DEFAULT true,
    grc_servid character varying(255)
);


ALTER TABLE public.types OWNER TO webgfc;

--
-- Name: types_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.types_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.types_id_seq OWNER TO webgfc;

--
-- Name: types_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.types_id_seq OWNED BY public.types.id;



--
-- Name: uniterifs; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.uniterifs (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    description character varying(255),
    active boolean DEFAULT true NOT NULL,
    created timestamp without time zone DEFAULT now() NOT NULL,
    modified timestamp without time zone DEFAULT now() NOT NULL
);


ALTER TABLE public.uniterifs OWNER TO webgfc;

--
-- Name: TABLE uniterifs; Type: COMMENT; Schema: public; Owner: webgfc
--

COMMENT ON TABLE public.uniterifs IS 'Table permettant de définir les Unités RIF (SAERP)';


--
-- Name: uniterifs_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.uniterifs_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.uniterifs_id_seq OWNER TO webgfc;

--
-- Name: uniterifs_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.uniterifs_id_seq OWNED BY public.uniterifs.id;



--
-- Name: users; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.users (
    id integer NOT NULL,
    username character varying(255) NOT NULL,
    password character varying(255) NOT NULL,
    nom character varying(255) NOT NULL,
    prenom character varying(255) NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone,
    mail character varying(255),
    desktop_id integer,
    last_login timestamp without time zone,
    last_logout timestamp without time zone,
    active boolean DEFAULT true NOT NULL,
    pwdmodified boolean DEFAULT false NOT NULL,
    numtel character varying(20),
    accept_notif boolean,
    mail_insertion boolean,
    mail_traitement boolean,
    mail_refus boolean,
    mail_retard_validation boolean,
    mail_copy boolean,
    mail_insertion_reponse boolean,
    mail_aiguillage boolean,
    mail_delegation boolean,
    typeabonnement character varying(255) DEFAULT 'event'::character varying,
    addressbook_private boolean DEFAULT false,
    mail_clos boolean DEFAULT true,
    mail_bloque_parapheur boolean DEFAULT true
);


ALTER TABLE public.users OWNER TO webgfc;

--
-- Name: users_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.users_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.users_id_seq OWNER TO webgfc;

--
-- Name: users_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.users_id_seq OWNED BY public.users.id;



--
-- Name: wkf_circuits; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.wkf_circuits (
    id integer NOT NULL,
    nom character varying(250) NOT NULL,
    description bytea,
    actif integer DEFAULT 1 NOT NULL,
    defaut integer DEFAULT 0 NOT NULL,
    created_user_id integer NOT NULL,
    modified_user_id integer,
    created timestamp without time zone,
    modified timestamp without time zone
);


ALTER TABLE public.wkf_circuits OWNER TO webgfc;

--
-- Name: wkf_circuits_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.wkf_circuits_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.wkf_circuits_id_seq OWNER TO webgfc;

--
-- Name: wkf_circuits_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.wkf_circuits_id_seq OWNED BY public.wkf_circuits.id;


--
-- Name: wkf_compositions; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.wkf_compositions (
    id integer NOT NULL,
    etape_id integer NOT NULL,
    type_validation character varying(1) NOT NULL,
    created_user_id integer,
    modified_user_id integer,
    created timestamp without time zone,
    modified timestamp without time zone,
    trigger_id integer,
    soustype character varying(255),
    type_composition character varying(20) DEFAULT 'USER'::character varying,
    type_document character varying(255),
    inforequired_type_document character varying(255)
);


ALTER TABLE public.wkf_compositions OWNER TO webgfc;

--
-- Name: wkf_compositions_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.wkf_compositions_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.wkf_compositions_id_seq OWNER TO webgfc;

--
-- Name: wkf_compositions_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.wkf_compositions_id_seq OWNED BY public.wkf_compositions.id;


--
-- Name: wkf_etapes; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.wkf_etapes (
    id integer NOT NULL,
    circuit_id integer NOT NULL,
    nom character varying(250) NOT NULL,
    description bytea,
    type integer NOT NULL,
    ordre integer NOT NULL,
    created_user_id integer NOT NULL,
    modified_user_id integer,
    created timestamp without time zone,
    modified timestamp without time zone,
    soustype character varying(255),
    cpt_retard integer,
    type_document character varying(255),
    inforequired_type_document character varying(255)
);


ALTER TABLE public.wkf_etapes OWNER TO webgfc;

--
-- Name: wkf_etapes_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.wkf_etapes_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.wkf_etapes_id_seq OWNER TO webgfc;

--
-- Name: wkf_etapes_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.wkf_etapes_id_seq OWNED BY public.wkf_etapes.id;


--
-- Name: wkf_signatures; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.wkf_signatures (
    id integer NOT NULL,
    type_signature character varying(100) NOT NULL,
    signature bytea NOT NULL,
    visa_id integer
);


ALTER TABLE public.wkf_signatures OWNER TO webgfc;

--
-- Name: wkf_signatures_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.wkf_signatures_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.wkf_signatures_id_seq OWNER TO webgfc;

--
-- Name: wkf_signatures_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.wkf_signatures_id_seq OWNED BY public.wkf_signatures.id;


--
-- Name: wkf_traitements; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.wkf_traitements (
    id integer NOT NULL,
    circuit_id integer NOT NULL,
    target_id integer NOT NULL,
    numero_traitement integer DEFAULT 1 NOT NULL,
    created_user_id integer,
    modified_user_id integer,
    created timestamp without time zone,
    modified timestamp without time zone,
    treated boolean
);


ALTER TABLE public.wkf_traitements OWNER TO webgfc;

--
-- Name: wkf_traitements_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.wkf_traitements_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.wkf_traitements_id_seq OWNER TO webgfc;

--
-- Name: wkf_traitements_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.wkf_traitements_id_seq OWNED BY public.wkf_traitements.id;


--
-- Name: wkf_visas; Type: TABLE; Schema: public; Owner: webgfc
--

CREATE TABLE public.wkf_visas (
    id integer NOT NULL,
    traitement_id integer NOT NULL,
    trigger_id integer NOT NULL,
    signature_id integer,
    etape_nom character varying(250),
    etape_type integer NOT NULL,
    action character varying(2) NOT NULL,
    commentaire text,
    date timestamp without time zone,
    type_validation character varying(1) NOT NULL,
    numero_traitement integer NOT NULL,
    etape_id integer,
    date_retard timestamp without time zone,
    soustype character varying(255),
    type_document character varying(255),
    inforequired_type_document character varying(255)
);


ALTER TABLE public.wkf_visas OWNER TO webgfc;

--
-- Name: wkf_visas_id_seq; Type: SEQUENCE; Schema: public; Owner: webgfc
--

CREATE SEQUENCE public.wkf_visas_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.wkf_visas_id_seq OWNER TO webgfc;

--
-- Name: wkf_visas_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: webgfc
--

ALTER SEQUENCE public.wkf_visas_id_seq OWNED BY public.wkf_visas.id;


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.acos ALTER COLUMN id SET DEFAULT nextval('public.acos_id_seq'::regclass);

--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.activites ALTER COLUMN id SET DEFAULT nextval('public.activites_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.activites_organismes ALTER COLUMN id SET DEFAULT nextval('public.activites_organismes_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.addressbooks ALTER COLUMN id SET DEFAULT nextval('public.addressbooks_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.affaires ALTER COLUMN id SET DEFAULT nextval('public.affaires_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.armodels ALTER COLUMN id SET DEFAULT nextval('public.armodels_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.aros ALTER COLUMN id SET DEFAULT nextval('public.aros_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.aros_acos ALTER COLUMN id SET DEFAULT nextval('public.aros_acos_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.ars ALTER COLUMN id SET DEFAULT nextval('public.ars_id_seq'::regclass);



--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.bancontenus ALTER COLUMN id SET DEFAULT nextval('public.bancontenus_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.bannettes ALTER COLUMN id SET DEFAULT nextval('public.bannettes_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.bans ALTER COLUMN id SET DEFAULT nextval('public.bans_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.bansadresses ALTER COLUMN id SET DEFAULT nextval('public.bansadresses_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.banscommunes ALTER COLUMN id SET DEFAULT nextval('public.banscommunes_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.comments ALTER COLUMN id SET DEFAULT nextval('public.comments_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.comments_readers ALTER COLUMN id SET DEFAULT nextval('public.comments_readers_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.compteurs ALTER COLUMN id SET DEFAULT nextval('public.compteurs_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.connecteurs ALTER COLUMN id SET DEFAULT nextval('public.connecteurs_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.consultations ALTER COLUMN id SET DEFAULT nextval('public.consultations_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.contactinfos ALTER COLUMN id SET DEFAULT nextval('public.contactinfos_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.contacts ALTER COLUMN id SET DEFAULT nextval('public.contacts_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.contacts_events ALTER COLUMN id SET DEFAULT nextval('public.contacts_events_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.contacts_operations ALTER COLUMN id SET DEFAULT nextval('public.contacts_operations_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.contractants ALTER COLUMN id SET DEFAULT nextval('public.contractants_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.courriers ALTER COLUMN id SET DEFAULT nextval('public.courriers_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.courriers_metadonnees ALTER COLUMN id SET DEFAULT nextval('public.courriers_metadonnees_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.courriers_repertoires ALTER COLUMN id SET DEFAULT nextval('public.courriers_repertoires_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.dacos ALTER COLUMN id SET DEFAULT nextval('public.dacos_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.daros ALTER COLUMN id SET DEFAULT nextval('public.daros_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.daros_dacos ALTER COLUMN id SET DEFAULT nextval('public.daros_dacos_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.desktops ALTER COLUMN id SET DEFAULT nextval('public.desktops_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.desktops_desktopsmanagers ALTER COLUMN id SET DEFAULT nextval('public.desktops_desktopsmanagers_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.desktops_services ALTER COLUMN id SET DEFAULT nextval('public.desktops_services_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.desktops_taches ALTER COLUMN id SET DEFAULT nextval('public.desktops_taches_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.desktops_users ALTER COLUMN id SET DEFAULT nextval('public.desktops_users_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.desktopsmanagers ALTER COLUMN id SET DEFAULT nextval('public.desktopsmanagers_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY public.desktopsmanagers_sousoperations ALTER COLUMN id SET DEFAULT nextval('public.desktopsmanagers_sousoperations_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.documents ALTER COLUMN id SET DEFAULT nextval('public.documents_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.dossiers ALTER COLUMN id SET DEFAULT nextval('public.dossiers_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.events ALTER COLUMN id SET DEFAULT nextval('public.events_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.fonctions ALTER COLUMN id SET DEFAULT nextval('public.fonctions_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.formatsreponse ALTER COLUMN id SET DEFAULT nextval('public.formatsreponse_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.formatsreponse_soustypes ALTER COLUMN id SET DEFAULT nextval('public.formatsreponse_soustypes_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY public.gabaritsdocuments ALTER COLUMN id SET DEFAULT nextval('public.gabaritsdocuments_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.inits_services ALTER COLUMN id SET DEFAULT nextval('public.inits_services_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY public.intituleagentbyoperations ALTER COLUMN id SET DEFAULT nextval('public.intituleagentbyoperations_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.intitulesagents ALTER COLUMN id SET DEFAULT nextval('public.intitulesagents_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.journalevents ALTER COLUMN id SET DEFAULT nextval('public.journalevents_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.keywordlists ALTER COLUMN id SET DEFAULT nextval('public.keywordlists_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.keywords ALTER COLUMN id SET DEFAULT nextval('public.keywords_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.keywordtypes ALTER COLUMN id SET DEFAULT nextval('public.keywordtypes_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.ldapm_groups ALTER COLUMN id SET DEFAULT nextval('public.ldapm_groups_id_seq'::regclass);

--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.ldapm_models_groups ALTER COLUMN id SET DEFAULT nextval('public.ldapm_models_groups_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.marches ALTER COLUMN id SET DEFAULT nextval('public.marches_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.metadonnees ALTER COLUMN id SET DEFAULT nextval('public.metadonnees_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.metadonnees_soustypes ALTER COLUMN id SET DEFAULT nextval('public.metadonnees_soustypes_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.notifications ALTER COLUMN id SET DEFAULT nextval('public.notifications_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.notifieddesktops ALTER COLUMN id SET DEFAULT nextval('public.notifieddesktops_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.notifquotidiens ALTER COLUMN id SET DEFAULT nextval('public.notifquotidiens_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.operations ALTER COLUMN id SET DEFAULT nextval('public.operations_id_seq'::regclass);

--
-- Name: id; Type: DEFAULT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY public.ordresservices ALTER COLUMN id SET DEFAULT nextval('public.ordresservices_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.organismes ALTER COLUMN id SET DEFAULT nextval('public.organismes_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.organismes_operations ALTER COLUMN id SET DEFAULT nextval('public.organismes_operations_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.originesflux ALTER COLUMN id SET DEFAULT nextval('public.originesflux_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.plandelegations ALTER COLUMN id SET DEFAULT nextval('public.plandelegations_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.plisconsultatifs ALTER COLUMN id SET DEFAULT nextval('public.plisconsultatifs_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.profils ALTER COLUMN id SET DEFAULT nextval('public.profils_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recheche_contactinfos ALTER COLUMN id SET DEFAULT nextval('public.recheche_contactinfos_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches ALTER COLUMN id SET DEFAULT nextval('public.recherches_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_addressbooks ALTER COLUMN id SET DEFAULT nextval('public.recherches_addressbooks_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_affaires ALTER COLUMN id SET DEFAULT nextval('public.recherches_affaires_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_circuits ALTER COLUMN id SET DEFAULT nextval('public.recherches_circuits_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_contacts ALTER COLUMN id SET DEFAULT nextval('public.recherches_contacts_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_desktops ALTER COLUMN id SET DEFAULT nextval('public.recherches_desktops_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_dossiers ALTER COLUMN id SET DEFAULT nextval('public.recherches_dossiers_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_metadonnees ALTER COLUMN id SET DEFAULT nextval('public.recherches_metadonnees_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_organismes ALTER COLUMN id SET DEFAULT nextval('public.recherches_organismes_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_selectvaluesmetadonnees ALTER COLUMN id SET DEFAULT nextval('public.recherches_selectvaluesmetadonnees_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_services ALTER COLUMN id SET DEFAULT nextval('public.recherches_services_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_soustypes ALTER COLUMN id SET DEFAULT nextval('public.recherches_soustypes_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_taches ALTER COLUMN id SET DEFAULT nextval('public.recherches_taches_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_types ALTER COLUMN id SET DEFAULT nextval('public.recherches_types_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.referentielsfantoir ALTER COLUMN id SET DEFAULT nextval('public.referentielsfantoir_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.referentielsfantoircom ALTER COLUMN id SET DEFAULT nextval('public.referentielsfantoircom_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.referentielsfantoirdir ALTER COLUMN id SET DEFAULT nextval('public.referentielsfantoirdir_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.referentielsfantoirvoie ALTER COLUMN id SET DEFAULT nextval('public.referentielsfantoirvoie_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.relements ALTER COLUMN id SET DEFAULT nextval('public.relements_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.repertoires ALTER COLUMN id SET DEFAULT nextval('public.repertoires_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.rgpds ALTER COLUMN id SET DEFAULT nextval('public.rgpds_id_seq'::regclass);



--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.rights ALTER COLUMN id SET DEFAULT nextval('public.rights_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.rmodelgabarits ALTER COLUMN id SET DEFAULT nextval('public.rmodelgabarits_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.rmodels ALTER COLUMN id SET DEFAULT nextval('public.rmodels_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.scanemails ALTER COLUMN id SET DEFAULT nextval('public.scanemails_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.selectvaluesmetadonnees ALTER COLUMN id SET DEFAULT nextval('public.selectvaluesmetadonnees_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.sendsmails ALTER COLUMN id SET DEFAULT nextval('public.sendsmails_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.sequences ALTER COLUMN id SET DEFAULT nextval('public.sequences_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.services ALTER COLUMN id SET DEFAULT nextval('public.services_id_seq'::regclass);

--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.sms ALTER COLUMN id SET DEFAULT nextval('public.sms_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.sousactivites ALTER COLUMN id SET DEFAULT nextval('public.sousactivites_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY public.sousoperations ALTER COLUMN id SET DEFAULT nextval('public.sousoperations_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.soustypes ALTER COLUMN id SET DEFAULT nextval('public.soustypes_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--


ALTER TABLE ONLY public.soustypes_soustypescibles ALTER COLUMN id SET DEFAULT nextval('public.soustypes_soustypescibles_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.taches ALTER COLUMN id SET DEFAULT nextval('public.taches_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.templatemails ALTER COLUMN id SET DEFAULT nextval('public.templatemails_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.templatenotifications ALTER COLUMN id SET DEFAULT nextval('public.templatenotifications_id_seq'::regclass);

--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.titres ALTER COLUMN id SET DEFAULT nextval('public.titres_id_seq'::regclass);

--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.typemetadonnees ALTER COLUMN id SET DEFAULT nextval('public.typemetadonnees_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.typenotifs ALTER COLUMN id SET DEFAULT nextval('public.typenotifs_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.typenotifs_users ALTER COLUMN id SET DEFAULT nextval('public.typenotifs_users_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.types ALTER COLUMN id SET DEFAULT nextval('public.types_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.uniterifs ALTER COLUMN id SET DEFAULT nextval('public.uniterifs_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.users ALTER COLUMN id SET DEFAULT nextval('public.users_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.wkf_circuits ALTER COLUMN id SET DEFAULT nextval('public.wkf_circuits_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.wkf_compositions ALTER COLUMN id SET DEFAULT nextval('public.wkf_compositions_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.wkf_etapes ALTER COLUMN id SET DEFAULT nextval('public.wkf_etapes_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.wkf_signatures ALTER COLUMN id SET DEFAULT nextval('public.wkf_signatures_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.wkf_traitements ALTER COLUMN id SET DEFAULT nextval('public.wkf_traitements_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.wkf_visas ALTER COLUMN id SET DEFAULT nextval('public.wkf_visas_id_seq'::regclass);


--
-- Name: acos_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.acos
    ADD CONSTRAINT acos_pkey PRIMARY KEY (id);


--
-- Name: activites_organismes_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.activites_organismes
    ADD CONSTRAINT activites_organismes_pkey PRIMARY KEY (id);


--
-- Name: activites_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.activites
    ADD CONSTRAINT activites_pkey PRIMARY KEY (id);


--
-- Name: addressbooks_name_key; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.addressbooks
    ADD CONSTRAINT addressbooks_name_key UNIQUE (name);


--
-- Name: addressbooks_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.addressbooks
    ADD CONSTRAINT addressbooks_pkey PRIMARY KEY (id);


--
-- Name: affaires_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.affaires
    ADD CONSTRAINT affaires_pkey PRIMARY KEY (id);


--
-- Name: armodels_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.armodels
    ADD CONSTRAINT armodels_pkey PRIMARY KEY (id);


--
-- Name: aros_acos_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.aros_acos
    ADD CONSTRAINT aros_acos_pkey PRIMARY KEY (id);


--
-- Name: aros_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.aros
    ADD CONSTRAINT aros_pkey PRIMARY KEY (id);


--
-- Name: ars_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.ars
    ADD CONSTRAINT ars_pkey PRIMARY KEY (id);



--
-- Name: bancontenus_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.bancontenus
    ADD CONSTRAINT bancontenus_pkey PRIMARY KEY (id);


--
-- Name: bannettes_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.bannettes
    ADD CONSTRAINT bannettes_pkey PRIMARY KEY (id);


--
-- Name: bans_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.bans
    ADD CONSTRAINT bans_pkey PRIMARY KEY (id);


--
-- Name: bansadresses_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.bansadresses
    ADD CONSTRAINT bansadresses_pkey PRIMARY KEY (id);


--
-- Name: banscommunes_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.banscommunes
    ADD CONSTRAINT banscommunes_pkey PRIMARY KEY (id);


--
-- Name: comments_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.comments
    ADD CONSTRAINT comments_pkey PRIMARY KEY (id);


--
-- Name: comments_readers_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.comments_readers
    ADD CONSTRAINT comments_readers_pkey PRIMARY KEY (id);


--
-- Name: compteurs_nom_key; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.compteurs
    ADD CONSTRAINT compteurs_nom_key UNIQUE (nom);


--
-- Name: compteurs_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.compteurs
    ADD CONSTRAINT compteurs_pkey PRIMARY KEY (id);


--
-- Name: connecteurs_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.connecteurs
    ADD CONSTRAINT connecteurs_pkey PRIMARY KEY (id);


--
-- Name: consultations_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.consultations
    ADD CONSTRAINT consultations_pkey PRIMARY KEY (id);


--
-- Name: contactinfos_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.contactinfos
    ADD CONSTRAINT contactinfos_pkey PRIMARY KEY (id);


--
-- Name: contacts_events_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.contacts_events
    ADD CONSTRAINT contacts_events_pkey PRIMARY KEY (id);


--
-- Name: contacts_operations_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.contacts_operations
    ADD CONSTRAINT contacts_operations_pkey PRIMARY KEY (id);


--
-- Name: contacts_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.contacts
    ADD CONSTRAINT contacts_pkey PRIMARY KEY (id);


--
-- Name: contractants_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.contractants
    ADD CONSTRAINT contractants_pkey PRIMARY KEY (id);


--
-- Name: courriers_metadonnees_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.courriers_metadonnees
    ADD CONSTRAINT courriers_metadonnees_pkey PRIMARY KEY (id);


--
-- Name: courriers_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.courriers
    ADD CONSTRAINT courriers_pkey PRIMARY KEY (id);


--
-- Name: courriers_repertoires_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.courriers_repertoires
    ADD CONSTRAINT courriers_repertoires_pkey PRIMARY KEY (id);


--
-- Name: dacos_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.dacos
    ADD CONSTRAINT dacos_pkey PRIMARY KEY (id);


--
-- Name: daros_dacos_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.daros_dacos
    ADD CONSTRAINT daros_dacos_pkey PRIMARY KEY (id);


--
-- Name: daros_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.daros
    ADD CONSTRAINT daros_pkey PRIMARY KEY (id);


--
-- Name: desktops_desktopsmanagers_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.desktops_desktopsmanagers
    ADD CONSTRAINT desktops_desktopsmanagers_pkey PRIMARY KEY (id);


--
-- Name: desktops_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.desktops
    ADD CONSTRAINT desktops_pkey PRIMARY KEY (id);


--
-- Name: desktops_services_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.desktops_services
    ADD CONSTRAINT desktops_services_pkey PRIMARY KEY (id);


--
-- Name: desktops_taches_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.desktops_taches
    ADD CONSTRAINT desktops_taches_pkey PRIMARY KEY (id);


--
-- Name: desktops_users_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.desktops_users
    ADD CONSTRAINT desktops_users_pkey PRIMARY KEY (id);


--
-- Name: desktopsmanagers_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.desktopsmanagers
    ADD CONSTRAINT desktopsmanagers_pkey PRIMARY KEY (id);


--
-- Name: desktopsmanagers_sousoperations_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY public.desktopsmanagers_sousoperations
    ADD CONSTRAINT desktopsmanagers_sousoperations_pkey PRIMARY KEY (id);


--
-- Name: documents_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.documents
    ADD CONSTRAINT documents_pkey PRIMARY KEY (id);


--
-- Name: dossiers_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.dossiers
    ADD CONSTRAINT dossiers_pkey PRIMARY KEY (id);


--
-- Name: events_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.events
    ADD CONSTRAINT events_pkey PRIMARY KEY (id);


--
-- Name: fonctions_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.fonctions
    ADD CONSTRAINT fonctions_pkey PRIMARY KEY (id);


--
-- Name: formatsreponse_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.formatsreponse
    ADD CONSTRAINT formatsreponse_pkey PRIMARY KEY (id);


--
-- Name: formatsreponse_soustypes_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.formatsreponse_soustypes
    ADD CONSTRAINT formatsreponse_soustypes_pkey PRIMARY KEY (id);


--
-- Name: gabaritsdocuments_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY public.gabaritsdocuments
    ADD CONSTRAINT gabaritsdocuments_pkey PRIMARY KEY (id);


--
-- Name: groups_name_unique; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.profils
    ADD CONSTRAINT profils_name_unique UNIQUE (name);


--
-- Name: groups_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.profils
    ADD CONSTRAINT profils_pkey PRIMARY KEY (id);


--
-- Name: inits_services_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.inits_services
    ADD CONSTRAINT inits_services_pkey PRIMARY KEY (id);


--
-- Name: intituleagentbyoperations_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY public.intituleagentbyoperations
    ADD CONSTRAINT intituleagentbyoperations_pkey PRIMARY KEY (id);


--
-- Name: intitulesagents_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.intitulesagents
    ADD CONSTRAINT intitulesagents_pkey PRIMARY KEY (id);


--
-- Name: journalevents_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.journalevents
    ADD CONSTRAINT journalevents_pkey PRIMARY KEY (id);


--
-- Name: keywordlists_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.keywordlists
    ADD CONSTRAINT keywordlists_pkey PRIMARY KEY (id);


--
-- Name: keywords_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.keywords
    ADD CONSTRAINT keywords_pkey PRIMARY KEY (id);


--
-- Name: keywordtypes_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.keywordtypes
    ADD CONSTRAINT keywordtypes_pkey PRIMARY KEY (id);


--
-- Name: ldapm_groups_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.ldapm_groups
    ADD CONSTRAINT ldapm_groups_pkey PRIMARY KEY (id);


--
-- Name: ldapm_models_groups_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.ldapm_models_groups
    ADD CONSTRAINT ldapm_models_groups_pkey PRIMARY KEY (id);


--
-- Name: marches_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.marches
    ADD CONSTRAINT marches_pkey PRIMARY KEY (id);


--
-- Name: metadonnees_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.metadonnees
    ADD CONSTRAINT metadonnees_pkey PRIMARY KEY (id);


--
-- Name: metadonnees_soustypes_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.metadonnees_soustypes
    ADD CONSTRAINT metadonnees_soustypes_pkey PRIMARY KEY (id);


--
-- Name: notifications_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.notifications
    ADD CONSTRAINT notifications_pkey PRIMARY KEY (id);


--
-- Name: notifieddesktops_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.notifieddesktops
    ADD CONSTRAINT notifieddesktops_pkey PRIMARY KEY (id);


--
-- Name: notifquotidiens_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.notifquotidiens
    ADD CONSTRAINT notifquotidiens_pkey PRIMARY KEY (id);


--
-- Name: operations_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.operations
    ADD CONSTRAINT operations_pkey PRIMARY KEY (id);


--
-- Name: ordresservices_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY public.ordresservices
    ADD CONSTRAINT ordresservices_pkey PRIMARY KEY (id);


--
-- Name: organismes_operations_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.organismes_operations
    ADD CONSTRAINT organismes_operations_pkey PRIMARY KEY (id);


--
-- Name: organismes_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.organismes
    ADD CONSTRAINT organismes_pkey PRIMARY KEY (id);


--
-- Name: originesflux_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.originesflux
    ADD CONSTRAINT originesflux_pkey PRIMARY KEY (id);


--
-- Name: plandelegations_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.plandelegations
    ADD CONSTRAINT plandelegations_pkey PRIMARY KEY (id);


--
-- Name: plisconsultatifs_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.plisconsultatifs
    ADD CONSTRAINT plisconsultatifs_pkey PRIMARY KEY (id);


--
-- Name: recheche_contactinfos_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recheche_contactinfos
    ADD CONSTRAINT recheche_contactinfos_pkey PRIMARY KEY (id);


--
-- Name: recherches_addressbooks_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_addressbooks
    ADD CONSTRAINT recherches_addressbooks_pkey PRIMARY KEY (id);


--
-- Name: recherches_circuits_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_circuits
    ADD CONSTRAINT recherches_circuits_pkey PRIMARY KEY (id);


--
-- Name: recherches_contacts_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_contacts
    ADD CONSTRAINT recherches_contacts_pkey PRIMARY KEY (id);


--
-- Name: recherches_desktops_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_desktops
    ADD CONSTRAINT recherches_desktops_pkey PRIMARY KEY (id);


--
-- Name: recherches_metadonnees_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_metadonnees
    ADD CONSTRAINT recherches_metadonnees_pkey PRIMARY KEY (id);


--
-- Name: recherches_organismes_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_organismes
    ADD CONSTRAINT recherches_organismes_pkey PRIMARY KEY (id);


--
-- Name: recherches_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches
    ADD CONSTRAINT recherches_pkey PRIMARY KEY (id);


--
-- Name: recherches_selectvaluesmetadonnees_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_selectvaluesmetadonnees
    ADD CONSTRAINT recherches_selectvaluesmetadonnees_pkey PRIMARY KEY (id);


--
-- Name: recherches_services_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_services
    ADD CONSTRAINT recherches_services_pkey PRIMARY KEY (id);


--
-- Name: referentielsfantoir_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.referentielsfantoir
    ADD CONSTRAINT referentielsfantoir_pkey PRIMARY KEY (id);


--
-- Name: referentielsfantoircom_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.referentielsfantoircom
    ADD CONSTRAINT referentielsfantoircom_pkey PRIMARY KEY (id);


--
-- Name: referentielsfantoirdir_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.referentielsfantoirdir
    ADD CONSTRAINT referentielsfantoirdir_pkey PRIMARY KEY (id);


--
-- Name: referentielsfantoirvoie_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.referentielsfantoirvoie
    ADD CONSTRAINT referentielsfantoirvoie_pkey PRIMARY KEY (id);


--
-- Name: relements_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.relements
    ADD CONSTRAINT relements_pkey PRIMARY KEY (id);


--
-- Name: repertoires_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.repertoires
    ADD CONSTRAINT repertoires_pkey PRIMARY KEY (id);


--
-- Name: rgpds_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.rgpds
    ADD CONSTRAINT rgpds_pkey PRIMARY KEY (id);


--
-- Name: rights_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.rights
    ADD CONSTRAINT rights_pkey PRIMARY KEY (id);


--
-- Name: rmodelgabarits_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.rmodelgabarits
    ADD CONSTRAINT rmodelgabarits_pkey PRIMARY KEY (id);


--
-- Name: rmodels_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.rmodels
    ADD CONSTRAINT rmodels_pkey PRIMARY KEY (id);


--
-- Name: scanemails_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.scanemails
    ADD CONSTRAINT scanemails_pkey PRIMARY KEY (id);


--
-- Name: selectvaluesmetadonnees_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.selectvaluesmetadonnees
    ADD CONSTRAINT selectvaluesmetadonnees_pkey PRIMARY KEY (id);


--
-- Name: sendsmails_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.sendsmails
    ADD CONSTRAINT sendsmails_pkey PRIMARY KEY (id);


--
-- Name: sequences_nom_key; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.sequences
    ADD CONSTRAINT sequences_nom_key UNIQUE (nom);


--
-- Name: sequences_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.sequences
    ADD CONSTRAINT sequences_pkey PRIMARY KEY (id);


--
-- Name: services_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.services
    ADD CONSTRAINT services_pkey PRIMARY KEY (id);


--
-- Name: sms_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.sms
    ADD CONSTRAINT sms_pkey PRIMARY KEY (id);


--
-- Name: sousactivites_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.sousactivites
    ADD CONSTRAINT sousactivites_pkey PRIMARY KEY (id);


--
-- Name: sousoperations_pkey; Type: CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY public.sousoperations
    ADD CONSTRAINT sousoperations_pkey PRIMARY KEY (id);


--
-- Name: soustypes_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.soustypes
    ADD CONSTRAINT soustypes_pkey PRIMARY KEY (id);


--
-- Name: soustypes_soustypescibles_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.soustypes_soustypescibles
    ADD CONSTRAINT soustypes_soustypescibles_pkey PRIMARY KEY (id);


--
-- Name: taches_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.taches
    ADD CONSTRAINT taches_pkey PRIMARY KEY (id);


--
-- Name: templatemails_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.templatemails
    ADD CONSTRAINT templatemails_pkey PRIMARY KEY (id);


--
-- Name: templatenotifications_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.templatenotifications
    ADD CONSTRAINT templatenotifications_pkey PRIMARY KEY (id);

--
-- Name: titres_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.titres
    ADD CONSTRAINT titres_pkey PRIMARY KEY (id);

--
-- Name: typemetadonnees_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.typemetadonnees
    ADD CONSTRAINT typemetadonnees_pkey PRIMARY KEY (id);


--
-- Name: typenotifs_name_key; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.typenotifs
    ADD CONSTRAINT typenotifs_name_key UNIQUE (name);


--
-- Name: typenotifs_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.typenotifs
    ADD CONSTRAINT typenotifs_pkey PRIMARY KEY (id);


--
-- Name: typenotifs_users_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.typenotifs_users
    ADD CONSTRAINT typenotifs_users_pkey PRIMARY KEY (id);


--
-- Name: types_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.types
    ADD CONSTRAINT types_pkey PRIMARY KEY (id);


--
-- Name: uniterifs_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.uniterifs
    ADD CONSTRAINT uniterifs_pkey PRIMARY KEY (id);


--
-- Name: users_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.users
    ADD CONSTRAINT users_pkey PRIMARY KEY (id);


--
-- Name: wkf_circuits_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.wkf_circuits
    ADD CONSTRAINT wkf_circuits_pkey PRIMARY KEY (id);


--
-- Name: wkf_compositions_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.wkf_compositions
    ADD CONSTRAINT wkf_compositions_pkey PRIMARY KEY (id);


--
-- Name: wkf_etapes_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.wkf_etapes
    ADD CONSTRAINT wkf_etapes_pkey PRIMARY KEY (id);


--
-- Name: wkf_signatures_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.wkf_signatures
    ADD CONSTRAINT wkf_signatures_pkey PRIMARY KEY (id);


--
-- Name: wkf_traitements_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.wkf_traitements
    ADD CONSTRAINT wkf_traitements_pkey PRIMARY KEY (id);


--
-- Name: wkf_visas_pkey; Type: CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.wkf_visas
    ADD CONSTRAINT wkf_visas_pkey PRIMARY KEY (id);


--
-- Name: acos_alias_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX acos_alias_idx ON public.acos USING btree (alias);


--
-- Name: acos_lft_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX acos_lft_idx ON public.acos USING btree (lft);


--
-- Name: acos_parent_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX acos_parent_id_idx ON public.acos USING btree (parent_id);


--
-- Name: acos_rght_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX acos_rght_idx ON public.acos USING btree (rght);


--
-- Name: affaires_dossier_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX affaires_dossier_id_idx ON public.affaires USING btree (dossier_id);


--
-- Name: aro_aco_key; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE UNIQUE INDEX aro_aco_key ON public.aros_acos USING btree (aro_id, aco_id);


--
-- Name: aros_acos_aco_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX aros_acos_aco_id_idx ON public.aros_acos USING btree (aco_id);


--
-- Name: aros_acos_aro_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX aros_acos_aro_id_idx ON public.aros_acos USING btree (aro_id);


--
-- Name: aros_alias_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX aros_alias_idx ON public.aros USING btree (alias);


--
-- Name: aros_aros_aco_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX aros_aros_aco_id_idx ON public.aros_acos USING btree (aco_id);


--
-- Name: aros_aros_aro_id_aco_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE UNIQUE INDEX aros_aros_aro_id_aco_id_idx ON public.aros_acos USING btree (aro_id, aco_id);


--
-- Name: aros_aros_aro_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX aros_aros_aro_id_idx ON public.aros_acos USING btree (aro_id);


--
-- Name: aros_foreign_key_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX aros_foreign_key_idx ON public.aros USING btree (foreign_key);


--
-- Name: aros_lft_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX aros_lft_idx ON public.aros USING btree (lft);


--
-- Name: aros_model_alias_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE UNIQUE INDEX aros_model_alias_idx ON public.aros USING btree (model, alias);


--
-- Name: aros_model_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX aros_model_idx ON public.aros USING btree (model);


--
-- Name: aros_parent_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX aros_parent_id_idx ON public.aros USING btree (parent_id);


--
-- Name: aros_rght_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX aros_rght_idx ON public.aros USING btree (rght);


--
-- Name: bancontenus_bannette_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX bancontenus_bannette_id_idx ON public.bancontenus USING btree (bannette_id);


--
-- Name: bancontenus_courrier_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX bancontenus_courrier_id_idx ON public.bancontenus USING btree (courrier_id);


--
-- Name: bancontenus_etat_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX bancontenus_etat_id_idx ON public.bancontenus USING btree (etat);


--
-- Name: bans_name_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE UNIQUE INDEX bans_name_idx ON public.bans USING btree (name);


--
-- Name: banscommunes_name_ban_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE UNIQUE INDEX banscommunes_name_ban_id_idx ON public.banscommunes USING btree (name, ban_id);


--
-- Name: comments_slug_noaccents_upper; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX comments_slug_noaccents_upper ON public.comments USING btree (public.noaccents_upper((slug)::text));


--
-- Name: compositions_etape_id_trigger_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE UNIQUE INDEX compositions_etape_id_trigger_id_idx ON public.wkf_compositions USING btree (etape_id, trigger_id);


--
-- Name: compteurs_sequence_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX compteurs_sequence_id_idx ON public.compteurs USING btree (sequence_id);


--
-- Name: consultations_name_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE UNIQUE INDEX consultations_name_idx ON public.consultations USING btree (name);


--
-- Name: consultations_operation_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX consultations_operation_id_idx ON public.consultations USING btree (operation_id);


--
-- Name: courriers_affaire_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX courriers_affaire_id_idx ON public.courriers USING btree (affaire_id);


--
-- Name: courriers_metadonnees_courrier_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX courriers_metadonnees_courrier_id_idx ON public.courriers_metadonnees USING btree (courrier_id);


--
-- Name: courriers_metadonnees_courrier_id_metadonnee_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE UNIQUE INDEX courriers_metadonnees_courrier_id_metadonnee_id_idx ON public.courriers_metadonnees USING btree (courrier_id, metadonnee_id);


--
-- Name: courriers_metadonnees_metadonnee_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX courriers_metadonnees_metadonnee_id_idx ON public.courriers_metadonnees USING btree (metadonnee_id);


--
-- Name: courriers_parent_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX courriers_parent_id_idx ON public.courriers USING btree (parent_id);


--
-- Name: courriers_reference_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE UNIQUE INDEX courriers_reference_idx ON public.courriers USING btree (reference);


--
-- Name: courriers_repertoires_courrier_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX courriers_repertoires_courrier_id_idx ON public.courriers_repertoires USING btree (courrier_id);


--
-- Name: courriers_repertoires_courrier_id_repertoire_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE UNIQUE INDEX courriers_repertoires_courrier_id_repertoire_id_idx ON public.courriers_repertoires USING btree (courrier_id, repertoire_id);


--
-- Name: courriers_repertoires_repertoire_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX courriers_repertoires_repertoire_id_idx ON public.courriers_repertoires USING btree (repertoire_id);


--
-- Name: desktops_desktopsmanagers_desktop_id_desktopmanager_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE UNIQUE INDEX desktops_desktopsmanagers_desktop_id_desktopmanager_id_idx ON public.desktops_desktopsmanagers USING btree (desktop_id, desktopmanager_id);


--
-- Name: desktops_desktopsmanagers_desktop_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX desktops_desktopsmanagers_desktop_id_idx ON public.desktops_desktopsmanagers USING btree (desktop_id);


--
-- Name: desktops_desktopsmanagers_desktopmanager_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX desktops_desktopsmanagers_desktopmanager_id_idx ON public.desktops_desktopsmanagers USING btree (desktopmanager_id);


--
-- Name: desktopsmanagers_sousoperations_desktopmanager_id_idx; Type: INDEX; Schema: public; Owner: postgres
--

CREATE INDEX desktopsmanagers_sousoperations_desktopmanager_id_idx ON public.desktopsmanagers_sousoperations USING btree (desktopmanager_id);


--
-- Name: desktopsmanagers_sousoperations_desktopmanager_id_sousoperation; Type: INDEX; Schema: public; Owner: postgres
--

CREATE UNIQUE INDEX desktopsmanagers_sousoperations_desktopmanager_id_sousoperation ON public.desktopsmanagers_sousoperations USING btree (desktopmanager_id, sousoperation_id);


--
-- Name: desktopsmanagers_sousoperations_intituleagent_id_idx; Type: INDEX; Schema: public; Owner: postgres
--

CREATE INDEX desktopsmanagers_sousoperations_intituleagent_id_idx ON public.desktopsmanagers_sousoperations USING btree (intituleagent_id);


--
-- Name: desktopsmanagers_sousoperations_sousoperation_id_idx; Type: INDEX; Schema: public; Owner: postgres
--

CREATE INDEX desktopsmanagers_sousoperations_sousoperation_id_idx ON public.desktopsmanagers_sousoperations USING btree (sousoperation_id);


--
-- Name: desktopsmanagers_sousoperations_sousoperation_id_ordre_idx; Type: INDEX; Schema: public; Owner: postgres
--

CREATE UNIQUE INDEX desktopsmanagers_sousoperations_sousoperation_id_ordre_idx ON public.desktopsmanagers_sousoperations USING btree (sousoperation_id, ordre);


--
-- Name: formatsreponse_soustypes_formatreponse_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX formatsreponse_soustypes_formatreponse_id_idx ON public.formatsreponse_soustypes USING btree (formatreponse_id);


--
-- Name: formatsreponse_soustypes_soustype_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX formatsreponse_soustypes_soustype_id_idx ON public.formatsreponse_soustypes USING btree (soustype_id);


--
-- Name: intituleagentbyoperations_desktopmanager_id_idx; Type: INDEX; Schema: public; Owner: postgres
--

CREATE INDEX intituleagentbyoperations_desktopmanager_id_idx ON public.intituleagentbyoperations USING btree (desktopmanager_id);


--
-- Name: intituleagentbyoperations_intituleagent_id_idx; Type: INDEX; Schema: public; Owner: postgres
--

CREATE INDEX intituleagentbyoperations_intituleagent_id_idx ON public.intituleagentbyoperations USING btree (intituleagent_id);


--
-- Name: intituleagentbyoperations_operation_id_idx; Type: INDEX; Schema: public; Owner: postgres
--

CREATE INDEX intituleagentbyoperations_operation_id_idx ON public.intituleagentbyoperations USING btree (operation_id);


--
-- Name: intituleagentbyoperations_operation_id_intituleagent_id_idx; Type: INDEX; Schema: public; Owner: postgres
--

CREATE UNIQUE INDEX intituleagentbyoperations_operation_id_intituleagent_id_idx ON public.intituleagentbyoperations USING btree (intituleagent_id, operation_id);


--
-- Name: marches_name_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE UNIQUE INDEX marches_name_idx ON public.marches USING btree (name);


--
-- Name: marches_operation_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX marches_operation_id_idx ON public.marches USING btree (operation_id);


--
-- Name: metadonnees_soustypes_metadonnee_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX metadonnees_soustypes_metadonnee_id_idx ON public.metadonnees_soustypes USING btree (metadonnee_id);


--
-- Name: metadonnees_soustypes_metadonnee_id_soustype_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE UNIQUE INDEX metadonnees_soustypes_metadonnee_id_soustype_id_idx ON public.metadonnees_soustypes USING btree (metadonnee_id, soustype_id);


--
-- Name: metadonnees_soustypes_soustype_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX metadonnees_soustypes_soustype_id_idx ON public.metadonnees_soustypes USING btree (soustype_id);


--
-- Name: metadonnees_typemetadonnee_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX metadonnees_typemetadonnee_id_idx ON public.metadonnees USING btree (typemetadonnee_id);


--
-- Name: ordresservices_marche_id_idx; Type: INDEX; Schema: public; Owner: postgres
--

CREATE INDEX ordresservices_marche_id_idx ON public.ordresservices USING btree (marche_id);


--
-- Name: organismes_addressbook_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX organismes_addressbook_id_idx ON public.organismes USING btree (addressbook_id);


--
-- Name: organismes_ban_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX organismes_ban_id_idx ON public.organismes USING btree (ban_id);


--
-- Name: organismes_bancommune_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX organismes_bancommune_id_idx ON public.organismes USING btree (bancommune_id);


--
-- Name: plisconsultatifs_consultation_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX plisconsultatifs_consultation_id_idx ON public.plisconsultatifs USING btree (consultation_id);


--
-- Name: plisconsultatifs_origineflux_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX plisconsultatifs_origineflux_id_idx ON public.plisconsultatifs USING btree (origineflux_id);


--
-- Name: recherches_affaires_affaire_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX recherches_affaires_affaire_id_idx ON public.recherches_affaires USING btree (affaire_id);


--
-- Name: recherches_affaires_recherche_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX recherches_affaires_recherche_id_idx ON public.recherches_affaires USING btree (recherche_id);


--
-- Name: recherches_circuits_circuit_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX recherches_circuits_circuit_id_idx ON public.recherches_circuits USING btree (circuit_id);


--
-- Name: recherches_circuits_recherche_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX recherches_circuits_recherche_id_idx ON public.recherches_circuits USING btree (recherche_id);


--
-- Name: recherches_desktops_desktop_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX recherches_desktops_desktop_id_idx ON public.recherches_desktops USING btree (desktop_id);


--
-- Name: recherches_desktops_recherche_id_desktop_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE UNIQUE INDEX recherches_desktops_recherche_id_desktop_id_idx ON public.recherches_desktops USING btree (recherche_id, desktop_id);


--
-- Name: recherches_desktops_recherche_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX recherches_desktops_recherche_id_idx ON public.recherches_desktops USING btree (recherche_id);


--
-- Name: recherches_dossiers_dossier_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX recherches_dossiers_dossier_id_idx ON public.recherches_dossiers USING btree (dossier_id);


--
-- Name: recherches_dossiers_recherche_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX recherches_dossiers_recherche_id_idx ON public.recherches_dossiers USING btree (recherche_id);


--
-- Name: recherches_metadonnees_metadonnee_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX recherches_metadonnees_metadonnee_id_idx ON public.recherches_metadonnees USING btree (metadonnee_id);


--
-- Name: recherches_metadonnees_recherche_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX recherches_metadonnees_recherche_id_idx ON public.recherches_metadonnees USING btree (recherche_id);


--
-- Name: recherches_organismes_organisme_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX recherches_organismes_organisme_id_idx ON public.recherches_organismes USING btree (organisme_id);


--
-- Name: recherches_organismes_recherche_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX recherches_organismes_recherche_id_idx ON public.recherches_organismes USING btree (recherche_id);


--
-- Name: recherches_organismes_recherche_id_organisme_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE UNIQUE INDEX recherches_organismes_recherche_id_organisme_id_idx ON public.recherches_organismes USING btree (recherche_id, organisme_id);


--
-- Name: recherches_selectvaluesmetadonnees_recherche_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX recherches_selectvaluesmetadonnees_recherche_id_idx ON public.recherches_selectvaluesmetadonnees USING btree (recherche_id);


--
-- Name: recherches_selectvaluesmetadonnees_recherche_id_selectvaluemeta; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE UNIQUE INDEX recherches_selectvaluesmetadonnees_recherche_id_selectvaluemeta ON public.recherches_selectvaluesmetadonnees USING btree (recherche_id, selectvaluemetadonnee_id);


--
-- Name: recherches_services_recherche_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX recherches_services_recherche_id_idx ON public.recherches_services USING btree (recherche_id);


--
-- Name: recherches_services_recherche_id_service_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE UNIQUE INDEX recherches_services_recherche_id_service_id_idx ON public.recherches_services USING btree (recherche_id, service_id);


--
-- Name: recherches_services_service_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX recherches_services_service_id_idx ON public.recherches_services USING btree (service_id);


--
-- Name: recherches_soustypes_recherche_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX recherches_soustypes_recherche_id_idx ON public.recherches_soustypes USING btree (recherche_id);


--
-- Name: recherches_soustypes_soustype_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX recherches_soustypes_soustype_id_idx ON public.recherches_soustypes USING btree (soustype_id);


--
-- Name: recherches_taches_recherche_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX recherches_taches_recherche_id_idx ON public.recherches_taches USING btree (recherche_id);


--
-- Name: recherches_taches_tache_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX recherches_taches_tache_id_idx ON public.recherches_taches USING btree (tache_id);


--
-- Name: recherches_types_recherche_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX recherches_types_recherche_id_idx ON public.recherches_types USING btree (recherche_id);


--
-- Name: recherches_types_type_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX recherches_types_type_id_idx ON public.recherches_types USING btree (type_id);


--
-- Name: recherches_user_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX recherches_user_id_idx ON public.recherches USING btree (user_id);


--
-- Name: repertoires_parent_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX repertoires_parent_id_idx ON public.repertoires USING btree (parent_id);


--
-- Name: repertoires_user_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX repertoires_user_id_idx ON public.repertoires USING btree (user_id);


--
-- Name: rmodelgabarits_soustype_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX rmodelgabarits_soustype_id_idx ON public.rmodelgabarits USING btree (soustype_id);


--
-- Name: scanemails_desktop_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX scanemails_desktop_id_idx ON public.scanemails USING btree (desktop_id);


--
-- Name: scanemails_name_desktop_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE UNIQUE INDEX scanemails_name_desktop_id_idx ON public.scanemails USING btree (name, desktop_id);


--
-- Name: scanemails_soustype_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX scanemails_soustype_id_idx ON public.scanemails USING btree (soustype_id);


--
-- Name: scanemails_type_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX scanemails_type_id_idx ON public.scanemails USING btree (type_id);


--
-- Name: sendsmails_courrier_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX sendsmails_courrier_id_idx ON public.sendsmails USING btree (courrier_id);


--
-- Name: sendsmails_desktop_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX sendsmails_desktop_id_idx ON public.sendsmails USING btree (desktop_id);


--
-- Name: sendsmails_document_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX sendsmails_document_id_idx ON public.sendsmails USING btree (document_id);


--
-- Name: services_parent_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX services_parent_id_idx ON public.services USING btree (parent_id);


--
-- Name: soustypes_circuit_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX soustypes_circuit_id_idx ON public.soustypes USING btree (circuit_id);


--
-- Name: soustypes_compteur_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX soustypes_compteur_id_idx ON public.soustypes USING btree (compteur_id);


--
-- Name: soustypes_parent_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX soustypes_parent_id_idx ON public.soustypes USING btree (parent_id);


--
-- Name: soustypes_type_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX soustypes_type_id_idx ON public.soustypes USING btree (type_id);


CREATE INDEX soustypes_soustypescibles_soustype_id_idx ON public.soustypes_soustypescibles USING btree ( soustype_id );

CREATE INDEX soustypes_soustypescibles_soustypecible_id_idx ON public.soustypes_soustypescibles USING btree ( soustypecible_id );

CREATE INDEX soustypes_soustypescibles_soustype_id_soustypecible_id_idx ON public.soustypes_soustypescibles USING btree (soustype_id,soustypecible_id);

--
-- Name: taches_act_user_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX taches_act_user_id_idx ON public.taches USING btree (act_desktop_id);


--
-- Name: taches_courrier_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX taches_courrier_id_idx ON public.taches USING btree (courrier_id);


--
-- Name: wkf_cicruits_unique_nom; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE UNIQUE INDEX wkf_cicruits_unique_nom ON public.wkf_circuits USING btree (nom);


--
-- Name: wkf_circuits_created_user_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX wkf_circuits_created_user_id_idx ON public.wkf_circuits USING btree (created_user_id);


--
-- Name: wkf_circuits_modified_user_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX wkf_circuits_modified_user_id_idx ON public.wkf_circuits USING btree (modified_user_id);


--
-- Name: wkf_compositions_created_user_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX wkf_compositions_created_user_id_idx ON public.wkf_compositions USING btree (created_user_id);


--
-- Name: wkf_compositions_etape_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX wkf_compositions_etape_id_idx ON public.wkf_compositions USING btree (etape_id);


--
-- Name: wkf_compositions_modified_user_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX wkf_compositions_modified_user_id_idx ON public.wkf_compositions USING btree (modified_user_id);


--
-- Name: wkf_compositions_trigger_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX wkf_compositions_trigger_id_idx ON public.wkf_compositions USING btree (trigger_id);


--
-- Name: wkf_etapes_circuit_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX wkf_etapes_circuit_id_idx ON public.wkf_etapes USING btree (circuit_id);


--
-- Name: wkf_etapes_created_user_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX wkf_etapes_created_user_id_idx ON public.wkf_etapes USING btree (created_user_id);


--
-- Name: wkf_etapes_modified_user_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX wkf_etapes_modified_user_id_idx ON public.wkf_etapes USING btree (modified_user_id);


--
-- Name: wkf_traitements_circuit_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX wkf_traitements_circuit_id_idx ON public.wkf_traitements USING btree (circuit_id);


--
-- Name: wkf_traitements_created_user_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX wkf_traitements_created_user_id_idx ON public.wkf_traitements USING btree (created_user_id);


--
-- Name: wkf_traitements_modified_user_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX wkf_traitements_modified_user_id_idx ON public.wkf_traitements USING btree (modified_user_id);


--
-- Name: wkf_traitements_target_id_key; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX wkf_traitements_target_id_key ON public.wkf_traitements USING btree (target_id);


--
-- Name: wkf_visas_signature_id_idx; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX wkf_visas_signature_id_idx ON public.wkf_visas USING btree (signature_id);


--
-- Name: wkf_visas_traitement_id_key; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX wkf_visas_traitement_id_key ON public.wkf_visas USING btree (traitement_id);


--
-- Name: wkf_visas_trigger_id_key; Type: INDEX; Schema: public; Owner: webgfc
--

CREATE INDEX wkf_visas_trigger_id_key ON public.wkf_visas USING btree (trigger_id);


--
-- Name: aco_fk; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.aros_acos
    ADD CONSTRAINT aco_fk FOREIGN KEY (aco_id) REFERENCES public.acos(id);


--
-- Name: activites_organismes_activite_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.activites_organismes
    ADD CONSTRAINT activites_organismes_activite_id_fkey FOREIGN KEY (activite_id) REFERENCES public.activites(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: activites_organismes_organisme_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.activites_organismes
    ADD CONSTRAINT activites_organismes_organisme_id_fkey FOREIGN KEY (organisme_id) REFERENCES public.organismes(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: affaires_dossiers_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.affaires
    ADD CONSTRAINT affaires_dossiers_fkey FOREIGN KEY (dossier_id) REFERENCES public.dossiers(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: armodels_soustype_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.armodels
    ADD CONSTRAINT armodels_soustype_id_fkey FOREIGN KEY (soustype_id) REFERENCES public.soustypes(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: aro_fk; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.aros_acos
    ADD CONSTRAINT aro_fk FOREIGN KEY (aro_id) REFERENCES public.aros(id);


--
-- Name: ars_courrier_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.ars
    ADD CONSTRAINT ars_courrier_id_fkey FOREIGN KEY (courrier_id) REFERENCES public.courriers(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: bancontenus_bannettes_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.bancontenus
    ADD CONSTRAINT bancontenus_bannettes_fkey FOREIGN KEY (bannette_id) REFERENCES public.bannettes(id);


--
-- Name: bancontenus_courriers_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.bancontenus
    ADD CONSTRAINT bancontenus_courriers_fkey FOREIGN KEY (courrier_id) REFERENCES public.courriers(id);


--
-- Name: bancontenus_desktop_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.bancontenus
    ADD CONSTRAINT bancontenus_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES public.desktops(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: bancontenus_desktopmanager_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.bancontenus
    ADD CONSTRAINT bancontenus_desktopmanager_id_fkey FOREIGN KEY (desktopmanager_id) REFERENCES public.desktopsmanagers(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: bansadresses_bancommune_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.bansadresses
    ADD CONSTRAINT bansadresses_bancommune_id_fkey FOREIGN KEY (bancommune_id) REFERENCES public.banscommunes(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: banscommunes_ban_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.banscommunes
    ADD CONSTRAINT banscommunes_ban_id_fkey FOREIGN KEY (ban_id) REFERENCES public.bans(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: comments_target_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.comments
    ADD CONSTRAINT comments_target_id_fkey FOREIGN KEY (target_id) REFERENCES public.courriers(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: comments_readers_comment_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.comments_readers
    ADD CONSTRAINT comments_readers_comment_id_fkey FOREIGN KEY (comment_id) REFERENCES public.comments(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: compteurs_sequence_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.compteurs
    ADD CONSTRAINT compteurs_sequence_id_fkey FOREIGN KEY (sequence_id) REFERENCES public.sequences(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: consultations_operation_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.consultations
    ADD CONSTRAINT consultations_operation_id_fkey FOREIGN KEY (operation_id) REFERENCES public.operations(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: contactinfos_contact_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.contactinfos
    ADD CONSTRAINT contactinfos_contact_id_fkey FOREIGN KEY (contact_id) REFERENCES public.contacts(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: contacts_addressbook_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.contacts
    ADD CONSTRAINT contacts_addressbook_id_fkey FOREIGN KEY (addressbook_id) REFERENCES public.addressbooks(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: contacts_ban_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.contacts
    ADD CONSTRAINT contacts_ban_id_fkey FOREIGN KEY (ban_id) REFERENCES public.bans(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: contacts_bancommune_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.contacts
    ADD CONSTRAINT contacts_bancommune_id_fkey FOREIGN KEY (bancommune_id) REFERENCES public.banscommunes(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: contacts_events_contact_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.contacts_events
    ADD CONSTRAINT contacts_events_contact_id_fkey FOREIGN KEY (contact_id) REFERENCES public.contacts(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: contacts_events_event_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.contacts_events
    ADD CONSTRAINT contacts_events_event_id_fkey FOREIGN KEY (event_id) REFERENCES public.events(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: contacts_fonction_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.contacts
    ADD CONSTRAINT contacts_fonction_id_fkey FOREIGN KEY (fonction_id) REFERENCES public.fonctions(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: contacts_operations_contact_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.contacts_operations
    ADD CONSTRAINT contacts_operations_contact_id_fkey FOREIGN KEY (contact_id) REFERENCES public.contacts(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: contacts_operations_operation_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.contacts_operations
    ADD CONSTRAINT contacts_operations_operation_id_fkey FOREIGN KEY (operation_id) REFERENCES public.operations(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: contacts_organismes_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.contacts
    ADD CONSTRAINT contacts_organismes_fkey FOREIGN KEY (organisme_id) REFERENCES public.organismes(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: contacts_titre_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.contacts
    ADD CONSTRAINT contacts_titre_id_fkey FOREIGN KEY (titre_id) REFERENCES public.titres(id) ON UPDATE CASCADE ON DELETE SET NULL;

--
-- Name: couriers_affaires_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.courriers
    ADD CONSTRAINT couriers_affaires_fkey FOREIGN KEY (affaire_id) REFERENCES public.affaires(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: courriers_affairesuiviepar_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.courriers
    ADD CONSTRAINT courriers_affairesuiviepar_id_fkey FOREIGN KEY (affairesuiviepar_id) REFERENCES public.desktops(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: courriers_consultation_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.courriers
    ADD CONSTRAINT courriers_consultation_id_fkey FOREIGN KEY (consultation_id) REFERENCES public.consultations(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: courriers_contact_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.courriers
    ADD CONSTRAINT courriers_contact_id_fkey FOREIGN KEY (contact_id) REFERENCES public.contacts(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: courriers_contactinfo_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.courriers
    ADD CONSTRAINT courriers_contactinfo_id_fkey FOREIGN KEY (contactinfo_id) REFERENCES public.contactinfos(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: courriers_desktop_creator_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.courriers
    ADD CONSTRAINT courriers_desktop_creator_id_fkey FOREIGN KEY (desktop_creator_id) REFERENCES public.desktops(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: courriers_marche_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.courriers
    ADD CONSTRAINT courriers_marche_id_fkey FOREIGN KEY (marche_id) REFERENCES public.marches(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: courriers_metadonnees_courriers_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.courriers_metadonnees
    ADD CONSTRAINT courriers_metadonnees_courriers_fkey FOREIGN KEY (courrier_id) REFERENCES public.courriers(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: courriers_metadonnees_metadonnees_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.courriers_metadonnees
    ADD CONSTRAINT courriers_metadonnees_metadonnees_fkey FOREIGN KEY (metadonnee_id) REFERENCES public.metadonnees(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: courriers_organisme_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.courriers
    ADD CONSTRAINT courriers_organisme_id_fkey FOREIGN KEY (organisme_id) REFERENCES public.organismes(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: courriers_repertoires_courriers_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.courriers_repertoires
    ADD CONSTRAINT courriers_repertoires_courriers_fkey FOREIGN KEY (courrier_id) REFERENCES public.courriers(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: courriers_repertoires_repertoires_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.courriers_repertoires
    ADD CONSTRAINT courriers_repertoires_repertoires_fkey FOREIGN KEY (repertoire_id) REFERENCES public.repertoires(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: courriers_service_creator_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.courriers
    ADD CONSTRAINT courriers_service_creator_id_fkey FOREIGN KEY (service_creator_id) REFERENCES public.services(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: courriers_soustypes_soustype_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.courriers
    ADD CONSTRAINT courriers_soustypes_soustype_id_fkey FOREIGN KEY (soustype_id) REFERENCES public.soustypes(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: courriers_taches_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.taches
    ADD CONSTRAINT courriers_taches_fkey FOREIGN KEY (courrier_id) REFERENCES public.courriers(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: courriers_user_creator_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.courriers
    ADD CONSTRAINT courriers_user_creator_id_fkey FOREIGN KEY (user_creator_id) REFERENCES public.users(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: desktops_desktopsmanagers_desktop_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.desktops_desktopsmanagers
    ADD CONSTRAINT desktops_desktopsmanagers_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES public.desktops(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: desktops_desktopsmanagers_desktopmanager_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.desktops_desktopsmanagers
    ADD CONSTRAINT desktops_desktopsmanagers_desktopmanager_id_fkey FOREIGN KEY (desktopmanager_id) REFERENCES public.desktopsmanagers(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: desktops_profil_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.desktops
    ADD CONSTRAINT desktops_profil_id_fkey FOREIGN KEY (profil_id) REFERENCES public.profils(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: desktops_services_desktop_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.desktops_services
    ADD CONSTRAINT desktops_services_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES public.desktops(id);


--
-- Name: desktops_services_service_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.desktops_services
    ADD CONSTRAINT desktops_services_service_id_fkey FOREIGN KEY (service_id) REFERENCES public.services(id);


--
-- Name: desktops_taches_desktop_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.desktops_taches
    ADD CONSTRAINT desktops_taches_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES public.desktops(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: desktops_taches_tache_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.desktops_taches
    ADD CONSTRAINT desktops_taches_tache_id_fkey FOREIGN KEY (tache_id) REFERENCES public.taches(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: desktops_users_desktop_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.desktops_users
    ADD CONSTRAINT desktops_users_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES public.desktops(id);


--
-- Name: desktops_users_user_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.desktops_users
    ADD CONSTRAINT desktops_users_user_id_fkey FOREIGN KEY (user_id) REFERENCES public.users(id);


--
-- Name: desktopsmanagers_intituleagent_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.desktopsmanagers
    ADD CONSTRAINT desktopsmanagers_intituleagent_id_fkey FOREIGN KEY (intituleagent_id) REFERENCES public.intitulesagents(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: desktopsmanagers_sousoperations_desktopmanager_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY public.desktopsmanagers_sousoperations
    ADD CONSTRAINT desktopsmanagers_sousoperations_desktopmanager_id_fkey FOREIGN KEY (desktopmanager_id) REFERENCES public.desktopsmanagers(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: desktopsmanagers_sousoperations_intituleagent_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY public.desktopsmanagers_sousoperations
    ADD CONSTRAINT desktopsmanagers_sousoperations_intituleagent_id_fkey FOREIGN KEY (intituleagent_id) REFERENCES public.intitulesagents(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: desktopsmanagers_sousoperations_sousoperation_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY public.desktopsmanagers_sousoperations
    ADD CONSTRAINT desktopsmanagers_sousoperations_sousoperation_id_fkey FOREIGN KEY (sousoperation_id) REFERENCES public.sousoperations(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: documents_courrier_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.documents
    ADD CONSTRAINT documents_courrier_id_fkey FOREIGN KEY (courrier_id) REFERENCES public.courriers(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: formatsreponse_soustypes_formatreponse_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.formatsreponse_soustypes
    ADD CONSTRAINT formatsreponse_soustypes_formatreponse_id_fkey FOREIGN KEY (formatreponse_id) REFERENCES public.formatsreponse(id);


--
-- Name: formatsreponse_soustypes_soustype_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.formatsreponse_soustypes
    ADD CONSTRAINT formatsreponse_soustypes_soustype_id_fkey FOREIGN KEY (soustype_id) REFERENCES public.soustypes(id);


--
-- Name: inits_services_desktop_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.inits_services
    ADD CONSTRAINT inits_services_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES public.desktops(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: inits_services_service_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.inits_services
    ADD CONSTRAINT inits_services_service_id_fkey FOREIGN KEY (service_id) REFERENCES public.services(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: intituleagentbyoperations_desktopmanager_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY public.intituleagentbyoperations
    ADD CONSTRAINT intituleagentbyoperations_desktopmanager_id_fkey FOREIGN KEY (desktopmanager_id) REFERENCES public.desktopsmanagers(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: intituleagentbyoperations_intituleagent_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY public.intituleagentbyoperations
    ADD CONSTRAINT intituleagentbyoperations_intituleagent_id_fkey FOREIGN KEY (intituleagent_id) REFERENCES public.intitulesagents(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: intituleagentbyoperations_operation_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY public.intituleagentbyoperations
    ADD CONSTRAINT intituleagentbyoperations_operation_id_fkey FOREIGN KEY (operation_id) REFERENCES public.operations(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: journalevents_desktop_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.journalevents
    ADD CONSTRAINT journalevents_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES public.desktops(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: journalevents_user_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.journalevents
    ADD CONSTRAINT journalevents_user_id_fkey FOREIGN KEY (user_id) REFERENCES public.users(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: keywordlists_keywodtype_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.keywordlists
    ADD CONSTRAINT keywordlists_keywodtype_id_fkey FOREIGN KEY (keywordtype_id) REFERENCES public.keywordtypes(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: keywords_keywordlist_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.keywords
    ADD CONSTRAINT keywords_keywordlist_id_fkey FOREIGN KEY (keywordlist_id) REFERENCES public.keywordlists(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: ldapm_models_groups_ldapm_groups_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.ldapm_models_groups
    ADD CONSTRAINT ldapm_models_groups_ldapm_groups_id_fkey FOREIGN KEY (ldapm_groups_id) REFERENCES public.ldapm_groups(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: marches_operation_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.marches
    ADD CONSTRAINT marches_operation_id_fkey FOREIGN KEY (operation_id) REFERENCES public.operations(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: metadonnees_soustypes_metadonnee_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.metadonnees_soustypes
    ADD CONSTRAINT metadonnees_soustypes_metadonnee_id_fkey FOREIGN KEY (metadonnee_id) REFERENCES public.metadonnees(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: metadonnees_soustypes_soustype_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.metadonnees_soustypes
    ADD CONSTRAINT metadonnees_soustypes_soustype_id_fkey FOREIGN KEY (soustype_id) REFERENCES public.soustypes(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: metadonnees_typemetadonnees_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.metadonnees
    ADD CONSTRAINT metadonnees_typemetadonnees_fkey FOREIGN KEY (typemetadonnee_id) REFERENCES public.typemetadonnees(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: notifications_desktop_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.notifications
    ADD CONSTRAINT notifications_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES public.desktops(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: notifications_typenotif_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.notifications
    ADD CONSTRAINT notifications_typenotif_id_fkey FOREIGN KEY (typenotif_id) REFERENCES public.typenotifs(id);


--
-- Name: notifieddesktops_courrier_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.notifieddesktops
    ADD CONSTRAINT notifieddesktops_courrier_id_fkey FOREIGN KEY (courrier_id) REFERENCES public.courriers(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: notifieddesktops_desktop_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.notifieddesktops
    ADD CONSTRAINT notifieddesktops_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES public.desktops(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: notifieddesktops_notification_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.notifieddesktops
    ADD CONSTRAINT notifieddesktops_notification_id_fkey FOREIGN KEY (notification_id) REFERENCES public.notifications(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: notifquotidiens_courrier_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.notifquotidiens
    ADD CONSTRAINT notifquotidiens_courrier_id_fkey FOREIGN KEY (courrier_id) REFERENCES public.courriers(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: notifquotidiens_desktop_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.notifquotidiens
    ADD CONSTRAINT notifquotidiens_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES public.desktops(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: notifquotidiens_notification_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.notifquotidiens
    ADD CONSTRAINT notifquotidiens_notification_id_fkey FOREIGN KEY (notification_id) REFERENCES public.notifications(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: notifquotidiens_user_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.notifquotidiens
    ADD CONSTRAINT notifquotidiens_user_id_fkey FOREIGN KEY (user_id) REFERENCES public.users(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: operations_type_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.operations
    ADD CONSTRAINT operations_type_id_fkey FOREIGN KEY (type_id) REFERENCES public.types(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: ordresservices_courrier_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY public.ordresservices
    ADD CONSTRAINT ordresservices_courrier_id_fkey FOREIGN KEY (courrier_id) REFERENCES public.courriers(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: ordresservices_marche_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY public.ordresservices
    ADD CONSTRAINT ordresservices_marche_id_fkey FOREIGN KEY (marche_id) REFERENCES public.marches(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: ordresservices_organisme_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY public.ordresservices
    ADD CONSTRAINT ordresservices_organisme_id_fkey FOREIGN KEY (organisme_id) REFERENCES public.organismes(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: organismes_addressbook_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.organismes
    ADD CONSTRAINT organismes_addressbook_id_fkey FOREIGN KEY (addressbook_id) REFERENCES public.addressbooks(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: organismes_ban_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.organismes
    ADD CONSTRAINT organismes_ban_id_fkey FOREIGN KEY (ban_id) REFERENCES public.bans(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: organismes_bancommune_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.organismes
    ADD CONSTRAINT organismes_bancommune_id_fkey FOREIGN KEY (bancommune_id) REFERENCES public.banscommunes(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: organismes_operations_operation_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.organismes_operations
    ADD CONSTRAINT organismes_operations_operation_id_fkey FOREIGN KEY (operation_id) REFERENCES public.operations(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: organismes_operations_organisme_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.organismes_operations
    ADD CONSTRAINT organismes_operations_organisme_id_fkey FOREIGN KEY (organisme_id) REFERENCES public.organismes(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: plandelegations_desktop_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.plandelegations
    ADD CONSTRAINT plandelegations_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES public.desktops(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: plandelegations_user_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.plandelegations
    ADD CONSTRAINT plandelegations_user_id_fkey FOREIGN KEY (user_id) REFERENCES public.users(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: plisconsultatifs_consultation_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.plisconsultatifs
    ADD CONSTRAINT plisconsultatifs_consultation_id_fkey FOREIGN KEY (consultation_id) REFERENCES public.consultations(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: plisconsultatifs_courrier_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.plisconsultatifs
    ADD CONSTRAINT plisconsultatifs_courrier_id_fkey FOREIGN KEY (courrier_id) REFERENCES public.courriers(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: plisconsultatifs_origineflux_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.plisconsultatifs
    ADD CONSTRAINT plisconsultatifs_origineflux_id_fkey FOREIGN KEY (origineflux_id) REFERENCES public.originesflux(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: profils_profil_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.profils
    ADD CONSTRAINT profils_profil_id_fkey FOREIGN KEY (profil_id) REFERENCES public.profils(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: reacherches_affaires_recherche_id; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_affaires
    ADD CONSTRAINT reacherches_affaires_recherche_id FOREIGN KEY (recherche_id) REFERENCES public.recherches(id);


--
-- Name: recheche_contactinfos_contactinfo_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recheche_contactinfos
    ADD CONSTRAINT recheche_contactinfos_contactinfo_id_fkey FOREIGN KEY (contactinfo_id) REFERENCES public.recherches(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recheche_contactinfos_recherche_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recheche_contactinfos
    ADD CONSTRAINT recheche_contactinfos_recherche_id_fkey FOREIGN KEY (recherche_id) REFERENCES public.recherches(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_addressbooks_addressbook_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_addressbooks
    ADD CONSTRAINT recherches_addressbooks_addressbook_id_fkey FOREIGN KEY (addressbook_id) REFERENCES public.addressbooks(id);


--
-- Name: recherches_addressbooks_recherche_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_addressbooks
    ADD CONSTRAINT recherches_addressbooks_recherche_id_fkey FOREIGN KEY (recherche_id) REFERENCES public.recherches(id);


--
-- Name: recherches_affaires_affaire_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_affaires
    ADD CONSTRAINT recherches_affaires_affaire_id_fkey FOREIGN KEY (affaire_id) REFERENCES public.affaires(id);


--
-- Name: recherches_circuits_circuits_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_circuits
    ADD CONSTRAINT recherches_circuits_circuits_fkey FOREIGN KEY (circuit_id) REFERENCES public.wkf_circuits(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_circuits_recherches_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_circuits
    ADD CONSTRAINT recherches_circuits_recherches_fkey FOREIGN KEY (recherche_id) REFERENCES public.recherches(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_contacts_contact_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_contacts
    ADD CONSTRAINT recherches_contacts_contact_id_fkey FOREIGN KEY (contact_id) REFERENCES public.contacts(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_contacts_recherche_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_contacts
    ADD CONSTRAINT recherches_contacts_recherche_id_fkey FOREIGN KEY (recherche_id) REFERENCES public.recherches(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_desktops_desktop_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_desktops
    ADD CONSTRAINT recherches_desktops_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES public.desktops(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_desktops_recherche_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_desktops
    ADD CONSTRAINT recherches_desktops_recherche_id_fkey FOREIGN KEY (recherche_id) REFERENCES public.recherches(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_dossiers_dossier_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_dossiers
    ADD CONSTRAINT recherches_dossiers_dossier_id_fkey FOREIGN KEY (dossier_id) REFERENCES public.dossiers(id);


--
-- Name: recherches_dossiers_recherche_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_dossiers
    ADD CONSTRAINT recherches_dossiers_recherche_id_fkey FOREIGN KEY (recherche_id) REFERENCES public.recherches(id);


--
-- Name: recherches_metadonnees_metadonnees_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_metadonnees
    ADD CONSTRAINT recherches_metadonnees_metadonnees_fkey FOREIGN KEY (metadonnee_id) REFERENCES public.metadonnees(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_metadonnees_recherches_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_metadonnees
    ADD CONSTRAINT recherches_metadonnees_recherches_fkey FOREIGN KEY (recherche_id) REFERENCES public.recherches(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_organismes_organisme_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_organismes
    ADD CONSTRAINT recherches_organismes_organisme_id_fkey FOREIGN KEY (organisme_id) REFERENCES public.organismes(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_organismes_recherche_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_organismes
    ADD CONSTRAINT recherches_organismes_recherche_id_fkey FOREIGN KEY (recherche_id) REFERENCES public.recherches(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_selectvaluesmetadonnee_selectvaluemetadonnee_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_selectvaluesmetadonnees
    ADD CONSTRAINT recherches_selectvaluesmetadonnee_selectvaluemetadonnee_id_fkey FOREIGN KEY (selectvaluemetadonnee_id) REFERENCES public.selectvaluesmetadonnees(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_selectvaluesmetadonnees_metadonnee_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_selectvaluesmetadonnees
    ADD CONSTRAINT recherches_selectvaluesmetadonnees_metadonnee_id_fkey FOREIGN KEY (metadonnee_id) REFERENCES public.metadonnees(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_selectvaluesmetadonnees_recherche_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_selectvaluesmetadonnees
    ADD CONSTRAINT recherches_selectvaluesmetadonnees_recherche_id_fkey FOREIGN KEY (recherche_id) REFERENCES public.recherches(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_services_recherche_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_services
    ADD CONSTRAINT recherches_services_recherche_id_fkey FOREIGN KEY (recherche_id) REFERENCES public.recherches(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_services_service_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_services
    ADD CONSTRAINT recherches_services_service_id_fkey FOREIGN KEY (service_id) REFERENCES public.services(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: recherches_soustypes_recherche_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_soustypes
    ADD CONSTRAINT recherches_soustypes_recherche_id_fkey FOREIGN KEY (recherche_id) REFERENCES public.recherches(id);


--
-- Name: recherches_soustypes_soustype_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_soustypes
    ADD CONSTRAINT recherches_soustypes_soustype_id_fkey FOREIGN KEY (soustype_id) REFERENCES public.soustypes(id);


--
-- Name: recherches_taches_recherche_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_taches
    ADD CONSTRAINT recherches_taches_recherche_id_fkey FOREIGN KEY (recherche_id) REFERENCES public.recherches(id);


--
-- Name: recherches_taches_tache_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_taches
    ADD CONSTRAINT recherches_taches_tache_id_fkey FOREIGN KEY (tache_id) REFERENCES public.taches(id);


--
-- Name: recherches_types_recherche_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_types
    ADD CONSTRAINT recherches_types_recherche_id_fkey FOREIGN KEY (recherche_id) REFERENCES public.recherches(id);


--
-- Name: recherches_types_type_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches_types
    ADD CONSTRAINT recherches_types_type_id_fkey FOREIGN KEY (type_id) REFERENCES public.types(id);


--
-- Name: recherches_users_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.recherches
    ADD CONSTRAINT recherches_users_fkey FOREIGN KEY (user_id) REFERENCES public.users(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: referentielsfantoircom_referentielfantoir_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.referentielsfantoircom
    ADD CONSTRAINT referentielsfantoircom_referentielfantoir_id_fkey FOREIGN KEY (referentielfantoir_id) REFERENCES public.referentielsfantoir(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: referentielsfantoircom_referentielfantoirdir_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.referentielsfantoircom
    ADD CONSTRAINT referentielsfantoircom_referentielfantoirdir_id_fkey FOREIGN KEY (referentielfantoirdir_id) REFERENCES public.referentielsfantoirdir(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: referentielsfantoirdir_referentielfantoir_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.referentielsfantoirdir
    ADD CONSTRAINT referentielsfantoirdir_referentielfantoir_id_fkey FOREIGN KEY (referentielfantoir_id) REFERENCES public.referentielsfantoir(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: referentielsfantoirvoie_referentielfantoir_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.referentielsfantoirvoie
    ADD CONSTRAINT referentielsfantoirvoie_referentielfantoir_id_fkey FOREIGN KEY (referentielfantoir_id) REFERENCES public.referentielsfantoir(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: referentielsfantoirvoie_referentielfantoircom_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.referentielsfantoirvoie
    ADD CONSTRAINT referentielsfantoirvoie_referentielfantoircom_id_fkey FOREIGN KEY (referentielfantoircom_id) REFERENCES public.referentielsfantoircom(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: relements_courrier_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.relements
    ADD CONSTRAINT relements_courrier_id_fkey FOREIGN KEY (courrier_id) REFERENCES public.courriers(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: repertoires_users_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.repertoires
    ADD CONSTRAINT repertoires_users_fkey FOREIGN KEY (user_id) REFERENCES public.users(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: rmodelgabarits_soustype_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.rmodelgabarits
    ADD CONSTRAINT rmodelgabarits_soustype_id_fkey FOREIGN KEY (soustype_id) REFERENCES public.soustypes(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: rmodels_soustype_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.rmodels
    ADD CONSTRAINT rmodels_soustype_id_fkey FOREIGN KEY (soustype_id) REFERENCES public.soustypes(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: scanemails_desktop_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.scanemails
    ADD CONSTRAINT scanemails_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES public.desktops(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: scanemails_desktopmanager_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.scanemails
    ADD CONSTRAINT scanemails_desktopmanager_id_fkey FOREIGN KEY (desktopmanager_id) REFERENCES public.desktopsmanagers(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: scanemails_soustype_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.scanemails
    ADD CONSTRAINT scanemails_soustype_id_fkey FOREIGN KEY (soustype_id) REFERENCES public.soustypes(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: scanemails_type_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.scanemails
    ADD CONSTRAINT scanemails_type_id_fkey FOREIGN KEY (type_id) REFERENCES public.types(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: selectvaluesmetadonnees_metadonnee_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.selectvaluesmetadonnees
    ADD CONSTRAINT selectvaluesmetadonnees_metadonnee_id_fkey FOREIGN KEY (metadonnee_id) REFERENCES public.metadonnees(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: sendsmails_courrier_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.sendsmails
    ADD CONSTRAINT sendsmails_courrier_id_fkey FOREIGN KEY (courrier_id) REFERENCES public.courriers(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: sendsmails_desktop_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.sendsmails
    ADD CONSTRAINT sendsmails_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES public.desktops(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: sendsmails_document_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.sendsmails
    ADD CONSTRAINT sendsmails_document_id_fkey FOREIGN KEY (document_id) REFERENCES public.documents(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: sendsmails_templatemail_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.sendsmails
    ADD CONSTRAINT sendsmails_templatemail_id_fkey FOREIGN KEY (templatemail_id) REFERENCES public.templatemails(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: sousactivites_activite_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.sousactivites
    ADD CONSTRAINT sousactivites_activite_id_fkey FOREIGN KEY (activite_id) REFERENCES public.activites(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: sousoperations_operation_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY public.sousoperations
    ADD CONSTRAINT sousoperations_operation_id_fkey FOREIGN KEY (operation_id) REFERENCES public.operations(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: sousoperations_soustype_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: postgres
--

ALTER TABLE ONLY public.sousoperations
    ADD CONSTRAINT sousoperations_soustype_id_fkey FOREIGN KEY (soustype_id) REFERENCES public.soustypes(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: soustypes_compteur_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.soustypes
    ADD CONSTRAINT soustypes_compteur_id_fkey FOREIGN KEY (compteur_id) REFERENCES public.compteurs(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: soustypes_types_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.soustypes
    ADD CONSTRAINT soustypes_types_fkey FOREIGN KEY (type_id) REFERENCES public.types(id) ON UPDATE CASCADE ON DELETE CASCADE;



ALTER TABLE ONLY public.soustypes_soustypescibles
    ADD CONSTRAINT soustypes_soustypescibles_soustype_id_fkey FOREIGN KEY (soustype_id) REFERENCES public.soustypes(id) ON UPDATE CASCADE ON DELETE CASCADE;


ALTER TABLE ONLY public.soustypes_soustypescibles
    ADD CONSTRAINT soustypes_soustypescibles_soustypecible_id_fkey FOREIGN KEY (soustypecible_id) REFERENCES public.soustypes(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: typenotifs_users_typenotif_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.typenotifs_users
    ADD CONSTRAINT typenotifs_users_typenotif_id_fkey FOREIGN KEY (typenotif_id) REFERENCES public.typenotifs(id);


--
-- Name: typenotifs_users_user_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.typenotifs_users
    ADD CONSTRAINT typenotifs_users_user_id_fkey FOREIGN KEY (user_id) REFERENCES public.users(id);


--
-- Name: users_desktop_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.users
    ADD CONSTRAINT users_desktop_id_fkey FOREIGN KEY (desktop_id) REFERENCES public.desktops(id) ON UPDATE CASCADE;


--
-- Name: users_wkf_circuits_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.wkf_circuits
    ADD CONSTRAINT users_wkf_circuits_fkey FOREIGN KEY (created_user_id) REFERENCES public.users(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: users_wkf_circuits_fkey2; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.wkf_circuits
    ADD CONSTRAINT users_wkf_circuits_fkey2 FOREIGN KEY (modified_user_id) REFERENCES public.users(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: wkf_circuits_wkf_compositions_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.wkf_etapes
    ADD CONSTRAINT wkf_circuits_wkf_compositions_fkey FOREIGN KEY (circuit_id) REFERENCES public.wkf_circuits(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: wkf_compositions_etape_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.wkf_compositions
    ADD CONSTRAINT wkf_compositions_etape_id_fkey FOREIGN KEY (etape_id) REFERENCES public.wkf_etapes(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: wkf_signatures_visa_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.wkf_signatures
    ADD CONSTRAINT wkf_signatures_visa_id_fkey FOREIGN KEY (visa_id) REFERENCES public.wkf_visas(id);


--
-- Name: wkf_traitements_circuit_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.wkf_traitements
    ADD CONSTRAINT wkf_traitements_circuit_id_fkey FOREIGN KEY (circuit_id) REFERENCES public.wkf_circuits(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: wkf_traitements_target_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.wkf_traitements
    ADD CONSTRAINT wkf_traitements_target_id_fkey FOREIGN KEY (target_id) REFERENCES public.courriers(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: wkf_traitements_wkf_visas_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.wkf_visas
    ADD CONSTRAINT wkf_traitements_wkf_visas_fkey FOREIGN KEY (traitement_id) REFERENCES public.wkf_traitements(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: wkf_visas_etape_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: webgfc
--

ALTER TABLE ONLY public.wkf_visas
    ADD CONSTRAINT wkf_visas_etape_id_fkey FOREIGN KEY (etape_id) REFERENCES public.wkf_etapes(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: SCHEMA public; Type: ACL; Schema: -; Owner: postgres
--

REVOKE ALL ON SCHEMA public FROM PUBLIC;
REVOKE ALL ON SCHEMA public FROM webgfc;
GRANT ALL ON SCHEMA public TO webgfc;
GRANT ALL ON SCHEMA public TO PUBLIC;


--
-- PostgreSQL database dump complete
--

-- DATAs

--
-- Data for Name: acos; Type: TABLE DATA; Schema: public; Owner: webgfc
--
INSERT INTO public.acos VALUES (893, 2, NULL, NULL, 'isExistName', 39, 40);
INSERT INTO public.acos VALUES (3, 2, NULL, NULL, 'index', 3, 4);
INSERT INTO public.acos VALUES (4, 2, NULL, NULL, 'get_addressbooks', 5, 6);
INSERT INTO public.acos VALUES (5, 2, NULL, NULL, 'add', 7, 8);
INSERT INTO public.acos VALUES (6, 2, NULL, NULL, 'edit', 9, 10);
INSERT INTO public.acos VALUES (7, 2, NULL, NULL, 'delete', 11, 12);
INSERT INTO public.acos VALUES (8, 2, NULL, NULL, 'userAclCheck', 13, 14);
INSERT INTO public.acos VALUES (9, 2, NULL, NULL, 'getSessionInitServices', 15, 16);
INSERT INTO public.acos VALUES (10, 2, NULL, NULL, 'getSessionAppDefinedServices', 17, 18);
INSERT INTO public.acos VALUES (11, 2, NULL, NULL, 'getSessionAppDefinedDesktops', 19, 20);
INSERT INTO public.acos VALUES (12, 2, NULL, NULL, 'getSessionInitDesktops', 21, 22);
INSERT INTO public.acos VALUES (13, 2, NULL, NULL, 'getSessionValDesktops', 23, 24);
INSERT INTO public.acos VALUES (14, 2, NULL, NULL, 'getSessionValeditDesktops', 25, 26);
INSERT INTO public.acos VALUES (15, 2, NULL, NULL, 'getSessionDispDesktops', 27, 28);
INSERT INTO public.acos VALUES (16, 2, NULL, NULL, 'getSessionArchDesktops', 29, 30);
INSERT INTO public.acos VALUES (17, 2, NULL, NULL, 'getSessionAllDesktops', 31, 32);
INSERT INTO public.acos VALUES (18, 2, NULL, NULL, 'getEnv', 33, 34);
INSERT INTO public.acos VALUES (19, 2, NULL, NULL, 'getAddParams', 35, 36);
INSERT INTO public.acos VALUES (20, 2, NULL, NULL, 'ldebug', 37, 38);
INSERT INTO public.acos VALUES (894, 2, NULL, NULL, 'removeRepeat', 41, 42);
INSERT INTO public.acos VALUES (870, 855, NULL, NULL, 'ldebug', 2058, 2059);
INSERT INTO public.acos VALUES (1850, 675, NULL, NULL, 'casLogout', 1689, 1690);
INSERT INTO public.acos VALUES (1363, 306, NULL, NULL, 'getFileScannePreview', 819, 820);
INSERT INTO public.acos VALUES (2, 1, NULL, NULL, 'Addressbooks', 2, 51);
INSERT INTO public.acos VALUES (22, 21, NULL, NULL, 'add', 53, 54);
INSERT INTO public.acos VALUES (23, 21, NULL, NULL, 'edit', 55, 56);
INSERT INTO public.acos VALUES (24, 21, NULL, NULL, 'delete', 57, 58);
INSERT INTO public.acos VALUES (25, 21, NULL, NULL, 'userAclCheck', 59, 60);
INSERT INTO public.acos VALUES (26, 21, NULL, NULL, 'getSessionInitServices', 61, 62);
INSERT INTO public.acos VALUES (27, 21, NULL, NULL, 'getSessionAppDefinedServices', 63, 64);
INSERT INTO public.acos VALUES (28, 21, NULL, NULL, 'getSessionAppDefinedDesktops', 65, 66);
INSERT INTO public.acos VALUES (29, 21, NULL, NULL, 'getSessionInitDesktops', 67, 68);
INSERT INTO public.acos VALUES (30, 21, NULL, NULL, 'getSessionValDesktops', 69, 70);
INSERT INTO public.acos VALUES (31, 21, NULL, NULL, 'getSessionValeditDesktops', 71, 72);
INSERT INTO public.acos VALUES (32, 21, NULL, NULL, 'getSessionDispDesktops', 73, 74);
INSERT INTO public.acos VALUES (33, 21, NULL, NULL, 'getSessionArchDesktops', 75, 76);
INSERT INTO public.acos VALUES (34, 21, NULL, NULL, 'getSessionAllDesktops', 77, 78);
INSERT INTO public.acos VALUES (21, 1, NULL, NULL, 'Affaires', 52, 89);
INSERT INTO public.acos VALUES (1348, 1344, NULL, NULL, 'ajaxformvalid', 2795, 2796);
INSERT INTO public.acos VALUES (1651, 1638, NULL, NULL, 'getEnv', 3329, 3330);
INSERT INTO public.acos VALUES (1112, 2, NULL, NULL, 'export', 43, 44);
INSERT INTO public.acos VALUES (158, 147, NULL, NULL, 'getEnv', 363, 364);
INSERT INTO public.acos VALUES (1364, 306, NULL, NULL, 'getFileListPreview', 821, 822);
INSERT INTO public.acos VALUES (1851, 675, NULL, NULL, 'casLogin', 1691, 1692);
INSERT INTO public.acos VALUES (887, 872, NULL, NULL, 'getEnv', 2094, 2095);
INSERT INTO public.acos VALUES (1365, 306, NULL, NULL, 'removeFromImport', 823, 824);
INSERT INTO public.acos VALUES (1113, 2, NULL, NULL, 'get_organisme', 45, 46);
INSERT INTO public.acos VALUES (408, 391, NULL, NULL, 'ldebug', 1023, 1024);
INSERT INTO public.acos VALUES (1852, 675, NULL, NULL, 'chooseCollForCas', 1693, 1694);
INSERT INTO public.acos VALUES (869, 855, NULL, NULL, 'getAddParams', 2056, 2057);
INSERT INTO public.acos VALUES (1366, 306, NULL, NULL, 'convertToPdf', 825, 826);
INSERT INTO public.acos VALUES (1853, 675, NULL, NULL, 'logincasafterchoosingcoll', 1695, 1696);
INSERT INTO public.acos VALUES (1114, 2, NULL, NULL, 'searchResultat', 47, 48);
INSERT INTO public.acos VALUES (1478, 1461, NULL, NULL, 'getAddParams', 3021, 3022);
INSERT INTO public.acos VALUES (707, 706, NULL, NULL, 'chk', 1705, 1706);
INSERT INTO public.acos VALUES (708, 706, NULL, NULL, 'service', 1707, 1708);
INSERT INTO public.acos VALUES (709, 706, NULL, NULL, 'echotest', 1709, 1710);
INSERT INTO public.acos VALUES (706, 1, NULL, NULL, 'Webservices', 1704, 1763);
INSERT INTO public.acos VALUES (35, 21, NULL, NULL, 'getEnv', 79, 80);
INSERT INTO public.acos VALUES (36, 21, NULL, NULL, 'getAddParams', 81, 82);
INSERT INTO public.acos VALUES (37, 21, NULL, NULL, 'ldebug', 83, 84);
INSERT INTO public.acos VALUES (1367, 306, NULL, NULL, 'index', 827, 828);
INSERT INTO public.acos VALUES (1854, 675, NULL, NULL, 'setRights', 1697, 1698);
INSERT INTO public.acos VALUES (895, 21, NULL, NULL, 'getComment', 85, 86);
INSERT INTO public.acos VALUES (1115, 2, NULL, NULL, 'isAuthorized', 49, 50);
INSERT INTO public.acos VALUES (1457, 1441, NULL, NULL, 'getEnv', 2979, 2980);
INSERT INTO public.acos VALUES (710, 706, NULL, NULL, 'getGFCCollectivites', 1711, 1712);
INSERT INTO public.acos VALUES (711, 706, NULL, NULL, 'getGFCTypes', 1713, 1714);
INSERT INTO public.acos VALUES (712, 706, NULL, NULL, 'getGFCSoustypes', 1715, 1716);
INSERT INTO public.acos VALUES (713, 706, NULL, NULL, 'createCourrier', 1717, 1718);
INSERT INTO public.acos VALUES (714, 706, NULL, NULL, 'scanMass', 1719, 1720);
INSERT INTO public.acos VALUES (715, 706, NULL, NULL, 'getStatut', 1721, 1722);
INSERT INTO public.acos VALUES (716, 706, NULL, NULL, 'userAclCheck', 1723, 1724);
INSERT INTO public.acos VALUES (717, 706, NULL, NULL, 'getSessionInitServices', 1725, 1726);
INSERT INTO public.acos VALUES (718, 706, NULL, NULL, 'getSessionAppDefinedServices', 1727, 1728);
INSERT INTO public.acos VALUES (719, 706, NULL, NULL, 'getSessionAppDefinedDesktops', 1729, 1730);
INSERT INTO public.acos VALUES (720, 706, NULL, NULL, 'getSessionInitDesktops', 1731, 1732);
INSERT INTO public.acos VALUES (721, 706, NULL, NULL, 'getSessionValDesktops', 1733, 1734);
INSERT INTO public.acos VALUES (722, 706, NULL, NULL, 'getSessionValeditDesktops', 1735, 1736);
INSERT INTO public.acos VALUES (723, 706, NULL, NULL, 'getSessionDispDesktops', 1737, 1738);
INSERT INTO public.acos VALUES (724, 706, NULL, NULL, 'getSessionArchDesktops', 1739, 1740);
INSERT INTO public.acos VALUES (725, 706, NULL, NULL, 'getSessionAllDesktops', 1741, 1742);
INSERT INTO public.acos VALUES (727, 706, NULL, NULL, 'getAddParams', 1745, 1746);
INSERT INTO public.acos VALUES (728, 706, NULL, NULL, 'ldebug', 1747, 1748);
INSERT INTO public.acos VALUES (1368, 306, NULL, NULL, 'add', 829, 830);
INSERT INTO public.acos VALUES (39, 38, NULL, NULL, 'app_version', 91, 92);
INSERT INTO public.acos VALUES (40, 38, NULL, NULL, 'version', 93, 94);
INSERT INTO public.acos VALUES (41, 38, NULL, NULL, 'userAclCheck', 95, 96);
INSERT INTO public.acos VALUES (42, 38, NULL, NULL, 'getSessionInitServices', 97, 98);
INSERT INTO public.acos VALUES (43, 38, NULL, NULL, 'getSessionAppDefinedServices', 99, 100);
INSERT INTO public.acos VALUES (44, 38, NULL, NULL, 'getSessionAppDefinedDesktops', 101, 102);
INSERT INTO public.acos VALUES (45, 38, NULL, NULL, 'getSessionInitDesktops', 103, 104);
INSERT INTO public.acos VALUES (46, 38, NULL, NULL, 'getSessionValDesktops', 105, 106);
INSERT INTO public.acos VALUES (47, 38, NULL, NULL, 'getSessionValeditDesktops', 107, 108);
INSERT INTO public.acos VALUES (48, 38, NULL, NULL, 'getSessionDispDesktops', 109, 110);
INSERT INTO public.acos VALUES (49, 38, NULL, NULL, 'getSessionArchDesktops', 111, 112);
INSERT INTO public.acos VALUES (50, 38, NULL, NULL, 'getSessionAllDesktops', 113, 114);
INSERT INTO public.acos VALUES (51, 38, NULL, NULL, 'getEnv', 115, 116);
INSERT INTO public.acos VALUES (52, 38, NULL, NULL, 'getAddParams', 117, 118);
INSERT INTO public.acos VALUES (53, 38, NULL, NULL, 'ldebug', 119, 120);
INSERT INTO public.acos VALUES (535, 519, NULL, NULL, 'getAddParams', 1297, 1298);
INSERT INTO public.acos VALUES (1855, 675, NULL, NULL, 'habilPerson', 1699, 1700);
INSERT INTO public.acos VALUES (780, 755, NULL, NULL, 'ldebug', 1870, 1871);
INSERT INTO public.acos VALUES (1116, 21, NULL, NULL, 'isAuthorized', 87, 88);
INSERT INTO public.acos VALUES (1369, 306, NULL, NULL, 'edit', 831, 832);
INSERT INTO public.acos VALUES (38, 1, NULL, NULL, 'Allos', 90, 123);
INSERT INTO public.acos VALUES (55, 54, NULL, NULL, 'add', 125, 126);
INSERT INTO public.acos VALUES (56, 54, NULL, NULL, 'delete', 127, 128);
INSERT INTO public.acos VALUES (57, 54, NULL, NULL, 'download', 129, 130);
INSERT INTO public.acos VALUES (58, 54, NULL, NULL, 'getPreview', 131, 132);
INSERT INTO public.acos VALUES (59, 54, NULL, NULL, 'userAclCheck', 133, 134);
INSERT INTO public.acos VALUES (60, 54, NULL, NULL, 'getSessionInitServices', 135, 136);
INSERT INTO public.acos VALUES (61, 54, NULL, NULL, 'getSessionAppDefinedServices', 137, 138);
INSERT INTO public.acos VALUES (62, 54, NULL, NULL, 'getSessionAppDefinedDesktops', 139, 140);
INSERT INTO public.acos VALUES (63, 54, NULL, NULL, 'getSessionInitDesktops', 141, 142);
INSERT INTO public.acos VALUES (64, 54, NULL, NULL, 'getSessionValDesktops', 143, 144);
INSERT INTO public.acos VALUES (65, 54, NULL, NULL, 'getSessionValeditDesktops', 145, 146);
INSERT INTO public.acos VALUES (66, 54, NULL, NULL, 'getSessionDispDesktops', 147, 148);
INSERT INTO public.acos VALUES (67, 54, NULL, NULL, 'getSessionArchDesktops', 149, 150);
INSERT INTO public.acos VALUES (68, 54, NULL, NULL, 'getSessionAllDesktops', 151, 152);
INSERT INTO public.acos VALUES (69, 54, NULL, NULL, 'getEnv', 153, 154);
INSERT INTO public.acos VALUES (70, 54, NULL, NULL, 'getAddParams', 155, 156);
INSERT INTO public.acos VALUES (71, 54, NULL, NULL, 'ldebug', 157, 158);
INSERT INTO public.acos VALUES (1856, 675, NULL, NULL, 'isAuthorized', 1701, 1702);
INSERT INTO public.acos VALUES (1117, 38, NULL, NULL, 'isAuthorized', 121, 122);
INSERT INTO public.acos VALUES (1370, 306, NULL, NULL, 'getDocuments', 833, 834);
INSERT INTO public.acos VALUES (1857, 706, NULL, NULL, 'createContact', 1749, 1750);
INSERT INTO public.acos VALUES (54, 1, NULL, NULL, 'Armodels', 124, 165);
INSERT INTO public.acos VALUES (73, 72, NULL, NULL, 'select', 167, 168);
INSERT INTO public.acos VALUES (74, 72, NULL, NULL, 'delete', 169, 170);
INSERT INTO public.acos VALUES (75, 72, NULL, NULL, 'download', 171, 172);
INSERT INTO public.acos VALUES (76, 72, NULL, NULL, 'getPreview', 173, 174);
INSERT INTO public.acos VALUES (77, 72, NULL, NULL, 'userAclCheck', 175, 176);
INSERT INTO public.acos VALUES (78, 72, NULL, NULL, 'getSessionInitServices', 177, 178);
INSERT INTO public.acos VALUES (79, 72, NULL, NULL, 'getSessionAppDefinedServices', 179, 180);
INSERT INTO public.acos VALUES (80, 72, NULL, NULL, 'getSessionAppDefinedDesktops', 181, 182);
INSERT INTO public.acos VALUES (81, 72, NULL, NULL, 'getSessionInitDesktops', 183, 184);
INSERT INTO public.acos VALUES (72, 1, NULL, NULL, 'Ars', 166, 207);
INSERT INTO public.acos VALUES (931, 915, NULL, NULL, 'ldebug', 2181, 2182);
INSERT INTO public.acos VALUES (1371, 306, NULL, NULL, 'getProgress', 835, 836);
INSERT INTO public.acos VALUES (1118, 54, NULL, NULL, 'deletefile', 159, 160);
INSERT INTO public.acos VALUES (1858, 706, NULL, NULL, 'createOrganisme', 1751, 1752);
INSERT INTO public.acos VALUES (373, 368, NULL, NULL, 'setInfos', 951, 952);
INSERT INTO public.acos VALUES (374, 368, NULL, NULL, 'setDroitsFunc', 953, 954);
INSERT INTO public.acos VALUES (375, 368, NULL, NULL, 'getDroitsFunc', 955, 956);
INSERT INTO public.acos VALUES (376, 368, NULL, NULL, 'delete', 957, 958);
INSERT INTO public.acos VALUES (377, 368, NULL, NULL, 'ajaxformvalid', 959, 960);
INSERT INTO public.acos VALUES (378, 368, NULL, NULL, 'userAclCheck', 961, 962);
INSERT INTO public.acos VALUES (379, 368, NULL, NULL, 'getSessionInitServices', 963, 964);
INSERT INTO public.acos VALUES (82, 72, NULL, NULL, 'getSessionValDesktops', 185, 186);
INSERT INTO public.acos VALUES (83, 72, NULL, NULL, 'getSessionValeditDesktops', 187, 188);
INSERT INTO public.acos VALUES (84, 72, NULL, NULL, 'getSessionDispDesktops', 189, 190);
INSERT INTO public.acos VALUES (85, 72, NULL, NULL, 'getSessionArchDesktops', 191, 192);
INSERT INTO public.acos VALUES (86, 72, NULL, NULL, 'getSessionAllDesktops', 193, 194);
INSERT INTO public.acos VALUES (87, 72, NULL, NULL, 'getEnv', 195, 196);
INSERT INTO public.acos VALUES (88, 72, NULL, NULL, 'getAddParams', 197, 198);
INSERT INTO public.acos VALUES (89, 72, NULL, NULL, 'ldebug', 199, 200);
INSERT INTO public.acos VALUES (91, 90, NULL, NULL, 'index', 209, 210);
INSERT INTO public.acos VALUES (92, 90, NULL, NULL, 'userAclCheck', 211, 212);
INSERT INTO public.acos VALUES (93, 90, NULL, NULL, 'getSessionInitServices', 213, 214);
INSERT INTO public.acos VALUES (94, 90, NULL, NULL, 'getSessionAppDefinedServices', 215, 216);
INSERT INTO public.acos VALUES (95, 90, NULL, NULL, 'getSessionAppDefinedDesktops', 217, 218);
INSERT INTO public.acos VALUES (96, 90, NULL, NULL, 'getSessionInitDesktops', 219, 220);
INSERT INTO public.acos VALUES (97, 90, NULL, NULL, 'getSessionValDesktops', 221, 222);
INSERT INTO public.acos VALUES (98, 90, NULL, NULL, 'getSessionValeditDesktops', 223, 224);
INSERT INTO public.acos VALUES (99, 90, NULL, NULL, 'getSessionDispDesktops', 225, 226);
INSERT INTO public.acos VALUES (100, 90, NULL, NULL, 'getSessionArchDesktops', 227, 228);
INSERT INTO public.acos VALUES (101, 90, NULL, NULL, 'getSessionAllDesktops', 229, 230);
INSERT INTO public.acos VALUES (102, 90, NULL, NULL, 'getEnv', 231, 232);
INSERT INTO public.acos VALUES (90, 1, NULL, NULL, 'Checks', 208, 251);
INSERT INTO public.acos VALUES (1372, 306, NULL, NULL, 'getProgressDocument', 837, 838);
INSERT INTO public.acos VALUES (420, 409, NULL, NULL, 'getEnv', 1049, 1050);
INSERT INTO public.acos VALUES (1119, 54, NULL, NULL, 'edit', 161, 162);
INSERT INTO public.acos VALUES (1859, 706, NULL, NULL, 'index', 1753, 1754);
INSERT INTO public.acos VALUES (103, 90, NULL, NULL, 'getAddParams', 233, 234);
INSERT INTO public.acos VALUES (104, 90, NULL, NULL, 'ldebug', 235, 236);
INSERT INTO public.acos VALUES (1373, 306, NULL, NULL, 'setASigner', 839, 840);
INSERT INTO public.acos VALUES (1120, 54, NULL, NULL, 'isAuthorized', 163, 164);
INSERT INTO public.acos VALUES (1860, 706, NULL, NULL, 'add', 1755, 1756);
INSERT INTO public.acos VALUES (306, 1, NULL, NULL, 'Documents', 770, 843);
INSERT INTO public.acos VALUES (326, 325, NULL, NULL, 'index', 845, 846);
INSERT INTO public.acos VALUES (327, 325, NULL, NULL, 'add', 847, 848);
INSERT INTO public.acos VALUES (328, 325, NULL, NULL, 'edit', 849, 850);
INSERT INTO public.acos VALUES (329, 325, NULL, NULL, 'delete', 851, 852);
INSERT INTO public.acos VALUES (330, 325, NULL, NULL, 'getDossiers', 853, 854);
INSERT INTO public.acos VALUES (331, 325, NULL, NULL, 'userAclCheck', 855, 856);
INSERT INTO public.acos VALUES (332, 325, NULL, NULL, 'getSessionInitServices', 857, 858);
INSERT INTO public.acos VALUES (333, 325, NULL, NULL, 'getSessionAppDefinedServices', 859, 860);
INSERT INTO public.acos VALUES (334, 325, NULL, NULL, 'getSessionAppDefinedDesktops', 861, 862);
INSERT INTO public.acos VALUES (335, 325, NULL, NULL, 'getSessionInitDesktops', 863, 864);
INSERT INTO public.acos VALUES (336, 325, NULL, NULL, 'getSessionValDesktops', 865, 866);
INSERT INTO public.acos VALUES (337, 325, NULL, NULL, 'getSessionValeditDesktops', 867, 868);
INSERT INTO public.acos VALUES (338, 325, NULL, NULL, 'getSessionDispDesktops', 869, 870);
INSERT INTO public.acos VALUES (339, 325, NULL, NULL, 'getSessionArchDesktops', 871, 872);
INSERT INTO public.acos VALUES (340, 325, NULL, NULL, 'getSessionAllDesktops', 873, 874);
INSERT INTO public.acos VALUES (341, 325, NULL, NULL, 'getEnv', 875, 876);
INSERT INTO public.acos VALUES (342, 325, NULL, NULL, 'getAddParams', 877, 878);
INSERT INTO public.acos VALUES (343, 325, NULL, NULL, 'ldebug', 879, 880);
INSERT INTO public.acos VALUES (1003, 987, NULL, NULL, 'getEnv', 2251, 2252);
INSERT INTO public.acos VALUES (944, 325, NULL, NULL, 'getComment', 881, 882);
INSERT INTO public.acos VALUES (1374, 306, NULL, NULL, 'isAuthorized', 841, 842);
INSERT INTO public.acos VALUES (1121, 72, NULL, NULL, 'convertToPdf', 201, 202);
INSERT INTO public.acos VALUES (1861, 706, NULL, NULL, 'getGFCDesktopsmanagers', 1757, 1758);
INSERT INTO public.acos VALUES (325, 1, NULL, NULL, 'Dossiers', 844, 885);
INSERT INTO public.acos VALUES (345, 344, NULL, NULL, 'getBancontenusServices', 887, 888);
INSERT INTO public.acos VALUES (346, 344, NULL, NULL, 'index', 889, 890);
INSERT INTO public.acos VALUES (347, 344, NULL, NULL, 'superadminenv', 891, 892);
INSERT INTO public.acos VALUES (348, 344, NULL, NULL, 'adminenv', 893, 894);
INSERT INTO public.acos VALUES (349, 344, NULL, NULL, 'getNotifications', 895, 896);
INSERT INTO public.acos VALUES (350, 344, NULL, NULL, 'setNotificationRead', 897, 898);
INSERT INTO public.acos VALUES (351, 344, NULL, NULL, 'setAllNotificationRead', 899, 900);
INSERT INTO public.acos VALUES (352, 344, NULL, NULL, 'userenv', 901, 902);
INSERT INTO public.acos VALUES (353, 344, NULL, NULL, 'dispenv', 903, 904);
INSERT INTO public.acos VALUES (354, 344, NULL, NULL, 'historique', 905, 906);
INSERT INTO public.acos VALUES (355, 344, NULL, NULL, 'userAclCheck', 907, 908);
INSERT INTO public.acos VALUES (356, 344, NULL, NULL, 'getSessionInitServices', 909, 910);
INSERT INTO public.acos VALUES (357, 344, NULL, NULL, 'getSessionAppDefinedServices', 911, 912);
INSERT INTO public.acos VALUES (358, 344, NULL, NULL, 'getSessionAppDefinedDesktops', 913, 914);
INSERT INTO public.acos VALUES (359, 344, NULL, NULL, 'getSessionInitDesktops', 915, 916);
INSERT INTO public.acos VALUES (360, 344, NULL, NULL, 'getSessionValDesktops', 917, 918);
INSERT INTO public.acos VALUES (361, 344, NULL, NULL, 'getSessionValeditDesktops', 919, 920);
INSERT INTO public.acos VALUES (362, 344, NULL, NULL, 'getSessionDispDesktops', 921, 922);
INSERT INTO public.acos VALUES (363, 344, NULL, NULL, 'getSessionArchDesktops', 923, 924);
INSERT INTO public.acos VALUES (364, 344, NULL, NULL, 'getSessionAllDesktops', 925, 926);
INSERT INTO public.acos VALUES (365, 344, NULL, NULL, 'getEnv', 927, 928);
INSERT INTO public.acos VALUES (366, 344, NULL, NULL, 'getAddParams', 929, 930);
INSERT INTO public.acos VALUES (367, 344, NULL, NULL, 'ldebug', 931, 932);
INSERT INTO public.acos VALUES (344, 1, NULL, NULL, 'Environnement', 886, 941);
INSERT INTO public.acos VALUES (369, 368, NULL, NULL, 'getProfils', 943, 944);
INSERT INTO public.acos VALUES (370, 368, NULL, NULL, 'view', 945, 946);
INSERT INTO public.acos VALUES (371, 368, NULL, NULL, 'add', 947, 948);
INSERT INTO public.acos VALUES (372, 368, NULL, NULL, 'edit', 949, 950);
INSERT INTO public.acos VALUES (380, 368, NULL, NULL, 'getSessionAppDefinedServices', 965, 966);
INSERT INTO public.acos VALUES (381, 368, NULL, NULL, 'getSessionAppDefinedDesktops', 967, 968);
INSERT INTO public.acos VALUES (382, 368, NULL, NULL, 'getSessionInitDesktops', 969, 970);
INSERT INTO public.acos VALUES (383, 368, NULL, NULL, 'getSessionValDesktops', 971, 972);
INSERT INTO public.acos VALUES (384, 368, NULL, NULL, 'getSessionValeditDesktops', 973, 974);
INSERT INTO public.acos VALUES (385, 368, NULL, NULL, 'getSessionDispDesktops', 975, 976);
INSERT INTO public.acos VALUES (386, 368, NULL, NULL, 'getSessionArchDesktops', 977, 978);
INSERT INTO public.acos VALUES (387, 368, NULL, NULL, 'getSessionAllDesktops', 979, 980);
INSERT INTO public.acos VALUES (1375, 325, NULL, NULL, 'isAuthorized', 883, 884);
INSERT INTO public.acos VALUES (1862, 706, NULL, NULL, 'table_correspondance', 1759, 1760);
INSERT INTO public.acos VALUES (1122, 72, NULL, NULL, 'deletefile', 203, 204);
INSERT INTO public.acos VALUES (246, 227, NULL, NULL, 'valid', 597, 598);
INSERT INTO public.acos VALUES (1376, 344, NULL, NULL, 'infosLogiciel', 933, 934);
INSERT INTO public.acos VALUES (1526, 1508, NULL, NULL, 'getEnv', 3109, 3110);
INSERT INTO public.acos VALUES (730, 729, NULL, NULL, 'index', 1765, 1766);
INSERT INTO public.acos VALUES (731, 729, NULL, NULL, 'getWorkflows', 1767, 1768);
INSERT INTO public.acos VALUES (732, 729, NULL, NULL, 'add', 1769, 1770);
INSERT INTO public.acos VALUES (733, 729, NULL, NULL, 'edit', 1771, 1772);
INSERT INTO public.acos VALUES (734, 729, NULL, NULL, 'setSchema', 1773, 1774);
INSERT INTO public.acos VALUES (735, 729, NULL, NULL, 'delete', 1775, 1776);
INSERT INTO public.acos VALUES (736, 729, NULL, NULL, 'moveEtape', 1777, 1778);
INSERT INTO public.acos VALUES (737, 729, NULL, NULL, 'copieCircuit', 1779, 1780);
INSERT INTO public.acos VALUES (738, 729, NULL, NULL, 'userEnough', 1781, 1782);
INSERT INTO public.acos VALUES (739, 729, NULL, NULL, 'userAclCheck', 1783, 1784);
INSERT INTO public.acos VALUES (740, 729, NULL, NULL, 'getSessionInitServices', 1785, 1786);
INSERT INTO public.acos VALUES (741, 729, NULL, NULL, 'getSessionAppDefinedServices', 1787, 1788);
INSERT INTO public.acos VALUES (742, 729, NULL, NULL, 'getSessionAppDefinedDesktops', 1789, 1790);
INSERT INTO public.acos VALUES (743, 729, NULL, NULL, 'getSessionInitDesktops', 1791, 1792);
INSERT INTO public.acos VALUES (744, 729, NULL, NULL, 'getSessionValDesktops', 1793, 1794);
INSERT INTO public.acos VALUES (745, 729, NULL, NULL, 'getSessionValeditDesktops', 1795, 1796);
INSERT INTO public.acos VALUES (746, 729, NULL, NULL, 'getSessionDispDesktops', 1797, 1798);
INSERT INTO public.acos VALUES (747, 729, NULL, NULL, 'getSessionArchDesktops', 1799, 1800);
INSERT INTO public.acos VALUES (748, 729, NULL, NULL, 'getSessionAllDesktops', 1801, 1802);
INSERT INTO public.acos VALUES (750, 729, NULL, NULL, 'getAddParams', 1805, 1806);
INSERT INTO public.acos VALUES (751, 729, NULL, NULL, 'ldebug', 1807, 1808);
INSERT INTO public.acos VALUES (118, 105, NULL, NULL, 'getSessionInitServices', 277, 278);
INSERT INTO public.acos VALUES (119, 105, NULL, NULL, 'getSessionAppDefinedServices', 279, 280);
INSERT INTO public.acos VALUES (120, 105, NULL, NULL, 'getSessionAppDefinedDesktops', 281, 282);
INSERT INTO public.acos VALUES (121, 105, NULL, NULL, 'getSessionInitDesktops', 283, 284);
INSERT INTO public.acos VALUES (726, 706, NULL, NULL, 'getEnv', 1743, 1744);
INSERT INTO public.acos VALUES (982, 729, NULL, NULL, 'getDescription', 1809, 1810);
INSERT INTO public.acos VALUES (388, 368, NULL, NULL, 'getEnv', 981, 982);
INSERT INTO public.acos VALUES (984, 90, NULL, NULL, 'readTimeout', 237, 238);
INSERT INTO public.acos VALUES (985, 90, NULL, NULL, 'sec2hms', 239, 240);
INSERT INTO public.acos VALUES (986, 90, NULL, NULL, 'timeout', 241, 242);
INSERT INTO public.acos VALUES (1863, 706, NULL, NULL, 'isAuthorized', 1761, 1762);
INSERT INTO public.acos VALUES (1123, 72, NULL, NULL, 'isAuthorized', 205, 206);
INSERT INTO public.acos VALUES (1173, 1172, NULL, NULL, 'index', 2509, 2510);
INSERT INTO public.acos VALUES (1377, 344, NULL, NULL, 'setSessionPagination', 935, 936);
INSERT INTO public.acos VALUES (729, 1, NULL, NULL, 'Workflows', 1764, 1815);
INSERT INTO public.acos VALUES (752, 1, NULL, NULL, 'AclExtras', 1816, 1817);
INSERT INTO public.acos VALUES (756, 755, NULL, NULL, 'index', 1822, 1823);
INSERT INTO public.acos VALUES (758, 755, NULL, NULL, 'add', 1826, 1827);
INSERT INTO public.acos VALUES (759, 755, NULL, NULL, 'edit', 1828, 1829);
INSERT INTO public.acos VALUES (760, 755, NULL, NULL, 'delete', 1830, 1831);
INSERT INTO public.acos VALUES (761, 755, NULL, NULL, 'visuCircuit', 1832, 1833);
INSERT INTO public.acos VALUES (762, 755, NULL, NULL, 'reorder', 1834, 1835);
INSERT INTO public.acos VALUES (763, 755, NULL, NULL, 'setCreatedModifiedUser', 1836, 1837);
INSERT INTO public.acos VALUES (764, 755, NULL, NULL, 'formatUser', 1838, 1839);
INSERT INTO public.acos VALUES (765, 755, NULL, NULL, 'formatLinkedModel', 1840, 1841);
INSERT INTO public.acos VALUES (766, 755, NULL, NULL, 'formatLinkedModels', 1842, 1843);
INSERT INTO public.acos VALUES (767, 755, NULL, NULL, 'listLinkedModel', 1844, 1845);
INSERT INTO public.acos VALUES (768, 755, NULL, NULL, 'userAclCheck', 1846, 1847);
INSERT INTO public.acos VALUES (769, 755, NULL, NULL, 'getSessionInitServices', 1848, 1849);
INSERT INTO public.acos VALUES (770, 755, NULL, NULL, 'getSessionAppDefinedServices', 1850, 1851);
INSERT INTO public.acos VALUES (771, 755, NULL, NULL, 'getSessionAppDefinedDesktops', 1852, 1853);
INSERT INTO public.acos VALUES (772, 755, NULL, NULL, 'getSessionInitDesktops', 1854, 1855);
INSERT INTO public.acos VALUES (773, 755, NULL, NULL, 'getSessionValDesktops', 1856, 1857);
INSERT INTO public.acos VALUES (774, 755, NULL, NULL, 'getSessionValeditDesktops', 1858, 1859);
INSERT INTO public.acos VALUES (775, 755, NULL, NULL, 'getSessionDispDesktops', 1860, 1861);
INSERT INTO public.acos VALUES (776, 755, NULL, NULL, 'getSessionArchDesktops', 1862, 1863);
INSERT INTO public.acos VALUES (777, 755, NULL, NULL, 'getSessionAllDesktops', 1864, 1865);
INSERT INTO public.acos VALUES (779, 755, NULL, NULL, 'getAddParams', 1868, 1869);
INSERT INTO public.acos VALUES (755, 754, NULL, NULL, 'Circuits', 1821, 1874);
INSERT INTO public.acos VALUES (783, 781, NULL, NULL, 'view', 1878, 1879);
INSERT INTO public.acos VALUES (784, 781, NULL, NULL, 'add', 1880, 1881);
INSERT INTO public.acos VALUES (786, 781, NULL, NULL, 'delete', 1884, 1885);
INSERT INTO public.acos VALUES (802, 781, NULL, NULL, 'getEnv', 1916, 1917);
INSERT INTO public.acos VALUES (754, 1, NULL, NULL, 'Cakeflow', 1820, 2025);
INSERT INTO public.acos VALUES (106, 105, NULL, NULL, 'index', 253, 254);
INSERT INTO public.acos VALUES (107, 105, NULL, NULL, 'getCollectivites', 255, 256);
INSERT INTO public.acos VALUES (108, 105, NULL, NULL, 'ajaxformvalid', 257, 258);
INSERT INTO public.acos VALUES (109, 105, NULL, NULL, 'getConnList', 259, 260);
INSERT INTO public.acos VALUES (110, 105, NULL, NULL, 'add', 261, 262);
INSERT INTO public.acos VALUES (111, 105, NULL, NULL, 'edit', 263, 264);
INSERT INTO public.acos VALUES (112, 105, NULL, NULL, 'delete', 265, 266);
INSERT INTO public.acos VALUES (113, 105, NULL, NULL, 'upload', 267, 268);
INSERT INTO public.acos VALUES (114, 105, NULL, NULL, 'getLogo', 269, 270);
INSERT INTO public.acos VALUES (115, 105, NULL, NULL, 'admin', 271, 272);
INSERT INTO public.acos VALUES (116, 105, NULL, NULL, 'deleteLogo', 273, 274);
INSERT INTO public.acos VALUES (117, 105, NULL, NULL, 'userAclCheck', 275, 276);
INSERT INTO public.acos VALUES (122, 105, NULL, NULL, 'getSessionValDesktops', 285, 286);
INSERT INTO public.acos VALUES (105, 1, NULL, NULL, 'Collectivites', 252, 305);
INSERT INTO public.acos VALUES (1864, 729, NULL, NULL, 'export', 1811, 1812);
INSERT INTO public.acos VALUES (1378, 344, NULL, NULL, 'putInCache', 937, 938);
INSERT INTO public.acos VALUES (1145, 90, NULL, NULL, 'get_test_mail', 243, 244);
INSERT INTO public.acos VALUES (804, 781, NULL, NULL, 'ldebug', 1920, 1921);
INSERT INTO public.acos VALUES (787, 781, NULL, NULL, 'setCreatedModifiedUser', 1886, 1887);
INSERT INTO public.acos VALUES (788, 781, NULL, NULL, 'formatUser', 1888, 1889);
INSERT INTO public.acos VALUES (789, 781, NULL, NULL, 'formatLinkedModel', 1890, 1891);
INSERT INTO public.acos VALUES (790, 781, NULL, NULL, 'formatLinkedModels', 1892, 1893);
INSERT INTO public.acos VALUES (792, 781, NULL, NULL, 'userAclCheck', 1896, 1897);
INSERT INTO public.acos VALUES (793, 781, NULL, NULL, 'getSessionInitServices', 1898, 1899);
INSERT INTO public.acos VALUES (794, 781, NULL, NULL, 'getSessionAppDefinedServices', 1900, 1901);
INSERT INTO public.acos VALUES (795, 781, NULL, NULL, 'getSessionAppDefinedDesktops', 1902, 1903);
INSERT INTO public.acos VALUES (796, 781, NULL, NULL, 'getSessionInitDesktops', 1904, 1905);
INSERT INTO public.acos VALUES (797, 781, NULL, NULL, 'getSessionValDesktops', 1906, 1907);
INSERT INTO public.acos VALUES (798, 781, NULL, NULL, 'getSessionValeditDesktops', 1908, 1909);
INSERT INTO public.acos VALUES (799, 781, NULL, NULL, 'getSessionDispDesktops', 1910, 1911);
INSERT INTO public.acos VALUES (800, 781, NULL, NULL, 'getSessionArchDesktops', 1912, 1913);
INSERT INTO public.acos VALUES (801, 781, NULL, NULL, 'getSessionAllDesktops', 1914, 1915);
INSERT INTO public.acos VALUES (803, 781, NULL, NULL, 'getAddParams', 1918, 1919);
INSERT INTO public.acos VALUES (389, 368, NULL, NULL, 'getAddParams', 983, 984);
INSERT INTO public.acos VALUES (390, 368, NULL, NULL, 'ldebug', 985, 986);
INSERT INTO public.acos VALUES (391, 1, NULL, NULL, 'Keywordlists', 990, 1027);
INSERT INTO public.acos VALUES (392, 391, NULL, NULL, 'index', 991, 992);
INSERT INTO public.acos VALUES (1142, 896, NULL, NULL, 'isAuthorized', 2147, 2148);
INSERT INTO public.acos VALUES (1144, 915, NULL, NULL, 'isAuthorized', 2183, 2184);
INSERT INTO public.acos VALUES (123, 105, NULL, NULL, 'getSessionValeditDesktops', 287, 288);
INSERT INTO public.acos VALUES (124, 105, NULL, NULL, 'getSessionDispDesktops', 289, 290);
INSERT INTO public.acos VALUES (125, 105, NULL, NULL, 'getSessionArchDesktops', 291, 292);
INSERT INTO public.acos VALUES (126, 105, NULL, NULL, 'getSessionAllDesktops', 293, 294);
INSERT INTO public.acos VALUES (127, 105, NULL, NULL, 'getEnv', 295, 296);
INSERT INTO public.acos VALUES (128, 105, NULL, NULL, 'getAddParams', 297, 298);
INSERT INTO public.acos VALUES (129, 105, NULL, NULL, 'ldebug', 299, 300);
INSERT INTO public.acos VALUES (1865, 729, NULL, NULL, 'isAuthorized', 1813, 1814);
INSERT INTO public.acos VALUES (1379, 344, NULL, NULL, 'isAuthorized', 939, 940);
INSERT INTO public.acos VALUES (1146, 90, NULL, NULL, 'testenvoimail', 245, 246);
INSERT INTO public.acos VALUES (1085, 1075, NULL, NULL, 'getSessionDispDesktops', 2379, 2380);
INSERT INTO public.acos VALUES (1086, 1075, NULL, NULL, 'getSessionArchDesktops', 2381, 2382);
INSERT INTO public.acos VALUES (1087, 1075, NULL, NULL, 'getSessionAllDesktops', 2383, 2384);
INSERT INTO public.acos VALUES (1088, 1075, NULL, NULL, 'getEnv', 2385, 2386);
INSERT INTO public.acos VALUES (1089, 1075, NULL, NULL, 'getAddParams', 2387, 2388);
INSERT INTO public.acos VALUES (1090, 1075, NULL, NULL, 'ldebug', 2389, 2390);
INSERT INTO public.acos VALUES (1091, 1045, NULL, NULL, 'getHomonymsOrgs', 2339, 2340);
INSERT INTO public.acos VALUES (1092, 1, NULL, NULL, 'Activites', 2394, 2433);
INSERT INTO public.acos VALUES (422, 409, NULL, NULL, 'ldebug', 1053, 1054);
INSERT INTO public.acos VALUES (438, 437, NULL, NULL, 'index', 1089, 1090);
INSERT INTO public.acos VALUES (132, 130, NULL, NULL, 'edit', 309, 310);
INSERT INTO public.acos VALUES (133, 130, NULL, NULL, 'delete', 311, 312);
INSERT INTO public.acos VALUES (134, 130, NULL, NULL, 'userAclCheck', 313, 314);
INSERT INTO public.acos VALUES (135, 130, NULL, NULL, 'getSessionInitServices', 315, 316);
INSERT INTO public.acos VALUES (136, 130, NULL, NULL, 'getSessionAppDefinedServices', 317, 318);
INSERT INTO public.acos VALUES (137, 130, NULL, NULL, 'getSessionAppDefinedDesktops', 319, 320);
INSERT INTO public.acos VALUES (138, 130, NULL, NULL, 'getSessionInitDesktops', 321, 322);
INSERT INTO public.acos VALUES (1623, 1608, NULL, NULL, 'getSessionDispDesktops', 3289, 3290);
INSERT INTO public.acos VALUES (1481, 391, NULL, NULL, 'isAuthorized', 1025, 1026);
INSERT INTO public.acos VALUES (1147, 90, NULL, NULL, 'testgedooo', 247, 248);
INSERT INTO public.acos VALUES (703, 675, NULL, NULL, 'getEnv', 1679, 1680);
INSERT INTO public.acos VALUES (282, 227, NULL, NULL, 'getEnv', 669, 670);
INSERT INTO public.acos VALUES (1867, 755, NULL, NULL, 'isAuthorized', 1872, 1873);
INSERT INTO public.acos VALUES (424, 423, NULL, NULL, 'userAclCheck', 1059, 1060);
INSERT INTO public.acos VALUES (425, 423, NULL, NULL, 'getSessionInitServices', 1061, 1062);
INSERT INTO public.acos VALUES (426, 423, NULL, NULL, 'getSessionAppDefinedServices', 1063, 1064);
INSERT INTO public.acos VALUES (427, 423, NULL, NULL, 'getSessionAppDefinedDesktops', 1065, 1066);
INSERT INTO public.acos VALUES (428, 423, NULL, NULL, 'getSessionInitDesktops', 1067, 1068);
INSERT INTO public.acos VALUES (429, 423, NULL, NULL, 'getSessionValDesktops', 1069, 1070);
INSERT INTO public.acos VALUES (430, 423, NULL, NULL, 'getSessionValeditDesktops', 1071, 1072);
INSERT INTO public.acos VALUES (431, 423, NULL, NULL, 'getSessionDispDesktops', 1073, 1074);
INSERT INTO public.acos VALUES (432, 423, NULL, NULL, 'getSessionArchDesktops', 1075, 1076);
INSERT INTO public.acos VALUES (433, 423, NULL, NULL, 'getSessionAllDesktops', 1077, 1078);
INSERT INTO public.acos VALUES (435, 423, NULL, NULL, 'getAddParams', 1081, 1082);
INSERT INTO public.acos VALUES (1185, 1172, NULL, NULL, 'getEnv', 2533, 2534);
INSERT INTO public.acos VALUES (806, 805, NULL, NULL, 'index', 1926, 1927);
INSERT INTO public.acos VALUES (807, 805, NULL, NULL, 'view', 1928, 1929);
INSERT INTO public.acos VALUES (808, 805, NULL, NULL, 'add', 1930, 1931);
INSERT INTO public.acos VALUES (809, 805, NULL, NULL, 'edit', 1932, 1933);
INSERT INTO public.acos VALUES (810, 805, NULL, NULL, 'delete', 1934, 1935);
INSERT INTO public.acos VALUES (811, 805, NULL, NULL, 'moveUp', 1936, 1937);
INSERT INTO public.acos VALUES (805, 754, NULL, NULL, 'Etapes', 1925, 1978);
INSERT INTO public.acos VALUES (812, 805, NULL, NULL, 'moveDown', 1938, 1939);
INSERT INTO public.acos VALUES (813, 805, NULL, NULL, 'setCreatedModifiedUser', 1940, 1941);
INSERT INTO public.acos VALUES (814, 805, NULL, NULL, 'formatUser', 1942, 1943);
INSERT INTO public.acos VALUES (815, 805, NULL, NULL, 'formatLinkedModel', 1944, 1945);
INSERT INTO public.acos VALUES (816, 805, NULL, NULL, 'formatLinkedModels', 1946, 1947);
INSERT INTO public.acos VALUES (817, 805, NULL, NULL, 'listLinkedModel', 1948, 1949);
INSERT INTO public.acos VALUES (818, 805, NULL, NULL, 'userAclCheck', 1950, 1951);
INSERT INTO public.acos VALUES (819, 805, NULL, NULL, 'getSessionInitServices', 1952, 1953);
INSERT INTO public.acos VALUES (820, 805, NULL, NULL, 'getSessionAppDefinedServices', 1954, 1955);
INSERT INTO public.acos VALUES (821, 805, NULL, NULL, 'getSessionAppDefinedDesktops', 1956, 1957);
INSERT INTO public.acos VALUES (822, 805, NULL, NULL, 'getSessionInitDesktops', 1958, 1959);
INSERT INTO public.acos VALUES (823, 805, NULL, NULL, 'getSessionValDesktops', 1960, 1961);
INSERT INTO public.acos VALUES (824, 805, NULL, NULL, 'getSessionValeditDesktops', 1962, 1963);
INSERT INTO public.acos VALUES (825, 805, NULL, NULL, 'getSessionDispDesktops', 1964, 1965);
INSERT INTO public.acos VALUES (826, 805, NULL, NULL, 'getSessionArchDesktops', 1966, 1967);
INSERT INTO public.acos VALUES (827, 805, NULL, NULL, 'getSessionAllDesktops', 1968, 1969);
INSERT INTO public.acos VALUES (828, 805, NULL, NULL, 'getEnv', 1970, 1971);
INSERT INTO public.acos VALUES (829, 805, NULL, NULL, 'getAddParams', 1972, 1973);
INSERT INTO public.acos VALUES (830, 805, NULL, NULL, 'ldebug', 1974, 1975);
INSERT INTO public.acos VALUES (1482, 409, NULL, NULL, 'isAuthorized', 1055, 1056);
INSERT INTO public.acos VALUES (1148, 90, NULL, NULL, 'isAuthorized', 249, 250);
INSERT INTO public.acos VALUES (1868, 781, NULL, NULL, 'isAuthorized', 1922, 1923);
INSERT INTO public.acos VALUES (423, 1, NULL, NULL, 'Keywordtypes', 1058, 1087);
INSERT INTO public.acos VALUES (437, 1, NULL, NULL, 'Metadonnees', 1088, 1127);
INSERT INTO public.acos VALUES (439, 437, NULL, NULL, 'getMetadonnees', 1091, 1092);
INSERT INTO public.acos VALUES (440, 437, NULL, NULL, 'add', 1093, 1094);
INSERT INTO public.acos VALUES (441, 437, NULL, NULL, 'edit', 1095, 1096);
INSERT INTO public.acos VALUES (442, 437, NULL, NULL, 'delete', 1097, 1098);
INSERT INTO public.acos VALUES (443, 437, NULL, NULL, 'userAclCheck', 1099, 1100);
INSERT INTO public.acos VALUES (444, 437, NULL, NULL, 'getSessionInitServices', 1101, 1102);
INSERT INTO public.acos VALUES (445, 437, NULL, NULL, 'getSessionAppDefinedServices', 1103, 1104);
INSERT INTO public.acos VALUES (446, 437, NULL, NULL, 'getSessionAppDefinedDesktops', 1105, 1106);
INSERT INTO public.acos VALUES (447, 437, NULL, NULL, 'getSessionInitDesktops', 1107, 1108);
INSERT INTO public.acos VALUES (448, 437, NULL, NULL, 'getSessionValDesktops', 1109, 1110);
INSERT INTO public.acos VALUES (449, 437, NULL, NULL, 'getSessionValeditDesktops', 1111, 1112);
INSERT INTO public.acos VALUES (450, 437, NULL, NULL, 'getSessionDispDesktops', 1113, 1114);
INSERT INTO public.acos VALUES (451, 437, NULL, NULL, 'getSessionArchDesktops', 1115, 1116);
INSERT INTO public.acos VALUES (452, 437, NULL, NULL, 'getSessionAllDesktops', 1117, 1118);
INSERT INTO public.acos VALUES (453, 437, NULL, NULL, 'getEnv', 1119, 1120);
INSERT INTO public.acos VALUES (454, 437, NULL, NULL, 'getAddParams', 1121, 1122);
INSERT INTO public.acos VALUES (458, 456, NULL, NULL, 'formulaire', 1131, 1132);
INSERT INTO public.acos VALUES (459, 456, NULL, NULL, 'getRecherches', 1133, 1134);
INSERT INTO public.acos VALUES (460, 456, NULL, NULL, 'delete', 1135, 1136);
INSERT INTO public.acos VALUES (461, 456, NULL, NULL, 'recherche', 1137, 1138);
INSERT INTO public.acos VALUES (462, 456, NULL, NULL, 'exportcsv', 1139, 1140);
INSERT INTO public.acos VALUES (463, 456, NULL, NULL, 'userAclCheck', 1141, 1142);
INSERT INTO public.acos VALUES (464, 456, NULL, NULL, 'getSessionInitServices', 1143, 1144);
INSERT INTO public.acos VALUES (465, 456, NULL, NULL, 'getSessionAppDefinedServices', 1145, 1146);
INSERT INTO public.acos VALUES (466, 456, NULL, NULL, 'getSessionAppDefinedDesktops', 1147, 1148);
INSERT INTO public.acos VALUES (467, 456, NULL, NULL, 'getSessionInitDesktops', 1149, 1150);
INSERT INTO public.acos VALUES (468, 456, NULL, NULL, 'getSessionValDesktops', 1151, 1152);
INSERT INTO public.acos VALUES (469, 456, NULL, NULL, 'getSessionValeditDesktops', 1153, 1154);
INSERT INTO public.acos VALUES (470, 456, NULL, NULL, 'getSessionDispDesktops', 1155, 1156);
INSERT INTO public.acos VALUES (471, 456, NULL, NULL, 'getSessionArchDesktops', 1157, 1158);
INSERT INTO public.acos VALUES (456, 1, NULL, NULL, 'Recherches', 1128, 1175);
INSERT INTO public.acos VALUES (1288, 1287, NULL, NULL, 'list', 2707, 2708);
INSERT INTO public.acos VALUES (1454, 1441, NULL, NULL, 'getSessionDispDesktops', 2973, 2974);
INSERT INTO public.acos VALUES (131, 130, NULL, NULL, 'add', 307, 308);
INSERT INTO public.acos VALUES (139, 130, NULL, NULL, 'getSessionValDesktops', 323, 324);
INSERT INTO public.acos VALUES (130, 1, NULL, NULL, 'Comments', 306, 341);
INSERT INTO public.acos VALUES (832, 831, NULL, NULL, 'visuTraitement', 1980, 1981);
INSERT INTO public.acos VALUES (833, 831, NULL, NULL, 'majTraitementsParapheur', 1982, 1983);
INSERT INTO public.acos VALUES (834, 831, NULL, NULL, 'traiterDelegationsPassees', 1984, 1985);
INSERT INTO public.acos VALUES (835, 831, NULL, NULL, 'setCreatedModifiedUser', 1986, 1987);
INSERT INTO public.acos VALUES (836, 831, NULL, NULL, 'formatUser', 1988, 1989);
INSERT INTO public.acos VALUES (837, 831, NULL, NULL, 'formatLinkedModel', 1990, 1991);
INSERT INTO public.acos VALUES (838, 831, NULL, NULL, 'formatLinkedModels', 1992, 1993);
INSERT INTO public.acos VALUES (839, 831, NULL, NULL, 'listLinkedModel', 1994, 1995);
INSERT INTO public.acos VALUES (840, 831, NULL, NULL, 'userAclCheck', 1996, 1997);
INSERT INTO public.acos VALUES (841, 831, NULL, NULL, 'getSessionInitServices', 1998, 1999);
INSERT INTO public.acos VALUES (842, 831, NULL, NULL, 'getSessionAppDefinedServices', 2000, 2001);
INSERT INTO public.acos VALUES (843, 831, NULL, NULL, 'getSessionAppDefinedDesktops', 2002, 2003);
INSERT INTO public.acos VALUES (844, 831, NULL, NULL, 'getSessionInitDesktops', 2004, 2005);
INSERT INTO public.acos VALUES (845, 831, NULL, NULL, 'getSessionValDesktops', 2006, 2007);
INSERT INTO public.acos VALUES (846, 831, NULL, NULL, 'getSessionValeditDesktops', 2008, 2009);
INSERT INTO public.acos VALUES (847, 831, NULL, NULL, 'getSessionDispDesktops', 2010, 2011);
INSERT INTO public.acos VALUES (848, 831, NULL, NULL, 'getSessionArchDesktops', 2012, 2013);
INSERT INTO public.acos VALUES (849, 831, NULL, NULL, 'getSessionAllDesktops', 2014, 2015);
INSERT INTO public.acos VALUES (850, 831, NULL, NULL, 'getEnv', 2016, 2017);
INSERT INTO public.acos VALUES (851, 831, NULL, NULL, 'getAddParams', 2018, 2019);
INSERT INTO public.acos VALUES (852, 831, NULL, NULL, 'ldebug', 2020, 2021);
INSERT INTO public.acos VALUES (831, 754, NULL, NULL, 'Traitements', 1979, 2024);
INSERT INTO public.acos VALUES (853, 1, NULL, NULL, 'DataAcl', 2026, 2027);
INSERT INTO public.acos VALUES (856, 855, NULL, NULL, 'history_state', 2030, 2031);
INSERT INTO public.acos VALUES (857, 855, NULL, NULL, 'sql_explain', 2032, 2033);
INSERT INTO public.acos VALUES (858, 855, NULL, NULL, 'userAclCheck', 2034, 2035);
INSERT INTO public.acos VALUES (859, 855, NULL, NULL, 'getSessionInitServices', 2036, 2037);
INSERT INTO public.acos VALUES (860, 855, NULL, NULL, 'getSessionAppDefinedServices', 2038, 2039);
INSERT INTO public.acos VALUES (861, 855, NULL, NULL, 'getSessionAppDefinedDesktops', 2040, 2041);
INSERT INTO public.acos VALUES (862, 855, NULL, NULL, 'getSessionInitDesktops', 2042, 2043);
INSERT INTO public.acos VALUES (863, 855, NULL, NULL, 'getSessionValDesktops', 2044, 2045);
INSERT INTO public.acos VALUES (864, 855, NULL, NULL, 'getSessionValeditDesktops', 2046, 2047);
INSERT INTO public.acos VALUES (865, 855, NULL, NULL, 'getSessionDispDesktops', 2048, 2049);
INSERT INTO public.acos VALUES (866, 855, NULL, NULL, 'getSessionArchDesktops', 2050, 2051);
INSERT INTO public.acos VALUES (867, 855, NULL, NULL, 'getSessionAllDesktops', 2052, 2053);
INSERT INTO public.acos VALUES (854, 1, NULL, NULL, 'DebugKit', 2028, 2063);
INSERT INTO public.acos VALUES (855, 854, NULL, NULL, 'ToolbarAccess', 2029, 2062);
INSERT INTO public.acos VALUES (873, 872, NULL, NULL, 'index', 2066, 2067);
INSERT INTO public.acos VALUES (874, 872, NULL, NULL, 'retrieve', 2068, 2069);
INSERT INTO public.acos VALUES (875, 872, NULL, NULL, 'getMail', 2070, 2071);
INSERT INTO public.acos VALUES (876, 872, NULL, NULL, 'getMailPart', 2072, 2073);
INSERT INTO public.acos VALUES (877, 872, NULL, NULL, 'userAclCheck', 2074, 2075);
INSERT INTO public.acos VALUES (878, 872, NULL, NULL, 'getSessionInitServices', 2076, 2077);
INSERT INTO public.acos VALUES (879, 872, NULL, NULL, 'getSessionAppDefinedServices', 2078, 2079);
INSERT INTO public.acos VALUES (880, 872, NULL, NULL, 'getSessionAppDefinedDesktops', 2080, 2081);
INSERT INTO public.acos VALUES (881, 872, NULL, NULL, 'getSessionInitDesktops', 2082, 2083);
INSERT INTO public.acos VALUES (871, 1, NULL, NULL, 'Imap', 2064, 2103);
INSERT INTO public.acos VALUES (872, 871, NULL, NULL, 'Imaps', 2065, 2102);
INSERT INTO public.acos VALUES (1483, 423, NULL, NULL, 'isAuthorized', 1085, 1086);
INSERT INTO public.acos VALUES (1869, 805, NULL, NULL, 'isAuthorized', 1976, 1977);
INSERT INTO public.acos VALUES (1168, 105, NULL, NULL, 'genApiKey', 301, 302);
INSERT INTO public.acos VALUES (472, 456, NULL, NULL, 'getSessionAllDesktops', 1159, 1160);
INSERT INTO public.acos VALUES (474, 456, NULL, NULL, 'getAddParams', 1163, 1164);
INSERT INTO public.acos VALUES (475, 456, NULL, NULL, 'ldebug', 1165, 1166);
INSERT INTO public.acos VALUES (477, 476, NULL, NULL, 'add', 1177, 1178);
INSERT INTO public.acos VALUES (476, 1, NULL, NULL, 'Relements', 1176, 1215);
INSERT INTO public.acos VALUES (140, 130, NULL, NULL, 'getSessionValeditDesktops', 325, 326);
INSERT INTO public.acos VALUES (141, 130, NULL, NULL, 'getSessionDispDesktops', 327, 328);
INSERT INTO public.acos VALUES (142, 130, NULL, NULL, 'getSessionArchDesktops', 329, 330);
INSERT INTO public.acos VALUES (143, 130, NULL, NULL, 'getSessionAllDesktops', 331, 332);
INSERT INTO public.acos VALUES (144, 130, NULL, NULL, 'getEnv', 333, 334);
INSERT INTO public.acos VALUES (145, 130, NULL, NULL, 'getAddParams', 335, 336);
INSERT INTO public.acos VALUES (146, 130, NULL, NULL, 'ldebug', 337, 338);
INSERT INTO public.acos VALUES (1507, 437, NULL, NULL, 'isAuthorized', 1125, 1126);
INSERT INTO public.acos VALUES (1870, 831, NULL, NULL, 'isAuthorized', 2022, 2023);
INSERT INTO public.acos VALUES (1215, 1213, NULL, NULL, 'add', 2583, 2584);
INSERT INTO public.acos VALUES (1216, 1213, NULL, NULL, 'edit', 2585, 2586);
INSERT INTO public.acos VALUES (1217, 1213, NULL, NULL, 'delete', 2587, 2588);
INSERT INTO public.acos VALUES (1218, 1213, NULL, NULL, 'getConsultations', 2589, 2590);
INSERT INTO public.acos VALUES (1219, 1213, NULL, NULL, 'ajaxformvalid', 2591, 2592);
INSERT INTO public.acos VALUES (1220, 1213, NULL, NULL, 'export', 2593, 2594);
INSERT INTO public.acos VALUES (1221, 1213, NULL, NULL, 'getPlis', 2595, 2596);
INSERT INTO public.acos VALUES (1222, 1213, NULL, NULL, 'userAclCheck', 2597, 2598);
INSERT INTO public.acos VALUES (1223, 1213, NULL, NULL, 'getSessionInitServices', 2599, 2600);
INSERT INTO public.acos VALUES (1224, 1213, NULL, NULL, 'getSessionAppDefinedServices', 2601, 2602);
INSERT INTO public.acos VALUES (1225, 1213, NULL, NULL, 'getSessionAppDefinedDesktops', 2603, 2604);
INSERT INTO public.acos VALUES (1226, 1213, NULL, NULL, 'getSessionInitDesktops', 2605, 2606);
INSERT INTO public.acos VALUES (1227, 1213, NULL, NULL, 'getSessionValDesktops', 2607, 2608);
INSERT INTO public.acos VALUES (1228, 1213, NULL, NULL, 'getSessionValeditDesktops', 2609, 2610);
INSERT INTO public.acos VALUES (1229, 1213, NULL, NULL, 'getSessionDispDesktops', 2611, 2612);
INSERT INTO public.acos VALUES (1230, 1213, NULL, NULL, 'getSessionArchDesktops', 2613, 2614);
INSERT INTO public.acos VALUES (1231, 1213, NULL, NULL, 'getSessionAllDesktops', 2615, 2616);
INSERT INTO public.acos VALUES (1232, 1213, NULL, NULL, 'getEnv', 2617, 2618);
INSERT INTO public.acos VALUES (1233, 1213, NULL, NULL, 'getAddParams', 2619, 2620);
INSERT INTO public.acos VALUES (1234, 1213, NULL, NULL, 'ldebug', 2621, 2622);
INSERT INTO public.acos VALUES (1237, 1236, NULL, NULL, 'list', 2627, 2628);
INSERT INTO public.acos VALUES (1238, 1236, NULL, NULL, 'find', 2629, 2630);
INSERT INTO public.acos VALUES (1239, 1236, NULL, NULL, 'add', 2631, 2632);
INSERT INTO public.acos VALUES (1240, 1236, NULL, NULL, 'delete', 2633, 2634);
INSERT INTO public.acos VALUES (1241, 1236, NULL, NULL, 'update', 2635, 2636);
INSERT INTO public.acos VALUES (1242, 1236, NULL, NULL, 'userAclCheck', 2637, 2638);
INSERT INTO public.acos VALUES (1243, 1236, NULL, NULL, 'getSessionInitServices', 2639, 2640);
INSERT INTO public.acos VALUES (1244, 1236, NULL, NULL, 'getSessionAppDefinedServices', 2641, 2642);
INSERT INTO public.acos VALUES (1245, 1236, NULL, NULL, 'getSessionAppDefinedDesktops', 2643, 2644);
INSERT INTO public.acos VALUES (1246, 1236, NULL, NULL, 'getSessionInitDesktops', 2645, 2646);
INSERT INTO public.acos VALUES (1247, 1236, NULL, NULL, 'getSessionValDesktops', 2647, 2648);
INSERT INTO public.acos VALUES (1248, 1236, NULL, NULL, 'getSessionValeditDesktops', 2649, 2650);
INSERT INTO public.acos VALUES (1249, 1236, NULL, NULL, 'getSessionDispDesktops', 2651, 2652);
INSERT INTO public.acos VALUES (1252, 1236, NULL, NULL, 'getEnv', 2657, 2658);
INSERT INTO public.acos VALUES (1169, 105, NULL, NULL, 'isAuthorized', 303, 304);
INSERT INTO public.acos VALUES (202, 187, NULL, NULL, 'getEnv', 465, 466);
INSERT INTO public.acos VALUES (882, 872, NULL, NULL, 'getSessionValDesktops', 2084, 2085);
INSERT INTO public.acos VALUES (883, 872, NULL, NULL, 'getSessionValeditDesktops', 2086, 2087);
INSERT INTO public.acos VALUES (884, 872, NULL, NULL, 'getSessionDispDesktops', 2088, 2089);
INSERT INTO public.acos VALUES (885, 872, NULL, NULL, 'getSessionArchDesktops', 2090, 2091);
INSERT INTO public.acos VALUES (886, 872, NULL, NULL, 'getSessionAllDesktops', 2092, 2093);
INSERT INTO public.acos VALUES (888, 872, NULL, NULL, 'getAddParams', 2096, 2097);
INSERT INTO public.acos VALUES (889, 872, NULL, NULL, 'ldebug', 2098, 2099);
INSERT INTO public.acos VALUES (1871, 855, NULL, NULL, 'isAuthorized', 2060, 2061);
INSERT INTO public.acos VALUES (148, 147, NULL, NULL, 'userAclCheck', 343, 344);
INSERT INTO public.acos VALUES (149, 147, NULL, NULL, 'getSessionInitServices', 345, 346);
INSERT INTO public.acos VALUES (150, 147, NULL, NULL, 'getSessionAppDefinedServices', 347, 348);
INSERT INTO public.acos VALUES (151, 147, NULL, NULL, 'getSessionAppDefinedDesktops', 349, 350);
INSERT INTO public.acos VALUES (152, 147, NULL, NULL, 'getSessionInitDesktops', 351, 352);
INSERT INTO public.acos VALUES (153, 147, NULL, NULL, 'getSessionValDesktops', 353, 354);
INSERT INTO public.acos VALUES (154, 147, NULL, NULL, 'getSessionValeditDesktops', 355, 356);
INSERT INTO public.acos VALUES (155, 147, NULL, NULL, 'getSessionDispDesktops', 357, 358);
INSERT INTO public.acos VALUES (156, 147, NULL, NULL, 'getSessionArchDesktops', 359, 360);
INSERT INTO public.acos VALUES (157, 147, NULL, NULL, 'getSessionAllDesktops', 361, 362);
INSERT INTO public.acos VALUES (159, 147, NULL, NULL, 'getAddParams', 365, 366);
INSERT INTO public.acos VALUES (160, 147, NULL, NULL, 'ldebug', 367, 368);
INSERT INTO public.acos VALUES (945, 1, NULL, NULL, 'Referentielsfantoir', 2186, 2221);
INSERT INTO public.acos VALUES (946, 945, NULL, NULL, 'index', 2187, 2188);
INSERT INTO public.acos VALUES (947, 945, NULL, NULL, 'getReferentiel', 2189, 2190);
INSERT INTO public.acos VALUES (948, 945, NULL, NULL, 'import', 2191, 2192);
INSERT INTO public.acos VALUES (949, 945, NULL, NULL, 'userAclCheck', 2193, 2194);
INSERT INTO public.acos VALUES (950, 945, NULL, NULL, 'getSessionInitServices', 2195, 2196);
INSERT INTO public.acos VALUES (951, 945, NULL, NULL, 'getSessionAppDefinedServices', 2197, 2198);
INSERT INTO public.acos VALUES (952, 945, NULL, NULL, 'getSessionAppDefinedDesktops', 2199, 2200);
INSERT INTO public.acos VALUES (953, 945, NULL, NULL, 'getSessionInitDesktops', 2201, 2202);
INSERT INTO public.acos VALUES (954, 945, NULL, NULL, 'getSessionValDesktops', 2203, 2204);
INSERT INTO public.acos VALUES (955, 945, NULL, NULL, 'getSessionValeditDesktops', 2205, 2206);
INSERT INTO public.acos VALUES (956, 945, NULL, NULL, 'getSessionDispDesktops', 2207, 2208);
INSERT INTO public.acos VALUES (957, 945, NULL, NULL, 'getSessionArchDesktops', 2209, 2210);
INSERT INTO public.acos VALUES (958, 945, NULL, NULL, 'getSessionAllDesktops', 2211, 2212);
INSERT INTO public.acos VALUES (959, 945, NULL, NULL, 'getEnv', 2213, 2214);
INSERT INTO public.acos VALUES (960, 945, NULL, NULL, 'getAddParams', 2215, 2216);
INSERT INTO public.acos VALUES (961, 945, NULL, NULL, 'ldebug', 2217, 2218);
INSERT INTO public.acos VALUES (987, 1, NULL, NULL, 'Desktopsmanagers', 2222, 2263);
INSERT INTO public.acos VALUES (988, 987, NULL, NULL, 'index', 2223, 2224);
INSERT INTO public.acos VALUES (990, 987, NULL, NULL, 'add', 2225, 2226);
INSERT INTO public.acos VALUES (991, 987, NULL, NULL, 'edit', 2227, 2228);
INSERT INTO public.acos VALUES (992, 987, NULL, NULL, 'delete', 2229, 2230);
INSERT INTO public.acos VALUES (993, 987, NULL, NULL, 'userAclCheck', 2231, 2232);
INSERT INTO public.acos VALUES (994, 987, NULL, NULL, 'getSessionInitServices', 2233, 2234);
INSERT INTO public.acos VALUES (995, 987, NULL, NULL, 'getSessionAppDefinedServices', 2235, 2236);
INSERT INTO public.acos VALUES (996, 987, NULL, NULL, 'getSessionAppDefinedDesktops', 2237, 2238);
INSERT INTO public.acos VALUES (997, 987, NULL, NULL, 'getSessionInitDesktops', 2239, 2240);
INSERT INTO public.acos VALUES (998, 987, NULL, NULL, 'getSessionValDesktops', 2241, 2242);
INSERT INTO public.acos VALUES (999, 987, NULL, NULL, 'getSessionValeditDesktops', 2243, 2244);
INSERT INTO public.acos VALUES (1000, 987, NULL, NULL, 'getSessionDispDesktops', 2245, 2246);
INSERT INTO public.acos VALUES (1001, 987, NULL, NULL, 'getSessionArchDesktops', 2247, 2248);
INSERT INTO public.acos VALUES (1002, 987, NULL, NULL, 'getSessionAllDesktops', 2249, 2250);
INSERT INTO public.acos VALUES (1005, 987, NULL, NULL, 'ldebug', 2255, 2256);
INSERT INTO public.acos VALUES (1006, 1, NULL, NULL, 'Rmodelgabarits', 2264, 2301);
INSERT INTO public.acos VALUES (1008, 1006, NULL, NULL, 'download', 2267, 2268);
INSERT INTO public.acos VALUES (1009, 1006, NULL, NULL, 'getPreview', 2269, 2270);
INSERT INTO public.acos VALUES (1010, 1006, NULL, NULL, 'add', 2271, 2272);
INSERT INTO public.acos VALUES (1011, 1006, NULL, NULL, 'userAclCheck', 2273, 2274);
INSERT INTO public.acos VALUES (1012, 1006, NULL, NULL, 'getSessionInitServices', 2275, 2276);
INSERT INTO public.acos VALUES (1013, 1006, NULL, NULL, 'getSessionAppDefinedServices', 2277, 2278);
INSERT INTO public.acos VALUES (1014, 1006, NULL, NULL, 'getSessionAppDefinedDesktops', 2279, 2280);
INSERT INTO public.acos VALUES (1015, 1006, NULL, NULL, 'getSessionInitDesktops', 2281, 2282);
INSERT INTO public.acos VALUES (1016, 1006, NULL, NULL, 'getSessionValDesktops', 2283, 2284);
INSERT INTO public.acos VALUES (1017, 1006, NULL, NULL, 'getSessionValeditDesktops', 2285, 2286);
INSERT INTO public.acos VALUES (1018, 1006, NULL, NULL, 'getSessionDispDesktops', 2287, 2288);
INSERT INTO public.acos VALUES (1019, 1006, NULL, NULL, 'getSessionArchDesktops', 2289, 2290);
INSERT INTO public.acos VALUES (1061, 1045, NULL, NULL, 'getEnv', 2333, 2334);
INSERT INTO public.acos VALUES (1556, 1045, NULL, NULL, 'export', 2345, 2346);
INSERT INTO public.acos VALUES (1738, 1726, NULL, NULL, 'getSessionValeditDesktops', 3469, 3470);
INSERT INTO public.acos VALUES (1739, 1726, NULL, NULL, 'getSessionDispDesktops', 3471, 3472);
INSERT INTO public.acos VALUES (1740, 1726, NULL, NULL, 'getSessionArchDesktops', 3473, 3474);
INSERT INTO public.acos VALUES (1741, 1726, NULL, NULL, 'getSessionAllDesktops', 3475, 3476);
INSERT INTO public.acos VALUES (1742, 1726, NULL, NULL, 'getEnv', 3477, 3478);
INSERT INTO public.acos VALUES (1743, 1726, NULL, NULL, 'getAddParams', 3479, 3480);
INSERT INTO public.acos VALUES (1744, 1726, NULL, NULL, 'ldebug', 3481, 3482);
INSERT INTO public.acos VALUES (1745, 1726, NULL, NULL, 'isAuthorized', 3483, 3484);
INSERT INTO public.acos VALUES (1746, 1, NULL, NULL, 'Templatenotifications', 3486, 3525);
INSERT INTO public.acos VALUES (1747, 1746, NULL, NULL, 'index', 3487, 3488);
INSERT INTO public.acos VALUES (1748, 1746, NULL, NULL, 'add', 3489, 3490);
INSERT INTO public.acos VALUES (1749, 1746, NULL, NULL, 'edit', 3491, 3492);
INSERT INTO public.acos VALUES (1750, 1746, NULL, NULL, 'delete', 3493, 3494);
INSERT INTO public.acos VALUES (1751, 1746, NULL, NULL, 'getTemplatenotifications', 3495, 3496);
INSERT INTO public.acos VALUES (1752, 1746, NULL, NULL, 'userAclCheck', 3497, 3498);
INSERT INTO public.acos VALUES (1753, 1746, NULL, NULL, 'getSessionInitServices', 3499, 3500);
INSERT INTO public.acos VALUES (1754, 1746, NULL, NULL, 'getSessionAppDefinedServices', 3501, 3502);
INSERT INTO public.acos VALUES (1755, 1746, NULL, NULL, 'getSessionAppDefinedDesktops', 3503, 3504);
INSERT INTO public.acos VALUES (1756, 1746, NULL, NULL, 'getSessionInitDesktops', 3505, 3506);
INSERT INTO public.acos VALUES (1757, 1746, NULL, NULL, 'getSessionValDesktops', 3507, 3508);
INSERT INTO public.acos VALUES (1758, 1746, NULL, NULL, 'getSessionValeditDesktops', 3509, 3510);
INSERT INTO public.acos VALUES (1759, 1746, NULL, NULL, 'getSessionDispDesktops', 3511, 3512);
INSERT INTO public.acos VALUES (1760, 1746, NULL, NULL, 'getSessionArchDesktops', 3513, 3514);
INSERT INTO public.acos VALUES (1761, 1746, NULL, NULL, 'getSessionAllDesktops', 3515, 3516);
INSERT INTO public.acos VALUES (1762, 1746, NULL, NULL, 'getEnv', 3517, 3518);
INSERT INTO public.acos VALUES (1763, 1746, NULL, NULL, 'getAddParams', 3519, 3520);
INSERT INTO public.acos VALUES (1764, 1746, NULL, NULL, 'ldebug', 3521, 3522);
INSERT INTO public.acos VALUES (1765, 1746, NULL, NULL, 'isAuthorized', 3523, 3524);
INSERT INTO public.acos VALUES (1766, 1, NULL, NULL, 'Titres', 3526, 3567);
INSERT INTO public.acos VALUES (1767, 1766, NULL, NULL, 'index', 3527, 3528);
INSERT INTO public.acos VALUES (1768, 1766, NULL, NULL, 'add', 3529, 3530);
INSERT INTO public.acos VALUES (1769, 1766, NULL, NULL, 'edit', 3531, 3532);
INSERT INTO public.acos VALUES (1770, 1766, NULL, NULL, 'delete', 3533, 3534);
INSERT INTO public.acos VALUES (1771, 1766, NULL, NULL, 'getTitre', 3535, 3536);
INSERT INTO public.acos VALUES (1772, 1766, NULL, NULL, 'ajaxformvalid', 3537, 3538);
INSERT INTO public.acos VALUES (1773, 1766, NULL, NULL, 'userAclCheck', 3539, 3540);
INSERT INTO public.acos VALUES (890, 1, NULL, NULL, 'Jsonmsg', 2104, 2105);
INSERT INTO public.acos VALUES (891, 1, NULL, NULL, 'Pgsqlcake', 2106, 2107);
INSERT INTO public.acos VALUES (892, 1, NULL, NULL, 'SimpleComment', 2108, 2109);
INSERT INTO public.acos VALUES (896, 1, NULL, NULL, 'Bans', 2110, 2149);
INSERT INTO public.acos VALUES (897, 896, NULL, NULL, 'index', 2111, 2112);
INSERT INTO public.acos VALUES (898, 896, NULL, NULL, 'getBan', 2113, 2114);
INSERT INTO public.acos VALUES (899, 896, NULL, NULL, 'edit', 2115, 2116);
INSERT INTO public.acos VALUES (900, 896, NULL, NULL, 'import', 2117, 2118);
INSERT INTO public.acos VALUES (901, 896, NULL, NULL, 'delete', 2119, 2120);
INSERT INTO public.acos VALUES (902, 896, NULL, NULL, 'userAclCheck', 2121, 2122);
INSERT INTO public.acos VALUES (903, 896, NULL, NULL, 'getSessionInitServices', 2123, 2124);
INSERT INTO public.acos VALUES (904, 896, NULL, NULL, 'getSessionAppDefinedServices', 2125, 2126);
INSERT INTO public.acos VALUES (905, 896, NULL, NULL, 'getSessionAppDefinedDesktops', 2127, 2128);
INSERT INTO public.acos VALUES (906, 896, NULL, NULL, 'getSessionInitDesktops', 2129, 2130);
INSERT INTO public.acos VALUES (907, 896, NULL, NULL, 'getSessionValDesktops', 2131, 2132);
INSERT INTO public.acos VALUES (908, 896, NULL, NULL, 'getSessionValeditDesktops', 2133, 2134);
INSERT INTO public.acos VALUES (909, 896, NULL, NULL, 'getSessionDispDesktops', 2135, 2136);
INSERT INTO public.acos VALUES (910, 896, NULL, NULL, 'getSessionArchDesktops', 2137, 2138);
INSERT INTO public.acos VALUES (911, 896, NULL, NULL, 'getSessionAllDesktops', 2139, 2140);
INSERT INTO public.acos VALUES (913, 896, NULL, NULL, 'getAddParams', 2143, 2144);
INSERT INTO public.acos VALUES (915, 1, NULL, NULL, 'Banscommunes', 2150, 2185);
INSERT INTO public.acos VALUES (917, 915, NULL, NULL, 'edit', 2153, 2154);
INSERT INTO public.acos VALUES (918, 915, NULL, NULL, 'delete', 2155, 2156);
INSERT INTO public.acos VALUES (919, 915, NULL, NULL, 'userAclCheck', 2157, 2158);
INSERT INTO public.acos VALUES (920, 915, NULL, NULL, 'getSessionInitServices', 2159, 2160);
INSERT INTO public.acos VALUES (921, 915, NULL, NULL, 'getSessionAppDefinedServices', 2161, 2162);
INSERT INTO public.acos VALUES (922, 915, NULL, NULL, 'getSessionAppDefinedDesktops', 2163, 2164);
INSERT INTO public.acos VALUES (923, 915, NULL, NULL, 'getSessionInitDesktops', 2165, 2166);
INSERT INTO public.acos VALUES (924, 915, NULL, NULL, 'getSessionValDesktops', 2167, 2168);
INSERT INTO public.acos VALUES (925, 915, NULL, NULL, 'getSessionValeditDesktops', 2169, 2170);
INSERT INTO public.acos VALUES (926, 915, NULL, NULL, 'getSessionDispDesktops', 2171, 2172);
INSERT INTO public.acos VALUES (927, 915, NULL, NULL, 'getSessionArchDesktops', 2173, 2174);
INSERT INTO public.acos VALUES (928, 915, NULL, NULL, 'getSessionAllDesktops', 2175, 2176);
INSERT INTO public.acos VALUES (1004, 987, NULL, NULL, 'getAddParams', 2253, 2254);
INSERT INTO public.acos VALUES (1020, 1006, NULL, NULL, 'getSessionAllDesktops', 2291, 2292);
INSERT INTO public.acos VALUES (1021, 1006, NULL, NULL, 'getEnv', 2293, 2294);
INSERT INTO public.acos VALUES (1022, 1006, NULL, NULL, 'getAddParams', 2295, 2296);
INSERT INTO public.acos VALUES (1023, 1006, NULL, NULL, 'ldebug', 2297, 2298);
INSERT INTO public.acos VALUES (1044, 987, NULL, NULL, 'get_bureaux', 2257, 2258);
INSERT INTO public.acos VALUES (1045, 1, NULL, NULL, 'Organismes', 2302, 2355);
INSERT INTO public.acos VALUES (1046, 1045, NULL, NULL, 'add', 2303, 2304);
INSERT INTO public.acos VALUES (1047, 1045, NULL, NULL, 'edit', 2305, 2306);
INSERT INTO public.acos VALUES (1048, 1045, NULL, NULL, 'delete', 2307, 2308);
INSERT INTO public.acos VALUES (1049, 1045, NULL, NULL, 'getAdresses', 2309, 2310);
INSERT INTO public.acos VALUES (1050, 1045, NULL, NULL, 'get_organismes', 2311, 2312);
INSERT INTO public.acos VALUES (1051, 1045, NULL, NULL, 'userAclCheck', 2313, 2314);
INSERT INTO public.acos VALUES (1052, 1045, NULL, NULL, 'getSessionInitServices', 2315, 2316);
INSERT INTO public.acos VALUES (1053, 1045, NULL, NULL, 'getSessionAppDefinedServices', 2317, 2318);
INSERT INTO public.acos VALUES (1054, 1045, NULL, NULL, 'getSessionAppDefinedDesktops', 2319, 2320);
INSERT INTO public.acos VALUES (1055, 1045, NULL, NULL, 'getSessionInitDesktops', 2321, 2322);
INSERT INTO public.acos VALUES (1056, 1045, NULL, NULL, 'getSessionValDesktops', 2323, 2324);
INSERT INTO public.acos VALUES (1057, 1045, NULL, NULL, 'getSessionValeditDesktops', 2325, 2326);
INSERT INTO public.acos VALUES (1058, 1045, NULL, NULL, 'getSessionDispDesktops', 2327, 2328);
INSERT INTO public.acos VALUES (1059, 1045, NULL, NULL, 'getSessionArchDesktops', 2329, 2330);
INSERT INTO public.acos VALUES (1060, 1045, NULL, NULL, 'getSessionAllDesktops', 2331, 2332);
INSERT INTO public.acos VALUES (1062, 1045, NULL, NULL, 'getAddParams', 2335, 2336);
INSERT INTO public.acos VALUES (1063, 1045, NULL, NULL, 'ldebug', 2337, 2338);
INSERT INTO public.acos VALUES (1071, 1, NULL, NULL, 'Addressbook', 2358, 2359);
INSERT INTO public.acos VALUES (1075, 1, NULL, NULL, 'Bansadresses', 2360, 2393);
INSERT INTO public.acos VALUES (1077, 1075, NULL, NULL, 'getCodePostal', 2363, 2364);
INSERT INTO public.acos VALUES (1078, 1075, NULL, NULL, 'userAclCheck', 2365, 2366);
INSERT INTO public.acos VALUES (1079, 1075, NULL, NULL, 'getSessionInitServices', 2367, 2368);
INSERT INTO public.acos VALUES (1080, 1075, NULL, NULL, 'getSessionAppDefinedServices', 2369, 2370);
INSERT INTO public.acos VALUES (1081, 1075, NULL, NULL, 'getSessionAppDefinedDesktops', 2371, 2372);
INSERT INTO public.acos VALUES (1082, 1075, NULL, NULL, 'getSessionInitDesktops', 2373, 2374);
INSERT INTO public.acos VALUES (1083, 1075, NULL, NULL, 'getSessionValDesktops', 2375, 2376);
INSERT INTO public.acos VALUES (1084, 1075, NULL, NULL, 'getSessionValeditDesktops', 2377, 2378);
INSERT INTO public.acos VALUES (1093, 1092, NULL, NULL, 'index', 2395, 2396);
INSERT INTO public.acos VALUES (1094, 1092, NULL, NULL, 'add', 2397, 2398);
INSERT INTO public.acos VALUES (1095, 1092, NULL, NULL, 'edit', 2399, 2400);
INSERT INTO public.acos VALUES (1096, 1092, NULL, NULL, 'delete', 2401, 2402);
INSERT INTO public.acos VALUES (1097, 1092, NULL, NULL, 'getActivites', 2403, 2404);
INSERT INTO public.acos VALUES (1098, 1092, NULL, NULL, 'userAclCheck', 2405, 2406);
INSERT INTO public.acos VALUES (1099, 1092, NULL, NULL, 'getSessionInitServices', 2407, 2408);
INSERT INTO public.acos VALUES (1100, 1092, NULL, NULL, 'getSessionAppDefinedServices', 2409, 2410);
INSERT INTO public.acos VALUES (1101, 1092, NULL, NULL, 'getSessionAppDefinedDesktops', 2411, 2412);
INSERT INTO public.acos VALUES (1102, 1092, NULL, NULL, 'getSessionInitDesktops', 2413, 2414);
INSERT INTO public.acos VALUES (1103, 1092, NULL, NULL, 'getSessionValDesktops', 2415, 2416);
INSERT INTO public.acos VALUES (1104, 1092, NULL, NULL, 'getSessionValeditDesktops', 2417, 2418);
INSERT INTO public.acos VALUES (1105, 1092, NULL, NULL, 'getSessionDispDesktops', 2419, 2420);
INSERT INTO public.acos VALUES (1106, 1092, NULL, NULL, 'getSessionArchDesktops', 2421, 2422);
INSERT INTO public.acos VALUES (1107, 1092, NULL, NULL, 'getSessionAllDesktops', 2423, 2424);
INSERT INTO public.acos VALUES (1108, 1092, NULL, NULL, 'getEnv', 2425, 2426);
INSERT INTO public.acos VALUES (1109, 1092, NULL, NULL, 'getAddParams', 2427, 2428);
INSERT INTO public.acos VALUES (1110, 1092, NULL, NULL, 'ldebug', 2429, 2430);
INSERT INTO public.acos VALUES (1111, 1092, NULL, NULL, 'isAuthorized', 2431, 2432);
INSERT INTO public.acos VALUES (1124, 1, NULL, NULL, 'Authentifications', 2434, 2469);
INSERT INTO public.acos VALUES (1125, 1124, NULL, NULL, 'index', 2435, 2436);
INSERT INTO public.acos VALUES (1126, 1124, NULL, NULL, 'getAuthentifications', 2437, 2438);
INSERT INTO public.acos VALUES (1127, 1124, NULL, NULL, 'makeconf', 2439, 2440);
INSERT INTO public.acos VALUES (1128, 1124, NULL, NULL, 'userAclCheck', 2441, 2442);
INSERT INTO public.acos VALUES (1129, 1124, NULL, NULL, 'getSessionInitServices', 2443, 2444);
INSERT INTO public.acos VALUES (1130, 1124, NULL, NULL, 'getSessionAppDefinedServices', 2445, 2446);
INSERT INTO public.acos VALUES (1131, 1124, NULL, NULL, 'getSessionAppDefinedDesktops', 2447, 2448);
INSERT INTO public.acos VALUES (1132, 1124, NULL, NULL, 'getSessionInitDesktops', 2449, 2450);
INSERT INTO public.acos VALUES (1133, 1124, NULL, NULL, 'getSessionValDesktops', 2451, 2452);
INSERT INTO public.acos VALUES (1134, 1124, NULL, NULL, 'getSessionValeditDesktops', 2453, 2454);
INSERT INTO public.acos VALUES (1135, 1124, NULL, NULL, 'getSessionDispDesktops', 2455, 2456);
INSERT INTO public.acos VALUES (1136, 1124, NULL, NULL, 'getSessionArchDesktops', 2457, 2458);
INSERT INTO public.acos VALUES (1137, 1124, NULL, NULL, 'getSessionAllDesktops', 2459, 2460);
INSERT INTO public.acos VALUES (1138, 1124, NULL, NULL, 'getEnv', 2461, 2462);
INSERT INTO public.acos VALUES (1139, 1124, NULL, NULL, 'getAddParams', 2463, 2464);
INSERT INTO public.acos VALUES (1140, 1124, NULL, NULL, 'ldebug', 2465, 2466);
INSERT INTO public.acos VALUES (1141, 1124, NULL, NULL, 'isAuthorized', 2467, 2468);
INSERT INTO public.acos VALUES (1149, 1, NULL, NULL, 'CollectiviteApi', 2470, 2507);
INSERT INTO public.acos VALUES (1150, 1149, NULL, NULL, 'list', 2471, 2472);
INSERT INTO public.acos VALUES (1151, 1149, NULL, NULL, 'find', 2473, 2474);
INSERT INTO public.acos VALUES (1152, 1149, NULL, NULL, 'add', 2475, 2476);
INSERT INTO public.acos VALUES (1153, 1149, NULL, NULL, 'delete', 2477, 2478);
INSERT INTO public.acos VALUES (1154, 1149, NULL, NULL, 'userAclCheck', 2479, 2480);
INSERT INTO public.acos VALUES (1155, 1149, NULL, NULL, 'getSessionInitServices', 2481, 2482);
INSERT INTO public.acos VALUES (1156, 1149, NULL, NULL, 'getSessionAppDefinedServices', 2483, 2484);
INSERT INTO public.acos VALUES (1157, 1149, NULL, NULL, 'getSessionAppDefinedDesktops', 2485, 2486);
INSERT INTO public.acos VALUES (1158, 1149, NULL, NULL, 'getSessionInitDesktops', 2487, 2488);
INSERT INTO public.acos VALUES (1159, 1149, NULL, NULL, 'getSessionValDesktops', 2489, 2490);
INSERT INTO public.acos VALUES (1160, 1149, NULL, NULL, 'getSessionValeditDesktops', 2491, 2492);
INSERT INTO public.acos VALUES (1161, 1149, NULL, NULL, 'getSessionDispDesktops', 2493, 2494);
INSERT INTO public.acos VALUES (1162, 1149, NULL, NULL, 'getSessionArchDesktops', 2495, 2496);
INSERT INTO public.acos VALUES (1163, 1149, NULL, NULL, 'getSessionAllDesktops', 2497, 2498);
INSERT INTO public.acos VALUES (1164, 1149, NULL, NULL, 'getEnv', 2499, 2500);
INSERT INTO public.acos VALUES (1165, 1149, NULL, NULL, 'getAddParams', 2501, 2502);
INSERT INTO public.acos VALUES (1166, 1149, NULL, NULL, 'ldebug', 2503, 2504);
INSERT INTO public.acos VALUES (1167, 1149, NULL, NULL, 'isAuthorized', 2505, 2506);
INSERT INTO public.acos VALUES (1172, 1, NULL, NULL, 'Configurations', 2508, 2541);
INSERT INTO public.acos VALUES (1174, 1172, NULL, NULL, 'config', 2511, 2512);
INSERT INTO public.acos VALUES (1175, 1172, NULL, NULL, 'userAclCheck', 2513, 2514);
INSERT INTO public.acos VALUES (1176, 1172, NULL, NULL, 'getSessionInitServices', 2515, 2516);
INSERT INTO public.acos VALUES (1177, 1172, NULL, NULL, 'getSessionAppDefinedServices', 2517, 2518);
INSERT INTO public.acos VALUES (1178, 1172, NULL, NULL, 'getSessionAppDefinedDesktops', 2519, 2520);
INSERT INTO public.acos VALUES (1179, 1172, NULL, NULL, 'getSessionInitDesktops', 2521, 2522);
INSERT INTO public.acos VALUES (1180, 1172, NULL, NULL, 'getSessionValDesktops', 2523, 2524);
INSERT INTO public.acos VALUES (1181, 1172, NULL, NULL, 'getSessionValeditDesktops', 2525, 2526);
INSERT INTO public.acos VALUES (1182, 1172, NULL, NULL, 'getSessionDispDesktops', 2527, 2528);
INSERT INTO public.acos VALUES (1183, 1172, NULL, NULL, 'getSessionArchDesktops', 2529, 2530);
INSERT INTO public.acos VALUES (1184, 1172, NULL, NULL, 'getSessionAllDesktops', 2531, 2532);
INSERT INTO public.acos VALUES (1186, 1172, NULL, NULL, 'getAddParams', 2535, 2536);
INSERT INTO public.acos VALUES (1187, 1172, NULL, NULL, 'ldebug', 2537, 2538);
INSERT INTO public.acos VALUES (1188, 1172, NULL, NULL, 'isAuthorized', 2539, 2540);
INSERT INTO public.acos VALUES (1194, 1, NULL, NULL, 'Consoles', 2542, 2579);
INSERT INTO public.acos VALUES (1195, 1194, NULL, NULL, 'index', 2543, 2544);
INSERT INTO public.acos VALUES (1196, 1194, NULL, NULL, 'ajaxlogs', 2545, 2546);
INSERT INTO public.acos VALUES (1197, 1194, NULL, NULL, 'tail', 2547, 2548);
INSERT INTO public.acos VALUES (1198, 1194, NULL, NULL, 'ajaxdisk', 2549, 2550);
INSERT INTO public.acos VALUES (1199, 1194, NULL, NULL, 'userAclCheck', 2551, 2552);
INSERT INTO public.acos VALUES (1200, 1194, NULL, NULL, 'getSessionInitServices', 2553, 2554);
INSERT INTO public.acos VALUES (1201, 1194, NULL, NULL, 'getSessionAppDefinedServices', 2555, 2556);
INSERT INTO public.acos VALUES (1202, 1194, NULL, NULL, 'getSessionAppDefinedDesktops', 2557, 2558);
INSERT INTO public.acos VALUES (1203, 1194, NULL, NULL, 'getSessionInitDesktops', 2559, 2560);
INSERT INTO public.acos VALUES (1205, 1194, NULL, NULL, 'getSessionValeditDesktops', 2563, 2564);
INSERT INTO public.acos VALUES (1206, 1194, NULL, NULL, 'getSessionDispDesktops', 2565, 2566);
INSERT INTO public.acos VALUES (1207, 1194, NULL, NULL, 'getSessionArchDesktops', 2567, 2568);
INSERT INTO public.acos VALUES (1208, 1194, NULL, NULL, 'getSessionAllDesktops', 2569, 2570);
INSERT INTO public.acos VALUES (1209, 1194, NULL, NULL, 'getEnv', 2571, 2572);
INSERT INTO public.acos VALUES (1210, 1194, NULL, NULL, 'getAddParams', 2573, 2574);
INSERT INTO public.acos VALUES (1211, 1194, NULL, NULL, 'ldebug', 2575, 2576);
INSERT INTO public.acos VALUES (1212, 1194, NULL, NULL, 'isAuthorized', 2577, 2578);
INSERT INTO public.acos VALUES (1213, 1, NULL, NULL, 'Consultations', 2580, 2625);
INSERT INTO public.acos VALUES (1214, 1213, NULL, NULL, 'index', 2581, 2582);
INSERT INTO public.acos VALUES (1250, 1236, NULL, NULL, 'getSessionArchDesktops', 2653, 2654);
INSERT INTO public.acos VALUES (1251, 1236, NULL, NULL, 'getSessionAllDesktops', 2655, 2656);
INSERT INTO public.acos VALUES (1253, 1236, NULL, NULL, 'getAddParams', 2659, 2660);
INSERT INTO public.acos VALUES (1254, 1236, NULL, NULL, 'ldebug', 2661, 2662);
INSERT INTO public.acos VALUES (1255, 1236, NULL, NULL, 'isAuthorized', 2663, 2664);
INSERT INTO public.acos VALUES (1267, 1, NULL, NULL, 'Contractants', 2666, 2705);
INSERT INTO public.acos VALUES (1269, 1267, NULL, NULL, 'add', 2669, 2670);
INSERT INTO public.acos VALUES (1270, 1267, NULL, NULL, 'edit', 2671, 2672);
INSERT INTO public.acos VALUES (1271, 1267, NULL, NULL, 'delete', 2673, 2674);
INSERT INTO public.acos VALUES (1272, 1267, NULL, NULL, 'getContractants', 2675, 2676);
INSERT INTO public.acos VALUES (1273, 1267, NULL, NULL, 'userAclCheck', 2677, 2678);
INSERT INTO public.acos VALUES (1274, 1267, NULL, NULL, 'getSessionInitServices', 2679, 2680);
INSERT INTO public.acos VALUES (1275, 1267, NULL, NULL, 'getSessionAppDefinedServices', 2681, 2682);
INSERT INTO public.acos VALUES (1276, 1267, NULL, NULL, 'getSessionAppDefinedDesktops', 2683, 2684);
INSERT INTO public.acos VALUES (1277, 1267, NULL, NULL, 'getSessionInitDesktops', 2685, 2686);
INSERT INTO public.acos VALUES (1278, 1267, NULL, NULL, 'getSessionValDesktops', 2687, 2688);
INSERT INTO public.acos VALUES (1279, 1267, NULL, NULL, 'getSessionValeditDesktops', 2689, 2690);
INSERT INTO public.acos VALUES (1280, 1267, NULL, NULL, 'getSessionDispDesktops', 2691, 2692);
INSERT INTO public.acos VALUES (1281, 1267, NULL, NULL, 'getSessionArchDesktops', 2693, 2694);
INSERT INTO public.acos VALUES (1282, 1267, NULL, NULL, 'getSessionAllDesktops', 2695, 2696);
INSERT INTO public.acos VALUES (1283, 1267, NULL, NULL, 'getEnv', 2697, 2698);
INSERT INTO public.acos VALUES (1284, 1267, NULL, NULL, 'getAddParams', 2699, 2700);
INSERT INTO public.acos VALUES (1285, 1267, NULL, NULL, 'ldebug', 2701, 2702);
INSERT INTO public.acos VALUES (1286, 1267, NULL, NULL, 'isAuthorized', 2703, 2704);
INSERT INTO public.acos VALUES (1287, 1, NULL, NULL, 'CourrierApi', 2706, 2747);
INSERT INTO public.acos VALUES (1289, 1287, NULL, NULL, 'find', 2709, 2710);
INSERT INTO public.acos VALUES (1290, 1287, NULL, NULL, 'add', 2711, 2712);
INSERT INTO public.acos VALUES (1291, 1287, NULL, NULL, 'delete', 2713, 2714);
INSERT INTO public.acos VALUES (1292, 1287, NULL, NULL, 'update', 2715, 2716);
INSERT INTO public.acos VALUES (1293, 1287, NULL, NULL, 'status', 2717, 2718);
INSERT INTO public.acos VALUES (1294, 1287, NULL, NULL, 'userAclCheck', 2719, 2720);
INSERT INTO public.acos VALUES (1295, 1287, NULL, NULL, 'getSessionInitServices', 2721, 2722);
INSERT INTO public.acos VALUES (1296, 1287, NULL, NULL, 'getSessionAppDefinedServices', 2723, 2724);
INSERT INTO public.acos VALUES (1297, 1287, NULL, NULL, 'getSessionAppDefinedDesktops', 2725, 2726);
INSERT INTO public.acos VALUES (1298, 1287, NULL, NULL, 'getSessionInitDesktops', 2727, 2728);
INSERT INTO public.acos VALUES (1299, 1287, NULL, NULL, 'getSessionValDesktops', 2729, 2730);
INSERT INTO public.acos VALUES (1300, 1287, NULL, NULL, 'getSessionValeditDesktops', 2731, 2732);
INSERT INTO public.acos VALUES (1301, 1287, NULL, NULL, 'getSessionDispDesktops', 2733, 2734);
INSERT INTO public.acos VALUES (1302, 1287, NULL, NULL, 'getSessionArchDesktops', 2735, 2736);
INSERT INTO public.acos VALUES (1303, 1287, NULL, NULL, 'getSessionAllDesktops', 2737, 2738);
INSERT INTO public.acos VALUES (1304, 1287, NULL, NULL, 'getEnv', 2739, 2740);
INSERT INTO public.acos VALUES (1305, 1287, NULL, NULL, 'getAddParams', 2741, 2742);
INSERT INTO public.acos VALUES (1306, 1287, NULL, NULL, 'ldebug', 2743, 2744);
INSERT INTO public.acos VALUES (1318, 1, NULL, NULL, 'Demarchessimplifiees', 2748, 2787);
INSERT INTO public.acos VALUES (1319, 1318, NULL, NULL, 'index', 2749, 2750);
INSERT INTO public.acos VALUES (1320, 1318, NULL, NULL, 'search', 2751, 2752);
INSERT INTO public.acos VALUES (1321, 1318, NULL, NULL, 'createCourrier', 2753, 2754);
INSERT INTO public.acos VALUES (1170, 130, NULL, NULL, 'isAuthorized', 339, 340);
INSERT INTO public.acos VALUES (1322, 1318, NULL, NULL, 'ajaxformvalid', 2755, 2756);
INSERT INTO public.acos VALUES (1323, 1318, NULL, NULL, 'get_string_between', 2757, 2758);
INSERT INTO public.acos VALUES (1324, 1318, NULL, NULL, 'userAclCheck', 2759, 2760);
INSERT INTO public.acos VALUES (1325, 1318, NULL, NULL, 'getSessionInitServices', 2761, 2762);
INSERT INTO public.acos VALUES (1326, 1318, NULL, NULL, 'getSessionAppDefinedServices', 2763, 2764);
INSERT INTO public.acos VALUES (1327, 1318, NULL, NULL, 'getSessionAppDefinedDesktops', 2765, 2766);
INSERT INTO public.acos VALUES (1328, 1318, NULL, NULL, 'getSessionInitDesktops', 2767, 2768);
INSERT INTO public.acos VALUES (1329, 1318, NULL, NULL, 'getSessionValDesktops', 2769, 2770);
INSERT INTO public.acos VALUES (1330, 1318, NULL, NULL, 'getSessionValeditDesktops', 2771, 2772);
INSERT INTO public.acos VALUES (1331, 1318, NULL, NULL, 'getSessionDispDesktops', 2773, 2774);
INSERT INTO public.acos VALUES (1332, 1318, NULL, NULL, 'getSessionArchDesktops', 2775, 2776);
INSERT INTO public.acos VALUES (1333, 1318, NULL, NULL, 'getSessionAllDesktops', 2777, 2778);
INSERT INTO public.acos VALUES (1334, 1318, NULL, NULL, 'getEnv', 2779, 2780);
INSERT INTO public.acos VALUES (1335, 1318, NULL, NULL, 'getAddParams', 2781, 2782);
INSERT INTO public.acos VALUES (1336, 1318, NULL, NULL, 'ldebug', 2783, 2784);
INSERT INTO public.acos VALUES (1337, 1318, NULL, NULL, 'isAuthorized', 2785, 2786);
INSERT INTO public.acos VALUES (1343, 987, NULL, NULL, 'isAuthorized', 2261, 2262);
INSERT INTO public.acos VALUES (1344, 1, NULL, NULL, 'Directmairies', 2788, 2825);
INSERT INTO public.acos VALUES (1345, 1344, NULL, NULL, 'index', 2789, 2790);
INSERT INTO public.acos VALUES (1346, 1344, NULL, NULL, 'search', 2791, 2792);
INSERT INTO public.acos VALUES (1347, 1344, NULL, NULL, 'createCourrier', 2793, 2794);
INSERT INTO public.acos VALUES (1349, 1344, NULL, NULL, 'userAclCheck', 2797, 2798);
INSERT INTO public.acos VALUES (1350, 1344, NULL, NULL, 'getSessionInitServices', 2799, 2800);
INSERT INTO public.acos VALUES (1351, 1344, NULL, NULL, 'getSessionAppDefinedServices', 2801, 2802);
INSERT INTO public.acos VALUES (1352, 1344, NULL, NULL, 'getSessionAppDefinedDesktops', 2803, 2804);
INSERT INTO public.acos VALUES (1353, 1344, NULL, NULL, 'getSessionInitDesktops', 2805, 2806);
INSERT INTO public.acos VALUES (1354, 1344, NULL, NULL, 'getSessionValDesktops', 2807, 2808);
INSERT INTO public.acos VALUES (1355, 1344, NULL, NULL, 'getSessionValeditDesktops', 2809, 2810);
INSERT INTO public.acos VALUES (1356, 1344, NULL, NULL, 'getSessionDispDesktops', 2811, 2812);
INSERT INTO public.acos VALUES (1357, 1344, NULL, NULL, 'getSessionArchDesktops', 2813, 2814);
INSERT INTO public.acos VALUES (1358, 1344, NULL, NULL, 'getSessionAllDesktops', 2815, 2816);
INSERT INTO public.acos VALUES (1360, 1344, NULL, NULL, 'getAddParams', 2819, 2820);
INSERT INTO public.acos VALUES (1361, 1344, NULL, NULL, 'ldebug', 2821, 2822);
INSERT INTO public.acos VALUES (1362, 1344, NULL, NULL, 'isAuthorized', 2823, 2824);
INSERT INTO public.acos VALUES (1380, 1, NULL, NULL, 'Events', 2826, 2865);
INSERT INTO public.acos VALUES (1381, 1380, NULL, NULL, 'index', 2827, 2828);
INSERT INTO public.acos VALUES (1382, 1380, NULL, NULL, 'add', 2829, 2830);
INSERT INTO public.acos VALUES (1383, 1380, NULL, NULL, 'edit', 2831, 2832);
INSERT INTO public.acos VALUES (1384, 1380, NULL, NULL, 'delete', 2833, 2834);
INSERT INTO public.acos VALUES (1385, 1380, NULL, NULL, 'getEvents', 2835, 2836);
INSERT INTO public.acos VALUES (1386, 1380, NULL, NULL, 'userAclCheck', 2837, 2838);
INSERT INTO public.acos VALUES (1387, 1380, NULL, NULL, 'getSessionInitServices', 2839, 2840);
INSERT INTO public.acos VALUES (1388, 1380, NULL, NULL, 'getSessionAppDefinedServices', 2841, 2842);
INSERT INTO public.acos VALUES (1389, 1380, NULL, NULL, 'getSessionAppDefinedDesktops', 2843, 2844);
INSERT INTO public.acos VALUES (1390, 1380, NULL, NULL, 'getSessionInitDesktops', 2845, 2846);
INSERT INTO public.acos VALUES (1391, 1380, NULL, NULL, 'getSessionValDesktops', 2847, 2848);
INSERT INTO public.acos VALUES (1392, 1380, NULL, NULL, 'getSessionValeditDesktops', 2849, 2850);
INSERT INTO public.acos VALUES (1393, 1380, NULL, NULL, 'getSessionDispDesktops', 2851, 2852);
INSERT INTO public.acos VALUES (1394, 1380, NULL, NULL, 'getSessionArchDesktops', 2853, 2854);
INSERT INTO public.acos VALUES (1395, 1380, NULL, NULL, 'getSessionAllDesktops', 2855, 2856);
INSERT INTO public.acos VALUES (1396, 1380, NULL, NULL, 'getEnv', 2857, 2858);
INSERT INTO public.acos VALUES (1397, 1380, NULL, NULL, 'getAddParams', 2859, 2860);
INSERT INTO public.acos VALUES (1398, 1380, NULL, NULL, 'ldebug', 2861, 2862);
INSERT INTO public.acos VALUES (1399, 1380, NULL, NULL, 'isAuthorized', 2863, 2864);
INSERT INTO public.acos VALUES (1400, 1, NULL, NULL, 'Fonctions', 2866, 2905);
INSERT INTO public.acos VALUES (1401, 1400, NULL, NULL, 'index', 2867, 2868);
INSERT INTO public.acos VALUES (1402, 1400, NULL, NULL, 'add', 2869, 2870);
INSERT INTO public.acos VALUES (1403, 1400, NULL, NULL, 'edit', 2871, 2872);
INSERT INTO public.acos VALUES (1404, 1400, NULL, NULL, 'delete', 2873, 2874);
INSERT INTO public.acos VALUES (1405, 1400, NULL, NULL, 'getFonctions', 2875, 2876);
INSERT INTO public.acos VALUES (1406, 1400, NULL, NULL, 'userAclCheck', 2877, 2878);
INSERT INTO public.acos VALUES (1407, 1400, NULL, NULL, 'getSessionInitServices', 2879, 2880);
INSERT INTO public.acos VALUES (1408, 1400, NULL, NULL, 'getSessionAppDefinedServices', 2881, 2882);
INSERT INTO public.acos VALUES (1409, 1400, NULL, NULL, 'getSessionAppDefinedDesktops', 2883, 2884);
INSERT INTO public.acos VALUES (1410, 1400, NULL, NULL, 'getSessionInitDesktops', 2885, 2886);
INSERT INTO public.acos VALUES (1411, 1400, NULL, NULL, 'getSessionValDesktops', 2887, 2888);
INSERT INTO public.acos VALUES (1412, 1400, NULL, NULL, 'getSessionValeditDesktops', 2889, 2890);
INSERT INTO public.acos VALUES (1413, 1400, NULL, NULL, 'getSessionDispDesktops', 2891, 2892);
INSERT INTO public.acos VALUES (1414, 1400, NULL, NULL, 'getSessionArchDesktops', 2893, 2894);
INSERT INTO public.acos VALUES (1415, 1400, NULL, NULL, 'getSessionAllDesktops', 2895, 2896);
INSERT INTO public.acos VALUES (1416, 1400, NULL, NULL, 'getEnv', 2897, 2898);
INSERT INTO public.acos VALUES (1417, 1400, NULL, NULL, 'getAddParams', 2899, 2900);
INSERT INTO public.acos VALUES (1418, 1400, NULL, NULL, 'ldebug', 2901, 2902);
INSERT INTO public.acos VALUES (1419, 1400, NULL, NULL, 'isAuthorized', 2903, 2904);
INSERT INTO public.acos VALUES (1420, 1, NULL, NULL, 'Gabaritsdocuments', 2906, 2947);
INSERT INTO public.acos VALUES (1421, 1420, NULL, NULL, 'index', 2907, 2908);
INSERT INTO public.acos VALUES (1422, 1420, NULL, NULL, 'add', 2909, 2910);
INSERT INTO public.acos VALUES (1423, 1420, NULL, NULL, 'edit', 2911, 2912);
INSERT INTO public.acos VALUES (1424, 1420, NULL, NULL, 'delete', 2913, 2914);
INSERT INTO public.acos VALUES (1425, 1420, NULL, NULL, 'getGabaritsdocuments', 2915, 2916);
INSERT INTO public.acos VALUES (1426, 1420, NULL, NULL, 'download', 2917, 2918);
INSERT INTO public.acos VALUES (1427, 1420, NULL, NULL, 'userAclCheck', 2919, 2920);
INSERT INTO public.acos VALUES (1428, 1420, NULL, NULL, 'getSessionInitServices', 2921, 2922);
INSERT INTO public.acos VALUES (1429, 1420, NULL, NULL, 'getSessionAppDefinedServices', 2923, 2924);
INSERT INTO public.acos VALUES (1430, 1420, NULL, NULL, 'getSessionAppDefinedDesktops', 2925, 2926);
INSERT INTO public.acos VALUES (1431, 1420, NULL, NULL, 'getSessionInitDesktops', 2927, 2928);
INSERT INTO public.acos VALUES (1432, 1420, NULL, NULL, 'getSessionValDesktops', 2929, 2930);
INSERT INTO public.acos VALUES (1433, 1420, NULL, NULL, 'getSessionValeditDesktops', 2931, 2932);
INSERT INTO public.acos VALUES (1434, 1420, NULL, NULL, 'getSessionDispDesktops', 2933, 2934);
INSERT INTO public.acos VALUES (1435, 1420, NULL, NULL, 'getSessionArchDesktops', 2935, 2936);
INSERT INTO public.acos VALUES (1436, 1420, NULL, NULL, 'getSessionAllDesktops', 2937, 2938);
INSERT INTO public.acos VALUES (1438, 1420, NULL, NULL, 'getAddParams', 2941, 2942);
INSERT INTO public.acos VALUES (1439, 1420, NULL, NULL, 'ldebug', 2943, 2944);
INSERT INTO public.acos VALUES (1440, 1420, NULL, NULL, 'isAuthorized', 2945, 2946);
INSERT INTO public.acos VALUES (1441, 1, NULL, NULL, 'Intituleagentbyoperations', 2948, 2987);
INSERT INTO public.acos VALUES (1442, 1441, NULL, NULL, 'index', 2949, 2950);
INSERT INTO public.acos VALUES (1443, 1441, NULL, NULL, 'add', 2951, 2952);
INSERT INTO public.acos VALUES (1444, 1441, NULL, NULL, 'edit', 2953, 2954);
INSERT INTO public.acos VALUES (1445, 1441, NULL, NULL, 'delete', 2955, 2956);
INSERT INTO public.acos VALUES (1446, 1441, NULL, NULL, 'getIntituleagentbyoperations', 2957, 2958);
INSERT INTO public.acos VALUES (1447, 1441, NULL, NULL, 'userAclCheck', 2959, 2960);
INSERT INTO public.acos VALUES (1448, 1441, NULL, NULL, 'getSessionInitServices', 2961, 2962);
INSERT INTO public.acos VALUES (1449, 1441, NULL, NULL, 'getSessionAppDefinedServices', 2963, 2964);
INSERT INTO public.acos VALUES (1450, 1441, NULL, NULL, 'getSessionAppDefinedDesktops', 2965, 2966);
INSERT INTO public.acos VALUES (1451, 1441, NULL, NULL, 'getSessionInitDesktops', 2967, 2968);
INSERT INTO public.acos VALUES (1452, 1441, NULL, NULL, 'getSessionValDesktops', 2969, 2970);
INSERT INTO public.acos VALUES (1453, 1441, NULL, NULL, 'getSessionValeditDesktops', 2971, 2972);
INSERT INTO public.acos VALUES (1456, 1441, NULL, NULL, 'getSessionAllDesktops', 2977, 2978);
INSERT INTO public.acos VALUES (1458, 1441, NULL, NULL, 'getAddParams', 2981, 2982);
INSERT INTO public.acos VALUES (1459, 1441, NULL, NULL, 'ldebug', 2983, 2984);
INSERT INTO public.acos VALUES (1460, 1441, NULL, NULL, 'isAuthorized', 2985, 2986);
INSERT INTO public.acos VALUES (1461, 1, NULL, NULL, 'Intitulesagents', 2988, 3027);
INSERT INTO public.acos VALUES (1462, 1461, NULL, NULL, 'index', 2989, 2990);
INSERT INTO public.acos VALUES (1463, 1461, NULL, NULL, 'add', 2991, 2992);
INSERT INTO public.acos VALUES (1464, 1461, NULL, NULL, 'edit', 2993, 2994);
INSERT INTO public.acos VALUES (1465, 1461, NULL, NULL, 'delete', 2995, 2996);
INSERT INTO public.acos VALUES (1466, 1461, NULL, NULL, 'getIntitulesagents', 2997, 2998);
INSERT INTO public.acos VALUES (1467, 1461, NULL, NULL, 'userAclCheck', 2999, 3000);
INSERT INTO public.acos VALUES (1468, 1461, NULL, NULL, 'getSessionInitServices', 3001, 3002);
INSERT INTO public.acos VALUES (1469, 1461, NULL, NULL, 'getSessionAppDefinedServices', 3003, 3004);
INSERT INTO public.acos VALUES (1470, 1461, NULL, NULL, 'getSessionAppDefinedDesktops', 3005, 3006);
INSERT INTO public.acos VALUES (1471, 1461, NULL, NULL, 'getSessionInitDesktops', 3007, 3008);
INSERT INTO public.acos VALUES (1472, 1461, NULL, NULL, 'getSessionValDesktops', 3009, 3010);
INSERT INTO public.acos VALUES (1473, 1461, NULL, NULL, 'getSessionValeditDesktops', 3011, 3012);
INSERT INTO public.acos VALUES (1474, 1461, NULL, NULL, 'getSessionDispDesktops', 3013, 3014);
INSERT INTO public.acos VALUES (1475, 1461, NULL, NULL, 'getSessionArchDesktops', 3015, 3016);
INSERT INTO public.acos VALUES (1476, 1461, NULL, NULL, 'getSessionAllDesktops', 3017, 3018);
INSERT INTO public.acos VALUES (1477, 1461, NULL, NULL, 'getEnv', 3019, 3020);
INSERT INTO public.acos VALUES (1479, 1461, NULL, NULL, 'ldebug', 3023, 3024);
INSERT INTO public.acos VALUES (1480, 1461, NULL, NULL, 'isAuthorized', 3025, 3026);
INSERT INTO public.acos VALUES (1485, 1484, NULL, NULL, 'index', 3029, 3030);
INSERT INTO public.acos VALUES (1486, 1484, NULL, NULL, 'add', 3031, 3032);
INSERT INTO public.acos VALUES (1487, 1484, NULL, NULL, 'edit', 3033, 3034);
INSERT INTO public.acos VALUES (1488, 1484, NULL, NULL, 'delete', 3035, 3036);
INSERT INTO public.acos VALUES (1489, 1484, NULL, NULL, 'getMarches', 3037, 3038);
INSERT INTO public.acos VALUES (1490, 1484, NULL, NULL, 'getOs', 3039, 3040);
INSERT INTO public.acos VALUES (1491, 1484, NULL, NULL, 'ajaxformvalid', 3041, 3042);
INSERT INTO public.acos VALUES (1492, 1484, NULL, NULL, 'export', 3043, 3044);
INSERT INTO public.acos VALUES (1493, 1484, NULL, NULL, 'userAclCheck', 3045, 3046);
INSERT INTO public.acos VALUES (1494, 1484, NULL, NULL, 'getSessionInitServices', 3047, 3048);
INSERT INTO public.acos VALUES (1495, 1484, NULL, NULL, 'getSessionAppDefinedServices', 3049, 3050);
INSERT INTO public.acos VALUES (1496, 1484, NULL, NULL, 'getSessionAppDefinedDesktops', 3051, 3052);
INSERT INTO public.acos VALUES (1497, 1484, NULL, NULL, 'getSessionInitDesktops', 3053, 3054);
INSERT INTO public.acos VALUES (1498, 1484, NULL, NULL, 'getSessionValDesktops', 3055, 3056);
INSERT INTO public.acos VALUES (1499, 1484, NULL, NULL, 'getSessionValeditDesktops', 3057, 3058);
INSERT INTO public.acos VALUES (1500, 1484, NULL, NULL, 'getSessionDispDesktops', 3059, 3060);
INSERT INTO public.acos VALUES (1501, 1484, NULL, NULL, 'getSessionArchDesktops', 3061, 3062);
INSERT INTO public.acos VALUES (1502, 1484, NULL, NULL, 'getSessionAllDesktops', 3063, 3064);
INSERT INTO public.acos VALUES (1503, 1484, NULL, NULL, 'getEnv', 3065, 3066);
INSERT INTO public.acos VALUES (1504, 1484, NULL, NULL, 'getAddParams', 3067, 3068);
INSERT INTO public.acos VALUES (1505, 1484, NULL, NULL, 'ldebug', 3069, 3070);
INSERT INTO public.acos VALUES (1506, 1484, NULL, NULL, 'isAuthorized', 3071, 3072);
INSERT INTO public.acos VALUES (1508, 1, NULL, NULL, 'Operations', 3074, 3117);
INSERT INTO public.acos VALUES (1509, 1508, NULL, NULL, 'index', 3075, 3076);
INSERT INTO public.acos VALUES (1510, 1508, NULL, NULL, 'add', 3077, 3078);
INSERT INTO public.acos VALUES (1511, 1508, NULL, NULL, 'edit', 3079, 3080);
INSERT INTO public.acos VALUES (1512, 1508, NULL, NULL, 'delete', 3081, 3082);
INSERT INTO public.acos VALUES (1513, 1508, NULL, NULL, 'getOperations', 3083, 3084);
INSERT INTO public.acos VALUES (1514, 1508, NULL, NULL, 'getSousoperations', 3085, 3086);
INSERT INTO public.acos VALUES (1515, 1508, NULL, NULL, 'export', 3087, 3088);
INSERT INTO public.acos VALUES (1516, 1508, NULL, NULL, 'userAclCheck', 3089, 3090);
INSERT INTO public.acos VALUES (1517, 1508, NULL, NULL, 'getSessionInitServices', 3091, 3092);
INSERT INTO public.acos VALUES (1518, 1508, NULL, NULL, 'getSessionAppDefinedServices', 3093, 3094);
INSERT INTO public.acos VALUES (1519, 1508, NULL, NULL, 'getSessionAppDefinedDesktops', 3095, 3096);
INSERT INTO public.acos VALUES (1520, 1508, NULL, NULL, 'getSessionInitDesktops', 3097, 3098);
INSERT INTO public.acos VALUES (1521, 1508, NULL, NULL, 'getSessionValDesktops', 3099, 3100);
INSERT INTO public.acos VALUES (1522, 1508, NULL, NULL, 'getSessionValeditDesktops', 3101, 3102);
INSERT INTO public.acos VALUES (1523, 1508, NULL, NULL, 'getSessionDispDesktops', 3103, 3104);
INSERT INTO public.acos VALUES (1524, 1508, NULL, NULL, 'getSessionArchDesktops', 3105, 3106);
INSERT INTO public.acos VALUES (1525, 1508, NULL, NULL, 'getSessionAllDesktops', 3107, 3108);
INSERT INTO public.acos VALUES (1527, 1508, NULL, NULL, 'getAddParams', 3111, 3112);
INSERT INTO public.acos VALUES (1528, 1508, NULL, NULL, 'ldebug', 3113, 3114);
INSERT INTO public.acos VALUES (1529, 1508, NULL, NULL, 'isAuthorized', 3115, 3116);
INSERT INTO public.acos VALUES (1530, 1, NULL, NULL, 'Ordresservices', 3118, 3165);
INSERT INTO public.acos VALUES (1531, 1530, NULL, NULL, 'index', 3119, 3120);
INSERT INTO public.acos VALUES (1532, 1530, NULL, NULL, 'add', 3121, 3122);
INSERT INTO public.acos VALUES (1533, 1530, NULL, NULL, 'edit', 3123, 3124);
INSERT INTO public.acos VALUES (1534, 1530, NULL, NULL, 'delete', 3125, 3126);
INSERT INTO public.acos VALUES (1535, 1530, NULL, NULL, 'getOrdresservices', 3127, 3128);
INSERT INTO public.acos VALUES (1536, 1530, NULL, NULL, 'ajaxformdata', 3129, 3130);
INSERT INTO public.acos VALUES (1537, 1530, NULL, NULL, 'export', 3131, 3132);
INSERT INTO public.acos VALUES (1538, 1530, NULL, NULL, 'getExistance', 3133, 3134);
INSERT INTO public.acos VALUES (1539, 1530, NULL, NULL, 'view', 3135, 3136);
INSERT INTO public.acos VALUES (1540, 1530, NULL, NULL, 'userAclCheck', 3137, 3138);
INSERT INTO public.acos VALUES (1541, 1530, NULL, NULL, 'getSessionInitServices', 3139, 3140);
INSERT INTO public.acos VALUES (1542, 1530, NULL, NULL, 'getSessionAppDefinedServices', 3141, 3142);
INSERT INTO public.acos VALUES (1543, 1530, NULL, NULL, 'getSessionAppDefinedDesktops', 3143, 3144);
INSERT INTO public.acos VALUES (1544, 1530, NULL, NULL, 'getSessionInitDesktops', 3145, 3146);
INSERT INTO public.acos VALUES (1545, 1530, NULL, NULL, 'getSessionValDesktops', 3147, 3148);
INSERT INTO public.acos VALUES (1546, 1530, NULL, NULL, 'getSessionValeditDesktops', 3149, 3150);
INSERT INTO public.acos VALUES (1547, 1530, NULL, NULL, 'getSessionDispDesktops', 3151, 3152);
INSERT INTO public.acos VALUES (1548, 1530, NULL, NULL, 'getSessionArchDesktops', 3153, 3154);
INSERT INTO public.acos VALUES (1549, 1530, NULL, NULL, 'getSessionAllDesktops', 3155, 3156);
INSERT INTO public.acos VALUES (1550, 1530, NULL, NULL, 'getEnv', 3157, 3158);
INSERT INTO public.acos VALUES (1551, 1530, NULL, NULL, 'getAddParams', 3159, 3160);
INSERT INTO public.acos VALUES (1552, 1530, NULL, NULL, 'ldebug', 3161, 3162);
INSERT INTO public.acos VALUES (1553, 1530, NULL, NULL, 'isAuthorized', 3163, 3164);
INSERT INTO public.acos VALUES (1554, 1045, NULL, NULL, 'import', 2341, 2342);
INSERT INTO public.acos VALUES (1555, 1045, NULL, NULL, 'import_organismes', 2343, 2344);
INSERT INTO public.acos VALUES (1560, 1045, NULL, NULL, 'isAuthorized', 2353, 2354);
INSERT INTO public.acos VALUES (1561, 1, NULL, NULL, 'Originesflux', 3166, 3207);
INSERT INTO public.acos VALUES (1562, 1561, NULL, NULL, 'index', 3167, 3168);
INSERT INTO public.acos VALUES (1563, 1561, NULL, NULL, 'add', 3169, 3170);
INSERT INTO public.acos VALUES (1564, 1561, NULL, NULL, 'edit', 3171, 3172);
INSERT INTO public.acos VALUES (1565, 1561, NULL, NULL, 'delete', 3173, 3174);
INSERT INTO public.acos VALUES (1566, 1561, NULL, NULL, 'getOrigineflux', 3175, 3176);
INSERT INTO public.acos VALUES (1567, 1561, NULL, NULL, 'ajaxformvalid', 3177, 3178);
INSERT INTO public.acos VALUES (1568, 1561, NULL, NULL, 'userAclCheck', 3179, 3180);
INSERT INTO public.acos VALUES (1569, 1561, NULL, NULL, 'getSessionInitServices', 3181, 3182);
INSERT INTO public.acos VALUES (1570, 1561, NULL, NULL, 'getSessionAppDefinedServices', 3183, 3184);
INSERT INTO public.acos VALUES (1571, 1561, NULL, NULL, 'getSessionAppDefinedDesktops', 3185, 3186);
INSERT INTO public.acos VALUES (1572, 1561, NULL, NULL, 'getSessionInitDesktops', 3187, 3188);
INSERT INTO public.acos VALUES (1573, 1561, NULL, NULL, 'getSessionValDesktops', 3189, 3190);
INSERT INTO public.acos VALUES (1574, 1561, NULL, NULL, 'getSessionValeditDesktops', 3191, 3192);
INSERT INTO public.acos VALUES (1575, 1561, NULL, NULL, 'getSessionDispDesktops', 3193, 3194);
INSERT INTO public.acos VALUES (1576, 1561, NULL, NULL, 'getSessionArchDesktops', 3195, 3196);
INSERT INTO public.acos VALUES (1577, 1561, NULL, NULL, 'getSessionAllDesktops', 3197, 3198);
INSERT INTO public.acos VALUES (1579, 1561, NULL, NULL, 'getAddParams', 3201, 3202);
INSERT INTO public.acos VALUES (1580, 1561, NULL, NULL, 'ldebug', 3203, 3204);
INSERT INTO public.acos VALUES (1581, 1561, NULL, NULL, 'isAuthorized', 3205, 3206);
INSERT INTO public.acos VALUES (1582, 1, NULL, NULL, 'Outils', 3208, 3259);
INSERT INTO public.acos VALUES (1583, 1582, NULL, NULL, 'index', 3209, 3210);
INSERT INTO public.acos VALUES (1584, 1582, NULL, NULL, 'searchParapheur', 3211, 3212);
INSERT INTO public.acos VALUES (1585, 1582, NULL, NULL, 'formulaireParapheur', 3213, 3214);
INSERT INTO public.acos VALUES (1586, 1582, NULL, NULL, 'result', 3215, 3216);
INSERT INTO public.acos VALUES (1587, 1582, NULL, NULL, 'inject', 3217, 3218);
INSERT INTO public.acos VALUES (1588, 1582, NULL, NULL, 'fluxNonClos', 3219, 3220);
INSERT INTO public.acos VALUES (1589, 1582, NULL, NULL, 'searchFluxnonclos', 3221, 3222);
INSERT INTO public.acos VALUES (1590, 1582, NULL, NULL, 'passageenlulot', 3223, 3224);
INSERT INTO public.acos VALUES (1591, 1582, NULL, NULL, 'searchFluxEncoursBannetteAInserer', 3225, 3226);
INSERT INTO public.acos VALUES (1592, 1582, NULL, NULL, 'fluxEncoursBannetteAInserer', 3227, 3228);
INSERT INTO public.acos VALUES (1593, 1582, NULL, NULL, 'detachenlotbannetteainserer', 3229, 3230);
INSERT INTO public.acos VALUES (1594, 1582, NULL, NULL, 'userAclCheck', 3231, 3232);
INSERT INTO public.acos VALUES (1595, 1582, NULL, NULL, 'getSessionInitServices', 3233, 3234);
INSERT INTO public.acos VALUES (1596, 1582, NULL, NULL, 'getSessionAppDefinedServices', 3235, 3236);
INSERT INTO public.acos VALUES (1597, 1582, NULL, NULL, 'getSessionAppDefinedDesktops', 3237, 3238);
INSERT INTO public.acos VALUES (1598, 1582, NULL, NULL, 'getSessionInitDesktops', 3239, 3240);
INSERT INTO public.acos VALUES (1599, 1582, NULL, NULL, 'getSessionValDesktops', 3241, 3242);
INSERT INTO public.acos VALUES (1600, 1582, NULL, NULL, 'getSessionValeditDesktops', 3243, 3244);
INSERT INTO public.acos VALUES (1601, 1582, NULL, NULL, 'getSessionDispDesktops', 3245, 3246);
INSERT INTO public.acos VALUES (1602, 1582, NULL, NULL, 'getSessionArchDesktops', 3247, 3248);
INSERT INTO public.acos VALUES (1603, 1582, NULL, NULL, 'getSessionAllDesktops', 3249, 3250);
INSERT INTO public.acos VALUES (1604, 1582, NULL, NULL, 'getEnv', 3251, 3252);
INSERT INTO public.acos VALUES (1605, 1582, NULL, NULL, 'getAddParams', 3253, 3254);
INSERT INTO public.acos VALUES (1606, 1582, NULL, NULL, 'ldebug', 3255, 3256);
INSERT INTO public.acos VALUES (1607, 1582, NULL, NULL, 'isAuthorized', 3257, 3258);
INSERT INTO public.acos VALUES (1608, 1, NULL, NULL, 'Plisconsultatifs', 3260, 3303);
INSERT INTO public.acos VALUES (1609, 1608, NULL, NULL, 'index', 3261, 3262);
INSERT INTO public.acos VALUES (1610, 1608, NULL, NULL, 'add', 3263, 3264);
INSERT INTO public.acos VALUES (1611, 1608, NULL, NULL, 'edit', 3265, 3266);
INSERT INTO public.acos VALUES (1612, 1608, NULL, NULL, 'delete', 3267, 3268);
INSERT INTO public.acos VALUES (1613, 1608, NULL, NULL, 'getPlisconsultatifs', 3269, 3270);
INSERT INTO public.acos VALUES (1614, 1608, NULL, NULL, 'ajaxformdata', 3271, 3272);
INSERT INTO public.acos VALUES (1615, 1608, NULL, NULL, 'export', 3273, 3274);
INSERT INTO public.acos VALUES (1616, 1608, NULL, NULL, 'userAclCheck', 3275, 3276);
INSERT INTO public.acos VALUES (1617, 1608, NULL, NULL, 'getSessionInitServices', 3277, 3278);
INSERT INTO public.acos VALUES (1618, 1608, NULL, NULL, 'getSessionAppDefinedServices', 3279, 3280);
INSERT INTO public.acos VALUES (1619, 1608, NULL, NULL, 'getSessionAppDefinedDesktops', 3281, 3282);
INSERT INTO public.acos VALUES (1620, 1608, NULL, NULL, 'getSessionInitDesktops', 3283, 3284);
INSERT INTO public.acos VALUES (1621, 1608, NULL, NULL, 'getSessionValDesktops', 3285, 3286);
INSERT INTO public.acos VALUES (1622, 1608, NULL, NULL, 'getSessionValeditDesktops', 3287, 3288);
INSERT INTO public.acos VALUES (1624, 1608, NULL, NULL, 'getSessionArchDesktops', 3291, 3292);
INSERT INTO public.acos VALUES (1625, 1608, NULL, NULL, 'getSessionAllDesktops', 3293, 3294);
INSERT INTO public.acos VALUES (1626, 1608, NULL, NULL, 'getEnv', 3295, 3296);
INSERT INTO public.acos VALUES (1627, 1608, NULL, NULL, 'getAddParams', 3297, 3298);
INSERT INTO public.acos VALUES (1628, 1608, NULL, NULL, 'ldebug', 3299, 3300);
INSERT INTO public.acos VALUES (1629, 1608, NULL, NULL, 'isAuthorized', 3301, 3302);
INSERT INTO public.acos VALUES (1638, 1, NULL, NULL, 'Rgpds', 3304, 3337);
INSERT INTO public.acos VALUES (1639, 1638, NULL, NULL, 'index', 3305, 3306);
INSERT INTO public.acos VALUES (1640, 1638, NULL, NULL, 'policy', 3307, 3308);
INSERT INTO public.acos VALUES (1641, 1638, NULL, NULL, 'userAclCheck', 3309, 3310);
INSERT INTO public.acos VALUES (1642, 1638, NULL, NULL, 'getSessionInitServices', 3311, 3312);
INSERT INTO public.acos VALUES (1643, 1638, NULL, NULL, 'getSessionAppDefinedServices', 3313, 3314);
INSERT INTO public.acos VALUES (1644, 1638, NULL, NULL, 'getSessionAppDefinedDesktops', 3315, 3316);
INSERT INTO public.acos VALUES (1645, 1638, NULL, NULL, 'getSessionInitDesktops', 3317, 3318);
INSERT INTO public.acos VALUES (1646, 1638, NULL, NULL, 'getSessionValDesktops', 3319, 3320);
INSERT INTO public.acos VALUES (1647, 1638, NULL, NULL, 'getSessionValeditDesktops', 3321, 3322);
INSERT INTO public.acos VALUES (1648, 1638, NULL, NULL, 'getSessionDispDesktops', 3323, 3324);
INSERT INTO public.acos VALUES (1649, 1638, NULL, NULL, 'getSessionArchDesktops', 3325, 3326);
INSERT INTO public.acos VALUES (1650, 1638, NULL, NULL, 'getSessionAllDesktops', 3327, 3328);
INSERT INTO public.acos VALUES (1652, 1638, NULL, NULL, 'getAddParams', 3331, 3332);
INSERT INTO public.acos VALUES (1653, 1638, NULL, NULL, 'ldebug', 3333, 3334);
INSERT INTO public.acos VALUES (1654, 1638, NULL, NULL, 'isAuthorized', 3335, 3336);
INSERT INTO public.acos VALUES (1655, 1006, NULL, NULL, 'isAuthorized', 2299, 2300);
INSERT INTO public.acos VALUES (1664, 1, NULL, NULL, 'Sms', 3338, 3369);
INSERT INTO public.acos VALUES (1665, 1664, NULL, NULL, 'sendSms', 3339, 3340);
INSERT INTO public.acos VALUES (1666, 1664, NULL, NULL, 'userAclCheck', 3341, 3342);
INSERT INTO public.acos VALUES (1557, 1045, NULL, NULL, 'get_contact', 2347, 2348);
INSERT INTO public.acos VALUES (1667, 1664, NULL, NULL, 'getSessionInitServices', 3343, 3344);
INSERT INTO public.acos VALUES (1668, 1664, NULL, NULL, 'getSessionAppDefinedServices', 3345, 3346);
INSERT INTO public.acos VALUES (1669, 1664, NULL, NULL, 'getSessionAppDefinedDesktops', 3347, 3348);
INSERT INTO public.acos VALUES (1670, 1664, NULL, NULL, 'getSessionInitDesktops', 3349, 3350);
INSERT INTO public.acos VALUES (1671, 1664, NULL, NULL, 'getSessionValDesktops', 3351, 3352);
INSERT INTO public.acos VALUES (1672, 1664, NULL, NULL, 'getSessionValeditDesktops', 3353, 3354);
INSERT INTO public.acos VALUES (1673, 1664, NULL, NULL, 'getSessionDispDesktops', 3355, 3356);
INSERT INTO public.acos VALUES (1674, 1664, NULL, NULL, 'getSessionArchDesktops', 3357, 3358);
INSERT INTO public.acos VALUES (1675, 1664, NULL, NULL, 'getSessionAllDesktops', 3359, 3360);
INSERT INTO public.acos VALUES (1676, 1664, NULL, NULL, 'getEnv', 3361, 3362);
INSERT INTO public.acos VALUES (1677, 1664, NULL, NULL, 'getAddParams', 3363, 3364);
INSERT INTO public.acos VALUES (1678, 1664, NULL, NULL, 'ldebug', 3365, 3366);
INSERT INTO public.acos VALUES (1679, 1664, NULL, NULL, 'isAuthorized', 3367, 3368);
INSERT INTO public.acos VALUES (1680, 1, NULL, NULL, 'Sousactivites', 3370, 3405);
INSERT INTO public.acos VALUES (1681, 1680, NULL, NULL, 'add', 3371, 3372);
INSERT INTO public.acos VALUES (1682, 1680, NULL, NULL, 'edit', 3373, 3374);
INSERT INTO public.acos VALUES (1683, 1680, NULL, NULL, 'delete', 3375, 3376);
INSERT INTO public.acos VALUES (1684, 1680, NULL, NULL, 'userAclCheck', 3377, 3378);
INSERT INTO public.acos VALUES (1685, 1680, NULL, NULL, 'getSessionInitServices', 3379, 3380);
INSERT INTO public.acos VALUES (1686, 1680, NULL, NULL, 'getSessionAppDefinedServices', 3381, 3382);
INSERT INTO public.acos VALUES (1687, 1680, NULL, NULL, 'getSessionAppDefinedDesktops', 3383, 3384);
INSERT INTO public.acos VALUES (1688, 1680, NULL, NULL, 'getSessionInitDesktops', 3385, 3386);
INSERT INTO public.acos VALUES (1689, 1680, NULL, NULL, 'getSessionValDesktops', 3387, 3388);
INSERT INTO public.acos VALUES (1690, 1680, NULL, NULL, 'getSessionValeditDesktops', 3389, 3390);
INSERT INTO public.acos VALUES (1691, 1680, NULL, NULL, 'getSessionDispDesktops', 3391, 3392);
INSERT INTO public.acos VALUES (1692, 1680, NULL, NULL, 'getSessionArchDesktops', 3393, 3394);
INSERT INTO public.acos VALUES (1693, 1680, NULL, NULL, 'getSessionAllDesktops', 3395, 3396);
INSERT INTO public.acos VALUES (1694, 1680, NULL, NULL, 'getEnv', 3397, 3398);
INSERT INTO public.acos VALUES (1695, 1680, NULL, NULL, 'getAddParams', 3399, 3400);
INSERT INTO public.acos VALUES (1696, 1680, NULL, NULL, 'ldebug', 3401, 3402);
INSERT INTO public.acos VALUES (1697, 1680, NULL, NULL, 'isAuthorized', 3403, 3404);
INSERT INTO public.acos VALUES (1698, 1, NULL, NULL, 'Sousoperations', 3406, 3445);
INSERT INTO public.acos VALUES (1699, 1698, NULL, NULL, 'add', 3407, 3408);
INSERT INTO public.acos VALUES (1700, 1698, NULL, NULL, 'edit', 3409, 3410);
INSERT INTO public.acos VALUES (1701, 1698, NULL, NULL, 'delete', 3411, 3412);
INSERT INTO public.acos VALUES (1702, 1698, NULL, NULL, 'addUsers', 3413, 3414);
INSERT INTO public.acos VALUES (1703, 1698, NULL, NULL, 'deleteAssociation', 3415, 3416);
INSERT INTO public.acos VALUES (1704, 1698, NULL, NULL, 'userAclCheck', 3417, 3418);
INSERT INTO public.acos VALUES (1705, 1698, NULL, NULL, 'getSessionInitServices', 3419, 3420);
INSERT INTO public.acos VALUES (1706, 1698, NULL, NULL, 'getSessionAppDefinedServices', 3421, 3422);
INSERT INTO public.acos VALUES (1707, 1698, NULL, NULL, 'getSessionAppDefinedDesktops', 3423, 3424);
INSERT INTO public.acos VALUES (1708, 1698, NULL, NULL, 'getSessionInitDesktops', 3425, 3426);
INSERT INTO public.acos VALUES (1709, 1698, NULL, NULL, 'getSessionValDesktops', 3427, 3428);
INSERT INTO public.acos VALUES (1710, 1698, NULL, NULL, 'getSessionValeditDesktops', 3429, 3430);
INSERT INTO public.acos VALUES (1711, 1698, NULL, NULL, 'getSessionDispDesktops', 3431, 3432);
INSERT INTO public.acos VALUES (1712, 1698, NULL, NULL, 'getSessionArchDesktops', 3433, 3434);
INSERT INTO public.acos VALUES (1713, 1698, NULL, NULL, 'getSessionAllDesktops', 3435, 3436);
INSERT INTO public.acos VALUES (1714, 1698, NULL, NULL, 'getEnv', 3437, 3438);
INSERT INTO public.acos VALUES (1715, 1698, NULL, NULL, 'getAddParams', 3439, 3440);
INSERT INTO public.acos VALUES (1716, 1698, NULL, NULL, 'ldebug', 3441, 3442);
INSERT INTO public.acos VALUES (1717, 1698, NULL, NULL, 'isAuthorized', 3443, 3444);
INSERT INTO public.acos VALUES (1726, 1, NULL, NULL, 'Templatemails', 3446, 3485);
INSERT INTO public.acos VALUES (1727, 1726, NULL, NULL, 'index', 3447, 3448);
INSERT INTO public.acos VALUES (1728, 1726, NULL, NULL, 'add', 3449, 3450);
INSERT INTO public.acos VALUES (1729, 1726, NULL, NULL, 'edit', 3451, 3452);
INSERT INTO public.acos VALUES (1730, 1726, NULL, NULL, 'delete', 3453, 3454);
INSERT INTO public.acos VALUES (1731, 1726, NULL, NULL, 'getTemplatemails', 3455, 3456);
INSERT INTO public.acos VALUES (1732, 1726, NULL, NULL, 'userAclCheck', 3457, 3458);
INSERT INTO public.acos VALUES (1733, 1726, NULL, NULL, 'getSessionInitServices', 3459, 3460);
INSERT INTO public.acos VALUES (1734, 1726, NULL, NULL, 'getSessionAppDefinedServices', 3461, 3462);
INSERT INTO public.acos VALUES (1735, 1726, NULL, NULL, 'getSessionAppDefinedDesktops', 3463, 3464);
INSERT INTO public.acos VALUES (1736, 1726, NULL, NULL, 'getSessionInitDesktops', 3465, 3466);
INSERT INTO public.acos VALUES (1737, 1726, NULL, NULL, 'getSessionValDesktops', 3467, 3468);
INSERT INTO public.acos VALUES (1774, 1766, NULL, NULL, 'getSessionInitServices', 3541, 3542);
INSERT INTO public.acos VALUES (1775, 1766, NULL, NULL, 'getSessionAppDefinedServices', 3543, 3544);
INSERT INTO public.acos VALUES (1776, 1766, NULL, NULL, 'getSessionAppDefinedDesktops', 3545, 3546);
INSERT INTO public.acos VALUES (1777, 1766, NULL, NULL, 'getSessionInitDesktops', 3547, 3548);
INSERT INTO public.acos VALUES (1778, 1766, NULL, NULL, 'getSessionValDesktops', 3549, 3550);
INSERT INTO public.acos VALUES (1779, 1766, NULL, NULL, 'getSessionValeditDesktops', 3551, 3552);
INSERT INTO public.acos VALUES (1780, 1766, NULL, NULL, 'getSessionDispDesktops', 3553, 3554);
INSERT INTO public.acos VALUES (1781, 1766, NULL, NULL, 'getSessionArchDesktops', 3555, 3556);
INSERT INTO public.acos VALUES (1782, 1766, NULL, NULL, 'getSessionAllDesktops', 3557, 3558);
INSERT INTO public.acos VALUES (1783, 1766, NULL, NULL, 'getEnv', 3559, 3560);
INSERT INTO public.acos VALUES (1784, 1766, NULL, NULL, 'getAddParams', 3561, 3562);
INSERT INTO public.acos VALUES (1785, 1766, NULL, NULL, 'ldebug', 3563, 3564);
INSERT INTO public.acos VALUES (1786, 1766, NULL, NULL, 'isAuthorized', 3565, 3566);
INSERT INTO public.acos VALUES (1787, 1, NULL, NULL, 'TypeApi', 3568, 3607);
INSERT INTO public.acos VALUES (1788, 1787, NULL, NULL, 'list', 3569, 3570);
INSERT INTO public.acos VALUES (1789, 1787, NULL, NULL, 'find', 3571, 3572);
INSERT INTO public.acos VALUES (1790, 1787, NULL, NULL, 'add', 3573, 3574);
INSERT INTO public.acos VALUES (1791, 1787, NULL, NULL, 'delete', 3575, 3576);
INSERT INTO public.acos VALUES (1792, 1787, NULL, NULL, 'update', 3577, 3578);
INSERT INTO public.acos VALUES (1793, 1787, NULL, NULL, 'userAclCheck', 3579, 3580);
INSERT INTO public.acos VALUES (1794, 1787, NULL, NULL, 'getSessionInitServices', 3581, 3582);
INSERT INTO public.acos VALUES (1795, 1787, NULL, NULL, 'getSessionAppDefinedServices', 3583, 3584);
INSERT INTO public.acos VALUES (1796, 1787, NULL, NULL, 'getSessionAppDefinedDesktops', 3585, 3586);
INSERT INTO public.acos VALUES (1797, 1787, NULL, NULL, 'getSessionInitDesktops', 3587, 3588);
INSERT INTO public.acos VALUES (1798, 1787, NULL, NULL, 'getSessionValDesktops', 3589, 3590);
INSERT INTO public.acos VALUES (1799, 1787, NULL, NULL, 'getSessionValeditDesktops', 3591, 3592);
INSERT INTO public.acos VALUES (1800, 1787, NULL, NULL, 'getSessionDispDesktops', 3593, 3594);
INSERT INTO public.acos VALUES (1801, 1787, NULL, NULL, 'getSessionArchDesktops', 3595, 3596);
INSERT INTO public.acos VALUES (1802, 1787, NULL, NULL, 'getSessionAllDesktops', 3597, 3598);
INSERT INTO public.acos VALUES (1803, 1787, NULL, NULL, 'getEnv', 3599, 3600);
INSERT INTO public.acos VALUES (1804, 1787, NULL, NULL, 'getAddParams', 3601, 3602);
INSERT INTO public.acos VALUES (1805, 1787, NULL, NULL, 'ldebug', 3603, 3604);
INSERT INTO public.acos VALUES (1806, 1787, NULL, NULL, 'isAuthorized', 3605, 3606);
INSERT INTO public.acos VALUES (1808, 1, NULL, NULL, 'Uniterifs', 3608, 3647);
INSERT INTO public.acos VALUES (1809, 1808, NULL, NULL, 'index', 3609, 3610);
INSERT INTO public.acos VALUES (1810, 1808, NULL, NULL, 'add', 3611, 3612);
INSERT INTO public.acos VALUES (1811, 1808, NULL, NULL, 'edit', 3613, 3614);
INSERT INTO public.acos VALUES (1812, 1808, NULL, NULL, 'delete', 3615, 3616);
INSERT INTO public.acos VALUES (1813, 1808, NULL, NULL, 'getUniterifs', 3617, 3618);
INSERT INTO public.acos VALUES (1814, 1808, NULL, NULL, 'userAclCheck', 3619, 3620);
INSERT INTO public.acos VALUES (1815, 1808, NULL, NULL, 'getSessionInitServices', 3621, 3622);
INSERT INTO public.acos VALUES (1816, 1808, NULL, NULL, 'getSessionAppDefinedServices', 3623, 3624);
INSERT INTO public.acos VALUES (1817, 1808, NULL, NULL, 'getSessionAppDefinedDesktops', 3625, 3626);
INSERT INTO public.acos VALUES (1818, 1808, NULL, NULL, 'getSessionInitDesktops', 3627, 3628);
INSERT INTO public.acos VALUES (1819, 1808, NULL, NULL, 'getSessionValDesktops', 3629, 3630);
INSERT INTO public.acos VALUES (1820, 1808, NULL, NULL, 'getSessionValeditDesktops', 3631, 3632);
INSERT INTO public.acos VALUES (1821, 1808, NULL, NULL, 'getSessionDispDesktops', 3633, 3634);
INSERT INTO public.acos VALUES (1822, 1808, NULL, NULL, 'getSessionArchDesktops', 3635, 3636);
INSERT INTO public.acos VALUES (1823, 1808, NULL, NULL, 'getSessionAllDesktops', 3637, 3638);
INSERT INTO public.acos VALUES (1824, 1808, NULL, NULL, 'getEnv', 3639, 3640);
INSERT INTO public.acos VALUES (1825, 1808, NULL, NULL, 'getAddParams', 3641, 3642);
INSERT INTO public.acos VALUES (1826, 1808, NULL, NULL, 'ldebug', 3643, 3644);
INSERT INTO public.acos VALUES (1827, 1808, NULL, NULL, 'isAuthorized', 3645, 3646);
INSERT INTO public.acos VALUES (1828, 1, NULL, NULL, 'UserApi', 3648, 3691);
INSERT INTO public.acos VALUES (1829, 1828, NULL, NULL, 'list', 3649, 3650);
INSERT INTO public.acos VALUES (1830, 1828, NULL, NULL, 'find', 3651, 3652);
INSERT INTO public.acos VALUES (1831, 1828, NULL, NULL, 'add', 3653, 3654);
INSERT INTO public.acos VALUES (1832, 1828, NULL, NULL, 'delete', 3655, 3656);
INSERT INTO public.acos VALUES (1833, 1828, NULL, NULL, 'update', 3657, 3658);
INSERT INTO public.acos VALUES (1834, 1828, NULL, NULL, 'associateDesktop', 3659, 3660);
INSERT INTO public.acos VALUES (1835, 1828, NULL, NULL, 'dissociateDesktop', 3661, 3662);
INSERT INTO public.acos VALUES (1836, 1828, NULL, NULL, 'userAclCheck', 3663, 3664);
INSERT INTO public.acos VALUES (1837, 1828, NULL, NULL, 'getSessionInitServices', 3665, 3666);
INSERT INTO public.acos VALUES (1838, 1828, NULL, NULL, 'getSessionAppDefinedServices', 3667, 3668);
INSERT INTO public.acos VALUES (1839, 1828, NULL, NULL, 'getSessionAppDefinedDesktops', 3669, 3670);
INSERT INTO public.acos VALUES (1840, 1828, NULL, NULL, 'getSessionInitDesktops', 3671, 3672);
INSERT INTO public.acos VALUES (1841, 1828, NULL, NULL, 'getSessionValDesktops', 3673, 3674);
INSERT INTO public.acos VALUES (1842, 1828, NULL, NULL, 'getSessionValeditDesktops', 3675, 3676);
INSERT INTO public.acos VALUES (1843, 1828, NULL, NULL, 'getSessionDispDesktops', 3677, 3678);
INSERT INTO public.acos VALUES (1844, 1828, NULL, NULL, 'getSessionArchDesktops', 3679, 3680);
INSERT INTO public.acos VALUES (1845, 1828, NULL, NULL, 'getSessionAllDesktops', 3681, 3682);
INSERT INTO public.acos VALUES (1846, 1828, NULL, NULL, 'getEnv', 3683, 3684);
INSERT INTO public.acos VALUES (1847, 1828, NULL, NULL, 'getAddParams', 3685, 3686);
INSERT INTO public.acos VALUES (1848, 1828, NULL, NULL, 'ldebug', 3687, 3688);
INSERT INTO public.acos VALUES (1849, 1828, NULL, NULL, 'isAuthorized', 3689, 3690);
INSERT INTO public.acos VALUES (1866, 1, NULL, NULL, 'AuthManager', 3692, 3693);
INSERT INTO public.acos VALUES (1872, 872, NULL, NULL, 'isAuthorized', 2100, 2101);
INSERT INTO public.acos VALUES (1873, 1, NULL, NULL, 'LdapManager', 3694, 3695);
INSERT INTO public.acos VALUES (147, 1, NULL, NULL, 'Compteurs', 342, 371);
INSERT INTO public.acos VALUES (162, 161, NULL, NULL, 'index', 373, 374);
INSERT INTO public.acos VALUES (163, 161, NULL, NULL, 'getConnecteurs', 375, 376);
INSERT INTO public.acos VALUES (164, 161, NULL, NULL, 'edit', 377, 378);
INSERT INTO public.acos VALUES (165, 161, NULL, NULL, 'makeconf', 379, 380);
INSERT INTO public.acos VALUES (166, 161, NULL, NULL, 'getCircuitsParapheur', 381, 382);
INSERT INTO public.acos VALUES (167, 161, NULL, NULL, 'check', 383, 384);
INSERT INTO public.acos VALUES (168, 161, NULL, NULL, 'getRepositoryInfo', 385, 386);
INSERT INTO public.acos VALUES (169, 161, NULL, NULL, 'getRepositoryRetrieveInfo', 387, 388);
INSERT INTO public.acos VALUES (170, 161, NULL, NULL, 'getRootFolder', 389, 390);
INSERT INTO public.acos VALUES (171, 161, NULL, NULL, 'testObject', 391, 392);
INSERT INTO public.acos VALUES (172, 161, NULL, NULL, 'getObjectByPath', 393, 394);
INSERT INTO public.acos VALUES (173, 161, NULL, NULL, 'getFolderRetrieveInfo', 395, 396);
INSERT INTO public.acos VALUES (174, 161, NULL, NULL, 'userAclCheck', 397, 398);
INSERT INTO public.acos VALUES (175, 161, NULL, NULL, 'getSessionInitServices', 399, 400);
INSERT INTO public.acos VALUES (176, 161, NULL, NULL, 'getSessionAppDefinedServices', 401, 402);
INSERT INTO public.acos VALUES (177, 161, NULL, NULL, 'getSessionAppDefinedDesktops', 403, 404);
INSERT INTO public.acos VALUES (178, 161, NULL, NULL, 'getSessionInitDesktops', 405, 406);
INSERT INTO public.acos VALUES (179, 161, NULL, NULL, 'getSessionValDesktops', 407, 408);
INSERT INTO public.acos VALUES (180, 161, NULL, NULL, 'getSessionValeditDesktops', 409, 410);
INSERT INTO public.acos VALUES (181, 161, NULL, NULL, 'getSessionDispDesktops', 411, 412);
INSERT INTO public.acos VALUES (182, 161, NULL, NULL, 'getSessionArchDesktops', 413, 414);
INSERT INTO public.acos VALUES (183, 161, NULL, NULL, 'getSessionAllDesktops', 415, 416);
INSERT INTO public.acos VALUES (184, 161, NULL, NULL, 'getEnv', 417, 418);
INSERT INTO public.acos VALUES (185, 161, NULL, NULL, 'getAddParams', 419, 420);
INSERT INTO public.acos VALUES (186, 161, NULL, NULL, 'ldebug', 421, 422);
INSERT INTO public.acos VALUES (1558, 1045, NULL, NULL, 'searchOrganismes', 2349, 2350);
INSERT INTO public.acos VALUES (1033, 161, NULL, NULL, 'delete', 423, 424);
INSERT INTO public.acos VALUES (1559, 1045, NULL, NULL, 'search_organismes_courriers', 2351, 2352);
INSERT INTO public.acos VALUES (1171, 147, NULL, NULL, 'isAuthorized', 369, 370);
INSERT INTO public.acos VALUES (1236, 1, NULL, NULL, 'ContactApi', 2626, 2665);
INSERT INTO public.acos VALUES (573, 557, NULL, NULL, 'ldebug', 1383, 1384);
INSERT INTO public.acos VALUES (161, 1, NULL, NULL, 'Connecteurs', 372, 435);
INSERT INTO public.acos VALUES (188, 187, NULL, NULL, 'add', 437, 438);
INSERT INTO public.acos VALUES (189, 187, NULL, NULL, 'getHomonyms', 439, 440);
INSERT INTO public.acos VALUES (187, 1, NULL, NULL, 'Contactinfos', 436, 477);
INSERT INTO public.acos VALUES (368, 1, NULL, NULL, 'Profils', 942, 989);
INSERT INTO public.acos VALUES (393, 391, NULL, NULL, 'get_keywordlists', 993, 994);
INSERT INTO public.acos VALUES (394, 391, NULL, NULL, 'edit', 995, 996);
INSERT INTO public.acos VALUES (395, 391, NULL, NULL, 'view', 997, 998);
INSERT INTO public.acos VALUES (396, 391, NULL, NULL, 'userAclCheck', 999, 1000);
INSERT INTO public.acos VALUES (397, 391, NULL, NULL, 'getSessionInitServices', 1001, 1002);
INSERT INTO public.acos VALUES (398, 391, NULL, NULL, 'getSessionAppDefinedServices', 1003, 1004);
INSERT INTO public.acos VALUES (399, 391, NULL, NULL, 'getSessionAppDefinedDesktops', 1005, 1006);
INSERT INTO public.acos VALUES (400, 391, NULL, NULL, 'getSessionInitDesktops', 1007, 1008);
INSERT INTO public.acos VALUES (401, 391, NULL, NULL, 'getSessionValDesktops', 1009, 1010);
INSERT INTO public.acos VALUES (402, 391, NULL, NULL, 'getSessionValeditDesktops', 1011, 1012);
INSERT INTO public.acos VALUES (403, 391, NULL, NULL, 'getSessionDispDesktops', 1013, 1014);
INSERT INTO public.acos VALUES (404, 391, NULL, NULL, 'getSessionArchDesktops', 1015, 1016);
INSERT INTO public.acos VALUES (405, 391, NULL, NULL, 'getSessionAllDesktops', 1017, 1018);
INSERT INTO public.acos VALUES (406, 391, NULL, NULL, 'getEnv', 1019, 1020);
INSERT INTO public.acos VALUES (407, 391, NULL, NULL, 'getAddParams', 1021, 1022);
INSERT INTO public.acos VALUES (409, 1, NULL, NULL, 'Keywords', 1028, 1057);
INSERT INTO public.acos VALUES (410, 409, NULL, NULL, 'userAclCheck', 1029, 1030);
INSERT INTO public.acos VALUES (411, 409, NULL, NULL, 'getSessionInitServices', 1031, 1032);
INSERT INTO public.acos VALUES (412, 409, NULL, NULL, 'getSessionAppDefinedServices', 1033, 1034);
INSERT INTO public.acos VALUES (413, 409, NULL, NULL, 'getSessionAppDefinedDesktops', 1035, 1036);
INSERT INTO public.acos VALUES (414, 409, NULL, NULL, 'getSessionInitDesktops', 1037, 1038);
INSERT INTO public.acos VALUES (415, 409, NULL, NULL, 'getSessionValDesktops', 1039, 1040);
INSERT INTO public.acos VALUES (416, 409, NULL, NULL, 'getSessionValeditDesktops', 1041, 1042);
INSERT INTO public.acos VALUES (417, 409, NULL, NULL, 'getSessionDispDesktops', 1043, 1044);
INSERT INTO public.acos VALUES (418, 409, NULL, NULL, 'getSessionArchDesktops', 1045, 1046);
INSERT INTO public.acos VALUES (419, 409, NULL, NULL, 'getSessionAllDesktops', 1047, 1048);
INSERT INTO public.acos VALUES (421, 409, NULL, NULL, 'getAddParams', 1051, 1052);
INSERT INTO public.acos VALUES (455, 437, NULL, NULL, 'ldebug', 1123, 1124);
INSERT INTO public.acos VALUES (1189, 161, NULL, NULL, 'getFolderRetrieveInfo11', 425, 426);
INSERT INTO public.acos VALUES (1630, 368, NULL, NULL, 'isAuthorized', 987, 988);
INSERT INTO public.acos VALUES (1190, 161, NULL, NULL, 'sync', 427, 428);
INSERT INTO public.acos VALUES (1631, 456, NULL, NULL, 'charge_soustypes', 1167, 1168);
INSERT INTO public.acos VALUES (781, 754, NULL, NULL, 'Compositions', 1875, 1924);
INSERT INTO public.acos VALUES (1484, 1, NULL, NULL, 'Marches', 3028, 3073);
INSERT INTO public.acos VALUES (1191, 161, NULL, NULL, 'listEntities', 429, 430);
INSERT INTO public.acos VALUES (1632, 456, NULL, NULL, 'export', 1169, 1170);
INSERT INTO public.acos VALUES (778, 755, NULL, NULL, 'getEnv', 1866, 1867);
INSERT INTO public.acos VALUES (1192, 161, NULL, NULL, 'listTypesIParapheur', 431, 432);
INSERT INTO public.acos VALUES (190, 187, NULL, NULL, 'edit', 441, 442);
INSERT INTO public.acos VALUES (191, 187, NULL, NULL, 'delete', 443, 444);
INSERT INTO public.acos VALUES (192, 187, NULL, NULL, 'userAclCheck', 445, 446);
INSERT INTO public.acos VALUES (193, 187, NULL, NULL, 'getSessionInitServices', 447, 448);
INSERT INTO public.acos VALUES (194, 187, NULL, NULL, 'getSessionAppDefinedServices', 449, 450);
INSERT INTO public.acos VALUES (195, 187, NULL, NULL, 'getSessionAppDefinedDesktops', 451, 452);
INSERT INTO public.acos VALUES (196, 187, NULL, NULL, 'getSessionInitDesktops', 453, 454);
INSERT INTO public.acos VALUES (197, 187, NULL, NULL, 'getSessionValDesktops', 455, 456);
INSERT INTO public.acos VALUES (198, 187, NULL, NULL, 'getSessionValeditDesktops', 457, 458);
INSERT INTO public.acos VALUES (199, 187, NULL, NULL, 'getSessionDispDesktops', 459, 460);
INSERT INTO public.acos VALUES (200, 187, NULL, NULL, 'getSessionArchDesktops', 461, 462);
INSERT INTO public.acos VALUES (201, 187, NULL, NULL, 'getSessionAllDesktops', 463, 464);
INSERT INTO public.acos VALUES (203, 187, NULL, NULL, 'getAddParams', 467, 468);
INSERT INTO public.acos VALUES (204, 187, NULL, NULL, 'ldebug', 469, 470);
INSERT INTO public.acos VALUES (1633, 456, NULL, NULL, 'exportLettre', 1171, 1172);
INSERT INTO public.acos VALUES (1034, 187, NULL, NULL, 'addFromCourrier', 471, 472);
INSERT INTO public.acos VALUES (1035, 187, NULL, NULL, 'editFromCourrier', 473, 474);
INSERT INTO public.acos VALUES (1455, 1441, NULL, NULL, 'getSessionArchDesktops', 2975, 2976);
INSERT INTO public.acos VALUES (1193, 161, NULL, NULL, 'isAuthorized', 433, 434);
INSERT INTO public.acos VALUES (478, 476, NULL, NULL, 'delete', 1179, 1180);
INSERT INTO public.acos VALUES (479, 476, NULL, NULL, 'download', 1181, 1182);
INSERT INTO public.acos VALUES (480, 476, NULL, NULL, 'getPreview', 1183, 1184);
INSERT INTO public.acos VALUES (481, 476, NULL, NULL, 'addFromFile', 1185, 1186);
INSERT INTO public.acos VALUES (482, 476, NULL, NULL, 'userAclCheck', 1187, 1188);
INSERT INTO public.acos VALUES (483, 476, NULL, NULL, 'getSessionInitServices', 1189, 1190);
INSERT INTO public.acos VALUES (484, 476, NULL, NULL, 'getSessionAppDefinedServices', 1191, 1192);
INSERT INTO public.acos VALUES (485, 476, NULL, NULL, 'getSessionAppDefinedDesktops', 1193, 1194);
INSERT INTO public.acos VALUES (486, 476, NULL, NULL, 'getSessionInitDesktops', 1195, 1196);
INSERT INTO public.acos VALUES (487, 476, NULL, NULL, 'getSessionValDesktops', 1197, 1198);
INSERT INTO public.acos VALUES (488, 476, NULL, NULL, 'getSessionValeditDesktops', 1199, 1200);
INSERT INTO public.acos VALUES (489, 476, NULL, NULL, 'getSessionDispDesktops', 1201, 1202);
INSERT INTO public.acos VALUES (490, 476, NULL, NULL, 'getSessionArchDesktops', 1203, 1204);
INSERT INTO public.acos VALUES (491, 476, NULL, NULL, 'getSessionAllDesktops', 1205, 1206);
INSERT INTO public.acos VALUES (492, 476, NULL, NULL, 'getEnv', 1207, 1208);
INSERT INTO public.acos VALUES (493, 476, NULL, NULL, 'getAddParams', 1209, 1210);
INSERT INTO public.acos VALUES (322, 306, NULL, NULL, 'getEnv', 801, 802);
INSERT INTO public.acos VALUES (206, 205, NULL, NULL, 'get_contacts', 479, 480);
INSERT INTO public.acos VALUES (207, 205, NULL, NULL, 'add', 481, 482);
INSERT INTO public.acos VALUES (208, 205, NULL, NULL, 'edit', 483, 484);
INSERT INTO public.acos VALUES (209, 205, NULL, NULL, 'delete', 485, 486);
INSERT INTO public.acos VALUES (210, 205, NULL, NULL, 'import', 487, 488);
INSERT INTO public.acos VALUES (211, 205, NULL, NULL, 'search', 489, 490);
INSERT INTO public.acos VALUES (212, 205, NULL, NULL, 'searchContactinfo', 491, 492);
INSERT INTO public.acos VALUES (213, 205, NULL, NULL, 'getHomonyms', 493, 494);
INSERT INTO public.acos VALUES (214, 205, NULL, NULL, 'userAclCheck', 495, 496);
INSERT INTO public.acos VALUES (215, 205, NULL, NULL, 'getSessionInitServices', 497, 498);
INSERT INTO public.acos VALUES (216, 205, NULL, NULL, 'getSessionAppDefinedServices', 499, 500);
INSERT INTO public.acos VALUES (217, 205, NULL, NULL, 'getSessionAppDefinedDesktops', 501, 502);
INSERT INTO public.acos VALUES (218, 205, NULL, NULL, 'getSessionInitDesktops', 503, 504);
INSERT INTO public.acos VALUES (219, 205, NULL, NULL, 'getSessionValDesktops', 505, 506);
INSERT INTO public.acos VALUES (220, 205, NULL, NULL, 'getSessionValeditDesktops', 507, 508);
INSERT INTO public.acos VALUES (221, 205, NULL, NULL, 'getSessionDispDesktops', 509, 510);
INSERT INTO public.acos VALUES (222, 205, NULL, NULL, 'getSessionArchDesktops', 511, 512);
INSERT INTO public.acos VALUES (223, 205, NULL, NULL, 'getSessionAllDesktops', 513, 514);
INSERT INTO public.acos VALUES (224, 205, NULL, NULL, 'getEnv', 515, 516);
INSERT INTO public.acos VALUES (225, 205, NULL, NULL, 'getAddParams', 517, 518);
INSERT INTO public.acos VALUES (226, 205, NULL, NULL, 'ldebug', 519, 520);
INSERT INTO public.acos VALUES (1634, 456, NULL, NULL, 'isAuthorized', 1173, 1174);
INSERT INTO public.acos VALUES (932, 205, NULL, NULL, 'download_model', 521, 522);
INSERT INTO public.acos VALUES (933, 205, NULL, NULL, 'import_contacts', 523, 524);
INSERT INTO public.acos VALUES (934, 205, NULL, NULL, 'searchAdresse', 525, 526);
INSERT INTO public.acos VALUES (935, 205, NULL, NULL, 'searchAdresseSpecifique', 527, 528);
INSERT INTO public.acos VALUES (936, 205, NULL, NULL, 'searchAdresseNomvoie', 529, 530);
INSERT INTO public.acos VALUES (1036, 205, NULL, NULL, 'addFromCourrier', 531, 532);
INSERT INTO public.acos VALUES (1037, 205, NULL, NULL, 'editFromCourrier', 533, 534);
INSERT INTO public.acos VALUES (1066, 205, NULL, NULL, 'searchOrganismeinfo', 535, 536);
INSERT INTO public.acos VALUES (1074, 205, NULL, NULL, 'get_contactform', 537, 538);
INSERT INTO public.acos VALUES (1256, 187, NULL, NULL, 'isAuthorized', 475, 476);
INSERT INTO public.acos VALUES (606, 596, NULL, NULL, 'userAclCheck', 1453, 1454);
INSERT INTO public.acos VALUES (749, 729, NULL, NULL, 'getEnv', 1803, 1804);
INSERT INTO public.acos VALUES (912, 896, NULL, NULL, 'getEnv', 2141, 2142);
INSERT INTO public.acos VALUES (497, 495, NULL, NULL, 'getRepertoires', 1219, 1220);
INSERT INTO public.acos VALUES (498, 495, NULL, NULL, 'add', 1221, 1222);
INSERT INTO public.acos VALUES (499, 495, NULL, NULL, 'edit', 1223, 1224);
INSERT INTO public.acos VALUES (502, 495, NULL, NULL, 'linkProfil', 1229, 1230);
INSERT INTO public.acos VALUES (503, 495, NULL, NULL, 'link', 1231, 1232);
INSERT INTO public.acos VALUES (504, 495, NULL, NULL, 'unlinkProfil', 1233, 1234);
INSERT INTO public.acos VALUES (505, 495, NULL, NULL, 'unlink', 1235, 1236);
INSERT INTO public.acos VALUES (506, 495, NULL, NULL, 'userAclCheck', 1237, 1238);
INSERT INTO public.acos VALUES (507, 495, NULL, NULL, 'getSessionInitServices', 1239, 1240);
INSERT INTO public.acos VALUES (508, 495, NULL, NULL, 'getSessionAppDefinedServices', 1241, 1242);
INSERT INTO public.acos VALUES (509, 495, NULL, NULL, 'getSessionAppDefinedDesktops', 1243, 1244);
INSERT INTO public.acos VALUES (510, 495, NULL, NULL, 'getSessionInitDesktops', 1245, 1246);
INSERT INTO public.acos VALUES (511, 495, NULL, NULL, 'getSessionValDesktops', 1247, 1248);
INSERT INTO public.acos VALUES (512, 495, NULL, NULL, 'getSessionValeditDesktops', 1249, 1250);
INSERT INTO public.acos VALUES (513, 495, NULL, NULL, 'getSessionDispDesktops', 1251, 1252);
INSERT INTO public.acos VALUES (514, 495, NULL, NULL, 'getSessionArchDesktops', 1253, 1254);
INSERT INTO public.acos VALUES (515, 495, NULL, NULL, 'getSessionAllDesktops', 1255, 1256);
INSERT INTO public.acos VALUES (516, 495, NULL, NULL, 'getEnv', 1257, 1258);
INSERT INTO public.acos VALUES (517, 495, NULL, NULL, 'getAddParams', 1259, 1260);
INSERT INTO public.acos VALUES (518, 495, NULL, NULL, 'ldebug', 1261, 1262);
INSERT INTO public.acos VALUES (495, 1, NULL, NULL, 'Repertoires', 1216, 1265);
INSERT INTO public.acos VALUES (520, 519, NULL, NULL, 'add', 1267, 1268);
INSERT INTO public.acos VALUES (521, 519, NULL, NULL, 'delete', 1269, 1270);
INSERT INTO public.acos VALUES (522, 519, NULL, NULL, 'download', 1271, 1272);
INSERT INTO public.acos VALUES (523, 519, NULL, NULL, 'getPreview', 1273, 1274);
INSERT INTO public.acos VALUES (524, 519, NULL, NULL, 'userAclCheck', 1275, 1276);
INSERT INTO public.acos VALUES (525, 519, NULL, NULL, 'getSessionInitServices', 1277, 1278);
INSERT INTO public.acos VALUES (526, 519, NULL, NULL, 'getSessionAppDefinedServices', 1279, 1280);
INSERT INTO public.acos VALUES (527, 519, NULL, NULL, 'getSessionAppDefinedDesktops', 1281, 1282);
INSERT INTO public.acos VALUES (528, 519, NULL, NULL, 'getSessionInitDesktops', 1283, 1284);
INSERT INTO public.acos VALUES (529, 519, NULL, NULL, 'getSessionValDesktops', 1285, 1286);
INSERT INTO public.acos VALUES (530, 519, NULL, NULL, 'getSessionValeditDesktops', 1287, 1288);
INSERT INTO public.acos VALUES (531, 519, NULL, NULL, 'getSessionDispDesktops', 1289, 1290);
INSERT INTO public.acos VALUES (532, 519, NULL, NULL, 'getSessionArchDesktops', 1291, 1292);
INSERT INTO public.acos VALUES (533, 519, NULL, NULL, 'getSessionAllDesktops', 1293, 1294);
INSERT INTO public.acos VALUES (536, 519, NULL, NULL, 'ldebug', 1299, 1300);
INSERT INTO public.acos VALUES (519, 1, NULL, NULL, 'Rmodels', 1266, 1307);
INSERT INTO public.acos VALUES (538, 537, NULL, NULL, 'index', 1309, 1310);
INSERT INTO public.acos VALUES (539, 537, NULL, NULL, 'add', 1311, 1312);
INSERT INTO public.acos VALUES (540, 537, NULL, NULL, 'edit', 1313, 1314);
INSERT INTO public.acos VALUES (541, 537, NULL, NULL, 'delete', 1315, 1316);
INSERT INTO public.acos VALUES (542, 537, NULL, NULL, 'getScanemails', 1317, 1318);
INSERT INTO public.acos VALUES (544, 537, NULL, NULL, 'userAclCheck', 1319, 1320);
INSERT INTO public.acos VALUES (545, 537, NULL, NULL, 'getSessionInitServices', 1321, 1322);
INSERT INTO public.acos VALUES (546, 537, NULL, NULL, 'getSessionAppDefinedServices', 1323, 1324);
INSERT INTO public.acos VALUES (547, 537, NULL, NULL, 'getSessionAppDefinedDesktops', 1325, 1326);
INSERT INTO public.acos VALUES (548, 537, NULL, NULL, 'getSessionInitDesktops', 1327, 1328);
INSERT INTO public.acos VALUES (549, 537, NULL, NULL, 'getSessionValDesktops', 1329, 1330);
INSERT INTO public.acos VALUES (550, 537, NULL, NULL, 'getSessionValeditDesktops', 1331, 1332);
INSERT INTO public.acos VALUES (551, 537, NULL, NULL, 'getSessionDispDesktops', 1333, 1334);
INSERT INTO public.acos VALUES (1257, 205, NULL, NULL, 'import_contacts_index', 539, 540);
INSERT INTO public.acos VALUES (1636, 476, NULL, NULL, 'isAuthorized', 1213, 1214);
INSERT INTO public.acos VALUES (496, 495, NULL, NULL, 'index', 1217, 1218);
INSERT INTO public.acos VALUES (1007, 1006, NULL, NULL, 'delete', 2265, 2266);
INSERT INTO public.acos VALUES (1204, 1194, NULL, NULL, 'getSessionValDesktops', 2561, 2562);
INSERT INTO public.acos VALUES (1635, 945, NULL, NULL, 'isAuthorized', 2219, 2220);
INSERT INTO public.acos VALUES (1258, 205, NULL, NULL, 'view', 541, 542);
INSERT INTO public.acos VALUES (1637, 495, NULL, NULL, 'isAuthorized', 1263, 1264);
INSERT INTO public.acos VALUES (702, 675, NULL, NULL, 'getSessionAllDesktops', 1677, 1678);
INSERT INTO public.acos VALUES (1259, 205, NULL, NULL, 'addParCarnet', 543, 544);
INSERT INTO public.acos VALUES (1656, 519, NULL, NULL, 'deletefile', 1301, 1302);
INSERT INTO public.acos VALUES (929, 915, NULL, NULL, 'getEnv', 2177, 2178);
INSERT INTO public.acos VALUES (554, 537, NULL, NULL, 'getEnv', 1339, 1340);
INSERT INTO public.acos VALUES (555, 537, NULL, NULL, 'getAddParams', 1341, 1342);
INSERT INTO public.acos VALUES (556, 537, NULL, NULL, 'ldebug', 1343, 1344);
INSERT INTO public.acos VALUES (558, 557, NULL, NULL, 'add', 1353, 1354);
INSERT INTO public.acos VALUES (1260, 205, NULL, NULL, 'getOrganismeOption', 545, 546);
INSERT INTO public.acos VALUES (436, 423, NULL, NULL, 'ldebug', 1083, 1084);
INSERT INTO public.acos VALUES (1657, 519, NULL, NULL, 'edit', 1303, 1304);
INSERT INTO public.acos VALUES (930, 915, NULL, NULL, 'getAddParams', 2179, 2180);
INSERT INTO public.acos VALUES (552, 537, NULL, NULL, 'getSessionArchDesktops', 1335, 1336);
INSERT INTO public.acos VALUES (553, 537, NULL, NULL, 'getSessionAllDesktops', 1337, 1338);
INSERT INTO public.acos VALUES (1261, 205, NULL, NULL, 'export', 547, 548);
INSERT INTO public.acos VALUES (494, 476, NULL, NULL, 'ldebug', 1211, 1212);
INSERT INTO public.acos VALUES (1658, 519, NULL, NULL, 'isAuthorized', 1305, 1306);
INSERT INTO public.acos VALUES (785, 781, NULL, NULL, 'edit', 1882, 1883);
INSERT INTO public.acos VALUES (1143, 1075, NULL, NULL, 'isAuthorized', 2391, 2392);
INSERT INTO public.acos VALUES (537, 1, NULL, NULL, 'Scanemails', 1308, 1351);
INSERT INTO public.acos VALUES (571, 557, NULL, NULL, 'getEnv', 1379, 1380);
INSERT INTO public.acos VALUES (581, 574, NULL, NULL, 'setDroitsMeta', 1401, 1402);
INSERT INTO public.acos VALUES (582, 574, NULL, NULL, 'delete', 1403, 1404);
INSERT INTO public.acos VALUES (583, 574, NULL, NULL, 'userAclCheck', 1405, 1406);
INSERT INTO public.acos VALUES (584, 574, NULL, NULL, 'getSessionInitServices', 1407, 1408);
INSERT INTO public.acos VALUES (585, 574, NULL, NULL, 'getSessionAppDefinedServices', 1409, 1410);
INSERT INTO public.acos VALUES (586, 574, NULL, NULL, 'getSessionAppDefinedDesktops', 1411, 1412);
INSERT INTO public.acos VALUES (587, 574, NULL, NULL, 'getSessionInitDesktops', 1413, 1414);
INSERT INTO public.acos VALUES (588, 574, NULL, NULL, 'getSessionValDesktops', 1415, 1416);
INSERT INTO public.acos VALUES (589, 574, NULL, NULL, 'getSessionValeditDesktops', 1417, 1418);
INSERT INTO public.acos VALUES (590, 574, NULL, NULL, 'getSessionDispDesktops', 1419, 1420);
INSERT INTO public.acos VALUES (591, 574, NULL, NULL, 'getSessionArchDesktops', 1421, 1422);
INSERT INTO public.acos VALUES (592, 574, NULL, NULL, 'getSessionAllDesktops', 1423, 1424);
INSERT INTO public.acos VALUES (594, 574, NULL, NULL, 'getAddParams', 1427, 1428);
INSERT INTO public.acos VALUES (595, 574, NULL, NULL, 'ldebug', 1429, 1430);
INSERT INTO public.acos VALUES (597, 596, NULL, NULL, 'index', 1435, 1436);
INSERT INTO public.acos VALUES (596, 1, NULL, NULL, 'Soustypes', 1434, 1487);
INSERT INTO public.acos VALUES (1262, 205, NULL, NULL, 'searchContacts', 549, 550);
INSERT INTO public.acos VALUES (1659, 537, NULL, NULL, 'encrypt', 1345, 1346);
INSERT INTO public.acos VALUES (598, 596, NULL, NULL, 'getTypesSoustypes', 1437, 1438);
INSERT INTO public.acos VALUES (599, 596, NULL, NULL, 'setInfos', 1439, 1440);
INSERT INTO public.acos VALUES (600, 596, NULL, NULL, 'setArmodels', 1441, 1442);
INSERT INTO public.acos VALUES (601, 596, NULL, NULL, 'setRmodels', 1443, 1444);
INSERT INTO public.acos VALUES (602, 596, NULL, NULL, 'add', 1445, 1446);
INSERT INTO public.acos VALUES (603, 596, NULL, NULL, 'edit', 1447, 1448);
INSERT INTO public.acos VALUES (604, 596, NULL, NULL, 'delete', 1449, 1450);
INSERT INTO public.acos VALUES (605, 596, NULL, NULL, 'getRefSimul', 1451, 1452);
INSERT INTO public.acos VALUES (607, 596, NULL, NULL, 'getSessionInitServices', 1455, 1456);
INSERT INTO public.acos VALUES (608, 596, NULL, NULL, 'getSessionAppDefinedServices', 1457, 1458);
INSERT INTO public.acos VALUES (609, 596, NULL, NULL, 'getSessionAppDefinedDesktops', 1459, 1460);
INSERT INTO public.acos VALUES (610, 596, NULL, NULL, 'getSessionInitDesktops', 1461, 1462);
INSERT INTO public.acos VALUES (611, 596, NULL, NULL, 'getSessionValDesktops', 1463, 1464);
INSERT INTO public.acos VALUES (612, 596, NULL, NULL, 'getSessionValeditDesktops', 1465, 1466);
INSERT INTO public.acos VALUES (613, 596, NULL, NULL, 'getSessionDispDesktops', 1467, 1468);
INSERT INTO public.acos VALUES (614, 596, NULL, NULL, 'getSessionArchDesktops', 1469, 1470);
INSERT INTO public.acos VALUES (615, 596, NULL, NULL, 'getSessionAllDesktops', 1471, 1472);
INSERT INTO public.acos VALUES (617, 596, NULL, NULL, 'getAddParams', 1475, 1476);
INSERT INTO public.acos VALUES (1263, 205, NULL, NULL, 'getOrganismeValues', 551, 552);
INSERT INTO public.acos VALUES (1660, 537, NULL, NULL, 'decrypt', 1347, 1348);
INSERT INTO public.acos VALUES (559, 557, NULL, NULL, 'edit', 1355, 1356);
INSERT INTO public.acos VALUES (560, 557, NULL, NULL, 'delete', 1357, 1358);
INSERT INTO public.acos VALUES (561, 557, NULL, NULL, 'userAclCheck', 1359, 1360);
INSERT INTO public.acos VALUES (562, 557, NULL, NULL, 'getSessionInitServices', 1361, 1362);
INSERT INTO public.acos VALUES (563, 557, NULL, NULL, 'getSessionAppDefinedServices', 1363, 1364);
INSERT INTO public.acos VALUES (564, 557, NULL, NULL, 'getSessionAppDefinedDesktops', 1365, 1366);
INSERT INTO public.acos VALUES (565, 557, NULL, NULL, 'getSessionInitDesktops', 1367, 1368);
INSERT INTO public.acos VALUES (566, 557, NULL, NULL, 'getSessionValDesktops', 1369, 1370);
INSERT INTO public.acos VALUES (567, 557, NULL, NULL, 'getSessionValeditDesktops', 1371, 1372);
INSERT INTO public.acos VALUES (568, 557, NULL, NULL, 'getSessionDispDesktops', 1373, 1374);
INSERT INTO public.acos VALUES (569, 557, NULL, NULL, 'getSessionArchDesktops', 1375, 1376);
INSERT INTO public.acos VALUES (570, 557, NULL, NULL, 'getSessionAllDesktops', 1377, 1378);
INSERT INTO public.acos VALUES (572, 557, NULL, NULL, 'getAddParams', 1381, 1382);
INSERT INTO public.acos VALUES (575, 574, NULL, NULL, 'index', 1389, 1390);
INSERT INTO public.acos VALUES (576, 574, NULL, NULL, 'getServices', 1391, 1392);
INSERT INTO public.acos VALUES (577, 574, NULL, NULL, 'add', 1393, 1394);
INSERT INTO public.acos VALUES (578, 574, NULL, NULL, 'edit', 1395, 1396);
INSERT INTO public.acos VALUES (579, 574, NULL, NULL, 'setInfos', 1397, 1398);
INSERT INTO public.acos VALUES (580, 574, NULL, NULL, 'setDroitsData', 1399, 1400);
INSERT INTO public.acos VALUES (574, 1, NULL, NULL, 'Services', 1388, 1433);
INSERT INTO public.acos VALUES (1264, 205, NULL, NULL, 'search_contacts_courriers', 553, 554);
INSERT INTO public.acos VALUES (1661, 537, NULL, NULL, 'isAuthorized', 1349, 1350);
INSERT INTO public.acos VALUES (434, 423, NULL, NULL, 'getEnv', 1079, 1080);
INSERT INTO public.acos VALUES (205, 1, NULL, NULL, 'Contacts', 478, 559);
INSERT INTO public.acos VALUES (247, 227, NULL, NULL, 'insert', 599, 600);
INSERT INTO public.acos VALUES (248, 227, NULL, NULL, 'refus', 601, 602);
INSERT INTO public.acos VALUES (249, 227, NULL, NULL, 'sendcpy', 603, 604);
INSERT INTO public.acos VALUES (250, 227, NULL, NULL, 'sendwkf', 605, 606);
INSERT INTO public.acos VALUES (251, 227, NULL, NULL, 'send', 607, 608);
INSERT INTO public.acos VALUES (252, 227, NULL, NULL, 'dispsend', 609, 610);
INSERT INTO public.acos VALUES (253, 227, NULL, NULL, 'delete', 611, 612);
INSERT INTO public.acos VALUES (254, 227, NULL, NULL, 'getInsertEnd', 613, 614);
INSERT INTO public.acos VALUES (255, 227, NULL, NULL, 'getArs', 615, 616);
INSERT INTO public.acos VALUES (256, 227, NULL, NULL, 'setArs', 617, 618);
INSERT INTO public.acos VALUES (257, 227, NULL, NULL, 'genResponse', 619, 620);
INSERT INTO public.acos VALUES (258, 227, NULL, NULL, 'getContext', 621, 622);
INSERT INTO public.acos VALUES (259, 227, NULL, NULL, 'getRelements', 623, 624);
INSERT INTO public.acos VALUES (235, 227, NULL, NULL, 'getAffaire', 575, 576);
INSERT INTO public.acos VALUES (236, 227, NULL, NULL, 'createInfos', 577, 578);
INSERT INTO public.acos VALUES (237, 227, NULL, NULL, 'setInfos', 579, 580);
INSERT INTO public.acos VALUES (238, 227, NULL, NULL, 'setComments', 581, 582);
INSERT INTO public.acos VALUES (239, 227, NULL, NULL, 'getComments', 583, 584);
INSERT INTO public.acos VALUES (240, 227, NULL, NULL, 'getContacts', 585, 586);
INSERT INTO public.acos VALUES (241, 227, NULL, NULL, 'getFlowControls', 587, 588);
INSERT INTO public.acos VALUES (242, 227, NULL, NULL, 'setMeta', 589, 590);
INSERT INTO public.acos VALUES (261, 227, NULL, NULL, 'getParent', 627, 628);
INSERT INTO public.acos VALUES (262, 227, NULL, NULL, 'acquisition', 629, 630);
INSERT INTO public.acos VALUES (263, 227, NULL, NULL, 'isInTraite', 631, 632);
INSERT INTO public.acos VALUES (264, 227, NULL, NULL, 'fluxIdInTraite', 633, 634);
INSERT INTO public.acos VALUES (269, 227, NULL, NULL, 'sendDocumentToGed', 643, 644);
INSERT INTO public.acos VALUES (270, 227, NULL, NULL, 'sendmail', 645, 646);
INSERT INTO public.acos VALUES (271, 227, NULL, NULL, 'sendwkfsignaturemanuelle', 647, 648);
INSERT INTO public.acos VALUES (272, 227, NULL, NULL, 'userAclCheck', 649, 650);
INSERT INTO public.acos VALUES (273, 227, NULL, NULL, 'getSessionInitServices', 651, 652);
INSERT INTO public.acos VALUES (274, 227, NULL, NULL, 'getSessionAppDefinedServices', 653, 654);
INSERT INTO public.acos VALUES (275, 227, NULL, NULL, 'getSessionAppDefinedDesktops', 655, 656);
INSERT INTO public.acos VALUES (276, 227, NULL, NULL, 'getSessionInitDesktops', 657, 658);
INSERT INTO public.acos VALUES (557, 1, NULL, NULL, 'Selectvaluesmetadonnees', 1352, 1387);
INSERT INTO public.acos VALUES (1662, 557, NULL, NULL, 'isAuthorized', 1385, 1386);
INSERT INTO public.acos VALUES (1265, 205, NULL, NULL, 'searchSansContact', 555, 556);
INSERT INTO public.acos VALUES (618, 596, NULL, NULL, 'ldebug', 1477, 1478);
INSERT INTO public.acos VALUES (277, 227, NULL, NULL, 'getSessionValDesktops', 659, 660);
INSERT INTO public.acos VALUES (278, 227, NULL, NULL, 'getSessionValeditDesktops', 661, 662);
INSERT INTO public.acos VALUES (279, 227, NULL, NULL, 'getSessionDispDesktops', 663, 664);
INSERT INTO public.acos VALUES (243, 227, NULL, NULL, 'setTache', 591, 592);
INSERT INTO public.acos VALUES (244, 227, NULL, NULL, 'setAffaire', 593, 594);
INSERT INTO public.acos VALUES (245, 227, NULL, NULL, 'reponse', 595, 596);
INSERT INTO public.acos VALUES (228, 227, NULL, NULL, 'view', 561, 562);
INSERT INTO public.acos VALUES (229, 227, NULL, NULL, 'edit', 563, 564);
INSERT INTO public.acos VALUES (230, 227, NULL, NULL, 'add', 565, 566);
INSERT INTO public.acos VALUES (231, 227, NULL, NULL, 'getInfos', 567, 568);
INSERT INTO public.acos VALUES (232, 227, NULL, NULL, 'getCircuit', 569, 570);
INSERT INTO public.acos VALUES (233, 227, NULL, NULL, 'getMeta', 571, 572);
INSERT INTO public.acos VALUES (234, 227, NULL, NULL, 'getTache', 573, 574);
INSERT INTO public.acos VALUES (280, 227, NULL, NULL, 'getSessionArchDesktops', 665, 666);
INSERT INTO public.acos VALUES (281, 227, NULL, NULL, 'getSessionAllDesktops', 667, 668);
INSERT INTO public.acos VALUES (283, 227, NULL, NULL, 'getAddParams', 671, 672);
INSERT INTO public.acos VALUES (284, 227, NULL, NULL, 'ldebug', 673, 674);
INSERT INTO public.acos VALUES (265, 227, NULL, NULL, 'sendBackToDisp', 635, 636);
INSERT INTO public.acos VALUES (266, 227, NULL, NULL, 'dispmultiplesend', 637, 638);
INSERT INTO public.acos VALUES (267, 227, NULL, NULL, 'historiqueGlobal', 639, 640);
INSERT INTO public.acos VALUES (268, 227, NULL, NULL, 'sendToGed', 641, 642);
INSERT INTO public.acos VALUES (260, 227, NULL, NULL, 'getDocuments', 625, 626);
INSERT INTO public.acos VALUES (1663, 574, NULL, NULL, 'isAuthorized', 1431, 1432);
INSERT INTO public.acos VALUES (937, 227, NULL, NULL, 'validEtapeLastCol', 675, 676);
INSERT INTO public.acos VALUES (916, 915, NULL, NULL, 'index', 2151, 2152);
INSERT INTO public.acos VALUES (983, 227, NULL, NULL, 'ajaxformvalid', 677, 678);
INSERT INTO public.acos VALUES (1038, 227, NULL, NULL, 'traitementlot', 679, 680);
INSERT INTO public.acos VALUES (1039, 227, NULL, NULL, 'execTraitementlot', 681, 682);
INSERT INTO public.acos VALUES (1040, 227, NULL, NULL, 'refusparlot', 683, 684);
INSERT INTO public.acos VALUES (1041, 227, NULL, NULL, 'sendcopylot', 685, 686);
INSERT INTO public.acos VALUES (1042, 227, NULL, NULL, 'detache', 687, 688);
INSERT INTO public.acos VALUES (1068, 227, NULL, NULL, 'add_contact', 689, 690);
INSERT INTO public.acos VALUES (1069, 227, NULL, NULL, 'edit_contact', 691, 692);
INSERT INTO public.acos VALUES (1072, 227, NULL, NULL, 'add_organisme', 693, 694);
INSERT INTO public.acos VALUES (1073, 227, NULL, NULL, 'edit_organisme', 695, 696);
INSERT INTO public.acos VALUES (1266, 205, NULL, NULL, 'isAuthorized', 557, 558);
INSERT INTO public.acos VALUES (457, 456, NULL, NULL, 'index', 1129, 1130);
INSERT INTO public.acos VALUES (1718, 596, NULL, NULL, 'getCircuit', 1479, 1480);
INSERT INTO public.acos VALUES (1076, 1075, NULL, NULL, 'index', 2361, 2362);
INSERT INTO public.acos VALUES (1235, 1213, NULL, NULL, 'isAuthorized', 2623, 2624);
INSERT INTO public.acos VALUES (1308, 227, NULL, NULL, 'deletelot', 697, 698);
INSERT INTO public.acos VALUES (620, 619, NULL, NULL, 'index', 1489, 1490);
INSERT INTO public.acos VALUES (623, 619, NULL, NULL, 'userAclCheck', 1491, 1492);
INSERT INTO public.acos VALUES (624, 619, NULL, NULL, 'getSessionInitServices', 1493, 1494);
INSERT INTO public.acos VALUES (625, 619, NULL, NULL, 'getSessionAppDefinedServices', 1495, 1496);
INSERT INTO public.acos VALUES (626, 619, NULL, NULL, 'getSessionAppDefinedDesktops', 1497, 1498);
INSERT INTO public.acos VALUES (627, 619, NULL, NULL, 'getSessionInitDesktops', 1499, 1500);
INSERT INTO public.acos VALUES (628, 619, NULL, NULL, 'getSessionValDesktops', 1501, 1502);
INSERT INTO public.acos VALUES (629, 619, NULL, NULL, 'getSessionValeditDesktops', 1503, 1504);
INSERT INTO public.acos VALUES (473, 456, NULL, NULL, 'getEnv', 1161, 1162);
INSERT INTO public.acos VALUES (1719, 596, NULL, NULL, 'getInformation', 1481, 1482);
INSERT INTO public.acos VALUES (914, 896, NULL, NULL, 'ldebug', 2145, 2146);
INSERT INTO public.acos VALUES (1309, 227, NULL, NULL, 'ajaxformtype', 699, 700);
INSERT INTO public.acos VALUES (630, 619, NULL, NULL, 'getSessionDispDesktops', 1505, 1506);
INSERT INTO public.acos VALUES (631, 619, NULL, NULL, 'getSessionArchDesktops', 1507, 1508);
INSERT INTO public.acos VALUES (632, 619, NULL, NULL, 'getSessionAllDesktops', 1509, 1510);
INSERT INTO public.acos VALUES (633, 619, NULL, NULL, 'getEnv', 1511, 1512);
INSERT INTO public.acos VALUES (634, 619, NULL, NULL, 'getAddParams', 1513, 1514);
INSERT INTO public.acos VALUES (635, 619, NULL, NULL, 'ldebug', 1515, 1516);
INSERT INTO public.acos VALUES (1720, 596, NULL, NULL, 'export', 1483, 1484);
INSERT INTO public.acos VALUES (1310, 227, NULL, NULL, 'detachelot', 701, 702);
INSERT INTO public.acos VALUES (1024, 619, NULL, NULL, 'indicateursGeo', 1517, 1518);
INSERT INTO public.acos VALUES (1025, 619, NULL, NULL, 'indicateursQuant', 1519, 1520);
INSERT INTO public.acos VALUES (1026, 619, NULL, NULL, 'indicateursQual', 1521, 1522);
INSERT INTO public.acos VALUES (1027, 619, NULL, NULL, 'exportqual', 1523, 1524);
INSERT INTO public.acos VALUES (1028, 619, NULL, NULL, 'exportqualattente', 1525, 1526);
INSERT INTO public.acos VALUES (1029, 619, NULL, NULL, 'exportquant', 1527, 1528);
INSERT INTO public.acos VALUES (1030, 619, NULL, NULL, 'exportquantmeta', 1529, 1530);
INSERT INTO public.acos VALUES (1031, 619, NULL, NULL, 'exportgeo', 1531, 1532);
INSERT INTO public.acos VALUES (1032, 619, NULL, NULL, 'getAdresses', 1533, 1534);
INSERT INTO public.acos VALUES (1721, 596, NULL, NULL, 'isAuthorized', 1485, 1486);
INSERT INTO public.acos VALUES (1307, 1287, NULL, NULL, 'isAuthorized', 2745, 2746);
INSERT INTO public.acos VALUES (619, 1, NULL, NULL, 'Statistiques', 1488, 1541);
INSERT INTO public.acos VALUES (647, 636, NULL, NULL, 'getSessionAppDefinedServices', 1563, 1564);
INSERT INTO public.acos VALUES (648, 636, NULL, NULL, 'getSessionAppDefinedDesktops', 1565, 1566);
INSERT INTO public.acos VALUES (649, 636, NULL, NULL, 'getSessionInitDesktops', 1567, 1568);
INSERT INTO public.acos VALUES (650, 636, NULL, NULL, 'getSessionValDesktops', 1569, 1570);
INSERT INTO public.acos VALUES (651, 636, NULL, NULL, 'getSessionValeditDesktops', 1571, 1572);
INSERT INTO public.acos VALUES (652, 636, NULL, NULL, 'getSessionDispDesktops', 1573, 1574);
INSERT INTO public.acos VALUES (653, 636, NULL, NULL, 'getSessionArchDesktops', 1575, 1576);
INSERT INTO public.acos VALUES (654, 636, NULL, NULL, 'getSessionAllDesktops', 1577, 1578);
INSERT INTO public.acos VALUES (655, 636, NULL, NULL, 'getEnv', 1579, 1580);
INSERT INTO public.acos VALUES (656, 636, NULL, NULL, 'getAddParams', 1581, 1582);
INSERT INTO public.acos VALUES (657, 636, NULL, NULL, 'ldebug', 1583, 1584);
INSERT INTO public.acos VALUES (1311, 227, NULL, NULL, 'isInCopy', 703, 704);
INSERT INTO public.acos VALUES (291, 285, NULL, NULL, 'rights', 729, 730);
INSERT INTO public.acos VALUES (534, 519, NULL, NULL, 'getEnv', 1295, 1296);
INSERT INTO public.acos VALUES (1722, 619, NULL, NULL, 'indicateursAgents', 1535, 1536);
INSERT INTO public.acos VALUES (1312, 227, NULL, NULL, 'getFluxPreview', 705, 706);
INSERT INTO public.acos VALUES (501, 495, NULL, NULL, 'delete', 1227, 1228);
INSERT INTO public.acos VALUES (1723, 619, NULL, NULL, 'exportagent', 1537, 1538);
INSERT INTO public.acos VALUES (791, 781, NULL, NULL, 'listLinkedModel', 1894, 1895);
INSERT INTO public.acos VALUES (1359, 1344, NULL, NULL, 'getEnv', 2817, 2818);
INSERT INTO public.acos VALUES (637, 636, NULL, NULL, 'add', 1543, 1544);
INSERT INTO public.acos VALUES (638, 636, NULL, NULL, 'edit', 1545, 1546);
INSERT INTO public.acos VALUES (639, 636, NULL, NULL, 'delete', 1547, 1548);
INSERT INTO public.acos VALUES (640, 636, NULL, NULL, 'checkTache', 1549, 1550);
INSERT INTO public.acos VALUES (641, 636, NULL, NULL, 'grabTache', 1551, 1552);
INSERT INTO public.acos VALUES (642, 636, NULL, NULL, 'releaseTache', 1553, 1554);
INSERT INTO public.acos VALUES (643, 636, NULL, NULL, 'getTaches', 1555, 1556);
INSERT INTO public.acos VALUES (644, 636, NULL, NULL, 'setFromTask', 1557, 1558);
INSERT INTO public.acos VALUES (645, 636, NULL, NULL, 'userAclCheck', 1559, 1560);
INSERT INTO public.acos VALUES (646, 636, NULL, NULL, 'getSessionInitServices', 1561, 1562);
INSERT INTO public.acos VALUES (1313, 227, NULL, NULL, 'updateContext', 707, 708);
INSERT INTO public.acos VALUES (1724, 619, NULL, NULL, 'isAuthorized', 1539, 1540);
INSERT INTO public.acos VALUES (636, 1, NULL, NULL, 'Taches', 1542, 1587);
INSERT INTO public.acos VALUES (659, 658, NULL, NULL, 'add', 1589, 1590);
INSERT INTO public.acos VALUES (660, 658, NULL, NULL, 'edit', 1591, 1592);
INSERT INTO public.acos VALUES (661, 658, NULL, NULL, 'delete', 1593, 1594);
INSERT INTO public.acos VALUES (662, 658, NULL, NULL, 'userAclCheck', 1595, 1596);
INSERT INTO public.acos VALUES (663, 658, NULL, NULL, 'getSessionInitServices', 1597, 1598);
INSERT INTO public.acos VALUES (664, 658, NULL, NULL, 'getSessionAppDefinedServices', 1599, 1600);
INSERT INTO public.acos VALUES (665, 658, NULL, NULL, 'getSessionAppDefinedDesktops', 1601, 1602);
INSERT INTO public.acos VALUES (666, 658, NULL, NULL, 'getSessionInitDesktops', 1603, 1604);
INSERT INTO public.acos VALUES (667, 658, NULL, NULL, 'getSessionValDesktops', 1605, 1606);
INSERT INTO public.acos VALUES (668, 658, NULL, NULL, 'getSessionValeditDesktops', 1607, 1608);
INSERT INTO public.acos VALUES (669, 658, NULL, NULL, 'getSessionDispDesktops', 1609, 1610);
INSERT INTO public.acos VALUES (670, 658, NULL, NULL, 'getSessionArchDesktops', 1611, 1612);
INSERT INTO public.acos VALUES (671, 658, NULL, NULL, 'getSessionAllDesktops', 1613, 1614);
INSERT INTO public.acos VALUES (672, 658, NULL, NULL, 'getEnv', 1615, 1616);
INSERT INTO public.acos VALUES (673, 658, NULL, NULL, 'getAddParams', 1617, 1618);
INSERT INTO public.acos VALUES (674, 658, NULL, NULL, 'ldebug', 1619, 1620);
INSERT INTO public.acos VALUES (675, 1, NULL, NULL, 'Users', 1624, 1703);
INSERT INTO public.acos VALUES (1, NULL, NULL, NULL, 'controllers', 1, 3696);
INSERT INTO public.acos VALUES (658, 1, NULL, NULL, 'Types', 1588, 1623);
INSERT INTO public.acos VALUES (676, 675, NULL, NULL, 'index', 1625, 1626);
INSERT INTO public.acos VALUES (677, 675, NULL, NULL, 'getUsers', 1627, 1628);
INSERT INTO public.acos VALUES (678, 675, NULL, NULL, 'add', 1629, 1630);
INSERT INTO public.acos VALUES (679, 675, NULL, NULL, 'ajaxformvalid', 1631, 1632);
INSERT INTO public.acos VALUES (680, 675, NULL, NULL, 'edit', 1633, 1634);
INSERT INTO public.acos VALUES (681, 675, NULL, NULL, 'delete', 1635, 1636);
INSERT INTO public.acos VALUES (1314, 227, NULL, NULL, 'sendlot', 709, 710);
INSERT INTO public.acos VALUES (1725, 636, NULL, NULL, 'isAuthorized', 1585, 1586);
INSERT INTO public.acos VALUES (500, 495, NULL, NULL, 'getFlux', 1225, 1226);
INSERT INTO public.acos VALUES (1437, 1420, NULL, NULL, 'getEnv', 2939, 2940);
INSERT INTO public.acos VALUES (1578, 1561, NULL, NULL, 'getEnv', 3199, 3200);
INSERT INTO public.acos VALUES (682, 675, NULL, NULL, 'setInfosPersos', 1637, 1638);
INSERT INTO public.acos VALUES (683, 675, NULL, NULL, 'getInfosCompte', 1639, 1640);
INSERT INTO public.acos VALUES (684, 675, NULL, NULL, 'editinfo', 1641, 1642);
INSERT INTO public.acos VALUES (685, 675, NULL, NULL, 'changemdp', 1643, 1644);
INSERT INTO public.acos VALUES (686, 675, NULL, NULL, 'rights', 1645, 1646);
INSERT INTO public.acos VALUES (687, 675, NULL, NULL, 'getRights', 1647, 1648);
INSERT INTO public.acos VALUES (688, 675, NULL, NULL, 'login', 1649, 1650);
INSERT INTO public.acos VALUES (689, 675, NULL, NULL, 'logout', 1651, 1652);
INSERT INTO public.acos VALUES (690, 675, NULL, NULL, 'isAway', 1653, 1654);
INSERT INTO public.acos VALUES (691, 675, NULL, NULL, 'mdplost', 1655, 1656);
INSERT INTO public.acos VALUES (692, 675, NULL, NULL, 'changenotif', 1657, 1658);
INSERT INTO public.acos VALUES (693, 675, NULL, NULL, 'userAclCheck', 1659, 1660);
INSERT INTO public.acos VALUES (694, 675, NULL, NULL, 'getSessionInitServices', 1661, 1662);
INSERT INTO public.acos VALUES (695, 675, NULL, NULL, 'getSessionAppDefinedServices', 1663, 1664);
INSERT INTO public.acos VALUES (696, 675, NULL, NULL, 'getSessionAppDefinedDesktops', 1665, 1666);
INSERT INTO public.acos VALUES (697, 675, NULL, NULL, 'getSessionInitDesktops', 1667, 1668);
INSERT INTO public.acos VALUES (698, 675, NULL, NULL, 'getSessionValDesktops', 1669, 1670);
INSERT INTO public.acos VALUES (699, 675, NULL, NULL, 'getSessionValeditDesktops', 1671, 1672);
INSERT INTO public.acos VALUES (700, 675, NULL, NULL, 'getSessionDispDesktops', 1673, 1674);
INSERT INTO public.acos VALUES (701, 675, NULL, NULL, 'getSessionArchDesktops', 1675, 1676);
INSERT INTO public.acos VALUES (704, 675, NULL, NULL, 'getAddParams', 1681, 1682);
INSERT INTO public.acos VALUES (705, 675, NULL, NULL, 'ldebug', 1683, 1684);
INSERT INTO public.acos VALUES (1064, 675, NULL, NULL, 'exportcsv', 1685, 1686);
INSERT INTO public.acos VALUES (1065, 675, NULL, NULL, 'cutCircuit', 1687, 1688);
INSERT INTO public.acos VALUES (1315, 227, NULL, NULL, 'sendToGrc', 711, 712);
INSERT INTO public.acos VALUES (1807, 658, NULL, NULL, 'isAuthorized', 1621, 1622);
INSERT INTO public.acos VALUES (1316, 227, NULL, NULL, 'getSms', 713, 714);
INSERT INTO public.acos VALUES (782, 781, NULL, NULL, 'index', 1876, 1877);
INSERT INTO public.acos VALUES (227, 1, NULL, NULL, 'Courriers', 560, 717);
INSERT INTO public.acos VALUES (286, 285, NULL, NULL, 'add', 719, 720);
INSERT INTO public.acos VALUES (287, 285, NULL, NULL, 'edit', 721, 722);
INSERT INTO public.acos VALUES (288, 285, NULL, NULL, 'setRights', 723, 724);
INSERT INTO public.acos VALUES (289, 285, NULL, NULL, 'setDeleg', 725, 726);
INSERT INTO public.acos VALUES (290, 285, NULL, NULL, 'getRights', 727, 728);
INSERT INTO public.acos VALUES (292, 285, NULL, NULL, 'delete', 731, 732);
INSERT INTO public.acos VALUES (293, 285, NULL, NULL, 'userAclCheck', 733, 734);
INSERT INTO public.acos VALUES (294, 285, NULL, NULL, 'getSessionInitServices', 735, 736);
INSERT INTO public.acos VALUES (295, 285, NULL, NULL, 'getSessionAppDefinedServices', 737, 738);
INSERT INTO public.acos VALUES (296, 285, NULL, NULL, 'getSessionAppDefinedDesktops', 739, 740);
INSERT INTO public.acos VALUES (297, 285, NULL, NULL, 'getSessionInitDesktops', 741, 742);
INSERT INTO public.acos VALUES (298, 285, NULL, NULL, 'getSessionValDesktops', 743, 744);
INSERT INTO public.acos VALUES (299, 285, NULL, NULL, 'getSessionValeditDesktops', 745, 746);
INSERT INTO public.acos VALUES (300, 285, NULL, NULL, 'getSessionDispDesktops', 747, 748);
INSERT INTO public.acos VALUES (301, 285, NULL, NULL, 'getSessionArchDesktops', 749, 750);
INSERT INTO public.acos VALUES (302, 285, NULL, NULL, 'getSessionAllDesktops', 751, 752);
INSERT INTO public.acos VALUES (303, 285, NULL, NULL, 'getEnv', 753, 754);
INSERT INTO public.acos VALUES (304, 285, NULL, NULL, 'getAddParams', 755, 756);
INSERT INTO public.acos VALUES (305, 285, NULL, NULL, 'ldebug', 757, 758);
INSERT INTO public.acos VALUES (1043, 285, NULL, NULL, 'setMain', 759, 760);
INSERT INTO public.acos VALUES (1317, 227, NULL, NULL, 'isAuthorized', 715, 716);
INSERT INTO public.acos VALUES (593, 574, NULL, NULL, 'getEnv', 1425, 1426);
INSERT INTO public.acos VALUES (868, 855, NULL, NULL, 'getEnv', 2054, 2055);
INSERT INTO public.acos VALUES (1338, 285, NULL, NULL, 'editDeleg', 761, 762);
INSERT INTO public.acos VALUES (757, 755, NULL, NULL, 'view', 1824, 1825);
INSERT INTO public.acos VALUES (1339, 285, NULL, NULL, 'deleteDeleg', 763, 764);
INSERT INTO public.acos VALUES (616, 596, NULL, NULL, 'getEnv', 1473, 1474);
INSERT INTO public.acos VALUES (1340, 285, NULL, NULL, 'nextFlux', 765, 766);
INSERT INTO public.acos VALUES (285, 1, NULL, NULL, 'Desktops', 718, 769);
INSERT INTO public.acos VALUES (307, 306, NULL, NULL, 'setMainDoc', 771, 772);
INSERT INTO public.acos VALUES (308, 306, NULL, NULL, 'upload', 773, 774);
INSERT INTO public.acos VALUES (309, 306, NULL, NULL, 'delete', 775, 776);
INSERT INTO public.acos VALUES (310, 306, NULL, NULL, 'download', 777, 778);
INSERT INTO public.acos VALUES (311, 306, NULL, NULL, 'getPreview', 779, 780);
INSERT INTO public.acos VALUES (312, 306, NULL, NULL, 'userAclCheck', 781, 782);
INSERT INTO public.acos VALUES (313, 306, NULL, NULL, 'getSessionInitServices', 783, 784);
INSERT INTO public.acos VALUES (314, 306, NULL, NULL, 'getSessionAppDefinedServices', 785, 786);
INSERT INTO public.acos VALUES (315, 306, NULL, NULL, 'getSessionAppDefinedDesktops', 787, 788);
INSERT INTO public.acos VALUES (316, 306, NULL, NULL, 'getSessionInitDesktops', 789, 790);
INSERT INTO public.acos VALUES (317, 306, NULL, NULL, 'getSessionValDesktops', 791, 792);
INSERT INTO public.acos VALUES (318, 306, NULL, NULL, 'getSessionValeditDesktops', 793, 794);
INSERT INTO public.acos VALUES (319, 306, NULL, NULL, 'getSessionDispDesktops', 795, 796);
INSERT INTO public.acos VALUES (320, 306, NULL, NULL, 'getSessionArchDesktops', 797, 798);
INSERT INTO public.acos VALUES (321, 306, NULL, NULL, 'getSessionAllDesktops', 799, 800);
INSERT INTO public.acos VALUES (323, 306, NULL, NULL, 'getAddParams', 803, 804);
INSERT INTO public.acos VALUES (324, 306, NULL, NULL, 'ldebug', 805, 806);
INSERT INTO public.acos VALUES (938, 306, NULL, NULL, 'getFichierScane', 807, 808);
INSERT INTO public.acos VALUES (939, 306, NULL, NULL, 'mergeFile', 809, 810);
INSERT INTO public.acos VALUES (940, 306, NULL, NULL, 'sousRepertoireMerge', 811, 812);
INSERT INTO public.acos VALUES (941, 306, NULL, NULL, 'nouveauFlux', 813, 814);
INSERT INTO public.acos VALUES (942, 306, NULL, NULL, 'getScanFilePreview', 815, 816);
INSERT INTO public.acos VALUES (943, 306, NULL, NULL, 'getFusionPreview', 817, 818);
INSERT INTO public.acos VALUES (1341, 285, NULL, NULL, 'isAuthorized', 767, 768);
INSERT INTO public.acos VALUES (1342, 987, NULL, NULL, 'getAllDesktopsByProfils', 2259, 2260);
INSERT INTO public.acos VALUES (1268, 1267, NULL, NULL, 'index', 2667, 2668);


--
-- Name: acos_id_seq; Type: SEQUENCE SET; Schema: public; Owner: webgfc
--

SELECT pg_catalog.setval('acos_id_seq', 1874, true);


--
-- Data for Name: aros; Type: TABLE DATA; Schema: public; Owner: webgfc
--

INSERT INTO aros VALUES (1, NULL, 'Profil', 1, 'Superadmin', 1, 2);
INSERT INTO aros VALUES (2, NULL, 'Profil', 2, 'Admin', 3, 6);
INSERT INTO aros VALUES (3, NULL, 'Profil', 3, 'Initiateur', 7, 160);
INSERT INTO aros VALUES (4, NULL, 'Profil', 4, 'Valideur', 161, 164);
INSERT INTO aros VALUES (5, NULL, 'Profil', 5, 'Valideur Editeur', 165, 214);
INSERT INTO aros VALUES (6, NULL, 'Profil', 6, 'Documentaliste', 215, 218);
INSERT INTO aros VALUES (7, NULL, 'Profil', 7, 'Aiguilleur', 219, 242);

-- Data for Name: aros_acos; Type: TABLE DATA; Schema: public; Owner: webgfc
--

INSERT INTO public.aros_acos VALUES (25595, 6, 345, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25596, 6, 456, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25597, 6, 253, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25598, 6, 2, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25599, 6, 619, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25600, 6, 205, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25601, 6, 1045, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25602, 6, 249, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25603, 6, 250, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25604, 6, 268, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25605, 6, 269, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25606, 6, 938, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25607, 6, 942, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25608, 6, 939, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25609, 6, 940, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25610, 6, 941, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25611, 6, 308, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25612, 6, 309, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25613, 6, 307, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25614, 6, 259, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25615, 6, 477, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25616, 6, 478, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25617, 6, 255, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25618, 6, 231, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25619, 6, 207, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25620, 6, 213, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25621, 6, 211, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25622, 6, 212, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25623, 6, 188, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25624, 6, 189, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25625, 6, 605, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25626, 6, 233, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25627, 6, 234, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25628, 6, 235, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25629, 6, 944, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25630, 6, 895, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25631, 6, 236, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25632, 6, 208, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25633, 6, 934, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25634, 6, 935, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25635, 6, 936, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25636, 6, 190, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25637, 6, 237, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25638, 6, 242, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25639, 6, 243, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (88, 3, 349, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (89, 3, 350, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (90, 3, 351, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (91, 3, 495, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (92, 3, 456, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (93, 3, 636, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (94, 3, 289, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (95, 3, 130, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (96, 3, 239, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (97, 3, 238, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (98, 3, 267, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (99, 3, 268, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (101, 3, 270, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (102, 3, 263, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (103, 3, 205, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (104, 3, 187, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (105, 3, 684, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (106, 3, 685, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (107, 3, 692, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (108, 3, 682, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (109, 3, 683, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (110, 3, 679, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (111, 3, 690, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (112, 3, 253, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (114, 3, 228, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (115, 3, 229, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (138, 3, 232, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (117, 3, 308, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (118, 3, 309, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (119, 3, 310, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (120, 3, 307, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (121, 3, 311, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (122, 3, 260, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (123, 3, 255, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (124, 3, 259, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (125, 3, 476, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (127, 3, 58, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (128, 3, 76, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (129, 3, 522, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (130, 3, 256, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (131, 3, 257, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (132, 3, 72, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (133, 3, 57, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (134, 3, 261, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (135, 3, 477, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (136, 3, 478, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (137, 3, 241, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (139, 3, 246, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (141, 3, 249, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (142, 3, 250, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (143, 3, 247, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (144, 3, 254, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (145, 3, 248, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (146, 3, 245, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (147, 3, 265, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (148, 3, 236, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (149, 3, 207, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (150, 3, 208, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (151, 3, 213, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (152, 3, 211, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (154, 3, 188, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (155, 3, 190, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (156, 3, 189, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (157, 3, 605, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (158, 3, 231, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (160, 3, 233, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (161, 3, 242, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (162, 3, 234, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (163, 3, 243, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (164, 3, 345, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (165, 3, 235, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (490, 7, 353, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (408, 7, 354, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (410, 7, 349, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (411, 7, 350, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (412, 7, 351, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (413, 7, 495, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (414, 7, 456, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (415, 7, 636, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (416, 7, 289, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (417, 7, 130, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (418, 7, 239, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (419, 7, 238, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (420, 7, 267, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (423, 7, 270, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18288, 7, 984, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18291, 3, 984, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18294, 7, 985, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18297, 3, 985, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18300, 7, 986, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18303, 3, 986, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18306, 7, 987, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18309, 3, 987, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18312, 7, 988, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18315, 3, 988, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18287, 2, 984, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18324, 7, 990, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18327, 3, 990, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18323, 2, 990, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18330, 7, 991, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18333, 3, 991, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18336, 7, 992, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18339, 3, 992, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18342, 7, 993, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18345, 3, 993, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18348, 7, 994, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18351, 3, 994, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18354, 7, 995, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18357, 3, 995, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18360, 7, 996, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18363, 3, 996, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18366, 7, 997, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18369, 3, 997, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18372, 7, 998, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18375, 3, 998, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18378, 7, 999, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18381, 3, 999, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18384, 7, 1000, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18387, 3, 1000, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18390, 7, 1001, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18393, 3, 1001, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18396, 7, 1002, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18399, 3, 1002, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18402, 7, 1003, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18405, 3, 1003, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18408, 7, 1004, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18411, 3, 1004, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18414, 7, 1005, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18417, 3, 1005, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18420, 7, 1006, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (489, 7, 244, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18423, 3, 1006, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (437, 7, 228, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (460, 7, 232, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (439, 7, 308, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (440, 7, 309, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (441, 7, 310, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (442, 7, 307, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (443, 7, 311, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (444, 7, 260, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (445, 7, 255, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (446, 7, 259, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (448, 7, 523, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (450, 7, 76, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (451, 7, 522, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (452, 7, 256, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (453, 7, 257, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (454, 7, 72, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (455, 7, 57, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (456, 7, 261, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (485, 7, 477, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (486, 7, 478, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (457, 7, 252, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (458, 7, 266, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (459, 7, 241, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (461, 7, 246, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (472, 7, 211, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (473, 7, 212, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (474, 7, 188, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (475, 7, 189, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (476, 7, 605, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (477, 7, 237, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (478, 7, 208, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (479, 7, 190, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (480, 7, 233, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (481, 7, 242, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (482, 7, 234, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (483, 7, 243, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (487, 7, 235, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (488, 7, 236, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (434, 7, 253, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18426, 7, 1007, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18429, 3, 1007, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18432, 7, 1008, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18435, 3, 1008, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18438, 7, 1009, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18441, 3, 1009, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18444, 7, 1010, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18447, 3, 1010, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18450, 7, 1011, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18453, 3, 1011, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18456, 7, 1012, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18459, 3, 1012, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18462, 7, 1013, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18465, 3, 1013, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18468, 7, 1014, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18471, 3, 1014, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18474, 7, 1015, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18477, 3, 1015, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18480, 7, 1016, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18483, 3, 1016, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18486, 7, 1017, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18489, 3, 1017, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18492, 7, 1018, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18495, 3, 1018, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18498, 7, 1019, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18501, 3, 1019, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18504, 7, 1020, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18507, 3, 1020, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18510, 7, 1021, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18513, 3, 1021, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18516, 7, 1022, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18519, 3, 1022, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18522, 7, 1023, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18525, 3, 1023, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18528, 7, 1024, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18531, 3, 1024, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18534, 7, 1025, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18537, 3, 1025, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18540, 7, 1026, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18543, 3, 1026, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18546, 7, 1027, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18549, 3, 1027, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18552, 7, 1028, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18555, 3, 1028, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18558, 7, 1029, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18561, 3, 1029, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18564, 7, 1030, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18567, 3, 1030, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18570, 7, 1031, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (14341, 3, 983, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (14343, 7, 983, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18573, 3, 1031, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18576, 7, 1032, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18579, 3, 1032, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (438, 7, 258, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (116, 3, 258, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19794, 7, 1033, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19796, 3, 1033, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19800, 7, 1034, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19802, 3, 1034, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19806, 7, 1035, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19808, 3, 1035, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19812, 7, 1036, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19814, 3, 1036, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19818, 7, 1037, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19820, 3, 1037, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19792, 2, 1033, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19798, 2, 1034, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19804, 2, 1035, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19810, 2, 1036, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19816, 2, 1037, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19854, 7, 1043, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19856, 3, 1043, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19860, 7, 1044, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19862, 3, 1044, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19872, 7, 1046, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19874, 3, 1046, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19834, 2, 1040, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19840, 2, 1041, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8803, 3, 938, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8804, 3, 939, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8805, 3, 940, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8806, 3, 941, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8807, 3, 943, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19850, 3, 1042, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19826, 3, 1038, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19832, 3, 1039, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19838, 3, 1040, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19844, 3, 1041, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8809, 3, 935, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8810, 3, 936, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (159, 3, 237, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19868, 3, 1045, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (8811, 3, 944, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (8812, 3, 895, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (409, 7, 352, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (422, 7, 269, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (435, 7, 230, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8833, 7, 938, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8834, 7, 939, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8835, 7, 940, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8836, 7, 941, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8837, 7, 943, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (449, 7, 58, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19848, 7, 1042, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19824, 7, 1038, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19836, 7, 1040, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19842, 7, 1041, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (462, 7, 251, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (464, 7, 250, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (465, 7, 247, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (466, 7, 254, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (467, 7, 248, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (468, 7, 245, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (469, 7, 231, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (470, 7, 207, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (471, 7, 213, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8838, 7, 934, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8840, 7, 936, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19866, 7, 1045, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8841, 7, 944, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (8842, 7, 895, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (19822, 2, 1038, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19878, 7, 1047, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19880, 3, 1047, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19884, 7, 1048, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19886, 3, 1048, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19890, 7, 1049, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19892, 3, 1049, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19896, 7, 1050, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19898, 3, 1050, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19902, 7, 1051, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19904, 3, 1051, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19908, 7, 1052, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19910, 3, 1052, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19914, 7, 1053, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19916, 3, 1053, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19920, 7, 1054, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19922, 3, 1054, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19926, 7, 1055, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19928, 3, 1055, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19932, 7, 1056, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19934, 3, 1056, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19938, 7, 1057, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19940, 3, 1057, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19944, 7, 1058, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19946, 3, 1058, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19950, 7, 1059, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19952, 3, 1059, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19956, 7, 1060, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19958, 3, 1060, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19962, 7, 1061, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19964, 3, 1061, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19968, 7, 1062, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19970, 3, 1062, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19974, 7, 1063, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19976, 3, 1063, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19980, 7, 1064, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19982, 3, 1064, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19986, 7, 1065, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19988, 3, 1065, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19852, 2, 1043, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19858, 2, 1044, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19870, 2, 1046, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19876, 2, 1047, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19882, 2, 1048, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19888, 2, 1049, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19894, 2, 1050, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19900, 2, 1051, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19906, 2, 1052, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19912, 2, 1053, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19918, 2, 1054, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19924, 2, 1055, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19930, 2, 1056, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19936, 2, 1057, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19942, 2, 1058, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19948, 2, 1059, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19954, 2, 1060, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20067, 7, 1066, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20069, 3, 1066, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20079, 7, 1068, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20081, 3, 1068, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20085, 7, 1069, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20087, 3, 1069, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20065, 2, 1066, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20077, 2, 1068, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20083, 2, 1069, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20208, 1, 2, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18293, 2, 985, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18299, 2, 986, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18305, 2, 987, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18311, 2, 988, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18329, 2, 991, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18335, 2, 992, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18341, 2, 993, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18347, 2, 994, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18353, 2, 995, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18359, 2, 996, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18365, 2, 997, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18371, 2, 998, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18377, 2, 999, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18383, 2, 1000, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18389, 2, 1001, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18395, 2, 1002, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18401, 2, 1003, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18407, 2, 1004, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18413, 2, 1005, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18419, 2, 1006, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18425, 2, 1007, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20336, 7, 1071, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20338, 3, 1071, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18431, 2, 1008, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18437, 2, 1009, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18443, 2, 1010, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18449, 2, 1011, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18455, 2, 1012, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18461, 2, 1013, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18467, 2, 1014, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18473, 2, 1015, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18479, 2, 1016, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18485, 2, 1017, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18491, 2, 1018, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18497, 2, 1019, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18503, 2, 1020, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18509, 2, 1021, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18515, 2, 1022, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18521, 2, 1023, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18527, 2, 1024, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18533, 2, 1025, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18539, 2, 1026, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18545, 2, 1027, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18551, 2, 1028, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18557, 2, 1029, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18563, 2, 1030, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18569, 2, 1031, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (14340, 2, 983, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18575, 2, 1032, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19960, 2, 1061, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19966, 2, 1062, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19972, 2, 1063, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19978, 2, 1064, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19984, 2, 1065, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20610, 1, 1071, '1', '1', '1', '0');
INSERT INTO public.aros_acos VALUES (20724, 2, 1072, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20726, 7, 1072, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20728, 3, 1072, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20730, 2, 1073, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20732, 7, 1073, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20734, 3, 1073, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20736, 2, 1074, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20738, 7, 1074, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20740, 3, 1074, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20743, 3, 1075, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20745, 7, 1075, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20748, 2, 1076, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20749, 3, 1076, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20751, 7, 1076, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (3, 2, 105, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (4, 2, 729, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (5, 2, 368, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (6, 2, 675, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (7, 2, 227, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8, 2, 306, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (10, 2, 519, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (11, 2, 476, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (12, 2, 72, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (13, 2, 574, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (14, 2, 596, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (15, 2, 658, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (17, 2, 636, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18, 2, 437, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19, 2, 325, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20, 2, 21, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21, 2, 456, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22, 2, 409, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23, 2, 391, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24, 2, 423, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25, 2, 205, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (26, 2, 187, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20334, 2, 1071, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (28, 2, 285, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (30, 2, 161, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (31, 2, 268, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (32, 2, 269, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8780, 2, 537, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8782, 2, 945, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8783, 2, 896, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8784, 2, 915, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20742, 2, 1075, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (33, 2, 230, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (34, 2, 228, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (35, 2, 229, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (36, 2, 253, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (58, 2, 232, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (38, 2, 308, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (39, 2, 309, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (40, 2, 310, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (41, 2, 307, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (42, 2, 311, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8785, 2, 938, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8786, 2, 939, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8787, 2, 940, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8788, 2, 941, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8789, 2, 943, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (43, 2, 260, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (45, 2, 259, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (46, 2, 523, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (47, 2, 58, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (48, 2, 76, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (49, 2, 522, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (50, 2, 256, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (51, 2, 257, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (52, 2, 270, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (53, 2, 57, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (54, 2, 261, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (55, 2, 477, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (56, 2, 478, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (59, 2, 246, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (60, 2, 251, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (61, 2, 249, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (62, 2, 250, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (63, 2, 247, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (64, 2, 254, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (65, 2, 248, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (66, 2, 245, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (67, 2, 265, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (68, 2, 236, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (69, 2, 207, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (70, 2, 208, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (72, 2, 211, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8790, 2, 934, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8791, 2, 935, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8792, 2, 936, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (73, 2, 212, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (74, 2, 188, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (75, 2, 190, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (76, 2, 189, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (77, 2, 605, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (78, 2, 231, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (79, 2, 237, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (80, 2, 233, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (82, 2, 234, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (83, 2, 243, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (84, 2, 345, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (1, 2, 90, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20754, 2, 1077, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20755, 3, 1077, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20757, 7, 1077, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20760, 2, 1078, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20761, 3, 1078, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20763, 7, 1078, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20766, 2, 1079, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20767, 3, 1079, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20769, 7, 1079, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20772, 2, 1080, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20773, 3, 1080, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20775, 7, 1080, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20778, 2, 1081, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20779, 3, 1081, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20781, 7, 1081, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20784, 2, 1082, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20785, 3, 1082, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20787, 7, 1082, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20790, 2, 1083, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20791, 3, 1083, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20793, 7, 1083, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20796, 2, 1084, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20797, 3, 1084, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20799, 7, 1084, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20802, 2, 1085, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20803, 3, 1085, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20805, 7, 1085, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20808, 2, 1086, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20809, 3, 1086, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20811, 7, 1086, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20814, 2, 1087, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20815, 3, 1087, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20817, 7, 1087, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20820, 2, 1088, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20821, 3, 1088, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20823, 7, 1088, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20826, 2, 1089, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20827, 3, 1089, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20829, 7, 1089, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20832, 2, 1090, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20833, 3, 1090, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20835, 7, 1090, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20838, 2, 1091, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20839, 3, 1091, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20841, 7, 1091, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20845, 2, 1092, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20846, 3, 1092, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20850, 7, 1092, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20851, 2, 1093, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20852, 3, 1093, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20856, 7, 1093, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20857, 2, 1094, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20858, 3, 1094, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20862, 7, 1094, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20863, 2, 1095, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20864, 3, 1095, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20868, 7, 1095, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20869, 2, 1096, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20870, 3, 1096, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20874, 7, 1096, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20875, 2, 1097, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20876, 3, 1097, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20880, 7, 1097, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20881, 2, 1098, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20882, 3, 1098, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20886, 7, 1098, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20887, 2, 1099, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20888, 3, 1099, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20892, 7, 1099, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20893, 2, 1100, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20894, 3, 1100, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20898, 7, 1100, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20899, 2, 1101, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20900, 3, 1101, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20904, 7, 1101, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20302, 7, 2, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20309, 3, 2, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (20905, 2, 1102, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20906, 3, 1102, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20910, 7, 1102, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20911, 2, 1103, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20912, 3, 1103, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20916, 7, 1103, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20917, 2, 1104, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20918, 3, 1104, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20922, 7, 1104, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20923, 2, 1105, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20924, 3, 1105, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20928, 7, 1105, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20929, 2, 1106, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20930, 3, 1106, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20934, 7, 1106, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20935, 2, 1107, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20936, 3, 1107, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20940, 7, 1107, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20941, 2, 1108, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20942, 3, 1108, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20946, 7, 1108, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20947, 2, 1109, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20948, 3, 1109, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20952, 7, 1109, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20953, 2, 1110, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20954, 3, 1110, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20958, 7, 1110, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20959, 2, 1111, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20960, 3, 1111, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20964, 7, 1111, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20965, 2, 1112, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20966, 3, 1112, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20970, 7, 1112, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20971, 2, 1113, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20972, 3, 1113, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20976, 7, 1113, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20977, 2, 1114, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20978, 3, 1114, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20982, 7, 1114, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20983, 2, 1115, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20984, 3, 1115, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20988, 7, 1115, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20989, 2, 1116, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20990, 3, 1116, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20994, 7, 1116, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20995, 2, 1117, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20996, 3, 1117, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21000, 7, 1117, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21001, 2, 1118, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21002, 3, 1118, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21006, 7, 1118, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21007, 2, 1119, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21008, 3, 1119, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21012, 7, 1119, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21013, 2, 1120, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21014, 3, 1120, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21018, 7, 1120, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21019, 2, 1121, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21020, 3, 1121, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21024, 7, 1121, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21025, 2, 1122, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21026, 3, 1122, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21030, 7, 1122, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21031, 2, 1123, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21032, 3, 1123, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21036, 7, 1123, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21037, 2, 1124, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21038, 3, 1124, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21042, 7, 1124, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21043, 2, 1125, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21044, 3, 1125, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21048, 7, 1125, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21049, 2, 1126, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21050, 3, 1126, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21054, 7, 1126, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21055, 2, 1127, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21056, 3, 1127, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21060, 7, 1127, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21061, 2, 1128, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21062, 3, 1128, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21066, 7, 1128, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21067, 2, 1129, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21068, 3, 1129, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21072, 7, 1129, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21073, 2, 1130, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21074, 3, 1130, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21078, 7, 1130, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21079, 2, 1131, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21080, 3, 1131, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21084, 7, 1131, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21085, 2, 1132, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21086, 3, 1132, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21090, 7, 1132, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21091, 2, 1133, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21092, 3, 1133, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21096, 7, 1133, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21097, 2, 1134, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21098, 3, 1134, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21102, 7, 1134, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21103, 2, 1135, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21104, 3, 1135, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21108, 7, 1135, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21109, 2, 1136, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21110, 3, 1136, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21114, 7, 1136, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21115, 2, 1137, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21116, 3, 1137, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21120, 7, 1137, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21121, 2, 1138, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21122, 3, 1138, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21126, 7, 1138, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21127, 2, 1139, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21128, 3, 1139, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21132, 7, 1139, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21133, 2, 1140, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21134, 3, 1140, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21138, 7, 1140, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21139, 2, 1141, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21140, 3, 1141, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21144, 7, 1141, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21145, 2, 1142, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21146, 3, 1142, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21150, 7, 1142, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21151, 2, 1143, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21152, 3, 1143, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21156, 7, 1143, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21157, 2, 1144, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21158, 3, 1144, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21162, 7, 1144, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21163, 2, 1145, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21164, 3, 1145, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21168, 7, 1145, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21169, 2, 1146, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21170, 3, 1146, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21174, 7, 1146, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21175, 2, 1147, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21176, 3, 1147, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21180, 7, 1147, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21181, 2, 1148, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21182, 3, 1148, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21186, 7, 1148, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21187, 2, 1149, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21188, 3, 1149, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21192, 7, 1149, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21193, 2, 1150, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21194, 3, 1150, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21198, 7, 1150, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21199, 2, 1151, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21200, 3, 1151, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21204, 7, 1151, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21205, 2, 1152, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21206, 3, 1152, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21210, 7, 1152, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21211, 2, 1153, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21212, 3, 1153, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21216, 7, 1153, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21217, 2, 1154, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21218, 3, 1154, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21222, 7, 1154, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21223, 2, 1155, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21224, 3, 1155, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21228, 7, 1155, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21229, 2, 1156, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21230, 3, 1156, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21234, 7, 1156, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21235, 2, 1157, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21236, 3, 1157, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21240, 7, 1157, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21241, 2, 1158, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21242, 3, 1158, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21246, 7, 1158, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21247, 2, 1159, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21248, 3, 1159, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21252, 7, 1159, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21253, 2, 1160, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21254, 3, 1160, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21258, 7, 1160, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21259, 2, 1161, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21260, 3, 1161, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21264, 7, 1161, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21265, 2, 1162, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21266, 3, 1162, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21270, 7, 1162, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21271, 2, 1163, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21272, 3, 1163, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21276, 7, 1163, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21277, 2, 1164, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21278, 3, 1164, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21282, 7, 1164, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21283, 2, 1165, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21284, 3, 1165, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21288, 7, 1165, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21289, 2, 1166, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21290, 3, 1166, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21294, 7, 1166, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21295, 2, 1167, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21296, 3, 1167, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21300, 7, 1167, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21301, 2, 1168, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21302, 3, 1168, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21306, 7, 1168, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21307, 2, 1169, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21308, 3, 1169, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21312, 7, 1169, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21313, 2, 1170, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21314, 3, 1170, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21318, 7, 1170, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21319, 2, 1171, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21320, 3, 1171, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21324, 7, 1171, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21325, 2, 1172, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21326, 3, 1172, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21330, 7, 1172, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21331, 2, 1173, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21332, 3, 1173, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21336, 7, 1173, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21337, 2, 1174, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21338, 3, 1174, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21342, 7, 1174, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21343, 2, 1175, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21344, 3, 1175, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21348, 7, 1175, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21349, 2, 1176, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21350, 3, 1176, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21354, 7, 1176, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21355, 2, 1177, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21356, 3, 1177, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21360, 7, 1177, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21361, 2, 1178, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21362, 3, 1178, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21366, 7, 1178, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21367, 2, 1179, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21368, 3, 1179, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21372, 7, 1179, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21373, 2, 1180, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21374, 3, 1180, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21378, 7, 1180, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21379, 2, 1181, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21380, 3, 1181, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21384, 7, 1181, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21385, 2, 1182, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21386, 3, 1182, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21390, 7, 1182, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21391, 2, 1183, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21392, 3, 1183, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21396, 7, 1183, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21397, 2, 1184, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21398, 3, 1184, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21402, 7, 1184, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21403, 2, 1185, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21404, 3, 1185, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21408, 7, 1185, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21409, 2, 1186, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21410, 3, 1186, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21414, 7, 1186, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21415, 2, 1187, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21416, 3, 1187, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21420, 7, 1187, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21421, 2, 1188, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21422, 3, 1188, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21426, 7, 1188, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21427, 2, 1189, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21428, 3, 1189, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21432, 7, 1189, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21433, 2, 1190, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21434, 3, 1190, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21438, 7, 1190, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21439, 2, 1191, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21440, 3, 1191, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21444, 7, 1191, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21445, 2, 1192, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21446, 3, 1192, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21450, 7, 1192, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21451, 2, 1193, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21452, 3, 1193, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21456, 7, 1193, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21458, 3, 1194, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21462, 7, 1194, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21463, 2, 1195, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21464, 3, 1195, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21468, 7, 1195, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21469, 2, 1196, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21470, 3, 1196, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21474, 7, 1196, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21475, 2, 1197, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21476, 3, 1197, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21480, 7, 1197, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21481, 2, 1198, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21482, 3, 1198, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21486, 7, 1198, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21487, 2, 1199, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21488, 3, 1199, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21492, 7, 1199, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21493, 2, 1200, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21494, 3, 1200, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21498, 7, 1200, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21499, 2, 1201, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21500, 3, 1201, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21504, 7, 1201, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21505, 2, 1202, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21506, 3, 1202, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21510, 7, 1202, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21511, 2, 1203, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21512, 3, 1203, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21516, 7, 1203, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21517, 2, 1204, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21518, 3, 1204, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21522, 7, 1204, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21523, 2, 1205, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21524, 3, 1205, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21528, 7, 1205, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21529, 2, 1206, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21530, 3, 1206, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21534, 7, 1206, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21535, 2, 1207, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21536, 3, 1207, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21540, 7, 1207, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21541, 2, 1208, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21542, 3, 1208, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21546, 7, 1208, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21547, 2, 1209, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21548, 3, 1209, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21552, 7, 1209, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21553, 2, 1210, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21554, 3, 1210, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21558, 7, 1210, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21559, 2, 1211, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21560, 3, 1211, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21564, 7, 1211, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21565, 2, 1212, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21566, 3, 1212, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21570, 7, 1212, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21571, 2, 1213, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21572, 3, 1213, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21576, 7, 1213, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21577, 2, 1214, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21578, 3, 1214, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21582, 7, 1214, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21583, 2, 1215, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21584, 3, 1215, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21588, 7, 1215, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21589, 2, 1216, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21590, 3, 1216, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21594, 7, 1216, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21595, 2, 1217, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21596, 3, 1217, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21600, 7, 1217, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21601, 2, 1218, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21602, 3, 1218, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21606, 7, 1218, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21607, 2, 1219, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21608, 3, 1219, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21612, 7, 1219, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21613, 2, 1220, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21614, 3, 1220, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21618, 7, 1220, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21619, 2, 1221, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21620, 3, 1221, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21624, 7, 1221, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21625, 2, 1222, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21626, 3, 1222, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21630, 7, 1222, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21631, 2, 1223, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21632, 3, 1223, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21636, 7, 1223, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21637, 2, 1224, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21638, 3, 1224, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21642, 7, 1224, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21643, 2, 1225, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21644, 3, 1225, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21648, 7, 1225, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21649, 2, 1226, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21650, 3, 1226, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21654, 7, 1226, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21655, 2, 1227, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21656, 3, 1227, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21660, 7, 1227, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21661, 2, 1228, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21662, 3, 1228, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21666, 7, 1228, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21667, 2, 1229, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21668, 3, 1229, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21672, 7, 1229, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21673, 2, 1230, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21674, 3, 1230, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21678, 7, 1230, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21679, 2, 1231, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21680, 3, 1231, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21684, 7, 1231, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21685, 2, 1232, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21686, 3, 1232, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21690, 7, 1232, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21691, 2, 1233, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21692, 3, 1233, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21696, 7, 1233, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21697, 2, 1234, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21698, 3, 1234, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21702, 7, 1234, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21703, 2, 1235, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21704, 3, 1235, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21708, 7, 1235, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21709, 2, 1236, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21710, 3, 1236, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21714, 7, 1236, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21715, 2, 1237, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21716, 3, 1237, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21720, 7, 1237, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21721, 2, 1238, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21722, 3, 1238, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21726, 7, 1238, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21727, 2, 1239, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21728, 3, 1239, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21732, 7, 1239, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21733, 2, 1240, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21734, 3, 1240, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21738, 7, 1240, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21739, 2, 1241, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21740, 3, 1241, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21744, 7, 1241, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21745, 2, 1242, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21746, 3, 1242, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21750, 7, 1242, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21751, 2, 1243, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21752, 3, 1243, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21756, 7, 1243, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21757, 2, 1244, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21758, 3, 1244, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21762, 7, 1244, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21763, 2, 1245, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21764, 3, 1245, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21768, 7, 1245, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21769, 2, 1246, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21770, 3, 1246, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21774, 7, 1246, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21775, 2, 1247, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21776, 3, 1247, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21780, 7, 1247, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21781, 2, 1248, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21782, 3, 1248, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21786, 7, 1248, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21787, 2, 1249, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21788, 3, 1249, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21792, 7, 1249, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21793, 2, 1250, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21794, 3, 1250, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21798, 7, 1250, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21799, 2, 1251, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21800, 3, 1251, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21804, 7, 1251, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21805, 2, 1252, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21806, 3, 1252, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21810, 7, 1252, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21811, 2, 1253, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21812, 3, 1253, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21816, 7, 1253, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21817, 2, 1254, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21818, 3, 1254, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21822, 7, 1254, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21823, 2, 1255, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21824, 3, 1255, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21828, 7, 1255, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21829, 2, 1256, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21830, 3, 1256, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21834, 7, 1256, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21835, 2, 1257, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21836, 3, 1257, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21840, 7, 1257, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21841, 2, 1258, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21842, 3, 1258, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21846, 7, 1258, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21847, 2, 1259, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21848, 3, 1259, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21852, 7, 1259, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21853, 2, 1260, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21854, 3, 1260, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21858, 7, 1260, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21859, 2, 1261, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21860, 3, 1261, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21864, 7, 1261, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21865, 2, 1262, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21866, 3, 1262, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21870, 7, 1262, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21871, 2, 1263, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21872, 3, 1263, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21876, 7, 1263, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21877, 2, 1264, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21878, 3, 1264, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21882, 7, 1264, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21883, 2, 1265, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21884, 3, 1265, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21888, 7, 1265, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21889, 2, 1266, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21890, 3, 1266, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21894, 7, 1266, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21895, 2, 1267, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21896, 3, 1267, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21900, 7, 1267, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21901, 2, 1268, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21902, 3, 1268, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21906, 7, 1268, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21907, 2, 1269, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21908, 3, 1269, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21912, 7, 1269, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21913, 2, 1270, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21914, 3, 1270, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21918, 7, 1270, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21919, 2, 1271, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21920, 3, 1271, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21924, 7, 1271, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21925, 2, 1272, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21926, 3, 1272, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21930, 7, 1272, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21931, 2, 1273, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21932, 3, 1273, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21936, 7, 1273, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21937, 2, 1274, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21938, 3, 1274, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21942, 7, 1274, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21943, 2, 1275, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21944, 3, 1275, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21948, 7, 1275, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21949, 2, 1276, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21950, 3, 1276, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21954, 7, 1276, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21955, 2, 1277, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21956, 3, 1277, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21960, 7, 1277, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21961, 2, 1278, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21962, 3, 1278, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21966, 7, 1278, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21967, 2, 1279, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21968, 3, 1279, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21972, 7, 1279, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21973, 2, 1280, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21974, 3, 1280, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21978, 7, 1280, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21979, 2, 1281, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21980, 3, 1281, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21984, 7, 1281, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21985, 2, 1282, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21986, 3, 1282, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21990, 7, 1282, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21991, 2, 1283, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21992, 3, 1283, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21996, 7, 1283, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21997, 2, 1284, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21998, 3, 1284, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22002, 7, 1284, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22003, 2, 1285, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22004, 3, 1285, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22008, 7, 1285, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22009, 2, 1286, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22010, 3, 1286, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22014, 7, 1286, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22015, 2, 1287, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22016, 3, 1287, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22020, 7, 1287, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22021, 2, 1288, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22022, 3, 1288, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22026, 7, 1288, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22027, 2, 1289, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22028, 3, 1289, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22032, 7, 1289, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22033, 2, 1290, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22034, 3, 1290, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22038, 7, 1290, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22039, 2, 1291, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22040, 3, 1291, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22044, 7, 1291, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22045, 2, 1292, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22046, 3, 1292, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22050, 7, 1292, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22051, 2, 1293, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22052, 3, 1293, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22056, 7, 1293, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22057, 2, 1294, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22058, 3, 1294, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22062, 7, 1294, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22063, 2, 1295, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22064, 3, 1295, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22068, 7, 1295, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22069, 2, 1296, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22070, 3, 1296, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22074, 7, 1296, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22075, 2, 1297, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22076, 3, 1297, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22080, 7, 1297, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22081, 2, 1298, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22082, 3, 1298, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22086, 7, 1298, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22087, 2, 1299, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22088, 3, 1299, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22092, 7, 1299, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22093, 2, 1300, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22094, 3, 1300, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22098, 7, 1300, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22099, 2, 1301, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22100, 3, 1301, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22104, 7, 1301, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22105, 2, 1302, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22106, 3, 1302, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22110, 7, 1302, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22111, 2, 1303, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22112, 3, 1303, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22116, 7, 1303, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22117, 2, 1304, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22118, 3, 1304, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22122, 7, 1304, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22123, 2, 1305, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22124, 3, 1305, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22128, 7, 1305, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22129, 2, 1306, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22130, 3, 1306, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22134, 7, 1306, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22135, 2, 1307, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22136, 3, 1307, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22140, 7, 1307, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22147, 2, 1309, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22148, 3, 1309, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22152, 7, 1309, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22159, 2, 1311, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22160, 3, 1311, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22142, 3, 1308, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22158, 7, 1310, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22146, 7, 1308, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22141, 2, 1308, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22164, 7, 1311, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22165, 2, 1312, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22166, 3, 1312, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22170, 7, 1312, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22171, 2, 1313, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22172, 3, 1313, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22176, 7, 1313, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22177, 2, 1314, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22178, 3, 1314, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22182, 7, 1314, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22183, 2, 1315, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22184, 3, 1315, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22188, 7, 1315, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22189, 2, 1316, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22190, 3, 1316, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22194, 7, 1316, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22195, 2, 1317, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22196, 3, 1317, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22200, 7, 1317, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22201, 2, 1318, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22202, 3, 1318, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22206, 7, 1318, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22207, 2, 1319, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22208, 3, 1319, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22212, 7, 1319, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22213, 2, 1320, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22214, 3, 1320, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22218, 7, 1320, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22219, 2, 1321, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22220, 3, 1321, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22224, 7, 1321, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22225, 2, 1322, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22226, 3, 1322, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22230, 7, 1322, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22231, 2, 1323, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22232, 3, 1323, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22236, 7, 1323, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22237, 2, 1324, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22238, 3, 1324, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22242, 7, 1324, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22243, 2, 1325, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22244, 3, 1325, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22248, 7, 1325, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22249, 2, 1326, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22250, 3, 1326, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22254, 7, 1326, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22255, 2, 1327, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22256, 3, 1327, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22260, 7, 1327, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22261, 2, 1328, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22262, 3, 1328, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22266, 7, 1328, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22267, 2, 1329, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22268, 3, 1329, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22272, 7, 1329, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22273, 2, 1330, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22274, 3, 1330, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22278, 7, 1330, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22279, 2, 1331, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22280, 3, 1331, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22284, 7, 1331, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22285, 2, 1332, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22286, 3, 1332, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22290, 7, 1332, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22291, 2, 1333, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22292, 3, 1333, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22296, 7, 1333, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22297, 2, 1334, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22298, 3, 1334, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22302, 7, 1334, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22303, 2, 1335, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22304, 3, 1335, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22308, 7, 1335, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22309, 2, 1336, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22310, 3, 1336, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22314, 7, 1336, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22315, 2, 1337, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22316, 3, 1337, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22320, 7, 1337, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22321, 2, 1338, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22322, 3, 1338, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22326, 7, 1338, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22327, 2, 1339, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22328, 3, 1339, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22332, 7, 1339, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22333, 2, 1340, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22334, 3, 1340, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22338, 7, 1340, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22339, 2, 1341, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22340, 3, 1341, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22344, 7, 1341, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22345, 2, 1342, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22346, 3, 1342, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22350, 7, 1342, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22351, 2, 1343, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22352, 3, 1343, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22356, 7, 1343, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22357, 2, 1344, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22358, 3, 1344, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22362, 7, 1344, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22363, 2, 1345, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22364, 3, 1345, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22368, 7, 1345, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22369, 2, 1346, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22370, 3, 1346, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22374, 7, 1346, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22375, 2, 1347, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22376, 3, 1347, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22380, 7, 1347, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22381, 2, 1348, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22382, 3, 1348, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22386, 7, 1348, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22387, 2, 1349, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22388, 3, 1349, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22392, 7, 1349, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22393, 2, 1350, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22394, 3, 1350, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22398, 7, 1350, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22399, 2, 1351, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22400, 3, 1351, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22404, 7, 1351, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22405, 2, 1352, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22406, 3, 1352, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22410, 7, 1352, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22411, 2, 1353, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22412, 3, 1353, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22416, 7, 1353, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22417, 2, 1354, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22418, 3, 1354, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22422, 7, 1354, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22423, 2, 1355, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22424, 3, 1355, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22428, 7, 1355, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22429, 2, 1356, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22430, 3, 1356, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22434, 7, 1356, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22435, 2, 1357, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22436, 3, 1357, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22440, 7, 1357, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22441, 2, 1358, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22442, 3, 1358, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22446, 7, 1358, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22447, 2, 1359, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22448, 3, 1359, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22452, 7, 1359, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22453, 2, 1360, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22454, 3, 1360, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22458, 7, 1360, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22459, 2, 1361, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22460, 3, 1361, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22464, 7, 1361, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22465, 2, 1362, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22466, 3, 1362, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22470, 7, 1362, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22471, 2, 1363, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22472, 3, 1363, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22476, 7, 1363, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22477, 2, 1364, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22478, 3, 1364, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22482, 7, 1364, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22483, 2, 1365, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22484, 3, 1365, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22488, 7, 1365, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22489, 2, 1366, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22490, 3, 1366, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22494, 7, 1366, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22495, 2, 1367, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22496, 3, 1367, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22500, 7, 1367, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22501, 2, 1368, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22502, 3, 1368, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22506, 7, 1368, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22507, 2, 1369, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22508, 3, 1369, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22512, 7, 1369, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22513, 2, 1370, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22514, 3, 1370, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22518, 7, 1370, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22519, 2, 1371, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22520, 3, 1371, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22524, 7, 1371, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22525, 2, 1372, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22526, 3, 1372, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22530, 7, 1372, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22531, 2, 1373, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22532, 3, 1373, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22536, 7, 1373, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22537, 2, 1374, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22538, 3, 1374, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22542, 7, 1374, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22543, 2, 1375, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22544, 3, 1375, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22548, 7, 1375, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22549, 2, 1376, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22555, 2, 1377, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22561, 2, 1378, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22562, 3, 1378, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22566, 7, 1378, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22567, 2, 1379, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22568, 3, 1379, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22572, 7, 1379, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22573, 2, 1380, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22574, 3, 1380, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22578, 7, 1380, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22579, 2, 1381, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22580, 3, 1381, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22584, 7, 1381, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22585, 2, 1382, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22586, 3, 1382, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22590, 7, 1382, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22591, 2, 1383, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22592, 3, 1383, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22596, 7, 1383, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22597, 2, 1384, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22598, 3, 1384, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22602, 7, 1384, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22603, 2, 1385, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22604, 3, 1385, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22608, 7, 1385, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22609, 2, 1386, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22610, 3, 1386, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22614, 7, 1386, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22615, 2, 1387, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22616, 3, 1387, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22620, 7, 1387, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22621, 2, 1388, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22622, 3, 1388, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22626, 7, 1388, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22627, 2, 1389, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22628, 3, 1389, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22560, 7, 1377, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22554, 7, 1376, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22550, 3, 1376, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22632, 7, 1389, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22633, 2, 1390, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22634, 3, 1390, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22638, 7, 1390, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22639, 2, 1391, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22640, 3, 1391, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22644, 7, 1391, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22645, 2, 1392, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22646, 3, 1392, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22650, 7, 1392, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22651, 2, 1393, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22652, 3, 1393, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22656, 7, 1393, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22657, 2, 1394, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22658, 3, 1394, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22662, 7, 1394, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22663, 2, 1395, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22664, 3, 1395, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22668, 7, 1395, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22669, 2, 1396, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22670, 3, 1396, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22674, 7, 1396, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22675, 2, 1397, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22676, 3, 1397, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22680, 7, 1397, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22681, 2, 1398, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22682, 3, 1398, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22686, 7, 1398, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22687, 2, 1399, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22688, 3, 1399, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22692, 7, 1399, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22693, 2, 1400, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22694, 3, 1400, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22698, 7, 1400, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22699, 2, 1401, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22700, 3, 1401, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22704, 7, 1401, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22705, 2, 1402, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22706, 3, 1402, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22710, 7, 1402, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22711, 2, 1403, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22712, 3, 1403, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22716, 7, 1403, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22717, 2, 1404, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22718, 3, 1404, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22722, 7, 1404, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22723, 2, 1405, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22724, 3, 1405, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22728, 7, 1405, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22729, 2, 1406, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22730, 3, 1406, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22734, 7, 1406, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22735, 2, 1407, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22736, 3, 1407, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22740, 7, 1407, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22741, 2, 1408, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22742, 3, 1408, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22746, 7, 1408, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22747, 2, 1409, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22748, 3, 1409, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22752, 7, 1409, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22753, 2, 1410, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22754, 3, 1410, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22758, 7, 1410, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22759, 2, 1411, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22760, 3, 1411, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22764, 7, 1411, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22765, 2, 1412, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22766, 3, 1412, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22770, 7, 1412, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22771, 2, 1413, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22772, 3, 1413, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22776, 7, 1413, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22777, 2, 1414, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22778, 3, 1414, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22782, 7, 1414, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22783, 2, 1415, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22784, 3, 1415, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22788, 7, 1415, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22789, 2, 1416, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22790, 3, 1416, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22794, 7, 1416, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22795, 2, 1417, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22796, 3, 1417, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22800, 7, 1417, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22801, 2, 1418, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22802, 3, 1418, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22806, 7, 1418, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22807, 2, 1419, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22808, 3, 1419, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22812, 7, 1419, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22813, 2, 1420, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22814, 3, 1420, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22818, 7, 1420, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22819, 2, 1421, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22820, 3, 1421, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22824, 7, 1421, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22825, 2, 1422, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22826, 3, 1422, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22830, 7, 1422, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22831, 2, 1423, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22832, 3, 1423, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22836, 7, 1423, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22837, 2, 1424, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22838, 3, 1424, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22842, 7, 1424, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22843, 2, 1425, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22844, 3, 1425, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22848, 7, 1425, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22849, 2, 1426, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22850, 3, 1426, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22854, 7, 1426, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22855, 2, 1427, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22856, 3, 1427, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22860, 7, 1427, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22861, 2, 1428, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22862, 3, 1428, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22866, 7, 1428, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22867, 2, 1429, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22868, 3, 1429, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22872, 7, 1429, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22873, 2, 1430, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22874, 3, 1430, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22878, 7, 1430, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22879, 2, 1431, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22880, 3, 1431, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22884, 7, 1431, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22885, 2, 1432, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22886, 3, 1432, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22890, 7, 1432, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22891, 2, 1433, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22892, 3, 1433, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22896, 7, 1433, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22897, 2, 1434, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22898, 3, 1434, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22902, 7, 1434, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22903, 2, 1435, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22904, 3, 1435, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22908, 7, 1435, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22909, 2, 1436, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22910, 3, 1436, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22914, 7, 1436, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22915, 2, 1437, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22916, 3, 1437, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22920, 7, 1437, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22921, 2, 1438, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22922, 3, 1438, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22926, 7, 1438, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22927, 2, 1439, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22928, 3, 1439, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22932, 7, 1439, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22933, 2, 1440, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22934, 3, 1440, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22938, 7, 1440, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22939, 2, 1441, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22940, 3, 1441, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22944, 7, 1441, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22945, 2, 1442, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22946, 3, 1442, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22950, 7, 1442, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22951, 2, 1443, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22952, 3, 1443, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22956, 7, 1443, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22957, 2, 1444, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22958, 3, 1444, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22962, 7, 1444, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22963, 2, 1445, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22964, 3, 1445, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22968, 7, 1445, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22969, 2, 1446, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22970, 3, 1446, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22974, 7, 1446, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22975, 2, 1447, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22976, 3, 1447, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22980, 7, 1447, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22981, 2, 1448, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22982, 3, 1448, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22986, 7, 1448, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22987, 2, 1449, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22988, 3, 1449, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22992, 7, 1449, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22993, 2, 1450, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22994, 3, 1450, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22998, 7, 1450, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22999, 2, 1451, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23000, 3, 1451, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23004, 7, 1451, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23005, 2, 1452, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23006, 3, 1452, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23010, 7, 1452, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23011, 2, 1453, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23012, 3, 1453, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23016, 7, 1453, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23017, 2, 1454, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23018, 3, 1454, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23022, 7, 1454, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23023, 2, 1455, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23024, 3, 1455, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23028, 7, 1455, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23029, 2, 1456, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23030, 3, 1456, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23034, 7, 1456, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23035, 2, 1457, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23036, 3, 1457, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23040, 7, 1457, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23041, 2, 1458, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23042, 3, 1458, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23046, 7, 1458, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23047, 2, 1459, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23048, 3, 1459, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23052, 7, 1459, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23053, 2, 1460, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23054, 3, 1460, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23058, 7, 1460, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23059, 2, 1461, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23060, 3, 1461, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23064, 7, 1461, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23065, 2, 1462, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23066, 3, 1462, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23070, 7, 1462, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23071, 2, 1463, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23072, 3, 1463, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23076, 7, 1463, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23077, 2, 1464, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23078, 3, 1464, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23082, 7, 1464, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23083, 2, 1465, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23084, 3, 1465, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23088, 7, 1465, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23089, 2, 1466, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23090, 3, 1466, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23094, 7, 1466, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23095, 2, 1467, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23096, 3, 1467, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23100, 7, 1467, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23101, 2, 1468, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23102, 3, 1468, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23106, 7, 1468, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23107, 2, 1469, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23108, 3, 1469, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23112, 7, 1469, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23113, 2, 1470, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23114, 3, 1470, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23118, 7, 1470, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23119, 2, 1471, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23120, 3, 1471, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23124, 7, 1471, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23125, 2, 1472, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23126, 3, 1472, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23130, 7, 1472, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23131, 2, 1473, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23132, 3, 1473, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23136, 7, 1473, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23137, 2, 1474, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23138, 3, 1474, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23142, 7, 1474, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23143, 2, 1475, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23144, 3, 1475, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23148, 7, 1475, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23149, 2, 1476, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23150, 3, 1476, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23154, 7, 1476, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23155, 2, 1477, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23156, 3, 1477, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23160, 7, 1477, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23161, 2, 1478, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23162, 3, 1478, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23166, 7, 1478, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23167, 2, 1479, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23168, 3, 1479, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23172, 7, 1479, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23173, 2, 1480, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23174, 3, 1480, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23178, 7, 1480, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23179, 2, 1481, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23180, 3, 1481, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23184, 7, 1481, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23185, 2, 1482, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23186, 3, 1482, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23190, 7, 1482, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23191, 2, 1483, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23192, 3, 1483, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23196, 7, 1483, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23197, 2, 1484, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23198, 3, 1484, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23202, 7, 1484, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23203, 2, 1485, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23204, 3, 1485, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23208, 7, 1485, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23209, 2, 1486, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23210, 3, 1486, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23214, 7, 1486, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23215, 2, 1487, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23216, 3, 1487, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23220, 7, 1487, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23221, 2, 1488, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23222, 3, 1488, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23226, 7, 1488, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23227, 2, 1489, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23228, 3, 1489, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23232, 7, 1489, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23233, 2, 1490, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23234, 3, 1490, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23238, 7, 1490, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23239, 2, 1491, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23240, 3, 1491, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23244, 7, 1491, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23245, 2, 1492, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23246, 3, 1492, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23250, 7, 1492, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23251, 2, 1493, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23252, 3, 1493, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23256, 7, 1493, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23257, 2, 1494, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23258, 3, 1494, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23262, 7, 1494, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23263, 2, 1495, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23264, 3, 1495, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23268, 7, 1495, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23269, 2, 1496, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23270, 3, 1496, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23274, 7, 1496, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23275, 2, 1497, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23276, 3, 1497, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23280, 7, 1497, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23281, 2, 1498, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23282, 3, 1498, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23286, 7, 1498, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23287, 2, 1499, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23288, 3, 1499, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23292, 7, 1499, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23293, 2, 1500, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23294, 3, 1500, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23298, 7, 1500, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23299, 2, 1501, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23300, 3, 1501, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23304, 7, 1501, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23305, 2, 1502, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23306, 3, 1502, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23310, 7, 1502, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23311, 2, 1503, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23312, 3, 1503, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23316, 7, 1503, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23317, 2, 1504, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23318, 3, 1504, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23322, 7, 1504, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23323, 2, 1505, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23324, 3, 1505, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23328, 7, 1505, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23329, 2, 1506, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23330, 3, 1506, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23334, 7, 1506, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23335, 2, 1507, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23336, 3, 1507, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23340, 7, 1507, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23341, 2, 1508, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23342, 3, 1508, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23346, 7, 1508, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23347, 2, 1509, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23348, 3, 1509, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23352, 7, 1509, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23353, 2, 1510, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23354, 3, 1510, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23358, 7, 1510, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23359, 2, 1511, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23360, 3, 1511, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23364, 7, 1511, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23365, 2, 1512, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23366, 3, 1512, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23370, 7, 1512, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23371, 2, 1513, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23372, 3, 1513, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23376, 7, 1513, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23377, 2, 1514, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23378, 3, 1514, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23382, 7, 1514, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23383, 2, 1515, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23384, 3, 1515, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23388, 7, 1515, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23389, 2, 1516, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23390, 3, 1516, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23394, 7, 1516, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23395, 2, 1517, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23396, 3, 1517, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23400, 7, 1517, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23401, 2, 1518, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23402, 3, 1518, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23406, 7, 1518, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23407, 2, 1519, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23408, 3, 1519, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23412, 7, 1519, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23413, 2, 1520, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23414, 3, 1520, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23418, 7, 1520, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23419, 2, 1521, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23420, 3, 1521, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23424, 7, 1521, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23425, 2, 1522, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23426, 3, 1522, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23430, 7, 1522, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23431, 2, 1523, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23432, 3, 1523, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23436, 7, 1523, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23437, 2, 1524, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23438, 3, 1524, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23442, 7, 1524, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23443, 2, 1525, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23444, 3, 1525, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23448, 7, 1525, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23449, 2, 1526, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23450, 3, 1526, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23454, 7, 1526, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23455, 2, 1527, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23456, 3, 1527, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23460, 7, 1527, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23461, 2, 1528, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23462, 3, 1528, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23466, 7, 1528, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23467, 2, 1529, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23468, 3, 1529, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23472, 7, 1529, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23473, 2, 1530, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23474, 3, 1530, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23478, 7, 1530, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23479, 2, 1531, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23480, 3, 1531, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23484, 7, 1531, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23485, 2, 1532, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23486, 3, 1532, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23490, 7, 1532, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23491, 2, 1533, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23492, 3, 1533, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23496, 7, 1533, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23497, 2, 1534, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23498, 3, 1534, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23502, 7, 1534, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23503, 2, 1535, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23504, 3, 1535, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23508, 7, 1535, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23509, 2, 1536, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23510, 3, 1536, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23514, 7, 1536, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23515, 2, 1537, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23516, 3, 1537, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23520, 7, 1537, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23521, 2, 1538, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23522, 3, 1538, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23526, 7, 1538, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23527, 2, 1539, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23528, 3, 1539, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23532, 7, 1539, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23533, 2, 1540, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23534, 3, 1540, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23538, 7, 1540, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23539, 2, 1541, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23540, 3, 1541, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23544, 7, 1541, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23545, 2, 1542, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23546, 3, 1542, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23550, 7, 1542, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23551, 2, 1543, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23552, 3, 1543, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23556, 7, 1543, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23557, 2, 1544, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23558, 3, 1544, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23562, 7, 1544, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23563, 2, 1545, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23564, 3, 1545, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23568, 7, 1545, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23569, 2, 1546, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23570, 3, 1546, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23574, 7, 1546, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23575, 2, 1547, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23576, 3, 1547, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23580, 7, 1547, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23581, 2, 1548, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23582, 3, 1548, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23586, 7, 1548, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23587, 2, 1549, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23588, 3, 1549, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23592, 7, 1549, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23593, 2, 1550, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23594, 3, 1550, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23598, 7, 1550, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23599, 2, 1551, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23600, 3, 1551, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23604, 7, 1551, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23605, 2, 1552, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23606, 3, 1552, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23610, 7, 1552, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23611, 2, 1553, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23612, 3, 1553, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23616, 7, 1553, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23617, 2, 1554, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23618, 3, 1554, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23622, 7, 1554, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23623, 2, 1555, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23624, 3, 1555, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23628, 7, 1555, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23629, 2, 1556, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23630, 3, 1556, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23634, 7, 1556, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23635, 2, 1557, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23636, 3, 1557, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23640, 7, 1557, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23641, 2, 1558, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23642, 3, 1558, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23646, 7, 1558, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23647, 2, 1559, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23648, 3, 1559, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23652, 7, 1559, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23653, 2, 1560, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23654, 3, 1560, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23658, 7, 1560, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23660, 3, 1561, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23664, 7, 1561, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23665, 2, 1562, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23666, 3, 1562, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23670, 7, 1562, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23671, 2, 1563, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23672, 3, 1563, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23676, 7, 1563, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23677, 2, 1564, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23678, 3, 1564, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23682, 7, 1564, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23683, 2, 1565, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23684, 3, 1565, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23688, 7, 1565, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23689, 2, 1566, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23690, 3, 1566, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23694, 7, 1566, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23695, 2, 1567, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23696, 3, 1567, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23700, 7, 1567, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23701, 2, 1568, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23702, 3, 1568, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23706, 7, 1568, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23707, 2, 1569, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23708, 3, 1569, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23712, 7, 1569, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23713, 2, 1570, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23714, 3, 1570, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23718, 7, 1570, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23719, 2, 1571, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23720, 3, 1571, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23724, 7, 1571, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23725, 2, 1572, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23726, 3, 1572, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23730, 7, 1572, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23731, 2, 1573, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23732, 3, 1573, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23736, 7, 1573, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23737, 2, 1574, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23738, 3, 1574, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23742, 7, 1574, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23743, 2, 1575, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23744, 3, 1575, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23748, 7, 1575, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23749, 2, 1576, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23750, 3, 1576, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23754, 7, 1576, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23755, 2, 1577, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23756, 3, 1577, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23760, 7, 1577, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23761, 2, 1578, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23762, 3, 1578, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23766, 7, 1578, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23767, 2, 1579, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23768, 3, 1579, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23772, 7, 1579, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23773, 2, 1580, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23774, 3, 1580, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25640, 6, 244, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (166, 3, 244, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23778, 7, 1580, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23779, 2, 1581, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23780, 3, 1581, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23784, 7, 1581, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23785, 2, 1582, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23786, 3, 1582, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23790, 7, 1582, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23791, 2, 1583, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23792, 3, 1583, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (27437, 3, 22, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (27438, 4, 22, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23796, 7, 1583, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23797, 2, 1584, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23798, 3, 1584, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (27440, 6, 22, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23802, 7, 1584, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23803, 2, 1585, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23804, 3, 1585, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (27441, 7, 22, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (27442, 3, 23, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23808, 7, 1585, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23809, 2, 1586, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23810, 3, 1586, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (27443, 4, 23, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23814, 7, 1586, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23815, 2, 1587, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23816, 3, 1587, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (27445, 6, 23, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (27446, 7, 23, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23820, 7, 1587, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23821, 2, 1588, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23822, 3, 1588, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (27447, 3, 327, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (27448, 3, 328, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23826, 7, 1588, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23827, 2, 1589, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23828, 3, 1589, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (27449, 4, 327, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (27450, 4, 328, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23832, 7, 1589, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23833, 2, 1590, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23834, 3, 1590, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23838, 7, 1590, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23839, 2, 1591, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23840, 3, 1591, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (27453, 6, 327, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (27454, 6, 328, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23844, 7, 1591, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23845, 2, 1592, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23846, 3, 1592, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (27455, 7, 327, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (27456, 7, 328, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23850, 7, 1592, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23851, 2, 1593, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23852, 3, 1593, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23856, 7, 1593, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23857, 2, 1594, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23858, 3, 1594, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23862, 7, 1594, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23863, 2, 1595, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23864, 3, 1595, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23868, 7, 1595, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23869, 2, 1596, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23870, 3, 1596, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23874, 7, 1596, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23875, 2, 1597, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23876, 3, 1597, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23880, 7, 1597, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23881, 2, 1598, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23882, 3, 1598, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23886, 7, 1598, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23887, 2, 1599, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23888, 3, 1599, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23892, 7, 1599, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23893, 2, 1600, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23894, 3, 1600, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23898, 7, 1600, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23899, 2, 1601, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23900, 3, 1601, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23904, 7, 1601, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23905, 2, 1602, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23906, 3, 1602, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23910, 7, 1602, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23911, 2, 1603, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23912, 3, 1603, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23916, 7, 1603, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23917, 2, 1604, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23918, 3, 1604, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23922, 7, 1604, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23923, 2, 1605, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23924, 3, 1605, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23928, 7, 1605, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23929, 2, 1606, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23930, 3, 1606, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23934, 7, 1606, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23935, 2, 1607, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23936, 3, 1607, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23940, 7, 1607, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23941, 2, 1608, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23942, 3, 1608, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23946, 7, 1608, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23947, 2, 1609, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23948, 3, 1609, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23952, 7, 1609, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23953, 2, 1610, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23954, 3, 1610, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23958, 7, 1610, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23959, 2, 1611, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23960, 3, 1611, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23964, 7, 1611, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23965, 2, 1612, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23966, 3, 1612, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23970, 7, 1612, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23971, 2, 1613, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23972, 3, 1613, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23976, 7, 1613, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23977, 2, 1614, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23978, 3, 1614, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23982, 7, 1614, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23983, 2, 1615, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23984, 3, 1615, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23988, 7, 1615, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23989, 2, 1616, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23990, 3, 1616, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23994, 7, 1616, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23995, 2, 1617, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23996, 3, 1617, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24000, 7, 1617, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24001, 2, 1618, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24002, 3, 1618, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24006, 7, 1618, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24007, 2, 1619, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24008, 3, 1619, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24012, 7, 1619, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24013, 2, 1620, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24014, 3, 1620, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24018, 7, 1620, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24019, 2, 1621, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24020, 3, 1621, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24024, 7, 1621, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24025, 2, 1622, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24026, 3, 1622, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24030, 7, 1622, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24031, 2, 1623, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24032, 3, 1623, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24036, 7, 1623, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24037, 2, 1624, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24038, 3, 1624, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24042, 7, 1624, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24043, 2, 1625, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24044, 3, 1625, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24048, 7, 1625, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24049, 2, 1626, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24050, 3, 1626, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24054, 7, 1626, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24055, 2, 1627, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24056, 3, 1627, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24060, 7, 1627, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24061, 2, 1628, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24062, 3, 1628, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24066, 7, 1628, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24067, 2, 1629, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24068, 3, 1629, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24072, 7, 1629, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24073, 2, 1630, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24074, 3, 1630, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24078, 7, 1630, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24079, 2, 1631, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24080, 3, 1631, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24084, 7, 1631, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24085, 2, 1632, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24086, 3, 1632, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24090, 7, 1632, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24091, 2, 1633, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24092, 3, 1633, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24096, 7, 1633, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24097, 2, 1634, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24098, 3, 1634, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24102, 7, 1634, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24103, 2, 1635, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24104, 3, 1635, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24108, 7, 1635, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24109, 2, 1636, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24110, 3, 1636, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24114, 7, 1636, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24115, 2, 1637, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24116, 3, 1637, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24120, 7, 1637, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24121, 2, 1638, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24122, 3, 1638, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24126, 7, 1638, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24127, 2, 1639, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24128, 3, 1639, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24132, 7, 1639, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24133, 2, 1640, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24134, 3, 1640, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24138, 7, 1640, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24139, 2, 1641, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24140, 3, 1641, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24144, 7, 1641, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24145, 2, 1642, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24146, 3, 1642, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24150, 7, 1642, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24151, 2, 1643, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24152, 3, 1643, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24156, 7, 1643, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24157, 2, 1644, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24158, 3, 1644, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24162, 7, 1644, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24163, 2, 1645, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24164, 3, 1645, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24168, 7, 1645, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24169, 2, 1646, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24170, 3, 1646, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24174, 7, 1646, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24175, 2, 1647, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24176, 3, 1647, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24180, 7, 1647, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24181, 2, 1648, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24182, 3, 1648, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24186, 7, 1648, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24187, 2, 1649, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24188, 3, 1649, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24192, 7, 1649, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24193, 2, 1650, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24194, 3, 1650, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24198, 7, 1650, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24199, 2, 1651, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24200, 3, 1651, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24204, 7, 1651, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24205, 2, 1652, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24206, 3, 1652, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24210, 7, 1652, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24211, 2, 1653, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24212, 3, 1653, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24216, 7, 1653, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24217, 2, 1654, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24218, 3, 1654, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24222, 7, 1654, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24223, 2, 1655, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24224, 3, 1655, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24228, 7, 1655, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24229, 2, 1656, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24230, 3, 1656, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24234, 7, 1656, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24235, 2, 1657, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24236, 3, 1657, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24240, 7, 1657, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24241, 2, 1658, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24242, 3, 1658, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24246, 7, 1658, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24247, 2, 1659, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24248, 3, 1659, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24252, 7, 1659, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24253, 2, 1660, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24254, 3, 1660, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24258, 7, 1660, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24259, 2, 1661, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24260, 3, 1661, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24264, 7, 1661, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24265, 2, 1662, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24266, 3, 1662, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24270, 7, 1662, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24271, 2, 1663, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24272, 3, 1663, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24276, 7, 1663, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24277, 2, 1664, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24278, 3, 1664, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24282, 7, 1664, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24283, 2, 1665, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24284, 3, 1665, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24288, 7, 1665, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24289, 2, 1666, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24290, 3, 1666, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24294, 7, 1666, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24295, 2, 1667, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24296, 3, 1667, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24300, 7, 1667, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24301, 2, 1668, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24302, 3, 1668, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24306, 7, 1668, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24307, 2, 1669, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24308, 3, 1669, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24312, 7, 1669, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24313, 2, 1670, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24314, 3, 1670, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24318, 7, 1670, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24319, 2, 1671, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24320, 3, 1671, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24324, 7, 1671, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24325, 2, 1672, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24326, 3, 1672, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24330, 7, 1672, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24331, 2, 1673, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24332, 3, 1673, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24336, 7, 1673, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24337, 2, 1674, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24338, 3, 1674, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24342, 7, 1674, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24343, 2, 1675, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24344, 3, 1675, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24348, 7, 1675, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24349, 2, 1676, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24350, 3, 1676, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24354, 7, 1676, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24355, 2, 1677, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24356, 3, 1677, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24360, 7, 1677, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24361, 2, 1678, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24362, 3, 1678, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24366, 7, 1678, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24367, 2, 1679, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24368, 3, 1679, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24372, 7, 1679, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24373, 2, 1680, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24374, 3, 1680, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24378, 7, 1680, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24379, 2, 1681, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24380, 3, 1681, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24384, 7, 1681, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24385, 2, 1682, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24386, 3, 1682, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24390, 7, 1682, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24391, 2, 1683, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24392, 3, 1683, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24396, 7, 1683, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24397, 2, 1684, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24398, 3, 1684, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24402, 7, 1684, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24403, 2, 1685, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24404, 3, 1685, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24408, 7, 1685, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24409, 2, 1686, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24410, 3, 1686, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24414, 7, 1686, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24415, 2, 1687, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24416, 3, 1687, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24420, 7, 1687, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24421, 2, 1688, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24422, 3, 1688, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24426, 7, 1688, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24427, 2, 1689, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24428, 3, 1689, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24432, 7, 1689, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24433, 2, 1690, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24434, 3, 1690, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24438, 7, 1690, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24439, 2, 1691, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24440, 3, 1691, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24444, 7, 1691, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24445, 2, 1692, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24446, 3, 1692, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24450, 7, 1692, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24451, 2, 1693, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24452, 3, 1693, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24456, 7, 1693, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24457, 2, 1694, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24458, 3, 1694, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24462, 7, 1694, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24463, 2, 1695, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24464, 3, 1695, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24468, 7, 1695, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24469, 2, 1696, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24470, 3, 1696, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24474, 7, 1696, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24475, 2, 1697, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24476, 3, 1697, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24480, 7, 1697, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24481, 2, 1698, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24482, 3, 1698, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24486, 7, 1698, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24487, 2, 1699, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24488, 3, 1699, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24492, 7, 1699, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24493, 2, 1700, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24494, 3, 1700, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24498, 7, 1700, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24499, 2, 1701, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24500, 3, 1701, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24504, 7, 1701, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24505, 2, 1702, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24506, 3, 1702, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24510, 7, 1702, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24511, 2, 1703, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24512, 3, 1703, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24516, 7, 1703, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24517, 2, 1704, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24518, 3, 1704, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24522, 7, 1704, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24523, 2, 1705, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24524, 3, 1705, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24528, 7, 1705, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24529, 2, 1706, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24530, 3, 1706, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24534, 7, 1706, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24535, 2, 1707, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24536, 3, 1707, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24540, 7, 1707, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24541, 2, 1708, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24542, 3, 1708, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24546, 7, 1708, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24547, 2, 1709, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24548, 3, 1709, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24552, 7, 1709, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24553, 2, 1710, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24554, 3, 1710, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24558, 7, 1710, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24559, 2, 1711, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24560, 3, 1711, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24564, 7, 1711, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24565, 2, 1712, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24566, 3, 1712, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24570, 7, 1712, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24571, 2, 1713, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24572, 3, 1713, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24576, 7, 1713, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24577, 2, 1714, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24578, 3, 1714, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24582, 7, 1714, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24583, 2, 1715, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24584, 3, 1715, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24588, 7, 1715, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24589, 2, 1716, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24590, 3, 1716, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24594, 7, 1716, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24595, 2, 1717, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24596, 3, 1717, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24600, 7, 1717, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24601, 2, 1718, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24602, 3, 1718, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24606, 7, 1718, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24607, 2, 1719, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24608, 3, 1719, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24612, 7, 1719, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24613, 2, 1720, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24614, 3, 1720, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24618, 7, 1720, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24619, 2, 1721, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24620, 3, 1721, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24624, 7, 1721, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24625, 2, 1722, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24626, 3, 1722, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24630, 7, 1722, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24631, 2, 1723, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24632, 3, 1723, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24636, 7, 1723, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24637, 2, 1724, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24638, 3, 1724, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24642, 7, 1724, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24643, 2, 1725, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24644, 3, 1725, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24648, 7, 1725, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24649, 2, 1726, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24650, 3, 1726, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24654, 7, 1726, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24655, 2, 1727, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24656, 3, 1727, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24660, 7, 1727, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24661, 2, 1728, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24662, 3, 1728, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24666, 7, 1728, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24667, 2, 1729, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24668, 3, 1729, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24672, 7, 1729, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24673, 2, 1730, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24674, 3, 1730, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24678, 7, 1730, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24679, 2, 1731, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24680, 3, 1731, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24684, 7, 1731, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24685, 2, 1732, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24686, 3, 1732, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24690, 7, 1732, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24691, 2, 1733, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24692, 3, 1733, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24696, 7, 1733, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24697, 2, 1734, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24698, 3, 1734, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24702, 7, 1734, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24703, 2, 1735, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24704, 3, 1735, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24708, 7, 1735, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24709, 2, 1736, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24710, 3, 1736, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24714, 7, 1736, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24715, 2, 1737, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24716, 3, 1737, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24720, 7, 1737, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24721, 2, 1738, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24722, 3, 1738, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24726, 7, 1738, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24727, 2, 1739, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24728, 3, 1739, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24732, 7, 1739, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24733, 2, 1740, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24734, 3, 1740, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24738, 7, 1740, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24739, 2, 1741, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24740, 3, 1741, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24744, 7, 1741, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24745, 2, 1742, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24746, 3, 1742, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24750, 7, 1742, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24751, 2, 1743, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24752, 3, 1743, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24756, 7, 1743, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24757, 2, 1744, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24758, 3, 1744, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24762, 7, 1744, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24763, 2, 1745, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24764, 3, 1745, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24768, 7, 1745, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24769, 2, 1746, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24770, 3, 1746, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24774, 7, 1746, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24775, 2, 1747, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24776, 3, 1747, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24780, 7, 1747, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24781, 2, 1748, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24782, 3, 1748, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24786, 7, 1748, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24787, 2, 1749, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24788, 3, 1749, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24792, 7, 1749, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24793, 2, 1750, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24794, 3, 1750, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24798, 7, 1750, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24799, 2, 1751, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24800, 3, 1751, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24804, 7, 1751, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24805, 2, 1752, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24806, 3, 1752, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24810, 7, 1752, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24811, 2, 1753, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24812, 3, 1753, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24816, 7, 1753, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24817, 2, 1754, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24818, 3, 1754, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24822, 7, 1754, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24823, 2, 1755, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24824, 3, 1755, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24828, 7, 1755, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24829, 2, 1756, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24830, 3, 1756, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24834, 7, 1756, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24835, 2, 1757, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24836, 3, 1757, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24840, 7, 1757, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24841, 2, 1758, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24842, 3, 1758, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24846, 7, 1758, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24847, 2, 1759, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24848, 3, 1759, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24852, 7, 1759, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24853, 2, 1760, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24854, 3, 1760, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24858, 7, 1760, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24859, 2, 1761, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24860, 3, 1761, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24864, 7, 1761, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24865, 2, 1762, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24866, 3, 1762, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24870, 7, 1762, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24871, 2, 1763, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24872, 3, 1763, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24876, 7, 1763, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24877, 2, 1764, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24878, 3, 1764, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24882, 7, 1764, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24883, 2, 1765, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24884, 3, 1765, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24888, 7, 1765, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24889, 2, 1766, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24890, 3, 1766, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24894, 7, 1766, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24895, 2, 1767, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24896, 3, 1767, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24900, 7, 1767, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24901, 2, 1768, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24902, 3, 1768, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24906, 7, 1768, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24907, 2, 1769, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24908, 3, 1769, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24912, 7, 1769, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24913, 2, 1770, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24914, 3, 1770, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24918, 7, 1770, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24919, 2, 1771, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24920, 3, 1771, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24924, 7, 1771, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24925, 2, 1772, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24926, 3, 1772, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24930, 7, 1772, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24931, 2, 1773, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24932, 3, 1773, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24936, 7, 1773, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24937, 2, 1774, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24938, 3, 1774, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24942, 7, 1774, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24943, 2, 1775, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24944, 3, 1775, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24948, 7, 1775, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24949, 2, 1776, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24950, 3, 1776, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24954, 7, 1776, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24955, 2, 1777, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24956, 3, 1777, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24960, 7, 1777, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24961, 2, 1778, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24962, 3, 1778, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24966, 7, 1778, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24967, 2, 1779, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24968, 3, 1779, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24972, 7, 1779, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24973, 2, 1780, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24974, 3, 1780, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24978, 7, 1780, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24979, 2, 1781, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24980, 3, 1781, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24984, 7, 1781, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24985, 2, 1782, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24986, 3, 1782, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24990, 7, 1782, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24991, 2, 1783, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24992, 3, 1783, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24996, 7, 1783, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24997, 2, 1784, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24998, 3, 1784, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25002, 7, 1784, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25003, 2, 1785, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25004, 3, 1785, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25008, 7, 1785, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25009, 2, 1786, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25010, 3, 1786, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25014, 7, 1786, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25015, 2, 1787, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25016, 3, 1787, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25020, 7, 1787, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25021, 2, 1788, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25022, 3, 1788, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25026, 7, 1788, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25027, 2, 1789, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25028, 3, 1789, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25032, 7, 1789, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25033, 2, 1790, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25034, 3, 1790, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25038, 7, 1790, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25039, 2, 1791, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25040, 3, 1791, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25044, 7, 1791, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25045, 2, 1792, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25046, 3, 1792, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25050, 7, 1792, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25051, 2, 1793, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25052, 3, 1793, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25056, 7, 1793, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25057, 2, 1794, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25058, 3, 1794, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25062, 7, 1794, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25063, 2, 1795, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25064, 3, 1795, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25068, 7, 1795, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25069, 2, 1796, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25070, 3, 1796, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25074, 7, 1796, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25075, 2, 1797, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25076, 3, 1797, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25080, 7, 1797, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25081, 2, 1798, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25082, 3, 1798, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25086, 7, 1798, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25087, 2, 1799, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25088, 3, 1799, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25092, 7, 1799, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25093, 2, 1800, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25094, 3, 1800, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25098, 7, 1800, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25099, 2, 1801, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25100, 3, 1801, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25104, 7, 1801, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25105, 2, 1802, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25106, 3, 1802, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25110, 7, 1802, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25111, 2, 1803, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25112, 3, 1803, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25117, 2, 1804, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25118, 3, 1804, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25122, 7, 1804, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25123, 2, 1805, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25124, 3, 1805, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25128, 7, 1805, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25129, 2, 1806, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25130, 3, 1806, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25134, 7, 1806, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25135, 2, 1807, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25136, 3, 1807, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25140, 7, 1807, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25141, 2, 1808, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25142, 3, 1808, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25146, 7, 1808, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25147, 2, 1809, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25148, 3, 1809, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25152, 7, 1809, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25153, 2, 1810, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25154, 3, 1810, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25158, 7, 1810, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25159, 2, 1811, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25160, 3, 1811, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25164, 7, 1811, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25165, 2, 1812, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25166, 3, 1812, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25170, 7, 1812, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25171, 2, 1813, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25172, 3, 1813, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25176, 7, 1813, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25177, 2, 1814, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25178, 3, 1814, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25182, 7, 1814, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25183, 2, 1815, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25184, 3, 1815, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25188, 7, 1815, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25189, 2, 1816, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25190, 3, 1816, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25194, 7, 1816, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25195, 2, 1817, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25196, 3, 1817, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25200, 7, 1817, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25201, 2, 1818, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25202, 3, 1818, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25206, 7, 1818, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25207, 2, 1819, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25208, 3, 1819, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25212, 7, 1819, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25213, 2, 1820, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25214, 3, 1820, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25218, 7, 1820, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25219, 2, 1821, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25220, 3, 1821, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25224, 7, 1821, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25225, 2, 1822, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25226, 3, 1822, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25230, 7, 1822, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25231, 2, 1823, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25232, 3, 1823, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25236, 7, 1823, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25237, 2, 1824, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25238, 3, 1824, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25242, 7, 1824, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25243, 2, 1825, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25244, 3, 1825, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25248, 7, 1825, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25249, 2, 1826, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25250, 3, 1826, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25254, 7, 1826, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25255, 2, 1827, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25256, 3, 1827, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25260, 7, 1827, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25261, 2, 1828, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25262, 3, 1828, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25266, 7, 1828, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25267, 2, 1829, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25268, 3, 1829, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25272, 7, 1829, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25273, 2, 1830, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25274, 3, 1830, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25278, 7, 1830, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25279, 2, 1831, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25280, 3, 1831, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25284, 7, 1831, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25285, 2, 1832, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25286, 3, 1832, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25290, 7, 1832, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25291, 2, 1833, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25292, 3, 1833, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25296, 7, 1833, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25297, 2, 1834, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25298, 3, 1834, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25302, 7, 1834, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25303, 2, 1835, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25304, 3, 1835, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25308, 7, 1835, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25309, 2, 1836, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25310, 3, 1836, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25314, 7, 1836, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25315, 2, 1837, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25316, 3, 1837, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25320, 7, 1837, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25321, 2, 1838, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25322, 3, 1838, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25326, 7, 1838, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25327, 2, 1839, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25328, 3, 1839, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25332, 7, 1839, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25333, 2, 1840, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25334, 3, 1840, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25338, 7, 1840, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25339, 2, 1841, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25340, 3, 1841, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25344, 7, 1841, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25345, 2, 1842, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25346, 3, 1842, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25350, 7, 1842, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25351, 2, 1843, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25352, 3, 1843, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25356, 7, 1843, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25357, 2, 1844, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25358, 3, 1844, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25362, 7, 1844, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25363, 2, 1845, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25364, 3, 1845, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25368, 7, 1845, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25369, 2, 1846, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25370, 3, 1846, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25374, 7, 1846, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25375, 2, 1847, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25376, 3, 1847, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25380, 7, 1847, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25381, 2, 1848, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25382, 3, 1848, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25386, 7, 1848, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25387, 2, 1849, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25388, 3, 1849, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25392, 7, 1849, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25393, 2, 1850, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25394, 3, 1850, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25398, 7, 1850, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25399, 2, 1851, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25400, 3, 1851, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25404, 7, 1851, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25405, 2, 1852, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25406, 3, 1852, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25410, 7, 1852, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25411, 2, 1853, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25412, 3, 1853, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25416, 7, 1853, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25417, 2, 1854, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25418, 3, 1854, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25422, 7, 1854, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25423, 2, 1855, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25424, 3, 1855, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25428, 7, 1855, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25429, 2, 1856, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25430, 3, 1856, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25434, 7, 1856, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25435, 2, 1857, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25436, 3, 1857, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25440, 7, 1857, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25441, 2, 1858, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25442, 3, 1858, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25446, 7, 1858, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25447, 2, 1859, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25448, 3, 1859, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25452, 7, 1859, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25453, 2, 1860, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25454, 3, 1860, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25458, 7, 1860, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25459, 2, 1861, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25460, 3, 1861, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25464, 7, 1861, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25465, 2, 1862, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25466, 3, 1862, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25470, 7, 1862, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25471, 2, 1863, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25472, 3, 1863, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25476, 7, 1863, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25477, 2, 1864, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25478, 3, 1864, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25482, 7, 1864, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25483, 2, 1865, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25484, 3, 1865, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25488, 7, 1865, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25489, 2, 1866, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25490, 3, 1866, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25494, 7, 1866, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25495, 2, 1867, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25496, 3, 1867, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25500, 7, 1867, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25501, 2, 1868, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25502, 3, 1868, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25506, 7, 1868, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25507, 2, 1869, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25508, 3, 1869, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25512, 7, 1869, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25513, 2, 1870, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25514, 3, 1870, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25518, 7, 1870, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25519, 2, 1871, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25520, 3, 1871, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25524, 7, 1871, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25525, 2, 1872, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25526, 3, 1872, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25530, 7, 1872, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25531, 2, 1873, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25532, 3, 1873, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25536, 7, 1873, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (9, 2, 54, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (29, 2, 557, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23659, 2, 1561, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25537, 2, 619, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20327, 2, 2, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21457, 2, 1194, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (37, 2, 258, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25538, 2, 942, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (44, 2, 255, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19846, 2, 1042, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22153, 2, 1310, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19828, 2, 1039, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (57, 2, 241, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (81, 2, 242, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19864, 2, 1045, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (85, 3, 346, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (87, 3, 352, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22556, 3, 1377, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (100, 3, 269, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25539, 3, 619, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (113, 3, 230, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25540, 3, 942, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (126, 3, 523, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22154, 3, 1310, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (140, 3, 251, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (153, 3, 212, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (407, 7, 346, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (421, 7, 268, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (424, 7, 263, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (425, 7, 205, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (426, 7, 187, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (2, 2, 344, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (16, 2, 754, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (71, 2, 213, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (86, 3, 354, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8808, 3, 934, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25547, 7, 619, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (427, 7, 684, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (428, 7, 685, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (429, 7, 692, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (430, 7, 682, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (431, 7, 683, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (432, 7, 679, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (433, 7, 690, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (436, 7, 229, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25548, 7, 942, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (447, 7, 476, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19830, 7, 1039, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (463, 7, 249, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (8839, 7, 935, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (484, 7, 345, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25549, 4, 345, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25550, 4, 456, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25551, 4, 253, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25552, 4, 2, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25553, 4, 619, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25554, 4, 205, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25555, 4, 1045, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25556, 4, 249, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25557, 4, 250, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25558, 4, 268, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25559, 4, 269, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25560, 4, 938, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25561, 4, 942, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25562, 4, 939, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25563, 4, 940, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25564, 4, 941, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25565, 4, 308, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25566, 4, 309, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25567, 4, 307, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25568, 4, 259, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25569, 4, 477, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25570, 4, 478, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25571, 4, 255, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25572, 4, 231, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25573, 4, 207, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25574, 4, 213, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25575, 4, 211, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25576, 4, 212, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25577, 4, 188, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25578, 4, 189, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25579, 4, 605, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25580, 4, 233, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25581, 4, 234, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25582, 4, 235, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25583, 4, 944, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25584, 4, 895, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25585, 4, 236, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25586, 4, 208, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25587, 4, 934, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25588, 4, 935, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25589, 4, 936, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25590, 4, 190, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25591, 4, 237, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25592, 4, 242, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25593, 4, 243, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (25594, 4, 244, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25396, 5, 1850, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22474, 5, 1363, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20297, 5, 2, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (27439, 5, 22, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (27444, 5, 23, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22384, 5, 1348, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24202, 5, 1651, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20968, 5, 1112, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22480, 5, 1364, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25402, 5, 1851, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22486, 5, 1365, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20974, 5, 1113, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25408, 5, 1852, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22492, 5, 1366, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25414, 5, 1853, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20980, 5, 1114, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23164, 5, 1478, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22498, 5, 1367, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25420, 5, 1854, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8819, 5, 895, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (20986, 5, 1115, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23038, 5, 1457, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22504, 5, 1368, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25426, 5, 1855, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20992, 5, 1116, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22510, 5, 1369, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (295, 5, 57, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (289, 5, 58, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25432, 5, 1856, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20998, 5, 1117, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22516, 5, 1370, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25438, 5, 1857, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (290, 5, 76, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (294, 5, 72, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22522, 5, 1371, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21004, 5, 1118, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25444, 5, 1858, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22528, 5, 1372, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21010, 5, 1119, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25450, 5, 1859, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22534, 5, 1373, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21016, 5, 1120, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25456, 5, 1860, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (27452, 5, 327, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (27451, 5, 328, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18404, 5, 1003, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8818, 5, 944, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (22540, 5, 1374, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21022, 5, 1121, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25462, 5, 1861, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (275, 5, 345, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (247, 5, 346, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (250, 5, 349, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (251, 5, 350, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (252, 5, 351, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (249, 5, 352, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (248, 5, 354, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22546, 5, 1375, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25468, 5, 1862, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21028, 5, 1122, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (301, 5, 246, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22552, 5, 1376, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23452, 5, 1526, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18290, 5, 984, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18296, 5, 985, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18302, 5, 986, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25474, 5, 1863, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21034, 5, 1123, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21334, 5, 1173, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22558, 5, 1377, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25480, 5, 1864, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22564, 5, 1378, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21166, 5, 1145, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21148, 5, 1142, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21160, 5, 1144, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25486, 5, 1865, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22570, 5, 1379, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21172, 5, 1146, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20804, 5, 1085, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20810, 5, 1086, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20816, 5, 1087, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20822, 5, 1088, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20828, 5, 1089, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20834, 5, 1090, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20840, 5, 1091, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20848, 5, 1092, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24034, 5, 1623, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23182, 5, 1481, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21178, 5, 1147, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25498, 5, 1867, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21406, 5, 1185, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23188, 5, 1482, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21184, 5, 1148, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25504, 5, 1868, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (254, 5, 456, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22024, 5, 1288, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23020, 5, 1454, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (257, 5, 130, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23194, 5, 1483, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25510, 5, 1869, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21304, 5, 1168, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (297, 5, 477, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (287, 5, 476, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23338, 5, 1507, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25516, 5, 1870, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21586, 5, 1215, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21592, 5, 1216, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21598, 5, 1217, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21604, 5, 1218, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21610, 5, 1219, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21616, 5, 1220, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21622, 5, 1221, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21628, 5, 1222, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21634, 5, 1223, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21640, 5, 1224, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21646, 5, 1225, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21652, 5, 1226, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21658, 5, 1227, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21664, 5, 1228, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21670, 5, 1229, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21676, 5, 1230, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21682, 5, 1231, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21688, 5, 1232, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21694, 5, 1233, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21700, 5, 1234, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21718, 5, 1237, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21724, 5, 1238, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21730, 5, 1239, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21736, 5, 1240, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21742, 5, 1241, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21748, 5, 1242, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21754, 5, 1243, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21760, 5, 1244, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21766, 5, 1245, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21772, 5, 1246, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21778, 5, 1247, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21784, 5, 1248, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21790, 5, 1249, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21808, 5, 1252, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21310, 5, 1169, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25522, 5, 1871, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18308, 5, 987, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18314, 5, 988, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18326, 5, 990, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18332, 5, 991, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18338, 5, 992, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18344, 5, 993, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18350, 5, 994, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18356, 5, 995, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18362, 5, 996, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18368, 5, 997, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18374, 5, 998, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18380, 5, 999, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18386, 5, 1000, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18392, 5, 1001, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18398, 5, 1002, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18416, 5, 1005, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18422, 5, 1006, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18434, 5, 1008, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18440, 5, 1009, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18446, 5, 1010, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18452, 5, 1011, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18458, 5, 1012, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18464, 5, 1013, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18470, 5, 1014, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18476, 5, 1015, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18482, 5, 1016, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18488, 5, 1017, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18494, 5, 1018, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18500, 5, 1019, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19961, 5, 1061, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23632, 5, 1556, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24724, 5, 1738, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24730, 5, 1739, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24736, 5, 1740, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24742, 5, 1741, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24748, 5, 1742, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24754, 5, 1743, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24760, 5, 1744, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24766, 5, 1745, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24772, 5, 1746, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24778, 5, 1747, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24784, 5, 1748, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24790, 5, 1749, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24796, 5, 1750, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24802, 5, 1751, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24808, 5, 1752, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24814, 5, 1753, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24820, 5, 1754, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24826, 5, 1755, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24832, 5, 1756, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24838, 5, 1757, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24844, 5, 1758, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24850, 5, 1759, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24856, 5, 1760, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24862, 5, 1761, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24868, 5, 1762, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24874, 5, 1763, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24880, 5, 1764, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24886, 5, 1765, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24892, 5, 1766, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24898, 5, 1767, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24904, 5, 1768, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24910, 5, 1769, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24916, 5, 1770, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24922, 5, 1771, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24928, 5, 1772, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24934, 5, 1773, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18410, 5, 1004, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18506, 5, 1020, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18512, 5, 1021, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18518, 5, 1022, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18524, 5, 1023, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19859, 5, 1044, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19865, 5, 1045, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (19871, 5, 1046, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19877, 5, 1047, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19883, 5, 1048, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19889, 5, 1049, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19895, 5, 1050, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19901, 5, 1051, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19907, 5, 1052, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19913, 5, 1053, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19919, 5, 1054, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19925, 5, 1055, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19931, 5, 1056, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19937, 5, 1057, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19943, 5, 1058, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19949, 5, 1059, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19955, 5, 1060, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19967, 5, 1062, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19973, 5, 1063, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20335, 5, 1071, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20744, 5, 1075, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20756, 5, 1077, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20762, 5, 1078, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20768, 5, 1079, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20774, 5, 1080, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20780, 5, 1081, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20786, 5, 1082, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20792, 5, 1083, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20798, 5, 1084, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20854, 5, 1093, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20860, 5, 1094, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20866, 5, 1095, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20872, 5, 1096, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20878, 5, 1097, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20884, 5, 1098, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20890, 5, 1099, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20896, 5, 1100, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20902, 5, 1101, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20908, 5, 1102, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20914, 5, 1103, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20920, 5, 1104, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20926, 5, 1105, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20932, 5, 1106, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20938, 5, 1107, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20944, 5, 1108, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20950, 5, 1109, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20956, 5, 1110, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20962, 5, 1111, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21040, 5, 1124, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21046, 5, 1125, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21052, 5, 1126, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21058, 5, 1127, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21064, 5, 1128, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21070, 5, 1129, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21076, 5, 1130, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21082, 5, 1131, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21088, 5, 1132, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21094, 5, 1133, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21100, 5, 1134, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21106, 5, 1135, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21112, 5, 1136, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21118, 5, 1137, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21124, 5, 1138, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21130, 5, 1139, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21136, 5, 1140, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21142, 5, 1141, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21190, 5, 1149, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21196, 5, 1150, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21202, 5, 1151, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21208, 5, 1152, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21214, 5, 1153, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21220, 5, 1154, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21226, 5, 1155, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21232, 5, 1156, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21238, 5, 1157, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21244, 5, 1158, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21250, 5, 1159, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21256, 5, 1160, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21262, 5, 1161, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21268, 5, 1162, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21274, 5, 1163, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21280, 5, 1164, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21286, 5, 1165, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21292, 5, 1166, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21298, 5, 1167, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21328, 5, 1172, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21340, 5, 1174, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21346, 5, 1175, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21352, 5, 1176, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21358, 5, 1177, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21364, 5, 1178, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21370, 5, 1179, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21376, 5, 1180, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21382, 5, 1181, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21388, 5, 1182, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21394, 5, 1183, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21400, 5, 1184, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21412, 5, 1186, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21418, 5, 1187, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21424, 5, 1188, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21460, 5, 1194, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21466, 5, 1195, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21472, 5, 1196, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21478, 5, 1197, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21484, 5, 1198, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21490, 5, 1199, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21496, 5, 1200, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21502, 5, 1201, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21508, 5, 1202, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21514, 5, 1203, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21526, 5, 1205, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21532, 5, 1206, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21538, 5, 1207, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21544, 5, 1208, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21550, 5, 1209, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21556, 5, 1210, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21562, 5, 1211, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21568, 5, 1212, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21574, 5, 1213, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21580, 5, 1214, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21796, 5, 1250, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21802, 5, 1251, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21814, 5, 1253, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21820, 5, 1254, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21826, 5, 1255, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21898, 5, 1267, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21910, 5, 1269, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21916, 5, 1270, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21922, 5, 1271, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21928, 5, 1272, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21934, 5, 1273, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21940, 5, 1274, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21946, 5, 1275, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21952, 5, 1276, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21958, 5, 1277, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21964, 5, 1278, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21970, 5, 1279, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21976, 5, 1280, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21982, 5, 1281, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21988, 5, 1282, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21994, 5, 1283, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22000, 5, 1284, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22006, 5, 1285, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22012, 5, 1286, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22018, 5, 1287, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22030, 5, 1289, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22036, 5, 1290, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22042, 5, 1291, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22048, 5, 1292, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22054, 5, 1293, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22060, 5, 1294, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22066, 5, 1295, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22072, 5, 1296, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22078, 5, 1297, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22084, 5, 1298, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22090, 5, 1299, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22096, 5, 1300, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22102, 5, 1301, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22108, 5, 1302, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22114, 5, 1303, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22120, 5, 1304, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22126, 5, 1305, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22132, 5, 1306, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22204, 5, 1318, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22210, 5, 1319, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22216, 5, 1320, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22222, 5, 1321, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21316, 5, 1170, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22228, 5, 1322, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22234, 5, 1323, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22240, 5, 1324, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22246, 5, 1325, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22252, 5, 1326, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22258, 5, 1327, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22264, 5, 1328, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22270, 5, 1329, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22276, 5, 1330, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22282, 5, 1331, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22288, 5, 1332, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22294, 5, 1333, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22300, 5, 1334, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22306, 5, 1335, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22312, 5, 1336, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22318, 5, 1337, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22354, 5, 1343, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22360, 5, 1344, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22366, 5, 1345, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22372, 5, 1346, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22378, 5, 1347, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22390, 5, 1349, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22396, 5, 1350, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22402, 5, 1351, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22408, 5, 1352, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22414, 5, 1353, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22420, 5, 1354, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22426, 5, 1355, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22432, 5, 1356, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22438, 5, 1357, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22444, 5, 1358, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22456, 5, 1360, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22462, 5, 1361, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22468, 5, 1362, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22576, 5, 1380, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22582, 5, 1381, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22588, 5, 1382, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22594, 5, 1383, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22600, 5, 1384, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22606, 5, 1385, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22612, 5, 1386, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22618, 5, 1387, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22624, 5, 1388, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22630, 5, 1389, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22636, 5, 1390, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22642, 5, 1391, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22648, 5, 1392, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22654, 5, 1393, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22660, 5, 1394, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22666, 5, 1395, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22672, 5, 1396, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22678, 5, 1397, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22684, 5, 1398, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22690, 5, 1399, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22696, 5, 1400, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22702, 5, 1401, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22708, 5, 1402, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22714, 5, 1403, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22720, 5, 1404, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22726, 5, 1405, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22732, 5, 1406, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22738, 5, 1407, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22744, 5, 1408, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22750, 5, 1409, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22756, 5, 1410, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22762, 5, 1411, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22768, 5, 1412, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22774, 5, 1413, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22780, 5, 1414, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22786, 5, 1415, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22792, 5, 1416, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22798, 5, 1417, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22804, 5, 1418, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22810, 5, 1419, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22816, 5, 1420, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22822, 5, 1421, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22828, 5, 1422, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22834, 5, 1423, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22840, 5, 1424, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22846, 5, 1425, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22852, 5, 1426, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22858, 5, 1427, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22864, 5, 1428, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22870, 5, 1429, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22876, 5, 1430, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22882, 5, 1431, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22888, 5, 1432, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22894, 5, 1433, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22900, 5, 1434, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22906, 5, 1435, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22912, 5, 1436, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22924, 5, 1438, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22930, 5, 1439, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22936, 5, 1440, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22942, 5, 1441, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22948, 5, 1442, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22954, 5, 1443, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22960, 5, 1444, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22966, 5, 1445, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22972, 5, 1446, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22978, 5, 1447, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22984, 5, 1448, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22990, 5, 1449, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22996, 5, 1450, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23002, 5, 1451, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23008, 5, 1452, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23014, 5, 1453, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23032, 5, 1456, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23044, 5, 1458, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23050, 5, 1459, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23056, 5, 1460, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23062, 5, 1461, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23068, 5, 1462, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23074, 5, 1463, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23080, 5, 1464, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23086, 5, 1465, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23092, 5, 1466, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23098, 5, 1467, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23104, 5, 1468, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23110, 5, 1469, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23116, 5, 1470, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23122, 5, 1471, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23128, 5, 1472, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23134, 5, 1473, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23140, 5, 1474, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23146, 5, 1475, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23152, 5, 1476, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23158, 5, 1477, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23170, 5, 1479, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23176, 5, 1480, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23206, 5, 1485, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23212, 5, 1486, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23218, 5, 1487, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23224, 5, 1488, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23230, 5, 1489, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23236, 5, 1490, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23242, 5, 1491, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23248, 5, 1492, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23254, 5, 1493, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23260, 5, 1494, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23266, 5, 1495, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23272, 5, 1496, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23278, 5, 1497, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23284, 5, 1498, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23290, 5, 1499, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23296, 5, 1500, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23302, 5, 1501, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23308, 5, 1502, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23314, 5, 1503, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23320, 5, 1504, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23326, 5, 1505, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23332, 5, 1506, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23344, 5, 1508, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23350, 5, 1509, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23356, 5, 1510, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23362, 5, 1511, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23368, 5, 1512, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23374, 5, 1513, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23380, 5, 1514, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23386, 5, 1515, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23392, 5, 1516, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23398, 5, 1517, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23404, 5, 1518, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23410, 5, 1519, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23416, 5, 1520, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23422, 5, 1521, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23428, 5, 1522, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23434, 5, 1523, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23440, 5, 1524, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23446, 5, 1525, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23458, 5, 1527, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23464, 5, 1528, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23470, 5, 1529, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23476, 5, 1530, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23482, 5, 1531, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23488, 5, 1532, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23494, 5, 1533, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23500, 5, 1534, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23506, 5, 1535, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23512, 5, 1536, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23518, 5, 1537, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23524, 5, 1538, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23530, 5, 1539, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23536, 5, 1540, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23542, 5, 1541, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23548, 5, 1542, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23554, 5, 1543, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23560, 5, 1544, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23566, 5, 1545, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23572, 5, 1546, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23578, 5, 1547, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23584, 5, 1548, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23590, 5, 1549, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23596, 5, 1550, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23602, 5, 1551, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23608, 5, 1552, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23614, 5, 1553, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23620, 5, 1554, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23626, 5, 1555, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23656, 5, 1560, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23662, 5, 1561, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23668, 5, 1562, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23674, 5, 1563, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23680, 5, 1564, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23686, 5, 1565, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23692, 5, 1566, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23698, 5, 1567, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23704, 5, 1568, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23710, 5, 1569, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23716, 5, 1570, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23722, 5, 1571, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23728, 5, 1572, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23734, 5, 1573, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23740, 5, 1574, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23746, 5, 1575, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23752, 5, 1576, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23758, 5, 1577, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23770, 5, 1579, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23776, 5, 1580, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23782, 5, 1581, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23788, 5, 1582, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23794, 5, 1583, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23800, 5, 1584, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23806, 5, 1585, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23812, 5, 1586, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23818, 5, 1587, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23824, 5, 1588, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23830, 5, 1589, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23836, 5, 1590, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23842, 5, 1591, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23848, 5, 1592, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23854, 5, 1593, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23860, 5, 1594, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23866, 5, 1595, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23872, 5, 1596, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23878, 5, 1597, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23884, 5, 1598, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23890, 5, 1599, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23896, 5, 1600, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23902, 5, 1601, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23908, 5, 1602, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23914, 5, 1603, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23920, 5, 1604, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23926, 5, 1605, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23932, 5, 1606, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23938, 5, 1607, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23944, 5, 1608, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23950, 5, 1609, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23956, 5, 1610, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23962, 5, 1611, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23968, 5, 1612, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23974, 5, 1613, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23980, 5, 1614, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23986, 5, 1615, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23992, 5, 1616, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23998, 5, 1617, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24004, 5, 1618, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24010, 5, 1619, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24016, 5, 1620, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24022, 5, 1621, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24028, 5, 1622, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24040, 5, 1624, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24046, 5, 1625, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24052, 5, 1626, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24058, 5, 1627, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24064, 5, 1628, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24070, 5, 1629, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24124, 5, 1638, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24130, 5, 1639, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24136, 5, 1640, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24142, 5, 1641, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24148, 5, 1642, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24154, 5, 1643, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24160, 5, 1644, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24166, 5, 1645, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24172, 5, 1646, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24178, 5, 1647, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24184, 5, 1648, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24190, 5, 1649, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24196, 5, 1650, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24208, 5, 1652, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24214, 5, 1653, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24220, 5, 1654, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24226, 5, 1655, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24280, 5, 1664, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24286, 5, 1665, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24292, 5, 1666, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23638, 5, 1557, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24298, 5, 1667, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24304, 5, 1668, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24310, 5, 1669, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24316, 5, 1670, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24322, 5, 1671, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24328, 5, 1672, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24334, 5, 1673, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24340, 5, 1674, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24346, 5, 1675, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24352, 5, 1676, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24358, 5, 1677, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24364, 5, 1678, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24370, 5, 1679, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24376, 5, 1680, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24382, 5, 1681, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24388, 5, 1682, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24394, 5, 1683, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24400, 5, 1684, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24406, 5, 1685, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24412, 5, 1686, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24418, 5, 1687, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24424, 5, 1688, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24430, 5, 1689, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24436, 5, 1690, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24442, 5, 1691, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24448, 5, 1692, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24454, 5, 1693, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24460, 5, 1694, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24466, 5, 1695, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24472, 5, 1696, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24478, 5, 1697, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24484, 5, 1698, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24490, 5, 1699, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24496, 5, 1700, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24502, 5, 1701, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24508, 5, 1702, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24514, 5, 1703, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24520, 5, 1704, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24526, 5, 1705, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24532, 5, 1706, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24538, 5, 1707, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24544, 5, 1708, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24550, 5, 1709, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24556, 5, 1710, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24562, 5, 1711, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24568, 5, 1712, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24574, 5, 1713, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24580, 5, 1714, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24586, 5, 1715, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24592, 5, 1716, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24598, 5, 1717, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24652, 5, 1726, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24658, 5, 1727, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24664, 5, 1728, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24670, 5, 1729, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24676, 5, 1730, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24682, 5, 1731, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24688, 5, 1732, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24694, 5, 1733, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24700, 5, 1734, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24706, 5, 1735, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24712, 5, 1736, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24718, 5, 1737, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24940, 5, 1774, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24946, 5, 1775, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24952, 5, 1776, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24958, 5, 1777, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24964, 5, 1778, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24970, 5, 1779, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24976, 5, 1780, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24982, 5, 1781, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24988, 5, 1782, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24994, 5, 1783, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25000, 5, 1784, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25006, 5, 1785, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25012, 5, 1786, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25018, 5, 1787, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25024, 5, 1788, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25030, 5, 1789, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25036, 5, 1790, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25042, 5, 1791, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25048, 5, 1792, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25054, 5, 1793, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25060, 5, 1794, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25066, 5, 1795, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25072, 5, 1796, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25078, 5, 1797, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25084, 5, 1798, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25090, 5, 1799, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25096, 5, 1800, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25102, 5, 1801, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25108, 5, 1802, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25114, 5, 1803, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25120, 5, 1804, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25126, 5, 1805, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25132, 5, 1806, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25144, 5, 1808, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25150, 5, 1809, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25156, 5, 1810, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25162, 5, 1811, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25168, 5, 1812, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25174, 5, 1813, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25180, 5, 1814, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25186, 5, 1815, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25192, 5, 1816, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25198, 5, 1817, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25204, 5, 1818, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25210, 5, 1819, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25216, 5, 1820, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25222, 5, 1821, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25228, 5, 1822, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25234, 5, 1823, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25240, 5, 1824, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25246, 5, 1825, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25252, 5, 1826, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25258, 5, 1827, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25264, 5, 1828, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25270, 5, 1829, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25276, 5, 1830, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25282, 5, 1831, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25288, 5, 1832, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25294, 5, 1833, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25300, 5, 1834, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25306, 5, 1835, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25312, 5, 1836, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25318, 5, 1837, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25324, 5, 1838, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25330, 5, 1839, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25336, 5, 1840, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25342, 5, 1841, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25348, 5, 1842, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25354, 5, 1843, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25360, 5, 1844, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25366, 5, 1845, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25372, 5, 1846, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25378, 5, 1847, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25384, 5, 1848, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25390, 5, 1849, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25492, 5, 1866, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25528, 5, 1872, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25534, 5, 1873, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23644, 5, 1558, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19793, 5, 1033, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23650, 5, 1559, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21322, 5, 1171, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21712, 5, 1236, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (314, 5, 188, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (315, 5, 189, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (266, 5, 187, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21430, 5, 1189, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24076, 5, 1630, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21436, 5, 1190, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24082, 5, 1631, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23200, 5, 1484, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21442, 5, 1191, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24088, 5, 1632, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21448, 5, 1192, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (324, 5, 190, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24094, 5, 1633, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19799, 5, 1034, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19805, 5, 1035, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23026, 5, 1455, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21454, 5, 1193, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (298, 5, 478, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (310, 5, 207, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (323, 5, 208, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (312, 5, 211, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (313, 5, 212, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (311, 5, 213, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24100, 5, 1634, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8820, 5, 934, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8821, 5, 935, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8822, 5, 936, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19811, 5, 1036, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19817, 5, 1037, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20066, 5, 1066, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20737, 5, 1074, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21832, 5, 1256, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (253, 5, 495, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (291, 5, 522, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (288, 5, 523, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21838, 5, 1257, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24112, 5, 1636, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18428, 5, 1007, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21520, 5, 1204, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24106, 5, 1635, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21844, 5, 1258, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24118, 5, 1637, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21850, 5, 1259, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24232, 5, 1656, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21856, 5, 1260, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24238, 5, 1657, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21862, 5, 1261, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24244, 5, 1658, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21154, 5, 1143, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21868, 5, 1262, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24250, 5, 1659, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (316, 5, 605, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21874, 5, 1263, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24256, 5, 1660, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21880, 5, 1264, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24262, 5, 1661, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (265, 5, 205, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (305, 5, 247, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (307, 5, 248, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (303, 5, 249, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (304, 5, 250, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (302, 5, 251, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (274, 5, 253, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (306, 5, 254, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (285, 5, 255, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (292, 5, 256, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (293, 5, 257, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (278, 5, 258, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (286, 5, 259, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (321, 5, 235, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (325, 5, 236, '-1', '-1', '-1', '-1');
INSERT INTO public.aros_acos VALUES (322, 5, 237, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (259, 5, 238, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (258, 5, 239, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (299, 5, 241, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (318, 5, 242, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (296, 5, 261, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (264, 5, 263, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (262, 5, 269, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (263, 5, 270, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24268, 5, 1662, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21886, 5, 1265, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (320, 5, 243, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (326, 5, 244, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (308, 5, 245, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (276, 5, 228, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (277, 5, 229, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (309, 5, 231, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (300, 5, 232, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (317, 5, 233, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (319, 5, 234, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (260, 5, 267, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (261, 5, 268, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (284, 5, 260, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24274, 5, 1663, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (14344, 5, 983, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19823, 5, 1038, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19829, 5, 1039, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19835, 5, 1040, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19841, 5, 1041, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19847, 5, 1042, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20078, 5, 1068, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20084, 5, 1069, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20725, 5, 1072, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20731, 5, 1073, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21892, 5, 1266, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24604, 5, 1718, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (20750, 5, 1076, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21706, 5, 1235, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22144, 5, 1308, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24610, 5, 1719, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22150, 5, 1309, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24616, 5, 1720, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22156, 5, 1310, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18530, 5, 1024, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18536, 5, 1025, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18542, 5, 1026, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18548, 5, 1027, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18554, 5, 1028, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18560, 5, 1029, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18566, 5, 1030, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18572, 5, 1031, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (18578, 5, 1032, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24622, 5, 1721, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22138, 5, 1307, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25543, 5, 619, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22162, 5, 1311, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24628, 5, 1722, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22168, 5, 1312, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24634, 5, 1723, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22450, 5, 1359, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22174, 5, 1313, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24640, 5, 1724, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (255, 5, 636, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (272, 5, 679, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22180, 5, 1314, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (24646, 5, 1725, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22918, 5, 1437, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (23764, 5, 1578, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (270, 5, 682, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (271, 5, 683, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (267, 5, 684, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (268, 5, 685, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (273, 5, 690, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (269, 5, 692, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19979, 5, 1064, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19985, 5, 1065, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22186, 5, 1315, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25138, 5, 1807, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22192, 5, 1316, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (256, 5, 289, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (19853, 5, 1043, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22198, 5, 1317, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22324, 5, 1338, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22330, 5, 1339, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22336, 5, 1340, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (282, 5, 307, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (279, 5, 308, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (280, 5, 309, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (281, 5, 310, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (283, 5, 311, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8813, 5, 938, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8814, 5, 939, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8815, 5, 940, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8816, 5, 941, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (25544, 5, 942, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (8817, 5, 943, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22342, 5, 1341, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (22348, 5, 1342, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (21904, 5, 1268, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (28246, 5, 21, '1', '1', '1', '1');
INSERT INTO public.aros_acos VALUES (28247, 5, 325, '1', '1', '1', '1');
--
-- Name: aros_acos_id_seq; Type: SEQUENCE SET; Schema: public; Owner: webgfc
--

SELECT pg_catalog.setval('aros_acos_id_seq', 27457, true);


--
-- Name: aros_id_seq; Type: SEQUENCE SET; Schema: public; Owner: webgfc
--

SELECT pg_catalog.setval('aros_id_seq', 125, true);


--
-- Data for Name: bannettes; Type: TABLE DATA; Schema: public; Owner: webgfc
--

INSERT INTO bannettes VALUES (1, 'docs');
INSERT INTO bannettes VALUES (2, 'refus');
INSERT INTO bannettes VALUES (3, 'reponses');
INSERT INTO bannettes VALUES (4, 'flux');
INSERT INTO bannettes VALUES (5, 'aiguillage');
INSERT INTO bannettes VALUES (6, 'infos');
INSERT INTO bannettes VALUES (7, 'copy');


--
-- Name: bannettes_id_seq; Type: SEQUENCE SET; Schema: public; Owner: webgfc
--

SELECT pg_catalog.setval('bannettes_id_seq', 8, true);


--
-- Data for Name: connecteurs; Type: TABLE DATA; Schema: public; Owner: webgfc
--

INSERT INTO connecteurs VALUES (1, 'Parapheur électronique (Signature)', '2014-08-05 13:53:10.780508', '2014-08-05 13:53:10.780508');
INSERT INTO connecteurs VALUES (2, 'GED (CMIS)', '2015-06-08 11:58:03.398832', '2015-06-08 11:58:03.398832');
INSERT INTO connecteurs ( id, name, ldap_note, ldap_username, ldap_nom, ldap_prenom, ldap_mail, ldap_numtel, ldap_portable, created, modified ) VALUES
(3, 'Annuaire LDAP', 'info', 'samaccountname', 'sn', 'givenname', 'mail', 'telephonenumber', 'mobile', '2018-08-01 11:58:03.398832', '2018-08-01 11:58:03.398832');

INSERT INTO connecteurs ( id, name, created, modified ) VALUES ( 4, 'GRC Localeo', NOW(), NOW() );

INSERT INTO connecteurs ( id, name, created, modified) VALUES ( 5, 'PASTELL', '2019-10-21 11:58:03.398832', '2019-10-21 11:58:03.398832');

INSERT INTO connecteurs ( id, name, host, created, modified) VALUES ( 6, 'Démarches-Simplifiées (DS)', 'https://www.demarches-simplifiees.fr/api/v1/', '2020-04-03 08:58:03.398832', '2020-04-03 08:58:03.398832');

INSERT INTO connecteurs ( id, name, host, created, modified) VALUES ( 7, 'LsMessage (SMS)', '', '2020-06-12 08:58:03.398832', '2020-06-12 08:58:03.398832');

INSERT INTO connecteurs ( id, name, host, created, modified) VALUES ( 8, 'Direct-Mairie (DM)', '', '2020-06-18 08:58:03.398832', '2020-06-18 08:58:03.398832');

--
-- Name: connecteurs_id_seq; Type: SEQUENCE SET; Schema: public; Owner: webgfc
--

SELECT pg_catalog.setval('connecteurs_id_seq', 9, true);


--
-- Data for Name: dacos; Type: TABLE DATA; Schema: public; Owner: webgfc
--

INSERT INTO dacos VALUES (1, NULL, NULL, NULL, 'data', 1, 2);
INSERT INTO dacos VALUES ( '-1', NULL, 'Type', '-1', 'Flux_sans_traitement.-1', 3, 6);
INSERT INTO dacos VALUES ( '-2', '-1', 'Soustype', '-1', 'Cloture_automatique.-2', 4, 5);

--
-- Name: dacos_id_seq; Type: SEQUENCE SET; Schema: public; Owner: webgfc
--

SELECT pg_catalog.setval('dacos_id_seq', 2, true);


--
-- Data for Name: formatsreponse; Type: TABLE DATA; Schema: public; Owner: webgfc
--

INSERT INTO formatsreponse VALUES (1, 'Modèle de document');
INSERT INTO formatsreponse VALUES (2, 'Fichier son (.wav)');
INSERT INTO formatsreponse VALUES (3, 'Fichier vidéo (.mpeg)');
INSERT INTO formatsreponse VALUES (4, 'SMS');
INSERT INTO formatsreponse VALUES (5, 'Email');


--
-- Name: formatsreponse_id_seq; Type: SEQUENCE SET; Schema: public; Owner: webgfc
--

SELECT pg_catalog.setval('formatsreponse_id_seq', 6, true);


--
-- Data for Name: profils; Type: TABLE DATA; Schema: public; Owner: webgfc
--

INSERT INTO profils VALUES (1, 'Superadmin', '2012-02-17 14:55:14.842819', '2012-02-17 14:55:14.842819', true, true, NULL);
INSERT INTO profils VALUES (2, 'Admin', '2012-02-17 15:15:43.236166', '2012-02-17 15:15:43.236166', true, true, NULL);
INSERT INTO profils VALUES (3, 'Initiateur', '2012-02-17 15:15:43.260401', '2012-02-17 15:15:43.260401', true, true, NULL);
INSERT INTO profils VALUES (4, 'Valideur', '2012-02-17 15:15:43.277671', '2012-02-17 15:15:43.277671', true, true, NULL);
INSERT INTO profils VALUES (5, 'Valideur Editeur', '2012-02-17 15:15:43.296607', '2012-02-17 15:15:43.296607', true, true, NULL);
INSERT INTO profils VALUES (6, 'Documentaliste', '2012-02-17 15:15:43.310943', '2012-02-17 15:15:43.310943', true, true, NULL);
INSERT INTO profils VALUES (7, 'Aiguilleur', '2012-07-06 11:51:30.769035', '2012-07-06 11:51:30.769035', true, true, NULL);


--
-- Name: profils_id_seq; Type: SEQUENCE SET; Schema: public; Owner: webgfc
--

SELECT pg_catalog.setval('profils_id_seq', 9, true);


--
-- Data for Name: keywordtypes; Type: TABLE DATA; Schema: public; Owner: webgfc
--

INSERT INTO keywordtypes VALUES (1, 'corpname', 'Collectivité', '2012-02-28 11:38:20.054222', '2012-02-28 11:38:20.054222');
INSERT INTO keywordtypes VALUES (2, 'famname', 'Nom de famille', '2012-02-28 11:38:20.088158', '2012-02-28 11:38:20.088158');
INSERT INTO keywordtypes VALUES (3, 'geogname', 'Nom géographique', '2012-02-28 11:38:20.096135', '2012-02-28 11:38:20.096135');
INSERT INTO keywordtypes VALUES (4, 'name', 'Nom', '2012-02-28 11:38:20.105176', '2012-02-28 11:38:20.105176');
INSERT INTO keywordtypes VALUES (5, 'occupation', 'Fonction', '2012-02-28 11:38:20.113056', '2012-02-28 11:38:20.113056');
INSERT INTO keywordtypes VALUES (6, 'persname', 'Nom de personne', '2012-02-28 11:38:20.12103', '2012-02-28 11:38:20.12103');
INSERT INTO keywordtypes VALUES (7, 'subject', 'Mot-matière', '2012-02-28 11:38:20.129994', '2012-02-28 11:38:20.129994');
INSERT INTO keywordtypes VALUES (8, 'genreform', 'Type de document', '2012-02-28 11:38:20.138126', '2012-02-28 11:38:20.138126');
INSERT INTO keywordtypes VALUES (9, 'function', 'Activité', '2012-02-28 11:38:20.146106', '2012-02-28 11:38:20.146106');


--
-- Data for Name: keywordlists; Type: TABLE DATA; Schema: public; Owner: webgfc
--

INSERT INTO keywordlists VALUES (3, 'Liste d''autorité « Contexte historique »', '\x5365727669636520696e7465726d696e697374c3a97269656c20646573204172636869766573206465204672616e6365203a20766f636162756c6169726520636f6e7472c3b46cc3a92064657320636f6e74657874657320686973746f726971756573', 1, true, 7, 'LISTE D''AUTORITE « CONTEXTE HISTORIQUE »', 'Liste d''autorité « Contexte historique »', 'Service interministériel des Archives de France', '2010-04-21', 'http://www.archivesdefrance.culture.gouv.fr/thesaurus/data/Contexte?includeSchemes=true', 'http://www.archivesdefrance.culture.gouv.fr/thesaurus/resource/Contexte', 1, 1, '2011-05-25 10:09:29', '2011-05-25 10:09:29');
INSERT INTO keywordlists VALUES (4, 'Liste d''autorités « Typologie documentaire »', '\x5365727669636520696e7465726d696e697374c3a97269656c20646573204172636869766573206465204672616e6365203a20766f636162756c6169726520636f6e7472c3b46cc3a9206465206c61207479706f6c6f67696520646f63756d656e74616972652064616e73206c6573206172636869766573', 1, true, 7, 'LISTE D''AUTORITES « TYPOLOGIE DOCUMENTAIRE »', 'Liste d''autorités « Typologie documentaire »', 'Service interministériel des Archives de France', '2011-04-18', 'http://www.archivesdefrance.culture.gouv.fr/thesaurus/data/Typologie?includeSchemes=true', 'http://www.archivesdefrance.culture.gouv.fr/thesaurus/resource/Typologie', 1, 1, '2011-05-25 10:13:13', '2011-05-25 10:13:13');
INSERT INTO keywordlists VALUES (2, 'Liste d''autorités « Actions »', '\x5365727669636520696e7465726d696e697374c3a97269656c20646573204172636869766573206465204672616e6365203a20766f636162756c6169726520636f6e7472c3b46cc3a92064657320616374696f6e732064616e73206c6573206172636869766573', 1, true, 7, 'LISTE D''AUTORITES « ACTIONS »', 'Liste d''autorités « Actions »', 'Service interministériel des Archives de France', '2011-04-07', 'http://www.archivesdefrance.culture.gouv.fr/thesaurus/data/Actions?includeSchemes=true', 'http://www.archivesdefrance.culture.gouv.fr/thesaurus/resource/Actions', 1, 1, '2011-05-25 10:06:25', '2011-05-25 10:14:22');
INSERT INTO keywordlists VALUES (1, 'Thésaurus-matières', '\x5365727669636520696e7465726d696e697374c3a97269656c20646573204172636869766573206465204672616e6365203a206465736372697074696f6e20647520766f636162756c61697265', 1, false, 7, 'THESAURUS-MATIERES', 'Thésaurus-matières', 'Service interministériel des Archives de France', '2011-04-19', 'http://www.archivesdefrance.culture.gouv.fr/thesaurus/data/Matiere?includeSchemes=true', 'http://www.archivesdefrance.culture.gouv.fr/thesaurus/resource/Matiere', 1, 1, '2015-10-14 07:54:57.221605', '2015-10-14 07:54:57.221605');


--
-- Name: keywordlists_id_seq; Type: SEQUENCE SET; Schema: public; Owner: webgfc
--

SELECT pg_catalog.setval('keywordlists_id_seq', 5, true);


--
-- Data for Name: keywords; Type: TABLE DATA; Schema: public; Owner: webgfc
--

INSERT INTO keywords VALUES (2, 1, 'T1-543', '\\xc3a9636f6e6f6d6965', NULL, 3, 4, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (1814, 1, 'T3-224', '\\x7461726966', NULL, 3627, 3628, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1826, 1, 'T3-221', '\\x7374617475742064276173736f63696174696f6e', NULL, 3651, 3652, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (297, 1, 'T1-90', '\\x66697363616c6974c3a92070c3a974726f6c69657265', NULL, 593, 594, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (1827, 1, 'T3-51', '\\x636172746f6e206427696e7669746174696f6e', NULL, 3653, 3654, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (424, 1, 'T1-1219', '\\x696d6d6575626c65206465206772616e64652068617574657572', NULL, 847, 848, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (620, 1, 'T1-1173', '\\x6c6f69736972', NULL, 1239, 1240, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (1126, 1, 'T1-743', '\\x76696520706173746f72616c65', NULL, 2251, 2252, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1777, 1, 'T3-150', '\\x6f75767261676520696d7072696dc3a9', NULL, 3553, 3554, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1746, 1, 'T3-44', '\\x6361727465206427c3a96c656374657572', NULL, 3491, 3492, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1327, 1, 'T1-1440', '\\x766f6c61696c6c65', NULL, 2653, 2654, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (546, 1, 'T1-1241', '\\x73616c6c6520646520737065637461636c6573', NULL, 1091, 1092, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (82, 1, 'T1-21', '\\x63c3a972c3a96d6f6e6965207075626c69717565', NULL, 163, 164, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (579, 1, 'T1-237', '\\x61756265726765206465206a65756e65737365', NULL, 1157, 1158, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (372, 1, 'T1-484', '\\x70617472696d6f696e6520736369656e7469666971756520657420746563686e69717565', NULL, 743, 744, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (250, 1, 'T1-80', '\\x6469736369706c696e652070c3a96e6974656e746961697265', NULL, 499, 500, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (218, 1, 'T1-284', '\\x696e63656e646965', NULL, 435, 436, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (219, 1, 'T1-698', '\\x636174617374726f706865206e61747572656c6c65', NULL, 437, 438, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (701, 1, 'T1-325', '\\x666f72c3aa7420636f6d6d756e616c65', NULL, 1401, 1402, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (889, 1, 'T1-1012', '\\x6173737572616e6365206d617465726e6974c3a9', NULL, 1777, 1778, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (261, 1, 'T1-85', '\\x656d706c6f6920c3a02074656d7073207061727469656c', NULL, 521, 522, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (302, 1, 'T1-1315', '\\xc3a96e65726769652072656e6f7576656c61626c65', NULL, 603, 604, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (373, 1, 'T1-226', '\\x6f6575767265206427617274', NULL, 745, 746, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (162, 1, 'T1-892', '\\x6a75737469636520636976696c65', NULL, 323, 324, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (301, 1, 'T1-92', '\\xc3a96e65726769652067c3a96f746865726d69717565', NULL, 601, 602, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (1569, 1, 'T4-68', '\\x6d6f7576656d656e7420646573204c756d69c3a8726573', NULL, 3137, 3138, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (750, 1, 'T1-357', '\\x73747275637475726520636f6d6d756e616c652064276169646520736f6369616c65', NULL, 1499, 1500, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (477, 1, 'T1-1264', '\\x73c3a96375726974c3a9206475207472617661696c', NULL, 953, 954, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (486, 1, 'T1-182', '\\x64c3a963656e7472616c69736174696f6e20696e647573747269656c6c65', NULL, 971, 972, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (970, 1, 'T1-514', '\\x617274206472616d617469717565', NULL, 1939, 1940, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1212, 1, 'T1-792', '\\x696e737472756374696f6e2063697669717565', NULL, 2423, 2424, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1114, 1, 'T1-1120', '\\x766f6965207069c3a9746f6e6e65', NULL, 2227, 2228, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (240, 1, 'T1-75', '\\x666f726d6174696f6e206d75736963616c65', NULL, 479, 480, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (165, 1, 'T1-48', '\\x6f757672616765206427617274', NULL, 329, 330, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (1816, 1, 'T3-36', '\\x62756c6c6574696e20646520766f7465', NULL, 3631, 3632, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (263, 1, 'T1-86', '\\xc3a96475636174696f6e20706f70756c61697265', NULL, 525, 526, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (288, 1, 'T1-803', '\\x6d6f7576656d656e74206175746f6e6f6d69737465', NULL, 575, 576, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (682, 1, 'T1-297', '\\x636972717565', NULL, 1363, 1364, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (1061, 1, 'T1-708', '\\x636972636f6e736372697074696f6e2066697363616c65', NULL, 2121, 2122, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1084, 1, 'T1-623', '\\x64726f69747320646f6d616e69617578206427416e6369656e2052c3a967696d65', NULL, 2167, 2168, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1110, 1, 'T1-657', '\\x6167656e74206e6f6e20746974756c61697265', NULL, 2219, 2220, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (738, 1, 'T1-476', '\\xc389746174', NULL, 1475, 1476, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (1288, 1, 'T1-1342', '\\x7465727261696e2064652063616d70696e67', NULL, 2575, 2576, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1122, 1, 'T1-1399', '\\x707570696c6c65206465206c61206e6174696f6e', NULL, 2243, 2244, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1149, 1, 'T1-701', '\\x736f6369c3a974c3a9206427c3a9636f6e6f6d6965206d69787465', NULL, 2297, 2298, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1678, 1, 'T3-170', '\\x70726f63c3a8732d76657262616c', NULL, 3355, 3356, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1006, 1, 'T1-627', '\\x70696c6f7465', NULL, 2011, 2012, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (711, 1, 'T1-336', '\\x617373697374616e636520c3a96475636174697665', NULL, 1421, 1422, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (712, 1, 'T1-824', '\\x70726f74656374696f6e206a756469636961697265206465206c61206a65756e65737365', NULL, 1423, 1424, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (723, 1, 'T1-959', '\\x626c6f637573', NULL, 1445, 1446, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (725, 1, 'T1-630', '\\x706f6c696365206465732066726f6e7469c3a8726573', NULL, 1449, 1450, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (749, 1, 'T1-356', '\\x70616c616973206465206a757374696365', NULL, 1497, 1498, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (994, 1, 'T1-1201', '\\x6d617269616765', NULL, 1987, 1988, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1140, 1, 'T1-1288', '\\x65617520706c757669616c65', NULL, 2279, 2280, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (247, 1, 'T1-78', '\\x6d696c6974616e7420706f6c697469717565', NULL, 493, 494, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (1530, 1, 'T4-128', '\\x7472616974c3a9206465205665727361696c6c657320283139313929', NULL, 3059, 3060, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1300, 1, 'T1-1445', '\\x73746174696f6e20726164696f20c3a96c65637472697175652070726976c3a965', NULL, 2599, 2600, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1331, 1, 'T1-1023', '\\x67617a206e61747572656c', NULL, 2661, 2662, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (292, 1, 'T1-752', '\\x696d7072696d657572', NULL, 583, 584, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (769, 1, 'T1-1283', '\\x73c3bb726574c3a9206d6f62696c69c3a87265', NULL, 1537, 1538, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (1539, 1, 'T4-97', '\\x63726973652070c3a974726f6c69c3a872652028313937332d3139373929', NULL, 3077, 3078, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1812, 1, 'T3-228', '\\x74697472652064652063697263756c6174696f6e', NULL, 3623, 3624, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (293, 1, 'T1-1212', '\\xc3a9646974657572', NULL, 585, 586, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (1813, 1, 'T3-12', '\\x61726d6f7269616c', NULL, 3625, 3626, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1219, 1, 'T1-794', '\\x68c3b474656c206465206c6120736f75732d7072c3a966656374757265', NULL, 2437, 2438, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1367, 1, 'T1-1161', '\\x636f6d6d616e6465207075626c69717565', NULL, 2733, 2734, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (751, 1, 'T1-358', '\\x7465727261696e206d696c697461697265', NULL, 1501, 1502, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (320, 1, 'T1-390', '\\x6875697373696572206465206a757374696365', NULL, 639, 640, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (996, 1, 'T1-537', '\\x65617520706f7461626c65', NULL, 1991, 1992, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (334, 1, 'T1-860', '\\xc3a97472616e676572', NULL, 667, 668, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (669, 1, 'T1-982', '\\x6d616c61646965206465732076c3a967c3a974617578', NULL, 1337, 1338, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (823, 1, 'T1-410', '\\x64726f69747320646520646f75616e65', NULL, 1645, 1646, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (1201, 1, 'T1-775', '\\xc3a96e657267696520736f6c61697265', NULL, 2401, 2402, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1365, 1, 'T1-1451', '\\xc3a9746174732070726f76696e6369617578', NULL, 2729, 2730, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1354, 1, 'T1-1142', '\\x6163636964656e742073636f6c61697265', NULL, 2707, 2708, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1355, 1, 'T1-1225', '\\x62617461696c6c6f6e2073636f6c61697265', NULL, 2709, 2710, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1358, 1, 'T1-1125', '\\x66c3aa7465206e6174696f6e616c65', NULL, 2715, 2716, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1326, 1, 'T1-1389', '\\x616c696d656e742064752062c3a97461696c', NULL, 2651, 2652, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1266, 1, 'T1-893', '\\x7072c3a9736964656e74206427c3a97461626c697373656d656e74207075626c696320646520636f6f70c3a9726174696f6e20696e746572636f6d6d756e616c65', NULL, 2531, 2532, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1292, 1, 'T1-916', '\\x696e74657272757074696f6e20766f6c6f6e74616972652064652067726f737365737365', NULL, 2583, 2584, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1376, 1, 'T1-1401', '\\x74616fc3af736d65', NULL, 2751, 2752, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1356, 1, 'T1-1122', '\\x636f6e7365696c20636f6d6d756e61757461697265', NULL, 2711, 2712, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1360, 1, 'T1-1230', '\\x63656e7472652076696c6c65', NULL, 2719, 2720, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1202, 1, 'T1-776', '\\x6368616d62726520636f6e73756c61697265', NULL, 2403, 2404, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (766, 1, 'T1-1250', '\\x7072c3a9736964656e7420647520636f6e7365696c2067c3a96ec3a972616c', NULL, 1531, 1532, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (399, 1, 'T1-135', '\\x706f6964732d65742d6d657375726573', NULL, 797, 798, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (1193, 1, 'T1-765', '\\x6cc3a96769736c6174696f6e', NULL, 2385, 2386, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1364, 1, 'T1-1156', '\\x6661757820656e20c3a963726974757265', NULL, 2727, 2728, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (763, 1, 'T1-364', '\\x666175782074c3a96d6f69676e616765', NULL, 1525, 1526, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (999, 1, 'T1-540', '\\x61726368c3a96f6c6f6769652061c3a97269656e6e65', NULL, 1997, 1998, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (950, 1, 'T1-857', '\\x656e736569676e656d656e74207072c3a9c3a96cc3a96d656e7461697265', NULL, 1899, 1900, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1368, 1, 'T1-1189', '\\x746f7869636f6d616e65', NULL, 2735, 2736, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1375, 1, 'T1-1367', '\\x70726f74657374616e7469736d65', NULL, 2749, 2750, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1745, 1, 'T3-178', '\\x70726f66657373696f6e20646520666f69', NULL, 3489, 3490, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1672, 1, 'T3-208', '\\x72c3a9706572746f69726520646573206f70c3a9726174696f6e7320646520646f75616e6573', NULL, 3343, 3344, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1209, 1, 'T1-1202', '\\x6175746f726974c3a92064276f636375706174696f6e', NULL, 2417, 2418, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1370, 1, 'T1-1362', '\\x6d616c616469652064657320616e696d617578', NULL, 2739, 2740, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (670, 1, 'T1-461', '\\x616e696d616c206e75697369626c65', NULL, 1339, 1340, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (671, 1, 'T1-290', '\\x6c6f67656d656e74', NULL, 1341, 1342, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (478, 1, 'T1-177', '\\x63616d7061676e6520c3a96c6563746f72616c65', NULL, 955, 956, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (1115, 1, 'T1-1340', '\\xc3a9636f6c65', NULL, 2229, 2230, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (164, 1, 'T1-47', '\\x66697363616c6974c3a92064697265637465206427416e6369656e2052c3a967696d65', NULL, 327, 328, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (1377, 1, 'T1-1180', '\\x636f6d7061676e6965207468c3a9c3a27472616c65', NULL, 2753, 2754, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1416, 1, 'T2-1', '\\x61626f6c6974696f6e', NULL, 2831, 2832, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1417, 1, 'T2-50', '\\x696e64656d6e69736174696f6e', NULL, 2833, 2834, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1488, 1, 'T4-46', '\\x323165207369c3a8636c65', NULL, 2975, 2976, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1329, 1, 'T1-1360', '\\x62617474616765', NULL, 2657, 2658, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1330, 1, 'T1-1020', '\\x6f657576726520646520677565727265', NULL, 2659, 2660, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1531, 1, 'T4-87', '\\x6166666169726520447265796675732028313839342d3139303629', NULL, 3061, 3062, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1323, 1, 'T1-1006', '\\xc3a9636f6c65206d696c697461697265', NULL, 2645, 2646, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (248, 1, 'T1-79', '\\x66c3aa746520666f7261696e65', NULL, 495, 496, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (824, 1, 'T1-456', '\\x636f6d6d6572636520657874c3a97269657572', NULL, 1647, 1648, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (556, 1, 'T1-220', '\\x636f757274696572206465206d61726368616e6469736573', NULL, 1111, 1112, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (557, 1, 'T1-221', '\\x6a657578206f6c796d706971756573', NULL, 1113, 1114, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (1234, 1, 'T1-821', '\\x646973736964656e74', NULL, 2467, 2468, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1205, 1, 'T1-1198', '\\x6dc3a963c3a96e6174', NULL, 2409, 2410, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1206, 1, 'T1-834', '\\x6d616e69666573746174696f6e2063756c747572656c6c65', NULL, 2411, 2412, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1207, 1, 'T1-1231', '\\x72c3a973697374616e6365', NULL, 2413, 2414, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1141, 1, 'T1-1276', '\\x6e61707065206427656175', NULL, 2281, 2282, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (264, 1, 'T1-786', '\\x63756c74757265', NULL, 527, 528, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (1418, 1, 'T2-21', '\\x636f6e737472756374696f6e', NULL, 2835, 2836, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1328, 1, 'T1-1026', '\\x6173736f6c656d656e74', NULL, 2655, 2656, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1529, 1, 'T4-111', '\\x6775657272652069737261c3a96c6f2d61726162652028313934382d2029', NULL, 3057, 3058, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1290, 1, 'T1-1119', '\\x617273656e616c', NULL, 2579, 2580, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (636, 1, 'T1-268', '\\x6d696e65726169', NULL, 1271, 1272, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (637, 1, 'T1-269', '\\x6a756966', NULL, 1273, 1274, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (1204, 1, 'T1-941', '\\x6d61726368c3a9206427696e74c3a972c3aa74206e6174696f6e616c', NULL, 2407, 2408, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1128, 1, 'T1-806', '\\x736167652066656d6d65', NULL, 2255, 2256, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1588, 1, 'T4-17', '\\x3965207369c3a8636c65', NULL, 3175, 3176, 3, 1, 1, '2011-05-25 10:09:31', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1131, 1, 'T1-681', '\\x6f70696e696f6e207075626c69717565', NULL, 2261, 2262, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (992, 1, 'T1-532', '\\x756e69766572736974c3a9', NULL, 1983, 1984, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1244, 1, 'T1-1165', '\\x66697363616c6974c3a920646972656374652072c3a9766f6c7574696f6e6e61697265', NULL, 2487, 2488, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (993, 1, 'T1-666', '\\x6166666169726520636976696c65', NULL, 1985, 1986, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1366, 1, 'T1-1160', '\\x68c3b474656c20636f6d6d756e61757461697265', NULL, 2731, 2732, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1371, 1, 'T1-1249', '\\x6169646520736f6369616c6520666163756c746174697665', NULL, 2741, 2742, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1359, 1, 'T1-1258', '\\x646f6d61696e65207075626c696320666c757669616c', NULL, 2717, 2718, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (715, 1, 'T1-1010', '\\x6f66666963696e6520706861726d616365757469717565', NULL, 1429, 1430, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (1361, 1, 'T1-1205', '\\x62617468796dc3a974726965', NULL, 2721, 2722, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1362, 1, 'T1-1147', '\\x726f7574652064c3a970617274656d656e74616c65', NULL, 2723, 2724, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1363, 1, 'T1-1380', '\\x766f696520696e746572636f6d6d756e616c65', NULL, 2725, 2726, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1232, 1, 'T1-818', '\\x7365637465', NULL, 2463, 2464, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (767, 1, 'T1-721', '\\x636f6e7365696c6c65722067c3a96ec3a972616c', NULL, 1533, 1534, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (1236, 1, 'T1-823', '\\x63617365726e65', NULL, 2471, 2472, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (768, 1, 'T1-368', '\\x61646d696e697374726174696f6e2063616e746f6e616c65206427c3a9706f7175652072c3a9766f6c7574696f6e6e61697265', NULL, 1535, 1536, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (1828, 1, 'T3-49', '\\x63617274652064276964656e746974c3a92070726f66657373696f6e6e656c6c65', NULL, 3655, 3656, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (794, 1, 'T1-1091', '\\x6368617373657572', NULL, 1587, 1588, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (1829, 1, 'T3-152', '\\x706172746974696f6e206d75736963616c65', NULL, 3657, 3658, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (316, 1, 'T1-100', '\\x707570696c6c65206465206c27c389746174', NULL, 631, 632, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (658, 1, 'T1-1179', '\\x63756c7465', NULL, 1315, 1316, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (1197, 1, 'T1-769', '\\x636f6e7365696c20642761646d696e697374726174696f6e', NULL, 2393, 2394, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1198, 1, 'T1-1280', '\\x6d696c696365732070726f76696e6369616c6573', NULL, 2395, 2396, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1658, 1, 'T3-16', '\\x617272c3aa74c3a9206475207072c3a9736964656e7420647520636f6e7365696c2072c3a967696f6e616c', NULL, 3315, 3316, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1659, 1, 'T3-27', '\\x62696c616e2070c3a96461676f6769717565', NULL, 3317, 3318, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1692, 1, 'T3-7', '\\x61666669636865', NULL, 3383, 3384, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1593, 1, 'T4-61', '\\x677565727265206465206c61204c6967756520642741756773626f7572672028313638382d3136393729', NULL, 3185, 3186, 3, 1, 1, '2011-05-25 10:09:31', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1645, 1, 'T3-156', '\\x7065726d69732064652064c3a96d6f6c6972', NULL, 3289, 3290, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1654, 1, 'T3-52', '\\x63617274756c61697265', NULL, 3307, 3308, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1655, 1, 'T3-132', '\\x6d617472696365206427696d706f736974696f6e', NULL, 3309, 3310, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1508, 1, 'T4-83', '\\x52c3a9766f6c7574696f6e20696e647573747269656c6c65', NULL, 3015, 3016, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1270, 1, 'T1-922', '\\x617373656d626cc3a965206575726f70c3a9656e6e65', NULL, 2539, 2540, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (604, 1, 'T1-441', '\\x7375727665696c6c616e6365206465732062c3a274696d656e7473', NULL, 1207, 1208, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (995, 1, 'T1-1233', '\\x656e7472657072697365206465207472617661696c2074656d706f7261697265', NULL, 1989, 1990, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1594, 1, 'T4-60', '\\x67756572726520646520486f6c6c616e64652028313637322d3136373829', NULL, 3187, 3188, 3, 1, 1, '2011-05-25 10:09:31', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (873, 1, 'T1-451', '\\x6175746f726f757465', NULL, 1745, 1746, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (1741, 1, 'T3-102', '\\x657461742064652073656374696f6e', NULL, 3481, 3482, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1666, 1, 'T3-212', '\\x72c3b46c65', NULL, 3331, 3332, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1667, 1, 'T3-129', '\\x6d61726368c3a9207075626c6963', NULL, 3333, 3334, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1665, 1, 'T3-192', '\\x7265676973747265206427617070656c', NULL, 3329, 3330, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1171, 1, 'T1-1185', '\\x7a6f6e65206465206d6172616973', NULL, 2341, 2342, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (300, 1, 'T1-739', '\\x657370616365206e61747572656c2073656e7369626c65', NULL, 599, 600, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (635, 1, 'T1-1141', '\\xc3a9636f6c6520646520636f6e6475697465', NULL, 1269, 1270, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (267, 1, 'T1-110', '\\x63656e747269736d65', NULL, 533, 534, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (949, 1, 'T1-952', '\\x656e736569676e656d656e742070726976c3a9', NULL, 1897, 1898, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1200, 1, 'T1-774', '\\x686162697461742070c3a9726975726261696e', NULL, 2399, 2400, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (296, 1, 'T1-672', '\\x70726f66657373696f6e206dc3a9646963616c65', NULL, 591, 592, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (951, 1, 'T1-1349', '\\x6f7269656e746174696f6e2073636f6c61697265', NULL, 1901, 1902, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (765, 1, 'T1-367', '\\x636f6e7365696c2067c3a96ec3a972616c', NULL, 1529, 1530, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (1235, 1, 'T1-822', '\\x6772c3a26365', NULL, 2469, 2470, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1563, 1, 'T4-30', '\\x313865207369c3a8636c65', NULL, 3125, 3126, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1567, 1, 'T4-63', '\\x677565727265206465205472656e746520416e732028313631382d3136343829', NULL, 3133, 3134, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1566, 1, 'T4-10', '\\x416e746971756974c3a9206761756c6f697365', NULL, 3131, 3132, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1568, 1, 'T4-66', '\\x6775657272652064652053756363657373696f6e20642741757472696368652028313734302d3137343829', NULL, 3135, 3136, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1589, 1, 'T4-22', '\\x313465207369c3a8636c65', NULL, 3177, 3178, 3, 1, 1, '2011-05-25 10:09:31', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (979, 1, 'T1-1190', '\\x736f6369c3a974c3a920636f6d6d65726369616c65', NULL, 1957, 1958, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1466, 1, 'T2-47', '\\x67657374696f6e20636f6d707461626c65', NULL, 2931, 2932, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (747, 1, 'T1-354', '\\x66696e616e63657320696e746572636f6d6d756e616c6573', NULL, 1493, 1494, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (1635, 1, 'T3-214', '\\x72c3b46c65206427696d706f736974696f6e', NULL, 3269, 3270, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1752, 1, 'T3-155', '\\x7065726d697320646520636f6e73747275697265', NULL, 3503, 3504, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1008, 1, 'T1-1050', '\\x63697263756c6174696f6e20726f757469c3a87265', NULL, 2015, 2016, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1029, 1, 'T1-1072', '\\x6d696e6575722064c3a96c696e7175616e74', NULL, 2057, 2058, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1035, 1, 'T1-561', '\\x6167656e7420736569676e65757269616c', NULL, 2069, 2070, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1668, 1, 'T3-231', '\\x7472616374', NULL, 3335, 3336, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1030, 1, 'T1-848', '\\x696e73657274696f6e2070726f66657373696f6e6e656c6c65', NULL, 2059, 2060, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1669, 1, 'T3-59', '\\x6368726f6e6f', NULL, 3337, 3338, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1199, 1, 'T1-799', '\\x63696d657469c3a87265206d696c697461697265', NULL, 2397, 2398, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1127, 1, 'T1-1032', '\\x617578696c6961697265206dc3a9646963616c', NULL, 2253, 2254, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (997, 1, 'T1-565', '\\x68796769c3a86e6520616c696d656e7461697265', NULL, 1993, 1994, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (998, 1, 'T1-539', '\\x696e717569736974696f6e', NULL, 1995, 1996, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (627, 1, 'T1-261', '\\x70617472696d6f696e652070726976c3a9', NULL, 1253, 1254, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (295, 1, 'T1-89', '\\x6dc3a9646563696e', NULL, 589, 590, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (1211, 1, 'T1-791', '\\x7265636f757672656d656e742066697363616c', NULL, 2421, 2422, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1111, 1, 'T1-659', '\\x64c3a96c69742064277573616765', NULL, 2221, 2222, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1664, 1, 'T3-179', '\\x70726f6772616d6d65', NULL, 3327, 3328, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (265, 1, 'T1-1407', '\\x756e69766572736974c3a920706f70756c61697265', NULL, 529, 530, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (1210, 1, 'T1-1274', '\\x70726f706167616e6465', NULL, 2419, 2420, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (771, 1, 'T1-1294', '\\x636f6d70746162696c6974c3a9206427656e7472657072697365', NULL, 1541, 1542, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (1554, 1, 'T4-33', '\\x417373656d626cc3a965206cc3a96769736c61746976652028313739312d3137393229', NULL, 3107, 3108, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1580, 1, 'T4-56', '\\x52c3a9666f726d652070726f74657374616e7465', NULL, 3159, 3160, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (13, 1, 'T1-764', '\\x706f70756c6174696f6e20727572616c65', NULL, 25, 26, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (158, 1, 'T1-1299', '\\x70726f74656374696f6e20736f6369616c65', NULL, 315, 316, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (262, 1, 'T1-696', '\\x656d706c6f69', NULL, 523, 524, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (1590, 1, 'T4-71', '\\x63616d7061676e652064652052757373696520283138313229', NULL, 3179, 3180, 3, 1, 1, '2011-05-25 10:09:31', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1591, 1, 'T4-70', '\\x63616d7061676e65206465204672616e636520283138313429', NULL, 3181, 3182, 3, 1, 1, '2011-05-25 10:09:31', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1608, 1, 'T3-203', '\\x72c3a8676c656d656e74206427656e7472657072697365', NULL, 3215, 3216, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1000, 1, 'T1-648', '\\x61726368c3a96f6c6f676965', NULL, 1999, 2000, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1527, 1, 'T4-118', '\\x4f636375706174696f6e20616c6c656d616e64652028313934302d3139343429', NULL, 3053, 3054, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1372, 1, 'T1-1425', '\\x6d6172c3a9636861757373c3a965', NULL, 2743, 2744, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1373, 1, 'T1-1200', '\\x64697374696e6374696f6e20686f6e6f72696669717565', NULL, 2745, 2746, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1374, 1, 'T1-1175', '\\x626f7564646869736d65', NULL, 2747, 2748, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1208, 1, 'T1-830', '\\x636f6c6c61626f726174696f6e', NULL, 2415, 2416, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (298, 1, 'T1-647', '\\x687964726f63617262757265', NULL, 595, 596, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (722, 1, 'T1-341', '\\x66726f6e7469c3a87265', NULL, 1443, 1444, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (463, 1, 'T1-1321', '\\x72c3a9717569736974696f6e73206d696c69746169726573', NULL, 925, 926, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (515, 1, 'T1-1096', '\\x6173736f63696174696f6e20736f63696f2dc3a96475636174697665', NULL, 1029, 1030, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (555, 1, 'T1-219', '\\x736f6369c3a974c3a92064652074c3a96cc3a9646966667573696f6e2070726976c3a965', NULL, 1109, 1110, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (1231, 1, 'T1-816', '\\x75736167657220646573207472616e73706f727473', NULL, 2461, 2462, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1233, 1, 'T1-819', '\\x64726f697420646573206f626c69676174696f6e73', NULL, 2465, 2466, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1172, 1, 'T1-740', '\\x636174686f6c696369736d65', NULL, 2343, 2344, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (299, 1, 'T1-91', '\\x64c3a9666f726573746174696f6e', NULL, 597, 598, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (1237, 1, 'T1-1444', '\\xc3a96475636174696f6e207375727665696c6cc3a965', NULL, 2473, 2474, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (770, 1, 'T1-524', '\\x6964656e746974c3a920636f6d6d65726369616c65', NULL, 1539, 1540, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (1679, 1, 'T3-209', '\\x72c3a9706572746f6972652064276f66666963696572207075626c6963206d696e697374c3a97269656c', NULL, 3357, 3358, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (815, 1, 'T1-626', '\\x666f796572', NULL, 1629, 1630, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (1001, 1, 'T1-541', '\\x6d6f7274616c6974c3a920696e66616e74696c65', NULL, 2001, 2002, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1190, 1, 'T1-1108', '\\x64c3a966656e736520636f6e747265206c61206d6572', NULL, 2379, 2380, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1191, 1, 'T1-951', '\\x7369676e616c69736174696f6e206d61726974696d65', NULL, 2381, 2382, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (294, 1, 'T1-397', '\\x6c69627261697265', NULL, 587, 588, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (1656, 1, 'T3-216', '\\x736368c3a96d6120646972656374657572', NULL, 3311, 3312, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (266, 1, 'T1-919', '\\x6661736369736d65', NULL, 531, 532, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (1189, 1, 'T1-761', '\\x72656c6174696f6e732061766563206c65732075736167657273', NULL, 2377, 2378, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (631, 1, 'T1-1082', '\\x736572766963652066696e616e63696572206465206c6120706f737465', NULL, 1261, 1262, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (249, 1, 'T1-1437', '\\x66c3aa7465', NULL, 497, 498, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (1203, 1, 'T1-781', '\\x726563656e73656d656e7420646520706f70756c6174696f6e', NULL, 2405, 2406, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1269, 1, 'T1-1063', '\\xc3a970696365726965', NULL, 2537, 2538, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (654, 1, 'T1-279', '\\x7472616e73667573696f6e2073616e6775696e65', NULL, 1307, 1308, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (679, 1, 'T1-1018', '\\x707261746971756573206167726169726573', NULL, 1357, 1358, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (1348, 1, 'T1-1093', '\\x706f6c6963652075726261696e65', NULL, 2695, 2696, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1429, 1, 'T2-55', '\\x6f7267616e69736174696f6e', NULL, 2857, 2858, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1509, 1, 'T4-37', '\\x4469726563746f6972652028313739352d3137393929', NULL, 3017, 3018, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1581, 1, 'T4-53', '\\x4772616e6465732064c3a9636f75766572746573', NULL, 3161, 3162, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1582, 1, 'T4-52', '\\xc3a9646974206465204e616e74657320283135393829', NULL, 3163, 3164, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1660, 1, 'T3-176', '\\x70726f63c3a8732d76657262616c2064276578616d656e', NULL, 3319, 3320, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1742, 1, 'T3-134', '\\x6dc3a96d6f697265', NULL, 3483, 3484, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1822, 1, 'T3-211', '\\x726576756520646520707265737365', NULL, 3643, 3644, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (3, 1, 'T1-246', '\\x6167726963756c74757265', NULL, 5, 6, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (4, 1, 'T1-1317', '\\x61646d696e697374726174696f6e', NULL, 7, 8, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (5, 1, 'T1-307', '\\x736f6369c3a974c3a9', NULL, 9, 10, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (7, 1, 'T1-1262', '\\xc3a971756970656d656e74', NULL, 13, 14, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (8, 1, 'T1-1302', '\\xc3a96475636174696f6e', NULL, 15, 16, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (9, 1, 'T1-483', '\\x6a757374696365', NULL, 17, 18, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (10, 1, 'T1-906', '\\x6f70696e696f6e', NULL, 19, 20, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (11, 1, 'T1-150', '\\x657874c3a97269657572', NULL, 21, 22, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (12, 1, 'T1-1', '\\x6861626974617420727572616c', NULL, 23, 24, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (304, 1, 'T1-94', '\\x686f75696c6c65', NULL, 607, 608, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (39, 1, 'T1-643', '\\x626f6973', NULL, 77, 78, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (40, 1, 'T1-1070', '\\x70726f6475697420646f6d616e69616c', NULL, 79, 80, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (41, 1, 'T1-8', '\\x6f70c3a9726174696f6e206d696c697461697265', NULL, 81, 82, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (42, 1, 'T1-250', '\\x64c3a966656e7365206475207465727269746f697265', NULL, 83, 84, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (43, 1, 'T1-9', '\\x6879647261756c697175652061677269636f6c65', NULL, 85, 86, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (44, 1, 'T1-1134', '\\xc3a9636f6e6f6d696520727572616c65', NULL, 87, 88, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (45, 1, 'T1-242', '\\x62617272616765206879647261756c69717565', NULL, 89, 90, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (46, 1, 'T1-133', '\\x69727269676174696f6e', NULL, 91, 92, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (47, 1, 'T1-211', '\\x647261696e616765', NULL, 93, 94, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (48, 1, 'T1-988', '\\x616dc3a96e6167656d656e74206465732065617578', NULL, 95, 96, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (49, 1, 'T1-10', '\\x696e647573747269652064752063756972', NULL, 97, 98, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (50, 1, 'T1-11', '\\x70726f707269c3a974c3a9207075626c69717565', NULL, 99, 100, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (51, 1, 'T1-1222', '\\x66696e616e636573207075626c6971756573', NULL, 101, 102, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (52, 1, 'T1-763', '\\x646f6d61696e65206d6f62696c696572', NULL, 103, 104, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (1195, 1, 'T1-1366', '\\x6d6167697374726174', NULL, 2389, 2390, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (61, 1, 'T1-383', '\\x646f6d61696e6520696d6d6f62696c696572', NULL, 121, 122, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (62, 1, 'T1-377', '\\x616c69c3a96e6174696f6e20646f6d616e69616c65', NULL, 123, 124, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (63, 1, 'T1-1113', '\\x636f6e63657373696f6e20646f6d616e69616c65', NULL, 125, 126, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (64, 1, 'T1-838', '\\x646f6e732d65742d6c656773', NULL, 127, 128, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (65, 1, 'T1-1095', '\\x666f72c3aa7420646f6d616e69616c65', NULL, 129, 130, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (66, 1, 'T1-12', '\\x67c3a96e6965206d696c697461697265', NULL, 131, 132, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (67, 1, 'T1-912', '\\x696e667261737472756374757265206d696c697461697265', NULL, 133, 134, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (68, 1, 'T1-13', '\\x76656e74652061757820656e6368c3a8726573', NULL, 135, 136, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (69, 1, 'T1-1248', '\\x636f6d6d65726365', NULL, 137, 138, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (70, 1, 'T1-1438', '\\x76656e7465207075626c6971756520696d6d6f62696c69c3a87265', NULL, 139, 140, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (71, 1, 'T1-1137', '\\x636f6d6d697373616972652070726973657572', NULL, 141, 142, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (72, 1, 'T1-14', '\\x706c616765', NULL, 143, 144, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (73, 1, 'T1-762', '\\x6c6974746f72616c', NULL, 145, 146, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (74, 1, 'T1-342', '\\x746f757269736d652062616c6ec3a961697265', NULL, 147, 148, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (75, 1, 'T1-15', '\\x6672616e636f70686f6e6965', NULL, 149, 150, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (76, 1, 'T1-974', '\\x72656c6174696f6e7320696e7465726e6174696f6e616c6573', NULL, 151, 152, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (77, 1, 'T1-16', '\\x736f6369c3a974c3a92072c3a9766f6c7574696f6e6e61697265', NULL, 153, 154, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (78, 1, 'T1-17', '\\x6d61696e7469656e206465206c276f72647265', NULL, 155, 156, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (79, 1, 'T1-471', '\\x706f6c696365', NULL, 157, 158, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (80, 1, 'T1-926', '\\x6d616e69666573746174696f6e2073706f7274697665', NULL, 159, 160, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (83, 1, 'T1-606', '\\x766973697465206f6666696369656c6c65', NULL, 165, 166, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (84, 1, 'T1-796', '\\x6d616e69666573746174696f6e2064652070726f746573746174696f6e', NULL, 167, 168, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (603, 1, 'T1-801', '\\x7365727669636573', NULL, 1205, 1206, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (1799, 1, 'T3-144', '\\x6e6f7465', NULL, 3597, 3598, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (14, 1, 'T1-40', '\\x62c3a274696d656e742061677269636f6c65', NULL, 27, 28, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (15, 1, 'T1-1047', '\\x616dc3a96e6167656d656e7420727572616c', NULL, 29, 30, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (16, 1, 'T1-366', '\\x67c3ae746520727572616c', NULL, 31, 32, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (17, 1, 'T1-2', '\\x766973697465757220646520707269736f6e', NULL, 33, 34, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (18, 1, 'T1-3', '\\x72656368657263686520736369656e74696669717565', NULL, 35, 36, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (19, 1, 'T1-1356', '\\x63756c7475726520657870c3a972696d656e74616c65', NULL, 37, 38, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (20, 1, 'T1-1413', '\\x726563686572636865206170706c697175c3a965', NULL, 39, 40, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (21, 1, 'T1-618', '\\x64c3a9636f75766572746520736369656e74696669717565', NULL, 41, 42, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (22, 1, 'T1-676', '\\x72c3a9756e696f6e20736369656e74696669717565', NULL, 43, 44, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (23, 1, 'T1-1073', '\\x736f6369c3a974c3a920736176616e7465', NULL, 45, 46, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (24, 1, 'T1-287', '\\x6469736369706c696e6520736369656e74696669717565', NULL, 47, 48, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (25, 1, 'T1-1005', '\\x736369656e6365732068756d61696e6573', NULL, 49, 50, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (26, 1, 'T1-319', '\\x72656368657263686520666f6e64616d656e74616c65', NULL, 51, 52, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (27, 1, 'T1-981', '\\x6f7267616e69736d6520646520726563686572636865', NULL, 53, 54, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (1449, 1, 'T2-15', '\\x63657373696f6e', NULL, 2897, 2898, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (846, 1, 'T1-428', '\\xc3a96cc3a87665', NULL, 1691, 1692, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (1613, 1, 'T3-18', '\\x61746c6173', NULL, 3225, 3226, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (85, 1, 'T1-18', '\\x6368616e736f6e2073756276657273697665', NULL, 169, 170, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (86, 1, 'T1-19', '\\x6361727269c3a87265', NULL, 171, 172, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (87, 1, 'T1-481', '\\x696e647573747269652065787472616374697665', NULL, 173, 174, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (88, 1, 'T1-20', '\\x7472617661696c6c65757220736f6369616c', NULL, 175, 176, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (28, 1, 'T1-448', '\\x636865726368657572', NULL, 55, 56, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (29, 1, 'T1-436', '\\x6469736369706c696e6520756e6976657273697461697265', NULL, 57, 58, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (30, 1, 'T1-896', '\\x76696520696e74656c6c65637475656c6c65', NULL, 59, 60, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (31, 1, 'T1-4', '\\x647572c3a965206475207472617661696c', NULL, 61, 62, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (37, 1, 'T1-1079', '\\x72c3a967696f6e', NULL, 73, 74, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (1788, 1, 'T3-220', '\\x737461747574', NULL, 3575, 3576, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (53, 1, 'T1-1433', '\\x6269656e7320696e746572636f6d6d756e617578', NULL, 105, 106, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (54, 1, 'T1-1051', '\\x6269656e73206e6174696f6e617578', NULL, 107, 108, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (55, 1, 'T1-690', '\\x6269656e7320636f6d6d756e617578', NULL, 109, 110, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (56, 1, 'T1-766', '\\x6269656e2061646d696e69737472c3a9', NULL, 111, 112, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (57, 1, 'T1-333', '\\x6163717569736974696f6e20646f6d616e69616c65', NULL, 113, 114, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (58, 1, 'T1-616', '\\x6269656e732072c3a967696f6e617578', NULL, 115, 116, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (59, 1, 'T1-93', '\\x636f6e63657373696f6e2066756ec3a97261697265', NULL, 117, 118, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (33, 1, 'T1-5', '\\x726f757465206e6174696f6e616c65', NULL, 65, 66, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (34, 1, 'T1-1149', '\\x72c3a97365617520726f7574696572', NULL, 67, 68, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (35, 1, 'T1-6', '\\x66696e616e6365732072c3a967696f6e616c6573', NULL, 69, 70, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (690, 1, 'T1-312', '\\x66656d6d65', NULL, 1379, 1380, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (1457, 1, 'T2-18', '\\x636f6d6d756e69636174696f6e', NULL, 2913, 2914, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (90, 1, 'T1-350', '\\xc3a96475636174657572', NULL, 179, 180, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (91, 1, 'T1-936', '\\x656e7472c3a96520736f6c656e6e656c6c65', NULL, 181, 182, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (93, 1, 'T1-194', '\\x72c3a963657074696f6e206f6666696369656c6c65', NULL, 185, 186, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (94, 1, 'T1-1335', '\\x696e61756775726174696f6e', NULL, 187, 188, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (95, 1, 'T1-22', '\\x76696520706f6c697469717565', NULL, 189, 190, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (96, 1, 'T1-1169', '\\x706172746920706f6c697469717565', NULL, 191, 192, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (97, 1, 'T1-87', '\\x6d6f7576656d656e7420706f6c697469717565', NULL, 193, 194, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (98, 1, 'T1-23', '\\x636f6d7061676e6f6e6e616765', NULL, 195, 196, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (99, 1, 'T1-1025', '\\x6f7267616e69736174696f6e2070726f66657373696f6e6e656c6c65', NULL, 197, 198, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (100, 1, 'T1-24', '\\x6368656d696e2064652072616e646f6e6ec3a965', NULL, 199, 200, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (101, 1, 'T1-632', '\\x746f757269736d65', NULL, 201, 202, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (102, 1, 'T1-908', '\\x6368656d696e20727572616c', NULL, 203, 204, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (103, 1, 'T1-25', '\\x696e63656e64696520766f6c6f6e7461697265', NULL, 205, 206, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (104, 1, 'T1-1403', '\\x6372696d65732065742064c3a96c697473', NULL, 207, 208, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (157, 1, 'T1-45', '\\x6578636c7573696f6e20736f6369616c65', NULL, 313, 314, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (243, 1, 'T1-76', '\\x766f69652070726976c3a965', NULL, 485, 486, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (112, 1, 'T1-418', '\\x64c3a974656e74696f6e20642761726d6573', NULL, 223, 224, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (113, 1, 'T1-204', '\\x73c3a96a6f75722064657320c3a97472616e67657273', NULL, 225, 226, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (115, 1, 'T1-545', '\\x7472616e73706f727420646520636f727073', NULL, 229, 230, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (116, 1, 'T1-518', '\\x706967656f6e20766f796167657572', NULL, 231, 232, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (117, 1, 'T1-29', '\\x70726976696cc3a86765732064657320636f6d6d756e617574c3a973', NULL, 233, 234, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (118, 1, 'T1-842', '\\x72c3a967696d6520736569676e65757269616c', NULL, 235, 236, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (119, 1, 'T1-155', '\\x636f7574756d6573', NULL, 237, 238, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (120, 1, 'T1-1333', '\\x64726f6974732064277573616765', NULL, 239, 240, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (121, 1, 'T1-30', '\\x766f69652066657272c3a9652070726976c3a965', NULL, 241, 242, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (122, 1, 'T1-31', '\\x766163616e7473', NULL, 243, 244, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (123, 1, 'T1-32', '\\x63697263756c6174696f6e2064657320706572736f6e6e6573', NULL, 245, 246, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (124, 1, 'T1-1269', '\\x7375727665696c6c616e6365206475207465727269746f697265', NULL, 247, 248, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (125, 1, 'T1-33', '\\x6d6f7576656d656e742064276964c3a96573', NULL, 249, 250, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (126, 1, 'T1-453', '\\x746f6cc3a972616e6365', NULL, 251, 252, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (1657, 1, 'T3-105', '\\xc3a974756465', NULL, 3313, 3314, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (127, 1, 'T1-1239', '\\x64726f697473206465206c27686f6d6d65', NULL, 253, 254, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (128, 1, 'T1-1247', '\\x617468c3a969736d65', NULL, 255, 256, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (129, 1, 'T1-143', '\\x7068696c6f736f70686965', NULL, 257, 258, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (130, 1, 'T1-34', '\\x62c3a274696d656e7420706f6c7976616c656e74', NULL, 259, 260, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (131, 1, 'T1-35', '\\x646f6d61696e65207075626c6963206d61726974696d65', NULL, 261, 262, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (132, 1, 'T1-36', '\\x64c3a96368657420726164696f6163746966', NULL, 263, 264, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (133, 1, 'T1-362', '\\x7472616974656d656e74206465732064c3a96368657473', NULL, 265, 266, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (134, 1, 'T1-1443', '\\x63656e7472616c65206e75636cc3a961697265', NULL, 267, 268, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (135, 1, 'T1-37', '\\x746f757269736d6520666c757669616c', NULL, 269, 270, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (136, 1, 'T1-255', '\\x6e617669676174696f6e20646520706c616973616e6365', NULL, 271, 272, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (137, 1, 'T1-38', '\\x656e736569676e616e74', NULL, 273, 274, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (138, 1, 'T1-499', '\\x6f7267616e69736174696f6e2073636f6c61697265', NULL, 275, 276, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (139, 1, 'T1-1029', '\\x696e737469747574657572', NULL, 277, 278, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (140, 1, 'T1-1456', '\\xc3a964756361746575722073706f72746966', NULL, 279, 280, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (141, 1, 'T1-614', '\\x6d61c3ae74726520642761707072656e74697373616765', NULL, 281, 282, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (142, 1, 'T1-300', '\\x70726f66657373657572', NULL, 283, 284, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (143, 1, 'T1-583', '\\x666f726d6174657572', NULL, 285, 286, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (144, 1, 'T1-777', '\\x6dc3a9646563696e20686f73706974616c696572', NULL, 287, 288, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (145, 1, 'T1-1056', '\\x666f6e6374696f6e6e61697265206465206c27c389746174', NULL, 289, 290, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (146, 1, 'T1-39', '\\xc3a9736f74657269736d65', NULL, 291, 292, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (147, 1, 'T1-172', '\\x736369656e63657320706172616c6cc3a86c6573', NULL, 293, 294, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (1695, 1, 'T3-21', '\\x6176616e742d70726f6a6574', NULL, 3389, 3390, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1784, 1, 'T3-92', '\\x646f63756d656e746174696f6e', NULL, 3567, 3568, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (149, 1, 'T1-42', '\\x72656472657373656d656e74206a756469636961697265', NULL, 297, 298, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (150, 1, 'T1-460', '\\x6166666169726520636f6d6d65726369616c65', NULL, 299, 300, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (151, 1, 'T1-724', '\\x656e747265707269736520656e20646966666963756c74c3a9', NULL, 301, 302, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (152, 1, 'T1-224', '\\x61646d696e697374726174657572206a756469636961697265', NULL, 303, 304, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (153, 1, 'T1-43', '\\x64616e7365', NULL, 305, 306, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (154, 1, 'T1-608', '\\x617274', NULL, 307, 308, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (155, 1, 'T1-1228', '\\x656e736569676e656d656e742061727469737469717565', NULL, 309, 310, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (156, 1, 'T1-44', '\\xc3a96d6967726174696f6e', NULL, 311, 312, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (400, 1, 'T1-136', '\\x76696e', NULL, 799, 800, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (105, 1, 'T1-26', '\\x70726f626174696f6e', NULL, 209, 210, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (106, 1, 'T1-996', '\\x6170706c69636174696f6e20646573207065696e6573', NULL, 211, 212, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (107, 1, 'T1-27', '\\x736f6369c3a974c3a920636976696c652070726f66657373696f6e6e656c6c65', NULL, 213, 214, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (108, 1, 'T1-885', '\\x656e7472657072697365', NULL, 215, 216, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (109, 1, 'T1-369', '\\x656e72656769737472656d656e74206a756469636961697265', NULL, 217, 218, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (110, 1, 'T1-28', '\\x706f6c6963652061646d696e697374726174697665', NULL, 219, 220, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (111, 1, 'T1-962', '\\x70726f66657373696f6e2072c3a9676c656d656e74c3a965', NULL, 221, 222, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (159, 1, 'T1-678', '\\x696e646967656e74', NULL, 317, 318, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (163, 1, 'T1-432', '\\x73c3a96375726974c3a920736f6369616c65', NULL, 325, 326, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (1789, 1, 'T3-138', '\\x6d696e757465206a75726964696374696f6e6e656c6c65', NULL, 3577, 3578, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (714, 1, 'T1-1028', '\\x7375627374616e63652076c3a96ec3a96e65757365', NULL, 1427, 1428, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (166, 1, 'T1-1334', '\\x766f696520646520636f6d6d756e69636174696f6e', NULL, 331, 332, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (167, 1, 'T1-277', '\\x696e66726173747275637475726520706f72747561697265', NULL, 333, 334, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (168, 1, 'T1-1089', '\\x6261727261676520687964726fc3a96c6563747269717565', NULL, 335, 336, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (169, 1, 'T1-49', '\\x7669746963756c74757265', NULL, 337, 338, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (170, 1, 'T1-421', '\\x70726f64756374696f6e2061677269636f6c65', NULL, 339, 340, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (171, 1, 'T1-50', '\\x76c3a9686963756c65206175746f6d6f62696c65', NULL, 341, 342, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (172, 1, 'T1-548', '\\x7472616e73706f7274', NULL, 343, 344, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (1630, 1, 'T3-23', '\\x6261696c', NULL, 3259, 3260, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (181, 1, 'T1-554', '\\x616dc3a96e6167656d656e74206475207465727269746f697265', NULL, 361, 362, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (182, 1, 'T1-334', '\\x6d6f6e7461676e65', NULL, 363, 364, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (183, 1, 'T1-53', '\\x61696465206dc3a9646963616c65', NULL, 365, 366, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (184, 1, 'T1-1168', '\\x6169646520736f6369616c65', NULL, 367, 368, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (185, 1, 'T1-54', '\\x7072c3a9736964656e7420647520636f6e7365696c2072c3a967696f6e616c', NULL, 369, 370, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (186, 1, 'T1-868', '\\xc3a96c75', NULL, 371, 372, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (187, 1, 'T1-1237', '\\x636f6e7365696c2072c3a967696f6e616c', NULL, 373, 374, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (188, 1, 'T1-55', '\\x72c3a967696f6e616c69736174696f6e', NULL, 375, 376, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (189, 1, 'T1-56', '\\x6dc3a9646563696e652076c3a974c3a972696e61697265', NULL, 377, 378, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (191, 1, 'T1-57', '\\x746572726520696e63756c7465', NULL, 381, 382, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (192, 1, 'T1-105', '\\x64c3a96672696368656d656e74', NULL, 383, 384, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (193, 1, 'T1-58', '\\x636f6e6672c3a9726965', NULL, 385, 386, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (194, 1, 'T1-74', '\\x6173736f63696174696f6e2063756c7475656c6c65', NULL, 387, 388, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (195, 1, 'T1-59', '\\x696e64757374726965206d61726974696d65', NULL, 389, 390, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (196, 1, 'T1-60', '\\x6e6f7461697265', NULL, 391, 392, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (197, 1, 'T1-101', '\\x6f66666963696572206d696e697374c3a97269656c', NULL, 393, 394, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (198, 1, 'T1-1215', '\\x7075626c69636974c3a920666f6e6369c3a87265', NULL, 395, 396, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (199, 1, 'T1-61', '\\x696e6672616374696f6e206d696c697461697265', NULL, 397, 398, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (200, 1, 'T1-872', '\\x726563727574656d656e74206d696c697461697265', NULL, 399, 400, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (201, 1, 'T1-359', '\\x736572766963652063697669717565', NULL, 401, 402, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (202, 1, 'T1-62', '\\x636f6e67c3a9732073636f6c6169726573', NULL, 403, 404, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (203, 1, 'T1-1404', '\\x727974686d652073636f6c61697265', NULL, 405, 406, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (204, 1, 'T1-684', '\\x63656e747265206465206c6f6973697273', NULL, 407, 408, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (205, 1, 'T1-311', '\\x63656e74726520646520766163616e636573', NULL, 409, 410, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (206, 1, 'T1-63', '\\xc3a9636c616972616765207075626c6963', NULL, 411, 412, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (207, 1, 'T1-755', '\\x72c3a97365617520646520646973747269627574696f6e', NULL, 413, 414, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (208, 1, 'T1-64', '\\x706c616e74652074696e63746f7269616c65', NULL, 415, 416, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (209, 1, 'T1-1325', '\\x63756c7475726520696e647573747269656c6c65', NULL, 417, 418, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (210, 1, 'T1-203', '\\x696e647573747269652074657874696c65', NULL, 419, 420, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (211, 1, 'T1-65', '\\x6d61676173696e2067c3a96ec3a972616c', NULL, 421, 422, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (212, 1, 'T1-66', '\\x636972636f6e736372697074696f6e206563636cc3a9736961737469717565', NULL, 423, 424, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (213, 1, 'T1-793', '\\x696e737469747574696f6e206563636cc3a9736961737469717565', NULL, 425, 426, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (214, 1, 'T1-67', '\\x73696e6973747265', NULL, 427, 428, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (215, 1, 'T1-147', '\\x70726f74656374696f6e20636976696c65', NULL, 429, 430, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (216, 1, 'T1-1393', '\\x696e6f6e646174696f6e', NULL, 431, 432, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (217, 1, 'T1-1055', '\\x636174617374726f70686520696e647573747269656c6c65', NULL, 433, 434, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (221, 1, 'T1-68', '\\x70726f707269c3a974c3a920696e647573747269656c6c65', NULL, 441, 442, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (222, 1, 'T1-69', '\\x696d7069c3a974c3a9', NULL, 443, 444, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (223, 1, 'T1-485', '\\x6d616e69666573746174696f6e20616e746972656c69676965757365', NULL, 445, 446, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (235, 1, 'T1-317', '\\x636f6d6d756e65', NULL, 469, 470, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (479, 1, 'T1-178', '\\x666572726f7574616765', NULL, 957, 958, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (736, 1, 'T1-345', '\\x62616e6e697373656d656e74', NULL, 1471, 1472, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (1511, 1, 'T4-96', '\\x63726973652064652031393239', NULL, 3021, 3022, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (224, 1, 'T1-70', '\\x6d6f7576656d656e74206c61c3af717565', NULL, 447, 448, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (225, 1, 'T1-1106', '\\x636f6e666c69742073636f6c61697265', NULL, 449, 450, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (226, 1, 'T1-71', '\\x696d70c3b47420737572206c65732062c3a96ec3a96669636573', NULL, 451, 452, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (227, 1, 'T1-669', '\\x66697363616c6974c3a92070726f66657373696f6e6e656c6c65', NULL, 453, 454, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (228, 1, 'T1-72', '\\x72c3a9717569736974696f6e206465206c6f67656d656e74', NULL, 455, 456, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (229, 1, 'T1-73', '\\x636f6c6c65637469766974c3a9206c6f63616c65', NULL, 457, 458, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (230, 1, 'T1-1345', '\\x61646d696e697374726174696f6e2067c3a96ec3a972616c65', NULL, 459, 460, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (231, 1, 'T1-594', '\\x726567726f7570656d656e7420646520636f6d6d756e6573', NULL, 461, 462, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (1268, 1, 'T1-1178', '\\x626f756c616e6765726965', NULL, 2535, 2536, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (176, 1, 'T1-628', '\\x73c3a96375726974c3a920726f757469c3a87265', NULL, 351, 352, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (177, 1, 'T1-51', '\\x7072756427686f6d6d65', NULL, 353, 354, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (178, 1, 'T1-767', '\\x6a756765', NULL, 355, 356, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (179, 1, 'T1-1061', '\\xc3a96c656374696f6e2070726f66657373696f6e6e656c6c65', NULL, 357, 358, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (180, 1, 'T1-52', '\\x706f6c697469717565206465206c61206d6f6e7461676e65', NULL, 359, 360, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (232, 1, 'T1-1449', '\\x64c3a970617274656d656e74', NULL, 463, 464, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (233, 1, 'T1-199', '\\x636f6f70c3a9726174696f6e20696e74657272c3a967696f6e616c65', NULL, 465, 466, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (1676, 1, 'T3-67', '\\x636f6e74726174', NULL, 3351, 3352, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (89, 1, 'T1-200', '\\x616374696f6e20736f6369616c65', NULL, 177, 178, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (324, 1, 'T1-446', '\\x6e6f626c65737365', NULL, 647, 648, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (582, 1, 'T1-239', '\\x686f6d6d616765', NULL, 1163, 1164, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (234, 1, 'T1-1238', '\\xc3a97461626c697373656d656e74207075626c696320646520636f6f70c3a9726174696f6e20696e746572636f6d6d756e616c65', NULL, 467, 468, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (236, 1, 'T1-804', '\\x7669652072656c69676965757365', NULL, 471, 472, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (237, 1, 'T1-1391', '\\x736f6369c3a974c3a920646520636861726974c3a9', NULL, 473, 474, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (241, 1, 'T1-1311', '\\x656e747265707269736520646520737065637461636c65', NULL, 481, 482, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (242, 1, 'T1-562', '\\x6c616e6775652072c3a967696f6e616c65', NULL, 483, 484, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (244, 1, 'T1-77', '\\x6e61697373616e6365', NULL, 487, 488, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (245, 1, 'T1-1139', '\\xc3a974617420636976696c', NULL, 489, 490, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (246, 1, 'T1-492', '\\x706f70756c6174696f6e20686f73706974616c69c3a87265', NULL, 491, 492, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (254, 1, 'T1-946', '\\x656e66616e74', NULL, 507, 508, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (774, 1, 'T1-782', '\\x616c696d656e746174696f6e', NULL, 1547, 1548, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (282, 1, 'T1-1390', '\\x7261646963616c69736d65', NULL, 563, 564, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (251, 1, 'T1-81', '\\x696e6475737472696520c3a96c656374726f6e69717565', NULL, 501, 502, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (252, 1, 'T1-82', '\\x656e66616e742068616e6469636170c3a9', NULL, 503, 504, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (253, 1, 'T1-677', '\\xc3a96475636174696f6e207370c3a96369616c65', NULL, 505, 506, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (255, 1, 'T1-83', '\\x6f70c3a9726174696f6e206427757262616e69736d65', NULL, 509, 510, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (256, 1, 'T1-209', '\\x757262616e69736d65', NULL, 511, 512, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (257, 1, 'T1-475', '\\x6c6f74697373656d656e74', NULL, 513, 514, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (258, 1, 'T1-1097', '\\x726573747275637475726174696f6e2075726261696e65', NULL, 515, 516, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (259, 1, 'T1-124', '\\x726573746175726174696f6e20696d6d6f62696c69c3a87265', NULL, 517, 518, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (260, 1, 'T1-84', '\\x757262616e69737465', NULL, 519, 520, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (575, 1, 'T1-233', '\\x73697465', NULL, 1149, 1150, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (283, 1, 'T1-1322', '\\x65787472c3aa6d652064726f697465', NULL, 565, 566, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (284, 1, 'T1-1207', '\\x65787472c3aa6d6520676175636865', NULL, 567, 568, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (285, 1, 'T1-229', '\\x726f79616c69736d65', NULL, 569, 570, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (286, 1, 'T1-174', '\\x626f6e617061727469736d65', NULL, 571, 572, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (287, 1, 'T1-1218', '\\x6d6f7576656d656e742066c3a96d696e69737465', NULL, 573, 574, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (289, 1, 'T1-115', '\\x6d6f7576656d656e7420686f6d6f73657875656c', NULL, 577, 578, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (290, 1, 'T1-318', '\\x6d6f7576656d656e7420706163696669737465', NULL, 579, 580, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (291, 1, 'T1-88', '\\x6dc3a97469657273206475206c69767265', NULL, 581, 582, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (354, 1, 'T1-114', '\\x6d7573c3a965', NULL, 707, 708, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (874, 1, 'T1-496', '\\x72616369736d65', NULL, 1747, 1748, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (274, 1, 'T1-538', '\\x636f6c6f6e69616c69736d65', NULL, 547, 548, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (275, 1, 'T1-526', '\\x636f6d6d756e69736d65', NULL, 549, 550, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (276, 1, 'T1-605', '\\x6761756c6c69736d65', NULL, 551, 552, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (277, 1, 'T1-720', '\\x6d6f7576656d656e742072c3a967696f6e616c69737465', NULL, 553, 554, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (278, 1, 'T1-1339', '\\x6d6f7576656d656e7420c3a974756469616e74', NULL, 555, 556, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (279, 1, 'T1-197', '\\x636cc3a9726963616c69736d65', NULL, 557, 558, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (280, 1, 'T1-391', '\\x6d6f7576656d656e74206f757672696572', NULL, 559, 560, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (281, 1, 'T1-266', '\\x6c6962c3a972616c69736d65', NULL, 561, 562, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (303, 1, 'T1-773', '\\x63696d657469c3a87265', NULL, 605, 606, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (305, 1, 'T1-1326', '\\x6d617469c3a87265207072656d69c3a87265', NULL, 609, 610, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (306, 1, 'T1-1441', '\\x63656e7472616c6520746865726d69717565', NULL, 611, 612, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (307, 1, 'T1-95', '\\x74c3a96cc3a96d617469717565', NULL, 613, 614, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (308, 1, 'T1-1327', '\\x74c3a96cc3a9636f6d6d756e69636174696f6e73', NULL, 615, 616, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (309, 1, 'T1-96', '\\x636f6e63657373696f6e20666f7265737469c3a87265', NULL, 617, 618, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (310, 1, 'T1-832', '\\x6578706c6f6974616e7420666f72657374696572', NULL, 619, 620, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (311, 1, 'T1-97', '\\x736f6369c3a974c3a9206d757475616c69737465', NULL, 621, 622, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (312, 1, 'T1-98', '\\x6173736f63696174696f6e20646520636f6e736f6d6d617465757273', NULL, 623, 624, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (313, 1, 'T1-581', '\\x636f6e736f6d6d6174696f6e', NULL, 625, 626, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (314, 1, 'T1-99', '\\x656e66616e74207365636f757275', NULL, 627, 628, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (315, 1, 'T1-566', '\\x6d696e657572207375727665696c6cc3a9', NULL, 629, 630, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (317, 1, 'T1-158', '\\x6f7267616e69736174696f6e206a756469636961697265', NULL, 633, 634, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (318, 1, 'T1-1166', '\\x61766f75c3a9', NULL, 635, 636, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (321, 1, 'T1-102', '\\x706f6c69636520646573207472616e73706f727473', NULL, 641, 642, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (322, 1, 'T1-103', '\\x636f6e646974696f6e20736f6369616c65', NULL, 643, 644, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (323, 1, 'T1-942', '\\x636f6e646974696f6e2064657320706572736f6e6e657320657420646573206269656e73', NULL, 645, 646, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (325, 1, 'T1-1394', '\\x6573636c6176616765', NULL, 649, 650, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (326, 1, 'T1-723', '\\x70726f6cc3a9746172696174', NULL, 651, 652, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (327, 1, 'T1-1181', '\\x73657276616765', NULL, 653, 654, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (328, 1, 'T1-151', '\\x746965727320c3a9746174', NULL, 655, 656, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (329, 1, 'T1-184', '\\x626f757267656f69736965', NULL, 657, 658, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (330, 1, 'T1-104', '\\x636f6e7365696c206a7572696469717565', NULL, 659, 660, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (331, 1, 'T1-487', '\\x617578696c6961697265206465206a757374696365', NULL, 661, 662, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (332, 1, 'T1-400', '\\x73796c766963756c74757265', NULL, 663, 664, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (333, 1, 'T1-106', '\\x72c3a966756769c3a9', NULL, 665, 666, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (335, 1, 'T1-1243', '\\x72c3a966756769c3a920646520677565727265', NULL, 669, 670, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (336, 1, 'T1-107', '\\xc3a96c656374696f6e20706f6c697469717565', NULL, 671, 672, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (173, 1, 'T1-1221', '\\x6163636964656e7420646573207472616e73706f727473', NULL, 345, 346, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (174, 1, 'T1-126', '\\x617070617265696c20c3a0207072657373696f6e20646520766170657572', NULL, 347, 348, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (175, 1, 'T1-1374', '\\x636f6e737472756374696f6e206175746f6d6f62696c65', NULL, 349, 350, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (337, 1, 'T1-1152', '\\xc3a96c656374696f6e', NULL, 673, 674, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (339, 1, 'T1-1081', '\\xc3a96c656374696f6e20726567696f6e616c65', NULL, 677, 678, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (340, 1, 'T1-624', '\\xc3a96c656374696f6e20707265736964656e7469656c6c65', NULL, 679, 680, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (341, 1, 'T1-1383', '\\xc3a96c656374696f6e2073c3a96e61746f7269616c65', NULL, 681, 682, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (342, 1, 'T1-923', '\\xc3a96c656374696f6e2063616e746f6e616c65', NULL, 683, 684, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (343, 1, 'T1-321', '\\x72c3a966c3a972656e64756d', NULL, 685, 686, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (773, 1, 'T1-372', '\\x6a75676520636f6e73756c61697265', NULL, 1545, 1546, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (344, 1, 'T1-401', '\\xc3a96c656374696f6e20617520636f6e7365696c2064276172726f6e64697373656d656e74', NULL, 687, 688, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (345, 1, 'T1-171', '\\xc3a96c656374696f6e206d756e69636970616c65', NULL, 689, 690, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (346, 1, 'T1-1254', '\\xc3a96c656374696f6e206cc3a96769736c6174697665', NULL, 691, 692, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (347, 1, 'T1-108', '\\x61726d7572696572', NULL, 693, 694, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (348, 1, 'T1-109', '\\x6dc3a964696174657572206465206a757374696365', NULL, 695, 696, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (349, 1, 'T1-111', '\\x696e74c3a972657373656d656e7420646573207472617661696c6c65757273', NULL, 697, 698, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (350, 1, 'T1-536', '\\x7472617661696c', NULL, 699, 700, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (351, 1, 'T1-112', '\\x72656365747465206e6f6e2066697363616c65', NULL, 701, 702, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (352, 1, 'T1-113', '\\x636f6e737469747574696f6e', NULL, 703, 704, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (353, 1, 'T1-1259', '\\x64726f6974207075626c6963', NULL, 705, 706, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (355, 1, 'T1-213', '\\xc3a971756970656d656e7420736f63696f2063756c747572656c', NULL, 709, 710, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (356, 1, 'T1-116', '\\xc3a9636f6e6f6d6965206d6f6e7461676e61726465', NULL, 711, 712, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (357, 1, 'T1-148', '\\x616374696f6e20c3a9636f6e6f6d69717565', NULL, 713, 714, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (358, 1, 'T1-1013', '\\xc3a96c6576616765', NULL, 715, 716, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (359, 1, 'T1-117', '\\x666f6e6473206575726f70c3a9656e', NULL, 717, 718, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (360, 1, 'T1-118', '\\x6d61676173696e20636f6c6c6563746966', NULL, 719, 720, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (361, 1, 'T1-1158', '\\x666f6e647320646520636f6d6d65726365', NULL, 721, 722, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (363, 1, 'T1-119', '\\x616c70686162c3a97469736174696f6e', NULL, 725, 726, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (364, 1, 'T1-544', '\\x616374696f6e2068756d616e697461697265', NULL, 727, 728, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (365, 1, 'T1-881', '\\x656e736569676e656d656e74', NULL, 729, 730, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (366, 1, 'T1-120', '\\x626f6d62617264656d656e74', NULL, 731, 732, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (367, 1, 'T1-1341', '\\x677565727265', NULL, 733, 734, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (368, 1, 'T1-121', '\\x726176697461696c6c656d656e74206d696c697461697265', NULL, 735, 736, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (369, 1, 'T1-122', '\\x6d6172696e65', NULL, 737, 738, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (370, 1, 'T1-551', '\\x61726dc3a965', NULL, 739, 740, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (371, 1, 'T1-123', '\\x70617472696d6f696e652063756c747572656c', NULL, 741, 742, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (374, 1, 'T1-889', '\\x70617472696d6f696e65206e756dc3a97269717565', NULL, 747, 748, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (375, 1, 'T1-995', '\\x6d6f6e756d656e7420686973746f7269717565', NULL, 749, 750, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (376, 1, 'T1-1343', '\\x70617472696d6f696e6520617564696f76697375656c', NULL, 751, 752, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (377, 1, 'T1-991', '\\x70617472696d6f696e65206574686e6f6c6f6769717565', NULL, 753, 754, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (378, 1, 'T1-1337', '\\x70617472696d6f696e6520c3a963726974', NULL, 755, 756, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (379, 1, 'T1-658', '\\xc3a964696669636520636c617373c3a9', NULL, 757, 758, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (380, 1, 'T1-825', '\\x6f626a6574206427617274', NULL, 759, 760, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (381, 1, 'T1-1223', '\\x70617472696d6f696e652069636f6e6f677261706869717565', NULL, 761, 762, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (1739, 1, 'T3-50', '\\x636172746520706f7374616c65', NULL, 3477, 3478, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (268, 1, 'T1-170', '\\x72c3a97075626c6963616e69736d65', NULL, 535, 536, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (269, 1, 'T1-1021', '\\x736f6369616c69736d65', NULL, 537, 538, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (270, 1, 'T1-288', '\\x616e74696361706974616c69736d65', NULL, 539, 540, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (271, 1, 'T1-634', '\\x616e6172636869736d65', NULL, 541, 542, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (272, 1, 'T1-1278', '\\x6e6174696f6e616c69736d65', NULL, 543, 544, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (273, 1, 'T1-1177', '\\x6d6f7576656d656e7420c3a9636f6c6f6769737465', NULL, 545, 546, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (385, 1, 'T1-127', '\\x6f757672696572', NULL, 769, 770, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (386, 1, 'T1-128', '\\x70726174697175652072656c69676965757365', NULL, 771, 772, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (387, 1, 'T1-129', '\\x706c616e696669636174696f6e', NULL, 773, 774, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (388, 1, 'T1-130', '\\x657370c3a863652070726f74c3a967c3a965', NULL, 775, 776, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (389, 1, 'T1-1395', '\\x70c3aa636865206d61726974696d65', NULL, 777, 778, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (390, 1, 'T1-389', '\\x636861737365', NULL, 779, 780, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (391, 1, 'T1-1182', '\\x70c3aa63686520c3a0206c61206c69676e65', NULL, 781, 782, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (392, 1, 'T1-425', '\\x70c3aa63686520656e2065617520646f756365', NULL, 783, 784, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (394, 1, 'T1-435', '\\x7374727563747572652061646d696e697374726174697665', NULL, 787, 788, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (395, 1, 'T1-132', '\\xc3a976616c756174696f6e20666f6e6369c3a87265', NULL, 789, 790, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (396, 1, 'T1-944', '\\x7072697365206427656175', NULL, 791, 792, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (397, 1, 'T1-134', '\\x7075c3a9726963756c7472696365', NULL, 793, 794, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (401, 1, 'T1-603', '\\x626f6973736f6e20616c636f6f6c6973c3a965', NULL, 801, 802, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (402, 1, 'T1-137', '\\x6368c3b46d616765207061727469656c', NULL, 803, 804, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (403, 1, 'T1-862', '\\x6368c3b46d616765', NULL, 805, 806, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (405, 1, 'T1-866', '\\x6173736f63696174696f6e2073796e646963616c652064652070726f707269c3a9746169726573', NULL, 809, 810, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (406, 1, 'T1-139', '\\x6c69676e65206465206368656d696e20646520666572', NULL, 811, 812, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (407, 1, 'T1-164', '\\x7472616e73706f727420666572726f766961697265', NULL, 813, 814, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (459, 1, 'T1-1019', '\\x62696f2d636172627572616e74', NULL, 917, 918, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (408, 1, 'T1-140', '\\x707265737365', NULL, 815, 816, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (409, 1, 'T1-813', '\\x64c3a970c3b474206cc3a967616c', NULL, 817, 818, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (410, 1, 'T1-1186', '\\x6a6f75726e616c69737465', NULL, 819, 820, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (411, 1, 'T1-968', '\\x7075626c69636174696f6e20696e7465726e65', NULL, 821, 822, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (412, 1, 'T1-576', '\\x7265706f727465722070686f746f677261706865', NULL, 823, 824, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (413, 1, 'T1-1358', '\\x7072657373652072c3a967696f6e616c65', NULL, 825, 826, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (414, 1, 'T1-141', '\\x646973747269627574696f6e206465732070726978', NULL, 827, 828, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (415, 1, 'T1-142', '\\x616772657373696f6e2073657875656c6c65', NULL, 829, 830, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (439, 1, 'T1-584', '\\x6d6172696e2070c3aa6368657572', NULL, 877, 878, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (440, 1, 'T1-157', '\\x706f6c6c7574696f6e2076697375656c6c65', NULL, 879, 880, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (441, 1, 'T1-286', '\\x706f6c6c7574696f6e', NULL, 881, 882, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (442, 1, 'T1-1132', '\\x6a75726964696374696f6e', NULL, 883, 884, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (443, 1, 'T1-159', '\\x636f6e7365696c206d756e69636970616c', NULL, 885, 886, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (444, 1, 'T1-1157', '\\x6f7267616e652064c3a96c6962c3a972616e74', NULL, 887, 888, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (445, 1, 'T1-1251', '\\x6d61697265', NULL, 889, 890, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (446, 1, 'T1-1188', '\\x636f6e7365696c6c6572206d756e69636970616c', NULL, 891, 892, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (447, 1, 'T1-160', '\\x617263686974656374757265', NULL, 893, 894, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (1289, 1, 'T1-910', '\\x6861726b69', NULL, 2577, 2578, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1616, 1, 'T3-8', '\\x6167656e6461', NULL, 3231, 3232, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (452, 1, 'T1-163', '\\x73796e646963206465206661696c6c697465', NULL, 903, 904, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (454, 1, 'T1-493', '\\x6c696169736f6e2066657272c3a96520696e7465726e6174696f6e616c65', NULL, 907, 908, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (455, 1, 'T1-165', '\\x6d697261636c65', NULL, 909, 910, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (456, 1, 'T1-1210', '\\x7669652073706972697475656c6c65', NULL, 911, 912, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (457, 1, 'T1-166', '\\xc3a96e6572676965', NULL, 913, 914, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (460, 1, 'T1-1373', '\\xc3a96e6572676965206e75636cc3a961697265', NULL, 919, 920, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (461, 1, 'T1-452', '\\xc3a9636f6e6f6d6965206427c3a96e6572676965', NULL, 921, 922, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (462, 1, 'T1-744', '\\xc3a971756970656d656e74206d696c697461697265', NULL, 923, 924, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (1740, 1, 'T3-108', '\\x66696368696572', NULL, 3479, 3480, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (425, 1, 'T1-887', '\\x696e7374616c6c6174696f6e20636c617373c3a965', NULL, 849, 850, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (426, 1, 'T1-1000', '\\x7365636f757273', NULL, 851, 852, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (427, 1, 'T1-511', '\\x64c3a976656c6f7070656d656e742064757261626c65', NULL, 853, 854, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (428, 1, 'T1-920', '\\x61696465207075626c697175652061757820656e747265707269736573', NULL, 855, 856, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (429, 1, 'T1-939', '\\x706f6c69636520c3a9636f6e6f6d69717565', NULL, 857, 858, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (430, 1, 'T1-1234', '\\x6c6162656c206465207175616c6974c3a9', NULL, 859, 860, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (431, 1, 'T1-568', '\\x70726978', NULL, 861, 862, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (432, 1, 'T1-149', '\\x65617520736f757465727261696e65', NULL, 863, 864, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (1343, 1, 'T1-1318', '\\x6d6572', NULL, 2685, 2686, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (464, 1, 'T1-506', '\\xc3a97461742d6d616a6f72', NULL, 927, 928, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (465, 1, 'T1-905', '\\x736572766963652073616e697461697265206d696c697461697265', NULL, 929, 930, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (466, 1, 'T1-168', '\\x64c3a9636f726174696f6e2075726261696e65', NULL, 931, 932, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (467, 1, 'T1-169', '\\x636f6e74656e746965757820c3a96c6563746f72616c', NULL, 933, 934, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (468, 1, 'T1-1154', '\\x636f6e74656e74696575782061646d696e6973747261746966', NULL, 935, 936, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (469, 1, 'T1-1439', '\\x63726f79616e63657320657420736369656e63657320706172616c6cc3a86c6573', NULL, 937, 938, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (470, 1, 'T1-1111', '\\x617374726f6c6f676965', NULL, 939, 940, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (471, 1, 'T1-784', '\\x616c6368696d6965', NULL, 941, 942, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (472, 1, 'T1-386', '\\x67726170686f6c6f676965', NULL, 943, 944, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (473, 1, 'T1-901', '\\x73706972697469736d65', NULL, 945, 946, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (474, 1, 'T1-173', '\\x726563686572636865206d696e69c3a87265', NULL, 947, 948, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (475, 1, 'T1-175', '\\x617373656d626cc3a965206e6174696f6e616c65', NULL, 949, 950, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (476, 1, 'T1-902', '\\x7061726c656d656e7461697265', NULL, 951, 952, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (634, 1, 'T1-267', '\\x6175746f6d6f62696c69737465', NULL, 1267, 1268, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (480, 1, 'T1-745', '\\x7472616e73706f7274206d756c74696d6f64616c', NULL, 959, 960, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (481, 1, 'T1-179', '\\x73616c617269c3a92061677269636f6c65', NULL, 961, 962, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (482, 1, 'T1-419', '\\x6167726963756c74657572', NULL, 963, 964, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (483, 1, 'T1-667', '\\x6d6967726174696f6e20736169736f6e6e69c3a87265', NULL, 965, 966, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (484, 1, 'T1-180', '\\x68616c7465206761726465726965', NULL, 967, 968, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (485, 1, 'T1-181', '\\x70657273c3a9637574696f6e2072656c69676965757365', NULL, 969, 970, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (487, 1, 'T1-183', '\\x61637469766974c3a9206dc3a96e6167c3a87265', NULL, 973, 974, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (488, 1, 'T1-185', '\\xc3a97461626c697373656d656e742064652073616e74c3a9', NULL, 975, 976, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (489, 1, 'T1-344', '\\x73616e74c3a9', NULL, 977, 978, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (490, 1, 'T1-1457', '\\xc3a97461626c697373656d656e742073616e6974616972652070726976c3a9', NULL, 979, 980, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (764, 1, 'T1-365', '\\x706861726d616369656e', NULL, 1527, 1528, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (382, 1, 'T1-125', '\\x726566756765206465206d6f6e7461676e65', NULL, 763, 764, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (383, 1, 'T1-909', '\\xc3a971756970656d656e7420746f757269737469717565', NULL, 765, 766, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (384, 1, 'T1-176', '\\x6d616368696e652064616e67657265757365', NULL, 767, 768, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (491, 1, 'T1-950', '\\x636f6e737472756374696f6e20686f73706974616c69c3a87265', NULL, 981, 982, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (495, 1, 'T1-894', '\\x73616e61746f7269756d', NULL, 989, 990, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (496, 1, 'T1-560', '\\xc3a971756970656d656e74206dc3a96469636f20636869727572676963616c', NULL, 991, 992, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (497, 1, 'T1-651', '\\x656e736569676e656d656e7420686f73706974616c696572', NULL, 993, 994, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (498, 1, 'T1-1414', '\\x666f6e6374696f6e6e6169726520686f73706974616c696572', NULL, 995, 996, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (1369, 1, 'T1-1164', '\\x766f6c', NULL, 2737, 2738, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (513, 1, 'T1-1090', '\\x7472616e73706f7274206d61726974696d65', NULL, 1025, 1026, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (514, 1, 'T1-198', '\\x63617274652073636f6c61697265', NULL, 1027, 1028, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (516, 1, 'T1-406', '\\x6173736f63696174696f6e2066616d696c69616c65', NULL, 1031, 1032, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (517, 1, 'T1-201', '\\x706f726e6f67726170686965', NULL, 1033, 1034, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (518, 1, 'T1-1396', '\\x706f6c69636520646573206d6f65757273', NULL, 1035, 1036, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (519, 1, 'T1-1110', '\\x63696ec3a96d61', NULL, 1037, 1038, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (520, 1, 'T1-202', '\\x63616c616d6974c3a92061677269636f6c65', NULL, 1039, 1040, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (521, 1, 'T1-309', '\\x73c3a96368657265737365', NULL, 1041, 1042, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (522, 1, 'T1-854', '\\x67656cc3a965', NULL, 1043, 1044, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (523, 1, 'T1-683', '\\x6772c3aa6c65', NULL, 1045, 1046, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (524, 1, 'T1-663', '\\x6dc3a974c3a96f726f6c6f676965', NULL, 1047, 1048, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (525, 1, 'T1-447', '\\x706c616e74652074657874696c65', NULL, 1049, 1050, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (526, 1, 'T1-205', '\\x7461786520737572206c6520636869666672652064276166666169726573', NULL, 1051, 1052, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (527, 1, 'T1-206', '\\x64c3a970656e7365206427696e76657374697373656d656e74', NULL, 1053, 1054, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (528, 1, 'T1-207', '\\xc3a96d616e6369706174696f6e', NULL, 1055, 1056, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (529, 1, 'T1-489', '\\x64726f6974206465206c612066616d696c6c65', NULL, 1057, 1058, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (619, 1, 'T1-1060', '\\x6a6575782d65742d7061726973', NULL, 1237, 1238, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (530, 1, 'T1-208', '\\xc3a96475636174696f6e2073657875656c6c65', NULL, 1059, 1060, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (531, 1, 'T1-940', '\\x70726f6772616d6d652073636f6c61697265', NULL, 1061, 1062, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (532, 1, 'T1-1131', '\\x6d6f62696c6965722075726261696e', NULL, 1063, 1064, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (533, 1, 'T1-1246', '\\xc3a974616c656d656e742075726261696e', NULL, 1065, 1066, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (534, 1, 'T1-444', '\\x7265636f6e737472756374696f6e', NULL, 1067, 1068, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (535, 1, 'T1-1105', '\\x7365637465757220736175766567617264c3a9', NULL, 1069, 1070, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (536, 1, 'T1-778', '\\x666c657572697373656d656e74', NULL, 1071, 1072, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (537, 1, 'T1-210', '\\x656d706c6f79c3a9206465206d6169736f6e', NULL, 1073, 1074, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (538, 1, 'T1-480', '\\x70726f66657373696f6e20706172746963756c69c3a87265', NULL, 1075, 1076, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (539, 1, 'T1-212', '\\x6a757374696365206d756e69636970616c65', NULL, 1077, 1078, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (540, 1, 'T1-1115', '\\x64726f69747320736569676e65757269617578', NULL, 1079, 1080, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (541, 1, 'T1-578', '\\x6dc3a96469617468c3a8717565', NULL, 1081, 1082, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (542, 1, 'T1-693', '\\x6b696f7371756520c3a0206d757369717565', NULL, 1083, 1084, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (543, 1, 'T1-254', '\\x6172636869766573', NULL, 1085, 1086, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (544, 1, 'T1-871', '\\x6d6169736f6e20646573206a65756e6573', NULL, 1087, 1088, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (545, 1, 'T1-305', '\\x73616c6c65206465732066c3aa746573', NULL, 1089, 1090, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (547, 1, 'T1-704', '\\x636f6e7365727661746f697265', NULL, 1093, 1094, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (549, 1, 'T1-332', '\\x666f79657220727572616c', NULL, 1097, 1098, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (416, 1, 'T1-144', '\\x656d706c6f6920646573206a65756e6573', NULL, 831, 832, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (417, 1, 'T1-558', '\\x6a65756e65', NULL, 833, 834, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (418, 1, 'T1-145', '\\x636c617373652064652064c3a9636f757665727465', NULL, 835, 836, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (419, 1, 'T1-146', '\\x726176697461696c6c656d656e74', NULL, 837, 838, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (420, 1, 'T1-1150', '\\x726174696f6e6e656d656e74', NULL, 839, 840, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (421, 1, 'T1-756', '\\x6d61726368c3a9206e6f6972', NULL, 841, 842, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (422, 1, 'T1-1450', '\\x73756273697374616e636573', NULL, 843, 844, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (423, 1, 'T1-1151', '\\x7472616e73706f7274206465206d617469c3a87265732064616e6765726575736573', NULL, 845, 846, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (438, 1, 'T1-231', '\\x706f7274', NULL, 875, 876, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (1662, 1, 'T3-135', '\\x6d656e75', NULL, 3323, 3324, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (499, 1, 'T1-186', '\\xc3a97461626c697373656d656e7420646520666f726d6174696f6e20646573206d61c3ae74726573', NULL, 997, 998, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (500, 1, 'T1-662', '\\xc3a97461626c697373656d656e74206427656e736569676e656d656e74', NULL, 999, 1000, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (501, 1, 'T1-188', '\\x696d70c3b47420737572206c6120666f7274756e65', NULL, 1001, 1002, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (502, 1, 'T1-1376', '\\x66697363616c6974c3a92064657320706572736f6e6e6573', NULL, 1003, 1004, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (503, 1, 'T1-189', '\\x7472617661696c2070726f74c3a967c3a9', NULL, 1005, 1006, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (504, 1, 'T1-1027', '\\x7472617661696c6c6575722068616e6469636170c3a9', NULL, 1007, 1008, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (505, 1, 'T1-190', '\\x6372c3a8636865', NULL, 1009, 1010, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (506, 1, 'T1-191', '\\x73756963696465', NULL, 1011, 1012, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (779, 1, 'T1-900', '\\x6d6f62696c696572', NULL, 1557, 1558, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (584, 1, 'T1-240', '\\x6772616e646520c3a9636f6c65', NULL, 1167, 1168, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (510, 1, 'T1-195', '\\x666f6e6374696f6e6e61697265206465206c27556e696f6e206575726f70c3a9656e6e65', NULL, 1019, 1020, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (511, 1, 'T1-805', '\\x706572736f6e6e656c', NULL, 1021, 1022, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (512, 1, 'T1-196', '\\x6c69676e65206d61726974696d65', NULL, 1023, 1024, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (448, 1, 'T1-738', '\\x61726368697465637465', NULL, 895, 896, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (449, 1, 'T1-161', '\\x7265706f7320686562646f6d616461697265', NULL, 897, 898, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (450, 1, 'T1-162', '\\x6672756974', NULL, 899, 900, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (451, 1, 'T1-631', '\\x686f72746963756c74757265', NULL, 901, 902, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (453, 1, 'T1-1232', '\\x6661696c6c697465', NULL, 905, 906, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (436, 1, 'T1-154', '\\x6173736f63696174696f6e20646520746f757269736d65', NULL, 871, 872, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (437, 1, 'T1-156', '\\x706f72742064652070c3aa636865', NULL, 873, 874, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (492, 1, 'T1-1263', '\\x6cc3a970726f7365726965', NULL, 983, 984, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (493, 1, 'T1-1062', '\\x68c3b4706974616c2070737963686961747269717565', NULL, 985, 986, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (578, 1, 'T1-525', '\\x6a7573746963652070c3a96e616c65', NULL, 1155, 1156, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (494, 1, 'T1-329', '\\xc3a97461626c697373656d656e74207075626c6963206427686f73706974616c69736174696f6e', NULL, 987, 988, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (548, 1, 'T1-1459', '\\x6269626c696f7468c3a8717565', NULL, 1095, 1096, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (550, 1, 'T1-1034', '\\x63656e74726520646520646f63756d656e746174696f6e', NULL, 1099, 1100, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (551, 1, 'T1-214', '\\x6d616c616469652073657875656c6c656d656e74207472616e736d69737369626c65', NULL, 1101, 1102, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (552, 1, 'T1-215', '\\x6162616e646f6e2064652066616d696c6c65', NULL, 1103, 1104, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (553, 1, 'T1-217', '\\x62617465617520646520706c616973616e6365', NULL, 1105, 1106, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (554, 1, 'T1-218', '\\x6167656e742064652064726f69742070726976c3a9', NULL, 1107, 1108, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (558, 1, 'T1-639', '\\x73706f7274', NULL, 1115, 1116, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (559, 1, 'T1-222', '\\x766f796167652073636f6c61697265', NULL, 1117, 1118, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (560, 1, 'T1-223', '\\x6561752d64652d766965', NULL, 1119, 1120, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (561, 1, 'T1-225', '\\x68c3b474656c20646520746f757269736d65', NULL, 1121, 1122, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (562, 1, 'T1-227', '\\x656e74726570726973652064752062c3a274696d656e74', NULL, 1123, 1124, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (563, 1, 'T1-915', '\\x6d657373616765726965', NULL, 1125, 1126, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (564, 1, 'T1-1350', '\\x7265646576616e6365207061726166697363616c65', NULL, 1127, 1128, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (565, 1, 'T1-230', '\\x6f7267616e69736d652064652073c3a96375726974c3a920736f6369616c65', NULL, 1129, 1130, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (566, 1, 'T1-1214', '\\xc3a96c656374696f6e20736f6369616c65', NULL, 1131, 1132, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (567, 1, 'T1-597', '\\x706f7274206d61726974696d65', NULL, 1133, 1134, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (568, 1, 'T1-1434', '\\x646f636b6572', NULL, 1135, 1136, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (569, 1, 'T1-567', '\\x706f727420646520636f6d6d65726365', NULL, 1137, 1138, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (570, 1, 'T1-948', '\\x706f727420666c757669616c', NULL, 1139, 1140, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (571, 1, 'T1-1253', '\\x706f727420646520706c616973616e6365', NULL, 1141, 1142, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (572, 1, 'T1-928', '\\x6f6666696369657220646520706f7274', NULL, 1143, 1144, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (573, 1, 'T1-1206', '\\x766f69652066657272c3a96520646573207175616973', NULL, 1145, 1146, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (698, 1, 'T1-322', '\\x6cc3a8707265', NULL, 1395, 1396, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (574, 1, 'T1-232', '\\x656e736569676e656d656e742072656c696769657578', NULL, 1147, 1148, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (576, 1, 'T1-234', '\\x6f66667265206427656d706c6f69', NULL, 1151, 1152, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (577, 1, 'T1-235', '\\x6372696d696e616c6974c3a9', NULL, 1153, 1154, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (580, 1, 'T1-1412', '\\x746f757269736d6520736f6369616c', NULL, 1159, 1160, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (581, 1, 'T1-238', '\\x76657220c3a020736f6965', NULL, 1161, 1162, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (583, 1, 'T1-718', '\\x66c3a96f64616c6974c3a9', NULL, 1165, 1166, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (585, 1, 'T1-241', '\\x696d70c3b47473206c6f63617578', NULL, 1169, 1170, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:02');
INSERT INTO keywords VALUES (586, 1, 'T1-1316', '\\x6f6374726f69', NULL, 1171, 1172, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (587, 1, 'T1-1016', '\\x696d70c3b474732070726f76696e6369617578206427416e6369656e2052c3a967696d65', NULL, 1173, 1174, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (588, 1, 'T1-641', '\\x64726f697420646520706c616365', NULL, 1175, 1176, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (589, 1, 'T1-298', '\\x64726f6974206465732070617576726573', NULL, 1177, 1178, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (590, 1, 'T1-1002', '\\x746178652064652073c3a96a6f7572', NULL, 1179, 1180, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (591, 1, 'T1-1298', '\\x7461786520642768616269746174696f6e', NULL, 1181, 1182, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (592, 1, 'T1-243', '\\x6368656e616c', NULL, 1183, 1184, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (593, 1, 'T1-244', '\\x696e64757374726965206167726f20616c696d656e7461697265', NULL, 1185, 1186, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (594, 1, 'T1-869', '\\x636f6d6d6572636520616c696d656e7461697265', NULL, 1187, 1188, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (595, 1, 'T1-1092', '\\x61626174746f6972', NULL, 1189, 1190, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (596, 1, 'T1-245', '\\x7472616e73706f72742073636f6c61697265', NULL, 1191, 1192, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (848, 1, 'T1-429', '\\x61637175697474656d656e74', NULL, 1695, 1696, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (1775, 1, 'T3-42', '\\x6361727465', NULL, 3549, 3550, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (611, 1, 'T1-251', '\\x656e736569676e656d656e7420737570c3a97269657572', NULL, 1221, 1222, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (612, 1, 'T1-294', '\\x64726f6974732064652073636f6c61726974c3a9', NULL, 1223, 1224, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (613, 1, 'T1-586', '\\x746974726520756e6976657273697461697265', NULL, 1225, 1226, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (614, 1, 'T1-976', '\\x74726f75626c6520756e6976657273697461697265', NULL, 1227, 1228, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (615, 1, 'T1-252', '\\x64726f697473206427757361676520666f7265737469657273', NULL, 1229, 1230, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (616, 1, 'T1-1324', '\\x7573616765732061677269636f6c6573206c6f63617578', NULL, 1231, 1232, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (617, 1, 'T1-637', '\\x6c6f7420646520636861737365', NULL, 1233, 1234, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (618, 1, 'T1-253', '\\x706f6c69636520646573206a657578', NULL, 1235, 1236, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (1188, 1, 'T1-1379', '\\x666f6e646174696f6e', NULL, 2375, 2376, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (597, 1, 'T1-1296', '\\x6f6575767265732073636f6c6169726573', NULL, 1193, 1194, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (598, 1, 'T1-1273', '\\x7472616e73706f727420656e20636f6d6d756e', NULL, 1195, 1196, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (599, 1, 'T1-1405', '\\x656e736569676e656d656e742061677269636f6c65', NULL, 1197, 1198, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (600, 1, 'T1-247', '\\x6f66666963696572206d696c697461697265', NULL, 1199, 1200, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (601, 1, 'T1-1103', '\\x6d696c697461697265', NULL, 1201, 1202, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (602, 1, 'T1-248', '\\x6167656e742064652073c3a96375726974c3a9', NULL, 1203, 1204, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (605, 1, 'T1-249', '\\x70726f706f7320737562766572736966', NULL, 1209, 1210, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (606, 1, 'T1-770', '\\x6d696c69636573', NULL, 1211, 1212, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (607, 1, 'T1-1293', '\\x6d6f62696c69736174696f6e', NULL, 1213, 1214, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (608, 1, 'T1-966', '\\x706f6c696365206d696c697461697265', NULL, 1215, 1216, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (684, 1, 'T1-303', '\\x68696e646f7569736d65', NULL, 1367, 1368, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (661, 1, 'T1-285', '\\x7265636f6e6e61697373616e636573', NULL, 1321, 1322, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (662, 1, 'T1-1074', '\\x656e7669726f6e6e656d656e74', NULL, 1323, 1324, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (663, 1, 'T1-714', '\\x706f6c6c7574696f6e2061746d6f737068c3a97269717565', NULL, 1325, 1326, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (664, 1, 'T1-1420', '\\x706f6c6c7574696f6e206465732065617578', NULL, 1327, 1328, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (665, 1, 'T1-375', '\\x706f6c6c7574696f6e20736f6e6f7265', NULL, 1329, 1330, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (666, 1, 'T1-327', '\\x706f6c6c7574696f6e206465206c61206d6572', NULL, 1331, 1332, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (667, 1, 'T1-1145', '\\x696e64757374726965206368696d69717565', NULL, 1333, 1334, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (668, 1, 'T1-289', '\\x70726f74656374696f6e206465732076c3a967c3a974617578', NULL, 1335, 1336, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (433, 1, 'T1-695', '\\x656175', NULL, 865, 866, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (434, 1, 'T1-152', '\\x63616e636572', NULL, 867, 868, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (609, 1, 'T1-1135', '\\x67656e6461726d65726965', NULL, 1217, 1218, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (610, 1, 'T1-301', '\\x61747465696e746520c3a0206c612073c3bb726574c3a9206465206c27c389746174', NULL, 1219, 1220, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (639, 1, 'T1-1297', '\\x616e746973c3a96d697469736d65', NULL, 1277, 1278, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (640, 1, 'T1-270', '\\x72c3a9636c7573696f6e206372696d696e656c6c65', NULL, 1279, 1280, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (641, 1, 'T1-615', '\\x7065696e65', NULL, 1281, 1282, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (642, 1, 'T1-271', '\\x676172646520c3a020767565', NULL, 1283, 1284, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (643, 1, 'T1-468', '\\x706f6c696365206a756469636961697265', NULL, 1285, 1286, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (644, 1, 'T1-272', '\\x636f6e737472756374696f6e2061c3a9726f6e61757469717565', NULL, 1287, 1288, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (645, 1, 'T1-849', '\\x696e64757374726965206dc3a963616e69717565', NULL, 1289, 1290, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (646, 1, 'T1-1406', '\\x61c3a9726f6e6566', NULL, 1291, 1292, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (647, 1, 'T1-273', '\\x68c3b474656c2064752064c3a970617274656d656e74', NULL, 1293, 1294, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (983, 1, 'T1-1381', '\\x636f6e74726176656e74696f6e', NULL, 1965, 1966, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (507, 1, 'T1-192', '\\x6173737572616e636520766965696c6c65737365', NULL, 1013, 1014, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (508, 1, 'T1-404', '\\x706572736f6e6e6520c3a267c3a965', NULL, 1015, 1016, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (509, 1, 'T1-193', '\\x636f6e74656e74696575782066697363616c', NULL, 1017, 1018, 1, 1, 1, '2011-05-25 10:03:02', '2012-03-07 09:41:01');
INSERT INTO keywords VALUES (621, 1, 'T1-256', '\\x626f75727365206427c3a97475646573', NULL, 1241, 1242, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (622, 1, 'T1-257', '\\x70726573627974c3a87265', NULL, 1243, 1244, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (623, 1, 'T1-258', '\\x73796e64696361742070726f66657373696f6e6e656c', NULL, 1245, 1246, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (624, 1, 'T1-847', '\\x72656c6174696f6e73206465207472617661696c', NULL, 1247, 1248, 1, 1, 1, '2011-05-25 10:03:03', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (625, 1, 'T1-259', '\\x6368697275726769656e2064656e7469737465', NULL, 1249, 1250, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (626, 1, 'T1-260', '\\x61646d696e697374726174696f6e2064c3a970617274656d656e74616c65', NULL, 1251, 1252, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (672, 1, 'T1-1130', '\\x6c6f79657220696d6d6f62696c696572', NULL, 1343, 1344, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (673, 1, 'T1-656', '\\x6c6f67656d656e7420696e646976696475656c', NULL, 1345, 1346, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (674, 1, 'T1-772', '\\x6c6f67656d656e7420636f6c6c6563746966', NULL, 1347, 1348, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (675, 1, 'T1-302', '\\x657870756c73696f6e206c6f636174697665', NULL, 1349, 1350, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (676, 1, 'T1-291', '\\x6173737572616e63652064c3a963c3a873', NULL, 1351, 1352, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (677, 1, 'T1-292', '\\x736f6e646167652067c3a96f6c6f6769717565', NULL, 1353, 1354, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (678, 1, 'T1-293', '\\xc3a9636f6275616765', NULL, 1355, 1356, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (680, 1, 'T1-295', '\\x6672617564652066697363616c65', NULL, 1359, 1360, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (681, 1, 'T1-296', '\\x657363726f717565726965', NULL, 1361, 1362, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (683, 1, 'T1-299', '\\x726573736f7274206a756469636961697265', NULL, 1365, 1366, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (628, 1, 'T1-262', '\\x636f7276c3a96520726f79616c65', NULL, 1255, 1256, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (629, 1, 'T1-263', '\\x636169737365206427c3a9706172676e65', NULL, 1257, 1258, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (959, 1, 'T1-549', '\\x61697265206465206a657578', NULL, 1917, 1918, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (630, 1, 'T1-314', '\\xc3a97461626c697373656d656e7420646520637265646974', NULL, 1259, 1260, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (632, 1, 'T1-264', '\\x6a75727920642761737369736573', NULL, 1263, 1264, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (633, 1, 'T1-265', '\\xc3a9636865632073636f6c61697265', NULL, 1265, 1266, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (638, 1, 'T1-281', '\\x6a756461c3af736d65', NULL, 1275, 1276, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:03');
INSERT INTO keywords VALUES (691, 1, 'T1-313', '\\x6a7567652061646d696e6973747261746966', NULL, 1381, 1382, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (692, 1, 'T1-622', '\\x7072c3aa74657572', NULL, 1383, 1384, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (693, 1, 'T1-1386', '\\x62616e717565206d757475616c69737465', NULL, 1385, 1386, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (694, 1, 'T1-315', '\\x62c3a274696d656e7420696e647573747269656c', NULL, 1387, 1388, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (695, 1, 'T1-316', '\\x636f7272757074696f6e', NULL, 1389, 1390, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (696, 1, 'T1-888', '\\x66696e616e63657320636f6d6d756e616c6573', NULL, 1391, 1392, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (697, 1, 'T1-320', '\\x746572726f7269736d65', NULL, 1393, 1394, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (699, 1, 'T1-323', '\\x6167726963756c747572652062696f6c6f6769717565', NULL, 1397, 1398, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (1601, 1, 'T3-53', '\\x636174616c6f677565', NULL, 3201, 3202, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (648, 1, 'T1-274', '\\x6d61726368c3a92064652064657461696c', NULL, 1295, 1296, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (649, 1, 'T1-1385', '\\x636f6c706f7274657572', NULL, 1297, 1298, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (650, 1, 'T1-1078', '\\x6d61726368616e6420666f7261696e', NULL, 1299, 1300, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (651, 1, 'T1-275', '\\xc3a9746174732067c3a96ec3a972617578', NULL, 1301, 1302, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (652, 1, 'T1-276', '\\x64c3a96c6f63616c69736174696f6e', NULL, 1303, 1304, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (653, 1, 'T1-278', '\\x706f6c696365207365636f757273', NULL, 1305, 1306, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (655, 1, 'T1-1423', '\\x6f7267616e69736174696f6e2073616e697461697265', NULL, 1309, 1310, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (656, 1, 'T1-280', '\\x656e736569676e656d656e7420636f6d6d65726369616c', NULL, 1311, 1312, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (657, 1, 'T1-1331', '\\x656e736569676e656d656e742070726f66657373696f6e6e656c', NULL, 1313, 1314, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (659, 1, 'T1-282', '\\x64726f69747320686f6e6f7269666971756573', NULL, 1317, 1318, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (660, 1, 'T1-283', '\\x6f6cc3a96167696e657578', NULL, 1319, 1320, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (685, 1, 'T1-304', '\\x63656e74726520646520666f726d6174696f6e', NULL, 1369, 1370, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (686, 1, 'T1-1196', '\\x666f726d6174696f6e2070726f66657373696f6e6e656c6c65', NULL, 1371, 1372, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (687, 1, 'T1-306', '\\x656e72656769737472656d656e74', NULL, 1373, 1374, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (688, 1, 'T1-308', '\\x636f6d6d756e69636174696f6e20696e737469747574696f6e6e656c6c65', NULL, 1375, 1376, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (689, 1, 'T1-310', '\\x726570726f64756374696f6e20616e696d616c65', NULL, 1377, 1378, 1, 1, 1, '2011-05-25 10:03:04', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (700, 1, 'T1-324', '\\x63616d70206427696e7465726e656d656e74', NULL, 1399, 1400, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (728, 1, 'T1-748', '\\x68796769c3a86e65', NULL, 1455, 1456, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (1072, 1, 'T1-611', '\\x6c6963656e6369656d656e74', NULL, 2143, 2144, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (704, 1, 'T1-679', '\\x7472616e73706f727420666c757669616c', NULL, 1407, 1408, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (705, 1, 'T1-328', '\\x6e6176697265', NULL, 1409, 1410, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (706, 1, 'T1-330', '\\x6372c3a96174696f6e20617564696f76697375656c6c65', NULL, 1411, 1412, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (795, 1, 'T1-1357', '\\x6661756e652073617576616765', NULL, 1589, 1590, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (707, 1, 'T1-331', '\\x7175616c696669636174696f6e20636f7272656374696f6e6e656c6c65', NULL, 1413, 1414, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (708, 1, 'T1-1004', '\\x726573746175726174696f6e20646573207465727261696e7320656e206d6f6e7461676e65', NULL, 1415, 1416, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (709, 1, 'T1-335', '\\x6469736369706c696e652073706f7274697665', NULL, 1417, 1418, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (710, 1, 'T1-687', '\\x746f757269736d65206465206d6f6e7461676e65', NULL, 1419, 1420, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (729, 1, 'T1-1170', '\\x616dc3a96c696f726174696f6e206465206c2768616269746174', NULL, 1457, 1458, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (730, 1, 'T1-500', '\\x6d616c616469652070726f66657373696f6e6e656c6c65', NULL, 1459, 1460, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (731, 1, 'T1-1163', '\\x68796769c3a86e6520736f6369616c65', NULL, 1461, 1462, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (732, 1, 'T1-702', '\\x6dc3a9646563696e652070c3a96e6974656e746961697265', NULL, 1463, 1464, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (733, 1, 'T1-841', '\\x6dc3a9646563696e65206475207472617661696c', NULL, 1465, 1466, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (734, 1, 'T1-998', '\\x6dc3a9646563696e652073636f6c61697265', NULL, 1467, 1468, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (735, 1, 'T1-1330', '\\x6173737572616e6365206d616c61646965', NULL, 1469, 1470, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (737, 1, 'T1-346', '\\x6d6f6e617263686965', NULL, 1473, 1474, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (739, 1, 'T1-1083', '\\x64c3a963656e7472616c69736174696f6e', NULL, 1477, 1478, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (740, 1, 'T1-1452', '\\x64c3a9636f6e63656e74726174696f6e', NULL, 1479, 1480, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (741, 1, 'T1-348', '\\x6c6f7465726965', NULL, 1481, 1482, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (742, 1, 'T1-349', '\\x6c696d697465207465727269746f7269616c65', NULL, 1483, 1484, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (743, 1, 'T1-351', '\\x636974c3a92061646d696e697374726174697665', NULL, 1485, 1486, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (744, 1, 'T1-352', '\\x616e6369656e20636f6d62617474616e74', NULL, 1487, 1488, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (745, 1, 'T1-1312', '\\x656d706c6f692072c3a973657276c3a9', NULL, 1489, 1490, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (746, 1, 'T1-353', '\\x666f6e6374696f6e6e61697265207465727269746f7269616c', NULL, 1491, 1492, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (702, 1, 'T1-326', '\\x766f6965206e6176696761626c65', NULL, 1403, 1404, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (713, 1, 'T1-337', '\\x706861726d61636965', NULL, 1425, 1426, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (716, 1, 'T1-728', '\\x70726f6475697420636f736dc3a97469717565', NULL, 1431, 1432, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (717, 1, 'T1-625', '\\x7370c3a96369616c6974c3a920706861726d616365757469717565', NULL, 1433, 1434, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (718, 1, 'T1-338', '\\x616374696f6e2063756c747572656c6c65', NULL, 1435, 1436, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (719, 1, 'T1-339', '\\x6e6f6d616465', NULL, 1437, 1438, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (720, 1, 'T1-1128', '\\x616972652064652073746174696f6e6e656d656e74', NULL, 1439, 1440, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (721, 1, 'T1-340', '\\x6c6f742064652070c3aa636865', NULL, 1441, 1442, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (1301, 1, 'T1-947', '\\x736f6c646573', NULL, 2601, 2602, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (778, 1, 'T1-416', '\\x65737061636520646f6d65737469717565', NULL, 1555, 1556, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (780, 1, 'T1-374', '\\x72c3a96d756ec3a9726174696f6e', NULL, 1559, 1560, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (782, 1, 'T1-379', '\\x656e736569676e656d656e7420c3a96cc3a96d656e7461697265', NULL, 1563, 1564, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (783, 1, 'T1-380', '\\x62616c207075626c6963', NULL, 1565, 1566, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (784, 1, 'T1-381', '\\x70c3aa6368657572', NULL, 1567, 1568, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (785, 1, 'T1-382', '\\x636c616e64657374696e6974c3a9', NULL, 1569, 1570, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (786, 1, 'T1-1193', '\\x72c3a9736572766520666f6e6369c3a87265', NULL, 1571, 1572, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (787, 1, 'T1-384', '\\x636f6c6f6e69736174696f6e', NULL, 1573, 1574, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (788, 1, 'T1-1069', '\\x636f6c6f6e6965', NULL, 1575, 1576, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (789, 1, 'T1-385', '\\x696e737469747574696f6e73', NULL, 1577, 1578, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (790, 1, 'T1-387', '\\x6f7267616e69736d6520706172617075626c6963', NULL, 1579, 1580, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (791, 1, 'T1-760', '\\x6173736f63696174696f6e', NULL, 1581, 1582, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (792, 1, 'T1-388', '\\x616d656e6465', NULL, 1583, 1584, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (796, 1, 'T1-1114', '\\x706f6c696365206465206c6120636861737365', NULL, 1591, 1592, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (797, 1, 'T1-886', '\\x636f6e666c6974206475207472617661696c', NULL, 1593, 1594, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (798, 1, 'T1-392', '\\x72c3a9686162696c69746174696f6e', NULL, 1595, 1596, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (799, 1, 'T1-393', '\\x746175726f6d6163686965', NULL, 1597, 1598, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (800, 1, 'T1-394', '\\x696e6672616374696f6e206d61726974696d65', NULL, 1599, 1600, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (801, 1, 'T1-395', '\\x766f696520636f6d6d756e616c65', NULL, 1601, 1602, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (802, 1, 'T1-595', '\\x6368616e74696572', NULL, 1603, 1604, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (804, 1, 'T1-654', '\\x736f6369c3a974c3a920646520636f6e737472756374696f6e', NULL, 1607, 1608, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (805, 1, 'T1-1346', '\\x6f7576726965722064752062c3a274696d656e74', NULL, 1609, 1610, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (806, 1, 'T1-975', '\\x6d6f6e6f706f6c65732066697363617578', NULL, 1611, 1612, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (807, 1, 'T1-1086', '\\x64726f6974732064652063697263756c6174696f6e', NULL, 1613, 1614, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (808, 1, 'T1-399', '\\x616dc3a96e6167656d656e74206475206c6974746f72616c', NULL, 1615, 1616, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (816, 1, 'T1-405', '\\x636f6e67c3a97320706179c3a973', NULL, 1631, 1632, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (726, 1, 'T1-1268', '\\x746865726d6f636c696d617469736d65', NULL, 1451, 1452, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (727, 1, 'T1-343', '\\x616d69616e7465', NULL, 1453, 1454, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (756, 1, 'T1-378', '\\x64c3a96368657420616e696d616c', NULL, 1511, 1512, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (757, 1, 'T1-442', '\\x64c3a96368657420696e647573747269656c', NULL, 1513, 1514, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (758, 1, 'T1-1129', '\\x6f726475726573206dc3a96e6167c3a8726573', NULL, 1515, 1516, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (759, 1, 'T1-800', '\\x64c3a96368657420686f73706974616c696572', NULL, 1517, 1518, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (760, 1, 'T1-722', '\\x7573696e65206427696e63696ec3a9726174696f6e', NULL, 1519, 1520, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (761, 1, 'T1-1159', '\\x64c3a9636861726765207075626c69717565', NULL, 1521, 1522, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (762, 1, 'T1-363', '\\x636f6e6772c3a87320706f6c697469717565', NULL, 1523, 1524, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (752, 1, 'T1-360', '\\x6c697474c3a9726174757265', NULL, 1503, 1504, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (753, 1, 'T1-361', '\\x6672616e63206d61c3a76f6e', NULL, 1505, 1506, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (754, 1, 'T1-370', '\\x736f6369c3a974c3a92073656372c3a87465', NULL, 1507, 1508, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (755, 1, 'T1-879', '\\x76696374696d6520646520677565727265', NULL, 1509, 1510, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (817, 1, 'T1-411', '\\x66616d696c6c65', NULL, 1633, 1634, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (818, 1, 'T1-407', '\\x617373656d626cc3a9652067c3a96ec3a972616c6520647520636c657267c3a9', NULL, 1635, 1636, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (819, 1, 'T1-408', '\\x616d6e6973746965', NULL, 1637, 1638, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (820, 1, 'T1-409', '\\x6f66666963696572206465206a757374696365206427416e6369656e2052c3a967696d65', NULL, 1639, 1640, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (821, 1, 'T1-1417', '\\x6167656e74206465206c2761646d696e697374726174696f6e20726f79616c65', NULL, 1641, 1642, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (822, 1, 'T1-780', '\\x6f666669636573', NULL, 1643, 1644, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (908, 1, 'T1-477', '\\x6578747261646974696f6e', NULL, 1815, 1816, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (772, 1, 'T1-371', '\\x61696465206a756469636961697265', NULL, 1543, 1544, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (775, 1, 'T1-1042', '\\x616e696d616c20646520636f6d7061676e6965', NULL, 1549, 1550, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (776, 1, 'T1-1252', '\\x76c3aa74656d656e74', NULL, 1551, 1552, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (777, 1, 'T1-686', '\\x636f6d70746162696c6974c3a92070726976c3a965', NULL, 1553, 1554, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (809, 1, 'T1-1213', '\\x7265626f6973656d656e74', NULL, 1617, 1618, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (810, 1, 'T1-874', '\\x636f6e7365696c2064276172726f6e64697373656d656e74', NULL, 1619, 1620, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (811, 1, 'T1-402', '\\x666f7263657320616c6c69c3a96573', NULL, 1621, 1622, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (812, 1, 'T1-403', '\\x6573706163652076657274', NULL, 1623, 1624, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (813, 1, 'T1-1057', '\\x70617263206e61747572656c', NULL, 1625, 1626, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (814, 1, 'T1-591', '\\x706c6163656d656e742066616d696c69616c', NULL, 1627, 1628, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (834, 1, 'T1-546', '\\xc3a96c6576657572', NULL, 1667, 1668, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (1691, 1, 'T3-147', '\\x6f626a6574', NULL, 3381, 3382, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (825, 1, 'T1-1415', '\\x6d6169736f6e2066616d696c69616c6520646520766163616e636573', NULL, 1649, 1650, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (826, 1, 'T1-533', '\\x616666616972652066616d696c69616c65', NULL, 1651, 1652, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (827, 1, 'T1-790', '\\x70726573746174696f6e2066616d696c69616c65', NULL, 1653, 1654, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (828, 1, 'T1-412', '\\x616c636f6f6c69717565', NULL, 1655, 1656, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (829, 1, 'T1-413', '\\x72c3a973656175206427696e666f726d6174696f6e', NULL, 1657, 1658, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (830, 1, 'T1-414', '\\x6578616d656e2070726f66657373696f6e6e656c', NULL, 1659, 1660, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (831, 1, 'T1-415', '\\x61706963756c74757265', NULL, 1661, 1662, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (832, 1, 'T1-417', '\\x636c616e64657374696e', NULL, 1663, 1664, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (835, 1, 'T1-449', '\\x6a65756e65206167726963756c74657572', NULL, 1669, 1670, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (836, 1, 'T1-1209', '\\x6578706c6f6974616e742061677269636f6c65', NULL, 1671, 1672, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (837, 1, 'T1-1107', '\\x626572676572', NULL, 1673, 1674, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (838, 1, 'T1-986', '\\x6171756163756c74757265', NULL, 1675, 1676, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (839, 1, 'T1-653', '\\x70c3aa636865', NULL, 1677, 1678, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (840, 1, 'T1-785', '\\x6d61726368c3a92061677269636f6c65', NULL, 1679, 1680, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (842, 1, 'T1-423', '\\x6f636375706174696f6e2074656d706f726169726520647520646f6d61696e65207075626c6963', NULL, 1683, 1684, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (843, 1, 'T1-424', '\\x6d696e69737472652064752063756c7465', NULL, 1685, 1686, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (844, 1, 'T1-426', '\\x73616e7320646f6d6963696c652066697865', NULL, 1687, 1688, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (845, 1, 'T1-427', '\\x666f7274696669636174696f6e', NULL, 1689, 1690, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (847, 1, 'T1-559', '\\x706f70756c6174696f6e2073636f6c61697265', NULL, 1693, 1694, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (849, 1, 'T1-1242', '\\x64c3a9636973696f6e206465206a757374696365', NULL, 1697, 1698, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (850, 1, 'T1-430', '\\xc3a96e6572676965206879647261756c69717565', NULL, 1699, 1700, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (851, 1, 'T1-431', '\\x636f6d6d657263652064652064c3a97461696c', NULL, 1701, 1702, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (1450, 1, 'T2-29', '\\x636f6f70c3a9726174696f6e', NULL, 2899, 2900, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (852, 1, 'T1-433', '\\x6c616e67756520c3a97472616e67c3a87265', NULL, 1703, 1704, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (853, 1, 'T1-434', '\\x65786f6ec3a9726174696f6e2066697363616c65', NULL, 1705, 1706, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (854, 1, 'T1-1109', '\\xc3a97461626c697373656d656e74207075626c6963206e6174696f6e616c', NULL, 1707, 1708, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (855, 1, 'T1-771', '\\x6f7267616e69736d652061646d696e6973747261746966206575726f70c3a9656e', NULL, 1709, 1710, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (856, 1, 'T1-1044', '\\x61646d696e697374726174696f6e2063656e7472616c65', NULL, 1711, 1712, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (857, 1, 'T1-575', '\\x7365727669636520657874c3a97269657572', NULL, 1713, 1714, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (858, 1, 'T1-1422', '\\xc3a97461626c697373656d656e74207075626c6963206c6f63616c', NULL, 1715, 1716, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (860, 1, 'T1-1040', '\\x61646d696e697374726174696f6e206c6f63616c65206427416e6369656e2052c3a967696d65', NULL, 1719, 1720, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (861, 1, 'T1-1088', '\\x736572766963652064c3a9636f6e63656e7472c3a9', NULL, 1721, 1722, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (862, 1, 'T1-621', '\\x61646d696e697374726174696f6e207072c3a9666563746f72616c65', NULL, 1723, 1724, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (863, 1, 'T1-437', '\\x6167656e7420696d6d6f62696c696572', NULL, 1725, 1726, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (864, 1, 'T1-706', '\\x6d61726368c3a920696d6d6f62696c696572', NULL, 1727, 1728, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (865, 1, 'T1-438', '\\x636f6e7365696c6c65722064276172726f6e64697373656d656e74', NULL, 1729, 1730, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (866, 1, 'T1-439', '\\x706861726d616369652076c3a974c3a972696e61697265', NULL, 1731, 1732, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (867, 1, 'T1-440', '\\x7472616e73706f727420c3a020646f73206427686f6d6d65', NULL, 1733, 1734, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (868, 1, 'T1-443', '\\x766f69652066657272c3a965206427696e74c3a972c3aa74206c6f63616c', NULL, 1735, 1736, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (1041, 1, 'T1-579', '\\x64656d692d736f6c6465', NULL, 2081, 2082, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (869, 1, 'T1-1453', '\\x646f6d6d6167657320646520677565727265', NULL, 1737, 1738, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (870, 1, 'T1-450', '\\xc3a96c656374657572', NULL, 1739, 1740, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (875, 1, 'T1-454', '\\x736569676e6575726965', NULL, 1749, 1750, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (876, 1, 'T1-455', '\\x6cc3a967696f6e20c3a97472616e67c3a87265', NULL, 1751, 1752, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (877, 1, 'T1-937', '\\x646973747269627574696f6e20c3a96c6563747269717565', NULL, 1753, 1754, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (878, 1, 'T1-458', '\\x6173736f63696174696f6e20646520646566656e7365206465206c27656e7669726f6e6e656d656e74', NULL, 1755, 1756, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (879, 1, 'T1-459', '\\x64c3a9636f757061676520c3a96c6563746f72616c', NULL, 1757, 1758, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (880, 1, 'T1-645', '\\x63616e746f6e', NULL, 1759, 1760, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (954, 1, 'T1-502', '\\x7069737465206379636c61626c65', NULL, 1907, 1908, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1042, 1, 'T1-580', '\\x6c7963c3a965', NULL, 2083, 2084, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (881, 1, 'T1-1338', '\\x6c69657574656e616e74206465206c6f7576657465726965', NULL, 1761, 1762, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (882, 1, 'T1-462', '\\x746178652065787472616f7264696e61697265206427416e6369656e2052c3a967696d65', NULL, 1763, 1764, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (883, 1, 'T1-463', '\\x6361646173747265', NULL, 1765, 1766, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (884, 1, 'T1-1143', '\\x636172746f67726170686965', NULL, 1767, 1768, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (885, 1, 'T1-464', '\\x64c3a970656e736520646520666f6e6374696f6e6e656d656e74', NULL, 1769, 1770, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (886, 1, 'T1-465', '\\x6368616d7020646520666f697265', NULL, 1771, 1772, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (887, 1, 'T1-1354', '\\x6173737572616e636520696e76616c69646974c3a9', NULL, 1773, 1774, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (888, 1, 'T1-779', '\\x6173737572616e6365206368c3b46d616765', NULL, 1775, 1776, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (1045, 1, 'T1-588', '\\x6162616e646f6e206427656e66616e74', NULL, 2089, 2090, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (890, 1, 'T1-673', '\\x6173737572616e63652076657576616765', NULL, 1779, 1780, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (891, 1, 'T1-467', '\\x656e66616e7420656e206761726465', NULL, 1781, 1782, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (892, 1, 'T1-1054', '\\x696e7465726469742064652073c3a96a6f7572', NULL, 1783, 1784, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (893, 1, 'T1-652', '\\x696e646976696475207265636865726368c3a9', NULL, 1785, 1786, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (894, 1, 'T1-1411', '\\x6172726573746174696f6e', NULL, 1787, 1788, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (896, 1, 'T1-629', '\\x6167656e7420646520706f6c696365206a756469636961697265', NULL, 1791, 1792, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (897, 1, 'T1-1162', '\\x6c6962c3a9726174696f6e20636f6e646974696f6e6e656c6c65', NULL, 1793, 1794, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (898, 1, 'T1-469', '\\x66726963686520696e647573747269656c6c65', NULL, 1795, 1796, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (1007, 1, 'T1-1022', '\\x61c3a9726f64726f6d65', NULL, 2013, 2014, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (841, 1, 'T1-422', '\\x61646d696e697374726174696f6e2072c3a967696f6e616c65', NULL, 1681, 1682, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (907, 1, 'T1-710', '\\x72c3a97075626c69717565', NULL, 1813, 1814, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (909, 1, 'T1-789', '\\x70617263206465206c6f6973697273', NULL, 1817, 1818, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (910, 1, 'T1-585', '\\x696e7374616c6c6174696f6e2073706f7274697665', NULL, 1819, 1820, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (911, 1, 'T1-826', '\\x64c3a962697420646520626f6973736f6e73', NULL, 1821, 1822, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (912, 1, 'T1-933', '\\x636173696e6f', NULL, 1823, 1824, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (913, 1, 'T1-1155', '\\x6772616e6465207375726661636520636f6d6d65726369616c65', NULL, 1825, 1826, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (914, 1, 'T1-820', '\\xc3a96469666963652063756c7475656c', NULL, 1827, 1828, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (915, 1, 'T1-479', '\\x7365727669747564652061c3a9726f6e61757469717565', NULL, 1829, 1830, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (919, 1, 'T1-1052', '\\x6d696e65757220646520666f6e64', NULL, 1837, 1838, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (920, 1, 'T1-642', '\\x6d6172696e20646520636f6d6d65726365', NULL, 1839, 1840, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (921, 1, 'T1-963', '\\x61727469737465', NULL, 1841, 1842, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (922, 1, 'T1-1307', '\\x617373697374616e7465206d617465726e656c6c65', NULL, 1843, 1844, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (924, 1, 'T1-535', '\\x70726f7370656374696f6e2070c3a974726f6c69c3a87265', NULL, 1847, 1848, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (925, 1, 'T1-1455', '\\x6578706c6f69746174696f6e206d696e69c3a87265', NULL, 1849, 1850, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (926, 1, 'T1-482', '\\x6c6f67696369656c20696e666f726d617469717565', NULL, 1851, 1852, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (927, 1, 'T1-903', '\\x73797374c3a86d65206427696e666f726d6174696f6e', NULL, 1853, 1854, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (928, 1, 'T1-486', '\\x74c3a96cc3a9677261706865', NULL, 1855, 1856, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (929, 1, 'T1-1323', '\\x636f6e63696c696174657572', NULL, 1857, 1858, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (930, 1, 'T1-994', '\\x61766f636174', NULL, 1859, 1860, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (931, 1, 'T1-882', '\\x6772656666696572', NULL, 1861, 1862, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (932, 1, 'T1-534', '\\x657870657274', NULL, 1863, 1864, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (933, 1, 'T1-488', '\\x7472616e73706f72742070617220616e696d616c', NULL, 1865, 1866, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (934, 1, 'T1-573', '\\x616e696d616c2064652062c3a274', NULL, 1867, 1868, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (1451, 1, 'T2-44', '\\x66696e616e63656d656e74', NULL, 2901, 2902, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (935, 1, 'T1-955', '\\x76c3a9686963756c6520c3a0207472616374696f6e20616e696d616c65', NULL, 1869, 1870, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (936, 1, 'T1-1284', '\\x656e66616e74206e61747572656c', NULL, 1871, 1872, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (937, 1, 'T1-934', '\\x72c3a967696d65206d617472696d6f6e69616c', NULL, 1873, 1874, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (938, 1, 'T1-490', '\\x7075626c69636974c3a9', NULL, 1875, 1876, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (939, 1, 'T1-491', '\\x636f6f70c3a97261746976652061677269636f6c65', NULL, 1877, 1878, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (940, 1, 'T1-1099', '\\x6578706c6f69746174696f6e2061677269636f6c65', NULL, 1879, 1880, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (941, 1, 'T1-817', '\\x736f6369c3a974c3a920636f6f70c3a9726174697665', NULL, 1881, 1882, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (942, 1, 'T1-1348', '\\x64c3a963c3a873', NULL, 1883, 1884, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (943, 1, 'T1-494', '\\xc3a97075726174696f6e', NULL, 1885, 1886, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (944, 1, 'T1-497', '\\x696e6475737472696520647520626f6973', NULL, 1887, 1888, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (945, 1, 'T1-498', '\\x706172656e74206427c3a96cc3a87665', NULL, 1889, 1890, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (946, 1, 'T1-954', '\\x656e736569676e656d656e74207075626c6963', NULL, 1891, 1892, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (947, 1, 'T1-1306', '\\x656e736569676e656d656e74207365636f6e6461697265', NULL, 1893, 1894, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (948, 1, 'T1-1077', '\\x6578616d656e', NULL, 1895, 1896, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1109, 1, 'T1-655', '\\x70726f64756974206c616974696572', NULL, 2217, 2218, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (952, 1, 'T1-574', '\\x656e736569676e656d656e7420c3a02064697374616e6365', NULL, 1903, 1904, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (953, 1, 'T1-501', '\\xc3a9706964c3a96d6965', NULL, 1905, 1906, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (955, 1, 'T1-1257', '\\x7665686963756c6520c3a0206465757820726f756573', NULL, 1909, 1910, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (956, 1, 'T1-504', '\\x6173736f63696174696f6e206465206d616c6661697465757273', NULL, 1911, 1912, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (957, 1, 'T1-505', '\\x64c3a96cc3a96775c3a920636f6e73756c61697265', NULL, 1913, 1914, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (958, 1, 'T1-507', '\\x636f6e7365696c206465206469736369706c696e65', NULL, 1915, 1916, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (960, 1, 'T1-1216', '\\x706c616365207075626c69717565', NULL, 1919, 1920, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (961, 1, 'T1-960', '\\x68616c6c65', NULL, 1921, 1922, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1216, 1, 'T1-918', '\\x706170617574c3a9', NULL, 2431, 2432, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1217, 1, 'T1-1291', '\\x6d697373696f6e73', NULL, 2433, 2434, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (962, 1, 'T1-884', '\\x7075697473', NULL, 1923, 1924, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (963, 1, 'T1-1320', '\\x6c61766f6972', NULL, 1925, 1926, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (964, 1, 'T1-570', '\\x686f726c6f6765207075626c69717565', NULL, 1927, 1928, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (965, 1, 'T1-835', '\\x666f6e7461696e65', NULL, 1929, 1930, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (966, 1, 'T1-921', '\\x636f6e737472756374696f6e2073636f6c61697265', NULL, 1931, 1932, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (967, 1, 'T1-509', '\\x666f726d6174696f6e207175616c696669616e7465', NULL, 1933, 1934, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (968, 1, 'T1-510', '\\x6368617566666167652075726261696e', NULL, 1935, 1936, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (969, 1, 'T1-512', '\\xc3a963686576696e', NULL, 1937, 1938, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1229, 1, 'T1-1204', '\\x656e74656e7465', NULL, 2457, 2458, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1033, 1, 'T1-1377', '\\x61707072656e7469', NULL, 2065, 2066, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (902, 1, 'T1-1172', '\\x6167656e74206465206c6120666f726365207075626c69717565', NULL, 1803, 1804, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (903, 1, 'T1-522', '\\x706f6c696365206465732065617578', NULL, 1805, 1806, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (904, 1, 'T1-472', '\\x72656d656d6272656d656e7420727572616c', NULL, 1807, 1808, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (905, 1, 'T1-473', '\\x706572736f6e6e616c6974c3a9', NULL, 1809, 1810, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (906, 1, 'T1-474', '\\x70726f6a657420c3a964756361746966', NULL, 1811, 1812, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (971, 1, 'T1-515', '\\x636f7374756d65206f6666696369656c', NULL, 1941, 1942, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (972, 1, 'T1-516', '\\x6167656e74206465206368616e6765', NULL, 1943, 1944, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (973, 1, 'T1-519', '\\x6761726465206e6174696f6e616c65', NULL, 1945, 1946, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (974, 1, 'T1-520', '\\x63756d756c206427656d706c6f69', NULL, 1947, 1948, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (975, 1, 'T1-809', '\\x696e6672616374696f6e20c3a9636f6e6f6d69717565', NULL, 1949, 1950, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (976, 1, 'T1-521', '\\x636f6d6d6572c3a7616e7420c3a97472616e676572', NULL, 1951, 1952, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (977, 1, 'T1-1038', '\\x63686566206427656e7472657072697365', NULL, 1953, 1954, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (978, 1, 'T1-523', '\\x6861626974617420696e73616c75627265', NULL, 1955, 1956, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (980, 1, 'T1-1014', '\\x76696374696d65206427696e6672616374696f6e', NULL, 1959, 1960, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (981, 1, 'T1-1458', '\\x64c3a96c696e7175616e6365', NULL, 1961, 1962, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1067, 1, 'T1-897', '\\x6172747320706c6173746971756573', NULL, 2133, 2134, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (982, 1, 'T1-1084', '\\x7175616c696669636174696f6e206372696d696e656c6c65', NULL, 1963, 1964, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1310, 1, 'T1-1049', '\\x7461626163', NULL, 2619, 2620, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (984, 1, 'T1-1116', '\\x696e64757374726965207370617469616c65', NULL, 1967, 1968, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (985, 1, 'T1-873', '\\x696e64757374726965206475207665727265', NULL, 1969, 1970, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (986, 1, 'T1-1267', '\\x696e6475737472696520647520666575', NULL, 1971, 1972, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (987, 1, 'T1-913', '\\x6dc3a974616c6c7572676965', NULL, 1973, 1974, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (988, 1, 'T1-528', '\\x63656e73757265', NULL, 1975, 1976, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (989, 1, 'T1-529', '\\x696e737469747574696f6e206d6f6e61737469717565', NULL, 1977, 1978, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (990, 1, 'T1-530', '\\x6d797374696369736d65', NULL, 1979, 1980, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (991, 1, 'T1-531', '\\x706973636963756c74757265', NULL, 1981, 1982, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:09');
INSERT INTO keywords VALUES (1406, 1, 'T2-65', '\\x726563727574656d656e74', NULL, 2811, 2812, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (916, 1, 'T1-1008', '\\x73657276697475646520726164696f20c3a96c6563747269717565', NULL, 1831, 1832, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (918, 1, 'T1-907', '\\x7472617661696c6c65757220c3a020646f6d6963696c65', NULL, 1835, 1836, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (1182, 1, 'T1-753', '\\x6a756d656c616765', NULL, 2363, 2364, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1002, 1, 'T1-1171', '\\x636f6c6c65637465207075626c69717565', NULL, 2003, 2004, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1003, 1, 'T1-547', '\\x646973747269627574696f6e20706f7374616c65', NULL, 2005, 2006, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1004, 1, 'T1-883', '\\x7472616e73706f727420726f7574696572', NULL, 2007, 2008, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1005, 1, 'T1-617', '\\x6368617566666575722064652074617869', NULL, 2009, 2010, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1009, 1, 'T1-1427', '\\x63697263756c6174696f6e2075726261696e65', NULL, 2017, 2018, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1010, 1, 'T1-1428', '\\x67617265', NULL, 2019, 2020, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1011, 1, 'T1-697', '\\x6167656e636520646520766f7961676573', NULL, 2021, 2022, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1012, 1, 'T1-550', '\\x656e747265707269736520696e647573747269656c6c65', NULL, 2023, 2024, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1013, 1, 'T1-977', '\\x61726dc3a965206465207465727265', NULL, 2025, 2026, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1014, 1, 'T1-1352', '\\x61726dc3a965206465206c27616972', NULL, 2027, 2028, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1015, 1, 'T1-552', '\\x73706972697475616c6974c3a9', NULL, 2029, 2030, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1016, 1, 'T1-553', '\\x7365726d656e7420646520666964c3a96c6974c3a9', NULL, 2031, 2032, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1017, 1, 'T1-1446', '\\x76696c6c65206e6f7576656c6c65', NULL, 2033, 2034, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1018, 1, 'T1-961', '\\x7a6f6e6520642761637469766974c3a973', NULL, 2035, 2036, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1698, 1, 'T3-154', '\\x7065726d6973', NULL, 3395, 3396, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1119, 1, 'T1-1361', '\\x657870726f7072696174696f6e', NULL, 2237, 2238, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1040, 1, 'T1-571', '\\x6d696c6963657320626f757267656f69736573', NULL, 2079, 2080, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1043, 1, 'T1-582', '\\x616dc3a96c696f726174696f6e2064657320736f6c73', NULL, 2085, 2086, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1044, 1, 'T1-587', '\\x636f696666657572', NULL, 2087, 2088, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1046, 1, 'T1-589', '\\x7472616974656d656e74206465732065617578207573c3a96573', NULL, 2091, 2092, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1047, 1, 'T1-590', '\\x63c3a972c3a9616c65', NULL, 2093, 2094, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1048, 1, 'T1-592', '\\x726164696f646966667573696f6e', NULL, 2095, 2096, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1049, 1, 'T1-593', '\\x747261666963206465206d61696e2064276f6575767265', NULL, 2097, 2098, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1050, 1, 'T1-596', '\\xc3a97461626c697373656d656e74206172746973616e616c', NULL, 2099, 2100, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1051, 1, 'T1-598', '\\x70726f737469747574696f6e', NULL, 2101, 2102, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1052, 1, 'T1-599', '\\x72c3a97365727665206e61747572656c6c65', NULL, 2103, 2104, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1832, 1, 'T3-235', '\\x64c3a9636973696f6e', NULL, 3663, 3664, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1489, 1, 'T4-95', '\\x636f6e66c3a972656e63652064652059616c746120283139343529', NULL, 2977, 2978, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1019, 1, 'T1-1192', '\\x706f6c697469717565206465206c612076696c6c65', NULL, 2037, 2038, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1020, 1, 'T1-1431', '\\x7a6f6e6520696e647573747269656c6c65', NULL, 2039, 2040, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1021, 1, 'T1-555', '\\x617373656d626cc3a9652067c3a96ec3a972616c65', NULL, 2041, 2042, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1022, 1, 'T1-1133', '\\x686162697461742075726261696e', NULL, 2043, 2044, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1023, 1, 'T1-1045', '\\x73796e64696320646520636f70726f707269c3a974c3a9', NULL, 2045, 2046, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1024, 1, 'T1-671', '\\x64c3a96d6f6c6974696f6e', NULL, 2047, 2048, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1025, 1, 'T1-1001', '\\x6c6f67656d656e7420736f6369616c', NULL, 2049, 2050, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1026, 1, 'T1-1448', '\\x6c6f67656d656e7420646520666f6e6374696f6e', NULL, 2051, 2052, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1392, 1, 'T1-1308', '\\x73696461', NULL, 2783, 2784, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1185, 1, 'T1-757', '\\x64726170656175', NULL, 2369, 2370, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (899, 1, 'T1-470', '\\x6d616e69666573746174696f6e20636f6d6d65726369616c65', NULL, 1797, 1798, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (900, 1, 'T1-735', '\\x706f6c696365206465206c612070c3aa636865', NULL, 1799, 1800, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (901, 1, 'T1-863', '\\x706f6c696365206465732063756c746573', NULL, 1801, 1802, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (1054, 1, 'T1-1064', '\\x6172726f6e64697373656d656e74', NULL, 2107, 2108, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1055, 1, 'T1-1328', '\\x64697374726963742072c3a9766f6c7574696f6e6e61697265', NULL, 2109, 2110, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1056, 1, 'T1-1043', '\\x646f6d61696e6520726f79616c', NULL, 2111, 2112, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1057, 1, 'T1-1035', '\\x6167676c6f6dc3a9726174696f6e2075726261696e65', NULL, 2113, 2114, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1058, 1, 'T1-812', '\\x73c3a96ec3a9636861757373c3a965', NULL, 2115, 2116, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1059, 1, 'T1-1372', '\\x70617973', NULL, 2117, 2118, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1060, 1, 'T1-638', '\\x67c3a96ec3a972616c6974c3a9', NULL, 2119, 2120, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1062, 1, 'T1-999', '\\x70726f76696e6365', NULL, 2123, 2124, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1063, 1, 'T1-602', '\\x616273656e74c3a969736d652073636f6c61697265', NULL, 2125, 2126, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1064, 1, 'T1-604', '\\x666c657572', NULL, 2127, 2128, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1065, 1, 'T1-607', '\\x636f757273206427656175', NULL, 2129, 2130, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1066, 1, 'T1-731', '\\x6d757369717565', NULL, 2131, 2132, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1068, 1, 'T1-609', '\\x656e736569676e656d656e742070c3a96e6974656e746961697265', NULL, 2135, 2136, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1069, 1, 'T1-610', '\\x70c3a96461676f676965', NULL, 2137, 2138, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1070, 1, 'T1-787', '\\x6269626c696f7468c3a87175652073636f6c61697265', NULL, 2139, 2140, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1071, 1, 'T1-1203', '\\x6d6174c3a97269656c2070c3a96461676f6769717565', NULL, 2141, 2142, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1073, 1, 'T1-839', '\\x61666661697265207072756427686f6d616c65', NULL, 2145, 2146, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1074, 1, 'T1-612', '\\x6f7267616e69736174696f6e20696e7465726e6174696f6e616c65', NULL, 2147, 2148, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1075, 1, 'T1-613', '\\x67616cc3a8726573', NULL, 2149, 2150, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1076, 1, 'T1-810', '\\x6172746973616e', NULL, 2151, 2152, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1077, 1, 'T1-758', '\\x7472617661757820666f7263c3a973', NULL, 2153, 2154, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1078, 1, 'T1-746', '\\x7065696e6520646520737562737469747574696f6e', NULL, 2155, 2156, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1079, 1, 'T1-1009', '\\x7065696e65206361706974616c65', NULL, 2157, 2158, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1080, 1, 'T1-1447', '\\x7065696e6520636f72706f72656c6c65', NULL, 2159, 2160, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1081, 1, 'T1-1432', '\\x61737369676e6174696f6e20c3a02072c3a9736964656e6365', NULL, 2161, 2162, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1082, 1, 'T1-619', '\\x6372696d6520646520677565727265', NULL, 2163, 2164, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1083, 1, 'T1-620', '\\x62c3a97461696c', NULL, 2165, 2166, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1085, 1, 'T1-815', '\\x696d6d6967726174696f6e', NULL, 2169, 2170, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1086, 1, 'T1-1220', '\\x70726f64756974206d617261c3ae63686572', NULL, 2171, 2172, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1087, 1, 'T1-1409', '\\x6a617264696e2066616d696c69616c', NULL, 2173, 2174, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1088, 1, 'T1-1068', '\\x677569646520696e7465727072c3a87465', NULL, 2175, 2176, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1089, 1, 'T1-1290', '\\x63616d70696e67206361726176616e696e67', NULL, 2177, 2178, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1090, 1, 'T1-1144', '\\x746f757269736d6520727572616c', NULL, 2179, 2180, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1550, 1, 'T4-89', '\\x42656c6c6520c389706f717565', NULL, 3099, 3100, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1091, 1, 'T1-827', '\\x746f757269736d652075726261696e', NULL, 2181, 2182, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1092, 1, 'T1-1102', '\\x6369726375697420746f757269737469717565', NULL, 2183, 2184, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1093, 1, 'T1-1378', '\\x6f7267616e69736d65206c6f63616c20646520746f757269736d65', NULL, 2185, 2186, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1094, 1, 'T1-1272', '\\x6379636c6f746f757269736d65', NULL, 2187, 2188, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1095, 1, 'T1-635', '\\x7472616e73706f72742073616e697461697265', NULL, 2189, 2190, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1096, 1, 'T1-636', '\\x69736c616d', NULL, 2191, 2192, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1097, 1, 'T1-850', '\\x68616e646973706f7274', NULL, 2193, 2194, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1098, 1, 'T1-1368', '\\x6173736f63696174696f6e2073706f7274697665', NULL, 2195, 2196, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1099, 1, 'T1-798', '\\x656475636174696f6e2073706f7274697665', NULL, 2197, 2198, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1100, 1, 'T1-640', '\\x746178652070726f66657373696f6e6e656c6c65', NULL, 2199, 2200, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1161, 1, 'T1-726', '\\x6d6f6e6e616965', NULL, 2321, 2322, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1592, 1, 'T4-15', '\\x3765207369c3a8636c65', NULL, 3183, 3184, 3, 1, 1, '2011-05-25 10:09:31', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1130, 1, 'T1-680', '\\x64c3a9706f7274c3a9', NULL, 2259, 2260, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (917, 1, 'T1-517', '\\x636f6e636965726765', NULL, 1833, 1834, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (1129, 1, 'T1-674', '\\xc3a97461626c697373656d656e742070c3a96e6974656e746961697265', NULL, 2257, 2258, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1038, 1, 'T1-1199', '\\x726573746175726174696f6e2073636f6c61697265', NULL, 2075, 2076, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1039, 1, 'T1-569', '\\x6a75737469636520726f79616c65', NULL, 2077, 2078, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1053, 1, 'T1-600', '\\x72c3a9736964656e636520646520746f757269736d65', NULL, 2105, 2106, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1311, 1, 'T1-978', '\\x6163636f756368656d656e74', NULL, 2621, 2622, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1394, 1, 'T1-1418', '\\x7375637265', NULL, 2787, 2788, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1102, 1, 'T1-644', '\\x626f75636865726965', NULL, 2203, 2204, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1103, 1, 'T1-1410', '\\x7669616e6465', NULL, 2205, 2206, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1104, 1, 'T1-646', '\\x61696465206dc3a9646963616c6520757267656e7465', NULL, 2207, 2208, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1105, 1, 'T1-664', '\\x61726368c3a96f6c6f67696520737562617175617469717565', NULL, 2209, 2210, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1106, 1, 'T1-1017', '\\x61726368c3a96f6c6f67696520736f75732d6d6172696e65', NULL, 2211, 2212, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1107, 1, 'T1-650', '\\x64c3a96cc3a96775c3a920647520706572736f6e6e656c', NULL, 2213, 2214, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1108, 1, 'T1-1067', '\\x706f6973736f6e', NULL, 2215, 2216, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1112, 1, 'T1-660', '\\x68796d6e65206e6174696f6e616c', NULL, 2223, 2224, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1113, 1, 'T1-754', '\\x70726f6d656e61646520706c616e74c3a965', NULL, 2225, 2226, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1116, 1, 'T1-715', '\\x636f6c6cc3a86765', NULL, 2231, 2232, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1117, 1, 'T1-1287', '\\xc3a9636f6c652063656e7472616c65', NULL, 2233, 2234, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1118, 1, 'T1-665', '\\x666f75726e69747572652073636f6c61697265', NULL, 2235, 2236, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1120, 1, 'T1-844', '\\x6261757820636f6d6d65726369617578', NULL, 2239, 2240, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1121, 1, 'T1-1235', '\\x6261757820727572617578', NULL, 2241, 2242, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1123, 1, 'T1-668', '\\x706f6c696365206d756e69636970616c65', NULL, 2245, 2246, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1124, 1, 'T1-1136', '\\x696d70c3b47420737572206c657320736f6369c3a974c3a973', NULL, 2247, 2248, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1125, 1, 'T1-1031', '\\x7461786520642761707072656e74697373616765', NULL, 2249, 2250, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1153, 1, 'T1-932', '\\x66726175646520636f6d6d65726369616c65', NULL, 2305, 2306, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1154, 1, 'T1-712', '\\x68c3b474656c2064652076696c6c65', NULL, 2307, 2308, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1155, 1, 'T1-716', '\\x6368616d616e69736d65', NULL, 2309, 2310, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1156, 1, 'T1-717', '\\x747574656c6c65206175782070726573746174696f6e7320736f6369616c6573', NULL, 2311, 2312, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1157, 1, 'T1-1176', '\\x747574656c6c65206a756469636961697265', NULL, 2313, 2314, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1158, 1, 'T1-1313', '\\x64c3a96e6f6d6272656d656e7420736569676e65757269616c', NULL, 2315, 2316, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1159, 1, 'T1-719', '\\x696e76616c696465206475207472617661696c', NULL, 2317, 2318, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1160, 1, 'T1-725', '\\x6269656e2073706f6c69c3a9', NULL, 2319, 2320, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1162, 1, 'T1-729', '\\xc3a96372697420737562766572736966', NULL, 2323, 2324, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1470, 1, 'T4-1', '\\x5072c3a9686973746f697265', NULL, 2939, 2940, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (38, 1, 'T1-7', '\\x636f75706520646520626f6973', NULL, 75, 76, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (1027, 1, 'T1-557', '\\x626967616d6965', NULL, 2053, 2054, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1028, 1, 'T1-709', '\\x6d6f7576656d656e74206465206a65756e65737365', NULL, 2055, 2056, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1031, 1, 'T1-699', '\\x6368616e74696572206465206c61206a65756e65737365', NULL, 2061, 2062, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1032, 1, 'T1-833', '\\xc3a974756469616e74', NULL, 2063, 2064, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1034, 1, 'T1-1244', '\\x696e7465726e65', NULL, 2067, 2068, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1036, 1, 'T1-563', '\\x707269736f6e6e69657220646520677565727265', NULL, 2071, 2072, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1037, 1, 'T1-564', '\\x66616d696e65', NULL, 2073, 2074, 1, 1, 1, '2011-05-25 10:03:09', '2012-03-07 09:41:10');
INSERT INTO keywords VALUES (1218, 1, 'T1-1183', '\\x636f6e63696c65', NULL, 2435, 2436, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1132, 1, 'T1-682', '\\x6163636964656e74206475207472617661696c', NULL, 2263, 2264, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1133, 1, 'T1-685', '\\x696e74657276656e74696f6e206427c3a96c75', NULL, 2265, 2266, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1134, 1, 'T1-688', '\\x616c6c6575', NULL, 2267, 2268, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1135, 1, 'T1-689', '\\x636f6d7061676e69652063686f72c3a9677261706869717565', NULL, 2269, 2270, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1136, 1, 'T1-691', '\\x656e7472657072697365207075626c69717565', NULL, 2271, 2272, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1137, 1, 'T1-692', '\\x617070617265696c20726164696f20c3a96c6563747269717565', NULL, 2273, 2274, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1138, 1, 'T1-945', '\\x7365727669636520726164696f20c3a96c6563747269717565', NULL, 2275, 2276, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1139, 1, 'T1-694', '\\x676172646520666f72657374696572', NULL, 2277, 2278, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1605, 1, 'T3-56', '\\x6368616e736f6e', NULL, 3209, 3210, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1163, 1, 'T1-730', '\\x696e7374616c6c6174696f6e20726164696f6c6f6769717565', NULL, 2325, 2326, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1164, 1, 'T1-732', '\\x6d616c616465206d656e74616c', NULL, 2327, 2328, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1165, 1, 'T1-733', '\\x64c3a9746563746976652070726976c3a9', NULL, 2329, 2330, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1166, 1, 'T1-734', '\\x726f75746520666f7265737469c3a87265', NULL, 2331, 2332, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1167, 1, 'T1-985', '\\x766f69652066657272c3a965206427696e74c3a972c3aa74206e6174696f6e616c', NULL, 2333, 2334, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1168, 1, 'T1-737', '\\x706f70756c6174696f6e2070c3a96e6974656e746961697265', NULL, 2335, 2336, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1169, 1, 'T1-855', '\\x696e63617263c3a9726174696f6e', NULL, 2337, 2338, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1170, 1, 'T1-1275', '\\xc3a9766173696f6e', NULL, 2339, 2340, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1101, 1, 'T1-1112', '\\x63686172626f6e20646520626f6973', NULL, 2201, 2202, 1, 1, 1, '2011-05-25 10:03:10', '2012-03-07 09:41:11');
INSERT INTO keywords VALUES (1223, 1, 'T1-802', '\\x736572766974756465', NULL, 2445, 2446, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1224, 1, 'T1-864', '\\x74656d706f72656c206563636cc3a9736961737469717565', NULL, 2447, 2448, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1225, 1, 'T1-1369', '\\x68c3a972c3a9736965', NULL, 2449, 2450, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1226, 1, 'T1-1071', '\\x636f6e636f7572732061646d696e6973747261746966', NULL, 2451, 2452, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1227, 1, 'T1-807', '\\x7461786520666f6e6369c3a87265', NULL, 2453, 2454, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1228, 1, 'T1-808', '\\x696e64c3a970656e64616e6365', NULL, 2455, 2456, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1230, 1, 'T1-814', '\\x6a65752d636f6e636f757273', NULL, 2459, 2460, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1145, 1, 'T1-929', '\\x72657472616974c3a9', NULL, 2289, 2290, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1146, 1, 'T1-713', '\\x706f70756c6174696f6e20616374697665', NULL, 2291, 2292, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1147, 1, 'T1-783', '\\x6174656c69657220646520636861726974c3a9', NULL, 2293, 2294, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1148, 1, 'T1-700', '\\x747562657263756c6f7365', NULL, 2295, 2296, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1150, 1, 'T1-703', '\\x636f75727365206869707069717565', NULL, 2299, 2300, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1151, 1, 'T1-705', '\\x6c696169736f6e20726f757469c3a8726520696e7465726e6174696f6e616c65', NULL, 2301, 2302, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1152, 1, 'T1-711', '\\x7075626c69636974c3a9206d656e736f6e67c3a87265', NULL, 2303, 2304, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1459, 1, 'T2-3', '\\x6163717569736974696f6e', NULL, 2917, 2918, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1239, 1, 'T1-935', '\\x72657374617572616e7420646520746f757269736d65', NULL, 2477, 2478, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1325, 1, 'T1-1011', '\\x6f657566', NULL, 2649, 2650, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1577, 1, 'T4-18', '\\x313065207369c3a8636c65', NULL, 3153, 3154, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1240, 1, 'T1-1347', '\\xc3a97461626c697373656d656e7420696e74657264697420617578206d696e65757273', NULL, 2479, 2480, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1241, 1, 'T1-1270', '\\x616c636f6f6c69736d65', NULL, 2481, 2482, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1242, 1, 'T1-828', '\\x617070656c', NULL, 2483, 2484, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1176, 1, 'T1-993', '\\x6465747465207075626c69717565', NULL, 2351, 2352, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1177, 1, 'T1-1245', '\\x726563657474652066697363616c65', NULL, 2353, 2354, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1179, 1, 'T1-749', '\\x676172646520706172746963756c696572', NULL, 2357, 2358, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1180, 1, 'T1-1036', '\\x616dc3a96e6167656d656e7420666f72657374696572', NULL, 2359, 2360, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1181, 1, 'T1-751', '\\x6f75747261676520617578206d6f65757273', NULL, 2361, 2362, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1183, 1, 'T1-856', '\\x63616e616c69736174696f6e', NULL, 2365, 2366, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1184, 1, 'T1-1295', '\\x646973747269627574696f6e2064652067617a', NULL, 2367, 2368, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1186, 1, 'T1-759', '\\x66617578206d6f6e6e6179616765', NULL, 2371, 2372, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1187, 1, 'T1-997', '\\x6173736f63696174696f6e207265636f6e6e75652064277574696c6974c3a9207075626c69717565', NULL, 2373, 2374, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1192, 1, 'T1-1454', '\\x6d6967726174696f6e20727572616c65', NULL, 2383, 2384, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1194, 1, 'T1-795', '\\x6a7567652064652070616978', NULL, 2387, 2388, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1196, 1, 'T1-768', '\\x74c3a96c6578', NULL, 2391, 2392, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1213, 1, 'T1-811', '\\x6661627269717565206427c3a9676c697365', NULL, 2425, 2426, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1214, 1, 'T1-1053', '\\x61646d696e697374726174696f6e2064696f63c3a97361696e65', NULL, 2427, 2428, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1215, 1, 'T1-967', '\\x73c3a96d696e616972652072656c696769657578', NULL, 2429, 2430, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1220, 1, 'T1-797', '\\x6d616c6164696520c3a02064c3a9636c61726174696f6e206f626c696761746f697265', NULL, 2439, 2440, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1221, 1, 'T1-1363', '\\x6167656e742064276173737572616e636573', NULL, 2441, 2442, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1222, 1, 'T1-949', '\\x736f6369c3a974c3a9206465207365727669636573', NULL, 2443, 2444, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1246, 1, 'T1-831', '\\x72657072c3a973656e746174696f6e206469706c6f6d617469717565', NULL, 2491, 2492, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1247, 1, 'T1-836', '\\x736f7263656c6c65726965', NULL, 2493, 2494, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1248, 1, 'T1-1364', '\\x6d61676965', NULL, 2495, 2496, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1249, 1, 'T1-840', '\\x636f6d6d6973736169726520656e7175c3aa74657572', NULL, 2497, 2498, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1250, 1, 'T1-843', '\\x636f6e7365696c2064652066616d696c6c65', NULL, 2499, 2500, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1794, 1, 'T3-40', '\\x63617269636174757265', NULL, 3587, 3588, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (36, 1, 'T1-376', '\\x66696e616e636573206c6f63616c6573', NULL, 71, 72, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (1251, 1, 'T1-846', '\\x636f6d6d6572c3a7616e74', NULL, 2501, 2502, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1252, 1, 'T1-1208', '\\x636f6d6974c3a9206427656e7472657072697365', NULL, 2503, 2504, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1253, 1, 'T1-987', '\\x696e73657274696f6e20736f6369616c65', NULL, 2505, 2506, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1254, 1, 'T1-925', '\\x6d6174c3a97269656c2061677269636f6c65', NULL, 2507, 2508, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1255, 1, 'T1-1217', '\\x6f727068656c696e6174', NULL, 2509, 2510, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1256, 1, 'T1-1281', '\\x61646f7074696f6e', NULL, 2511, 2512, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1257, 1, 'T1-858', '\\x7a6f6e65206427616dc3a96e6167656d656e74', NULL, 2513, 2514, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1258, 1, 'T1-859', '\\x6e617669676174696f6e2061c3a97269656e6e65', NULL, 2515, 2516, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1259, 1, 'T1-1240', '\\x6e61747572616c69736174696f6e', NULL, 2517, 2518, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1260, 1, 'T1-1140', '\\x7472617661696c206465206e756974', NULL, 2519, 2520, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1261, 1, 'T1-898', '\\x686f7261697265206465207472617661696c', NULL, 2521, 2522, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1262, 1, 'T1-1100', '\\x656d706c6f692064657320656e66616e7473', NULL, 2523, 2524, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1263, 1, 'T1-1194', '\\x64656d616e64657572206427656d706c6f69', NULL, 2525, 2526, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1264, 1, 'T1-865', '\\x7265736964656e636520756e6976657273697461697265', NULL, 2527, 2528, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1265, 1, 'T1-867', '\\x7265636f6e647569746520c3a0206c612066726f6e7469c3a87265', NULL, 2529, 2530, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1267, 1, 'T1-1382', '\\x636f6e7365696c6c65722072c3a967696f6e616c', NULL, 2533, 2534, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1271, 1, 'T1-1408', '\\x636f6e736372697074696f6e', NULL, 2541, 2542, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1272, 1, 'T1-1039', '\\x656e72c3b46c656d656e74', NULL, 2543, 2544, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1357, 1, 'T1-1124', '\\x62617074c3aa6d6520636976696c', NULL, 2713, 2714, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1273, 1, 'T1-875', '\\x67726f7570656d656e742064652070726f6475637465757273', NULL, 2545, 2546, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1274, 1, 'T1-876', '\\x6173736f63696174696f6e2064652070c3aa636865', NULL, 2547, 2548, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1275, 1, 'T1-877', '\\x61747465696e746520c3a0206c61206469676e6974c3a92064657320706572736f6e6e6573', NULL, 2549, 2550, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1238, 1, 'T1-1351', '\\x62726f63616e74657572', NULL, 2475, 2476, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:13');
INSERT INTO keywords VALUES (1280, 1, 'T1-880', '\\x6e6f6e2d6c696575', NULL, 2559, 2560, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1281, 1, 'T1-1003', '\\x706f696473206c6f757264', NULL, 2561, 2562, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1282, 1, 'T1-938', '\\x6372c3a96174696f6e206427656e7472657072697365', NULL, 2563, 2564, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1283, 1, 'T1-890', '\\x636f6e6368796c6963756c74757265', NULL, 2565, 2566, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1284, 1, 'T1-1153', '\\x6d6f7576656d656e7420706f70756c61697265', NULL, 2567, 2568, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1285, 1, 'T1-895', '\\x6163637565696c2070c3a9726973636f6c61697265', NULL, 2569, 2570, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1286, 1, 'T1-1426', '\\x6d6174c3a97269656c20696e666f726d617469717565', NULL, 2571, 2572, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1287, 1, 'T1-904', '\\x7365727669636520c3a964756361746966', NULL, 2573, 2574, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1243, 1, 'T1-1387', '\\x74696d6272652066697363616c', NULL, 2485, 2486, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1245, 1, 'T1-1123', '\\x636f6e747269627561626c65', NULL, 2489, 2490, 1, 1, 1, '2011-05-25 10:03:12', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1276, 1, 'T1-969', '\\x766575766520646520677565727265', NULL, 2551, 2552, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1277, 1, 'T1-1424', '\\x676974616e', NULL, 2553, 2554, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1278, 1, 'T1-1104', '\\x696e76616c69646520646520677565727265', NULL, 2555, 2556, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1279, 1, 'T1-1436', '\\x70726973652064276f74616765', NULL, 2557, 2558, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1304, 1, 'T1-958', '\\x61707072656e74697373616765', NULL, 2607, 2608, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1305, 1, 'T1-1384', '\\x64c3a9626974206465207461626163', NULL, 2609, 2610, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1306, 1, 'T1-965', '\\x64726f697420636f6d6d756e61757461697265', NULL, 2611, 2612, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1307, 1, 'T1-970', '\\x6a75737469636520736569676e65757269616c65', NULL, 2613, 2614, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1308, 1, 'T1-972', '\\x636f6e74726163657074696f6e', NULL, 2615, 2616, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1309, 1, 'T1-973', '\\xc3a96e6572676965206465206c61206d6572', NULL, 2617, 2618, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1312, 1, 'T1-979', '\\x73657276696365206475207472617661696c206f626c696761746f697265', NULL, 2623, 2624, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1313, 1, 'T1-980', '\\x72656c6967696f6e206f7274686f646f7865', NULL, 2625, 2626, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1314, 1, 'T1-984', '\\x746162616769736d65', NULL, 2627, 2628, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1315, 1, 'T1-989', '\\x64c3a966656e73652070617373697665', NULL, 2629, 2630, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1316, 1, 'T1-990', '\\x6c7574746520636f6e747265206c27c3a9726f73696f6e', NULL, 2631, 2632, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1317, 1, 'T1-1015', '\\x63726f79616e63657320706f70756c6169726573', NULL, 2633, 2634, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1424, 1, 'T2-10', '\\x6173736965747465', NULL, 2847, 2848, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1142, 1, 'T1-1419', '\\x7472617661696c6c6575722066726f6e74616c696572', NULL, 2283, 2284, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1143, 1, 'T1-1344', '\\x7472617661696c20636c616e64657374696e', NULL, 2285, 2286, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1144, 1, 'T1-1146', '\\x6361727269c3a872652070726f66657373696f6e6e656c6c65', NULL, 2287, 2288, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1332, 1, 'T1-1098', '\\x636f72706f726174696f6e', NULL, 2663, 2664, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1333, 1, 'T1-1037', '\\x636c617373656d656e742073616e73207375697465', NULL, 2665, 2666, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1334, 1, 'T1-1041', '\\x656e736569676e656d656e7420746563686e69717565', NULL, 2667, 2668, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1335, 1, 'T1-1048', '\\x61747465696e746520c3a02070617069657273207075626c696373', NULL, 2669, 2670, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1336, 1, 'T1-1076', '\\x7369676e616c69736174696f6e20726f757469c3a87265', NULL, 2671, 2672, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1337, 1, 'T1-1059', '\\x68c3b474656c2064652072c3a967696f6e', NULL, 2673, 2674, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1338, 1, 'T1-1304', '\\x6a657520646520636172746573', NULL, 2675, 2676, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1339, 1, 'T1-1309', '\\x617070617265696c206465206a6575', NULL, 2677, 2678, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1340, 1, 'T1-1271', '\\x706f70756c6174696f6e2075726261696e65', NULL, 2679, 2680, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1341, 1, 'T1-1314', '\\x64c3a96d6f67726170686965', NULL, 2681, 2682, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1342, 1, 'T1-1191', '\\x72617061747269c3a9', NULL, 2683, 2684, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1173, 1, 'T1-741', '\\x666f72c3aa742070726976c3a965', NULL, 2345, 2346, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1174, 1, 'T1-742', '\\x6169646520736f6369616c65206cc3a967616c65', NULL, 2347, 2348, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1175, 1, 'T1-1075', '\\x7472616e7368756d616e6365', NULL, 2349, 2350, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1318, 1, 'T1-992', '\\x6465677261646174696f6e206465206269656e73', NULL, 2635, 2636, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1319, 1, 'T1-1353', '\\x70737963686f70c3a96461676f676965', NULL, 2637, 2638, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1320, 1, 'T1-1033', '\\x73617576657461676520656e206d6572', NULL, 2639, 2640, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1321, 1, 'T1-1430', '\\x73617065757220706f6d70696572', NULL, 2641, 2642, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1322, 1, 'T1-1065', '\\x68616269746174696f6e20c3a0206c6f796572206d6f64c3a972c3a9', NULL, 2643, 2644, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (160, 1, 'T1-420', '\\x6d656e6469636974c3a9', NULL, 319, 320, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (1291, 1, 'T1-1397', '\\x626173652061c3a97269656e6e65', NULL, 2581, 2582, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1293, 1, 'T1-924', '\\x736169736965', NULL, 2585, 2586, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1294, 1, 'T1-927', '\\x6f7267616e69736d6520646520666f726d6174696f6e', NULL, 2587, 2588, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1295, 1, 'T1-930', '\\x696d70c3b47420736f6369616c', NULL, 2589, 2590, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:14');
INSERT INTO keywords VALUES (1296, 1, 'T1-1024', '\\x636f6e63656e74726174696f6e', NULL, 2591, 2592, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1297, 1, 'T1-1398', '\\x656e736569676e656d656e74206dc3a96e61676572', NULL, 2593, 2594, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1298, 1, 'T1-1260', '\\x7472617661696c206d616e75656c', NULL, 2595, 2596, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1299, 1, 'T1-1429', '\\x72c3a9666f726d6174696f6e', NULL, 2597, 2598, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1344, 1, 'T1-1080', '\\x6d657572747265', NULL, 2687, 2688, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1345, 1, 'T1-1085', '\\x76616363696e6174696f6e', NULL, 2689, 2690, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1346, 1, 'T1-1087', '\\x74c3a96cc3a97068c3a97269717565', NULL, 2691, 2692, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1350, 1, 'T1-1286', '\\x67656e7320646520677565727265', NULL, 2699, 2700, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1351, 1, 'T1-1148', '\\x7472617661696c2070c3a96e6974656e746961697265', NULL, 2701, 2702, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1352, 1, 'T1-1118', '\\x696e63617061626c65206d616a657572', NULL, 2703, 2704, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1353, 1, 'T1-1282', '\\x72656e7472c3a9652073636f6c61697265', NULL, 2705, 2706, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1378, 1, 'T1-1416', '\\x74726920706f7374616c', NULL, 2755, 2756, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1379, 1, 'T1-1187', '\\x74c3a96cc3a970686f6e65', NULL, 2757, 2758, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1386, 1, 'T1-1229', '\\x64c3a96d6f637261746965', NULL, 2771, 2772, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1387, 1, 'T1-1236', '\\x6c69676e652061c3a97269656e6e65', NULL, 2773, 2774, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1642, 1, 'T3-202', '\\x72c3a8676c656d656e74', NULL, 3283, 3284, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1388, 1, 'T1-1285', '\\x76656e7465206a756469636961697265', NULL, 2775, 2776, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1389, 1, 'T1-1255', '\\x6269656e2073696e69737472c3a9', NULL, 2777, 2778, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1390, 1, 'T1-1265', '\\x6c61626f7261746f697265206427616e616c797365', NULL, 2779, 2780, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1391, 1, 'T1-1303', '\\x62696f6469766572736974c3a9', NULL, 2781, 2782, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1393, 1, 'T1-1355', '\\xc3a96e657267696520c3a96f6c69656e6e65', NULL, 2785, 2786, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1395, 1, 'T1-1375', '\\x696d70c3b47420737572206c6520726576656e75', NULL, 2789, 2790, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1396, 1, 'T1-1392', '\\x636f75707320657420626c65737375726573', NULL, 2791, 2792, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1397, 1, 'T1-1400', '\\x6361726e6176616c', NULL, 2793, 2794, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1398, 1, 'T1-1421', '\\x61766f7274656d656e74', NULL, 2795, 2796, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1399, 1, 'T2-38', '\\x656e7472657469656e', NULL, 2797, 2798, 2, 1, 1, '2011-05-25 10:06:25', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1400, 1, 'T2-17', '\\x636f6d6d697373696f6e6e656d656e74', NULL, 2799, 2800, 2, 1, 1, '2011-05-25 10:06:25', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1401, 1, 'T2-42', '\\x6578c3a9637574696f6e2062756467c3a97461697265', NULL, 2801, 2802, 2, 1, 1, '2011-05-25 10:06:25', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1402, 1, 'T2-40', '\\xc3a9766163756174696f6e', NULL, 2803, 2804, 2, 1, 1, '2011-05-25 10:06:25', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1403, 1, 'T2-62', '\\x726563656e73656d656e74', NULL, 2805, 2806, 2, 1, 1, '2011-05-25 10:06:25', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1404, 1, 'T2-20', '\\x636f6e63696c696174696f6e', NULL, 2807, 2808, 2, 1, 1, '2011-05-25 10:06:25', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1405, 1, 'T2-53', '\\x6c6f636174696f6e', NULL, 2809, 2810, 2, 1, 1, '2011-05-25 10:06:25', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1407, 1, 'T2-51', '\\x696e666f726d617469736174696f6e', NULL, 2813, 2814, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1408, 1, 'T2-31', '\\x64c3a96cc3a9676174696f6e', NULL, 2815, 2816, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1409, 1, 'T2-59', '\\x70726f6772616d6d6174696f6e', NULL, 2817, 2818, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1410, 1, 'T2-28', '\\x636f6e7472c3b46c652073616e697461697265', NULL, 2819, 2820, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1411, 1, 'T2-43', '\\x6578706c6f69746174696f6e20636f6d6d65726369616c65', NULL, 2821, 2822, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1412, 1, 'T2-68', '\\x726573746175726174696f6e', NULL, 2823, 2824, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1414, 1, 'T2-16', '\\x636c617373656d656e74', NULL, 2827, 2828, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1415, 1, 'T2-39', '\\xc3a971756970656d656e74206d6174c3a97269656c', NULL, 2829, 2830, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1420, 1, 'T2-6', '\\x616772c3a96d656e74', NULL, 2839, 2840, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1421, 1, 'T2-19', '\\x636f6e63657373696f6e', NULL, 2841, 2842, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1422, 1, 'T2-4', '\\x616468c3a973696f6e', NULL, 2843, 2844, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1423, 1, 'T2-67', '\\x72c3a9717569736974696f6e', NULL, 2845, 2846, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1425, 1, 'T2-63', '\\x7265636f757273206869c3a9726172636869717565', NULL, 2849, 2850, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1426, 1, 'T2-26', '\\x636f6e7472c3b46c652064652073c3a96375726974c3a9', NULL, 2851, 2852, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1427, 1, 'T2-33', '\\x64c3a97369676e6174696f6e', NULL, 2853, 2854, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1430, 1, 'T2-48', '\\x67657374696f6e20647520706572736f6e6e656c', NULL, 2859, 2860, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1431, 1, 'T2-2', '\\x61627374656e74696f6e', NULL, 2861, 2862, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1432, 1, 'T2-64', '\\x7265636f757672656d656e74', NULL, 2863, 2864, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1433, 1, 'T2-9', '\\x617270656e74616765', NULL, 2865, 2866, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1434, 1, 'T2-69', '\\x746172696669636174696f6e', NULL, 2867, 2868, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1435, 1, 'T2-27', '\\x636f6e7472c3b46c652066697363616c', NULL, 2869, 2870, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1436, 1, 'T2-35', '\\x646973736f6c7574696f6e', NULL, 2871, 2872, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1437, 1, 'T2-24', '\\x636f6e7472c3b46c652064652067657374696f6e', NULL, 2873, 2874, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1419, 1, 'T2-45', '\\x666f6e6374696f6e6e656d656e74', NULL, 2837, 2838, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1446, 1, 'T2-56', '\\x706c6163656d656e74', NULL, 2891, 2892, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1447, 1, 'T2-32', '\\x64c3a96e6f6d696e6174696f6e', NULL, 2893, 2894, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1448, 1, 'T2-66', '\\x72c3a9676c656d656e746174696f6e', NULL, 2895, 2896, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1452, 1, 'T2-22', '\\x636f6e7472c3b46c65', NULL, 2903, 2904, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1453, 1, 'T2-11', '\\x61737369676e6174696f6e', NULL, 2905, 2906, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1454, 1, 'T2-60', '\\x70726f6d6f74696f6e', NULL, 2907, 2908, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1455, 1, 'T2-7', '\\x616dc3a96e6167656d656e74', NULL, 2909, 2910, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1456, 1, 'T2-37', '\\x656e7175c3aa7465207075626c69717565', NULL, 2911, 2912, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1458, 1, 'T2-58', '\\x7072c3a976656e74696f6e', NULL, 2915, 2916, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1460, 1, 'T2-14', '\\x63617574696f6e', NULL, 2919, 2920, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1461, 1, 'T2-41', '\\xc3a976616c756174696f6e', NULL, 2921, 2922, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1462, 1, 'T2-52', '\\x696e7370656374696f6e', NULL, 2923, 2924, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1463, 1, 'T2-70', '\\x747574656c6c652061646d696e697374726174697665', NULL, 2925, 2926, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1464, 1, 'T2-30', '\\x636f6f7264696e6174696f6e', NULL, 2927, 2928, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1465, 1, 'T2-13', '\\x6175746f7269736174696f6e', NULL, 2929, 2930, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1467, 1, 'T2-8', '\\x617070656c2064276f6666726573', NULL, 2933, 2934, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1468, 1, 'T2-25', '\\x636f6e7472c3b46c65206465206cc3a967616c6974c3a9', NULL, 2935, 2936, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1469, 1, 'T2-36', '\\x656e7175c3aa7465', NULL, 2937, 2938, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1471, 1, 'T4-8', '\\xc3a9706f71756520636f6e74656d706f7261696e65', NULL, 2941, 2942, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1472, 1, 'T4-3', '\\x4d6f79656e20c3826765', NULL, 2943, 2944, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1473, 1, 'T4-2', '\\x416e746971756974c3a9', NULL, 2945, 2946, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1474, 1, 'T4-5', '\\x52c3a9766f6c7574696f6e2064652031373839', NULL, 2947, 2948, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1475, 1, 'T4-7', '\\x5072656d69657220456d706972652028313830342d3138313529', NULL, 2949, 2950, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1476, 1, 'T4-4', '\\xc3a9706f717565206d6f6465726e65', NULL, 2951, 2952, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1477, 1, 'T4-6', '\\x436f6e73756c61742028313739392d3138303429', NULL, 2953, 2954, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1478, 1, 'T4-26', '\\x6775657272652064652043656e7420416e732028313333372d3134353329', NULL, 2955, 2956, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1479, 1, 'T4-25', '\\x657870616e73696f6e206465206c2769736c616d20283632322d3134393229', NULL, 2957, 2958, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1480, 1, 'T4-58', '\\x46726f6e64652028313634382d3136353229', NULL, 2959, 2960, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1481, 1, 'T4-29', '\\x313765207369c3a8636c65', NULL, 2961, 2962, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1482, 1, 'T4-80', '\\x526573746175726174696f6e2028313831352d3138333029', NULL, 2963, 2964, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1483, 1, 'T4-44', '\\x313965207369c3a8636c65', NULL, 2965, 2966, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1484, 1, 'T4-36', '\\x436f6e76656e74696f6e206e6174696f6e616c652028313739322d3137393529', NULL, 2967, 2968, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1485, 1, 'T4-47', '\\x43696e717569c3a86d652052c3a97075626c697175652028313935382d2029', NULL, 2969, 2970, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1486, 1, 'T4-72', '\\x61626f6c6974696f6e206465206c276573636c617661676520283138343829', NULL, 2971, 2972, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1487, 1, 'T4-84', '\\x5365636f6e6420456d706972652028313835322d3138373029', NULL, 2973, 2974, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1661, 1, 'T3-206', '\\x72c3a9706572746f69726520636976696c', NULL, 3321, 3322, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1440, 1, 'T2-12', '\\x6173737572616e6365', NULL, 2879, 2880, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1586, 1, 'T4-14', '\\x3665207369c3a8636c65', NULL, 3171, 3172, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1490, 1, 'T4-45', '\\x323065207369c3a8636c65', NULL, 2979, 2980, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1382, 1, 'T1-1305', '\\x6c69767265207361696e74', NULL, 2763, 2764, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1383, 1, 'T1-1211', '\\x666c6f72652073617576616765', NULL, 2765, 2766, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1384, 1, 'T1-1224', '\\x6162757320646520666f6e6374696f6e', NULL, 2767, 2768, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1439, 1, 'T2-5', '\\x61646a756469636174696f6e', NULL, 2877, 2878, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1441, 1, 'T2-46', '\\x676172616e746965206427656d7072756e74', NULL, 2881, 2882, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1442, 1, 'T2-49', '\\x696d6d6174726963756c6174696f6e', NULL, 2883, 2884, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1443, 1, 'T2-34', '\\x6465737472756374696f6e', NULL, 2885, 2886, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1444, 1, 'T2-54', '\\x6e756dc3a97269736174696f6e', NULL, 2887, 2888, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1445, 1, 'T2-61', '\\x70726f74656374696f6e', NULL, 2889, 2890, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1491, 1, 'T4-24', '\\x63726f6973616465732028313039352d3132393129', NULL, 2981, 2982, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1492, 1, 'T4-35', '\\x63616d7061676e652064274974616c69652028313739362d3137393729', NULL, 2983, 2984, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1493, 1, 'T4-57', '\\x52656e61697373616e6365', NULL, 2985, 2986, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1494, 1, 'T4-28', '\\x313665207369c3a8636c65', NULL, 2987, 2988, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1495, 1, 'T4-27', '\\x696e766173696f6e73207363616e64696e6176657320283739302d3130333029', NULL, 2989, 2990, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1496, 1, 'T4-94', '\\x636f6e66c3a972656e6365206465204d756e69636820283139333829', NULL, 2991, 2992, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1497, 1, 'T4-76', '\\x677565727265206465204372696dc3a9652028313835332d3138353629', NULL, 2993, 2994, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1498, 1, 'T4-74', '\\x636f7570206427c38974617420647520322064c3a963656d6272652031383531', NULL, 2995, 2996, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1499, 1, 'T4-73', '\\x436f6d6d756e6520646520506172697320283138373129', NULL, 2997, 2998, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1500, 1, 'T4-82', '\\x52c3a9766f6c7574696f6e2064652031383438', NULL, 2999, 3000, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1501, 1, 'T4-81', '\\x52c3a9766f6c7574696f6e2064652031383330', NULL, 3001, 3002, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1502, 1, 'T4-85', '\\x756e6974c3a920616c6c656d616e646520283138373129', NULL, 3003, 3004, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1503, 1, 'T4-86', '\\x756e6974c3a9206974616c69656e6e652028313965207369c3a8636c6529', NULL, 3005, 3006, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1504, 1, 'T4-77', '\\x6775657272652064652053c3a963657373696f6e2028313836312d3138363529', NULL, 3007, 3008, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1505, 1, 'T4-78', '\\x6775657272652064652031383730', NULL, 3009, 3010, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1821, 1, 'T3-189', '\\x726170706f7274206465207374616765', NULL, 3641, 3642, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1518, 1, 'T4-114', '\\x67756572726520313933392d31393435', NULL, 3035, 3036, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1519, 1, 'T4-125', '\\x52c3a9766f6c7574696f6e20727573736520283139313729', NULL, 3037, 3038, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1520, 1, 'T4-99', '\\xc3a976c3a96e656d656e7473206465206d61692031393638', NULL, 3039, 3040, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1521, 1, 'T4-105', '\\x677565727265206427496e646f6368696e652028313934362d3139353429', NULL, 3041, 3042, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1699, 1, 'T3-58', '\\x6368726f6e69717565', NULL, 3397, 3398, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1522, 1, 'T4-127', '\\x7472616974c3a9206465206c2741746c616e7469717565204e6f726420283139343929', NULL, 3043, 3044, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1523, 1, 'T4-110', '\\x6775657272652066726f6964652028313934352d3139383929', NULL, 3045, 3046, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1524, 1, 'T4-88', '\\x61666661697265206465205375657a20283139353629', NULL, 3047, 3048, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1525, 1, 'T4-121', '\\x7072696e74656d70732064652050726167756520283139363829', NULL, 3049, 3050, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1526, 1, 'T4-116', '\\x4c6962c3a9726174696f6e2028313934342d3139343529', NULL, 3051, 3052, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1528, 1, 'T4-107', '\\x67756572726520646573204d616c6f75696e657320283139383229', NULL, 3055, 3056, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1622, 1, 'T3-74', '\\x636f757075726520646520707265737365', NULL, 3243, 3244, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1532, 1, 'T4-109', '\\x67756572726520647520566965746e616d2028313936322d3139373329', NULL, 3063, 3064, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1533, 1, 'T4-92', '\\x636f6e63696c65205661746963616e20322028313936322d3139363529', NULL, 3065, 3066, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1534, 1, 'T4-108', '\\x67756572726520647520476f6c66652028313939302d3139393129', NULL, 3067, 3068, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1535, 1, 'T4-119', '\\x7061637465206765726d616e6f2d736f7669c3a9746971756520283139333929', NULL, 3069, 3070, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1536, 1, 'T4-122', '\\x517561747269c3a86d652052c3a97075626c697175652028313934362d3139353829', NULL, 3071, 3072, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1537, 1, 'T4-101', '\\x676f757665726e656d656e742064652056696368792028313934302d3139343429', NULL, 3073, 3074, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1538, 1, 'T4-112', '\\x67756572726520727573736f2d6a61706f6e616973652028313930342d3139303529', NULL, 3075, 3076, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1380, 1, 'T1-1292', '\\x737461676961697265', NULL, 2759, 2760, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1381, 1, 'T1-1197', '\\x7468c3a96f6c6f676965', NULL, 2761, 2762, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1542, 1, 'T4-120', '\\x706c616e204d61727368616c6c20283139343729', NULL, 3083, 3084, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1543, 1, 'T4-102', '\\x476f757665726e656d656e742070726f7669736f6972652028313934342d3139343629', NULL, 3085, 3086, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1728, 1, 'T3-141', '\\x6d6f74696f6e', NULL, 3455, 3456, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1544, 1, 'T4-123', '\\x52c3a97075626c69717565206465205765696d61722028313931392d3139333329', NULL, 3087, 3088, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1545, 1, 'T4-113', '\\x67756572726520313931342d31393138', NULL, 3089, 3090, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1546, 1, 'T4-104', '\\x67756572726520642745737061676e652028313933362d3139333929', NULL, 3091, 3092, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1719, 1, 'T3-41', '\\x6361726e65742064652073616e74c3a9', NULL, 3437, 3438, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1547, 1, 'T4-115', '\\x696e73757272656374696f6e20686f6e67726f69736520283139353629', NULL, 3093, 3094, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1632, 1, 'T3-125', '\\x6d61696e20636f7572616e7465', NULL, 3263, 3264, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1548, 1, 'T4-126', '\\x73c3a97061726174696f6e2064657320c389676c69736573206574206465206c27c38974617420283139303529', NULL, 3095, 3096, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1549, 1, 'T4-124', '\\x72c3a9756e696669636174696f6e20616c6c656d616e64652028313938392d3139393029', NULL, 3097, 3098, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1551, 1, 'T4-38', '\\x67756572726573206465206c612052c3a9766f6c7574696f6e2028313739322d3137393929', NULL, 3101, 3102, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1552, 1, 'T4-39', '\\x677565727265732064652056656e64c3a9652028313739332d3137393929', NULL, 3103, 3104, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1553, 1, 'T4-34', '\\x63616d7061676e65206427c38967797074652028313739382d3138303129', NULL, 3105, 3106, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1737, 1, 'T3-124', '\\x6c6976726574206f757672696572', NULL, 3473, 3474, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1555, 1, 'T4-32', '\\x417373656d626cc3a96520636f6e7374697475616e74652028313738392d3137393129', NULL, 3109, 3110, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1556, 1, 'T4-40', '\\x546572726575722028313739332d3137393429', NULL, 3111, 3112, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1557, 1, 'T4-49', '\\x54726f697369c3a86d652052c3a97075626c697175652028313837302d3139343029', NULL, 3113, 3114, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1558, 1, 'T4-11', '\\x4261732d456d70697265', NULL, 3115, 3116, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1559, 1, 'T4-48', '\\x636f6e737472756374696f6e206575726f70c3a9656e6e65', NULL, 3117, 3118, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1800, 1, 'T3-11', '\\x616e6e7561697265', NULL, 3599, 3600, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1506, 1, 'T4-75', '\\x4465757869c3a86d652052c3a97075626c697175652028313834382d3138353229', NULL, 3011, 3012, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1507, 1, 'T4-79', '\\x6d6f6e617263686965206465204a75696c6c65742028313833302d3138343829', NULL, 3013, 3014, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1510, 1, 'T4-59', '\\x6775657272652064652044c3a9766f6c7574696f6e2028313636372d3136363829', NULL, 3019, 3020, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1512, 1, 'T4-91', '\\x436f6c6c61626f726174696f6e2028313934302d3139343429', NULL, 3023, 3024, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1513, 1, 'T4-98', '\\x656e7472652d646575782d67756572726573', NULL, 3025, 3026, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1514, 1, 'T4-90', '\\x626c6f637573206465204265726c696e2028313934382d3139343929', NULL, 3027, 3028, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1515, 1, 'T4-103', '\\x677565727265206427416c67c3a97269652028313935342d3139363229', NULL, 3029, 3030, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1516, 1, 'T4-117', '\\x6d7572206465204265726c696e2028313936312d3139383929', NULL, 3031, 3032, 3, 1, 1, '2011-05-25 10:09:29', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1517, 1, 'T4-106', '\\x67756572726520646520436f72c3a9652028313935302d3139353329', NULL, 3033, 3034, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:18');
INSERT INTO keywords VALUES (1673, 1, 'T3-148', '\\x6f72647265206465206d697373696f6e', NULL, 3345, 3346, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1576, 1, 'T4-42', '\\x636f6e6772c3a873206465205669656e6e6520283138313529', NULL, 3151, 3152, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1578, 1, 'T4-54', '\\x677565727265732064274974616c69652028313439342d3135353929', NULL, 3155, 3156, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1579, 1, 'T4-55', '\\x677565727265732064652072656c6967696f6e2028313536322d3135393829', NULL, 3157, 3158, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1583, 1, 'T4-51', '\\x436f6e7472652d52c3a9666f726d65', NULL, 3165, 3166, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1584, 1, 'T4-13', '\\x3565207369c3a8636c65', NULL, 3167, 3168, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1585, 1, 'T4-19', '\\x313165207369c3a8636c65', NULL, 3169, 3170, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1587, 1, 'T4-9', '\\x416e746971756974c3a92067616c6c6f2d726f6d61696e65', NULL, 3173, 3174, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1705, 1, 'T3-30', '\\x62726f6368757265', NULL, 3409, 3410, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1571, 1, 'T4-65', '\\x677565727265206465205365707420416e732028313735362d3137363329', NULL, 3141, 3142, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1572, 1, 'T4-62', '\\x6775657272652064652053756363657373696f6e20642745737061676e652028313730312d3137313429', NULL, 3143, 3144, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1573, 1, 'T4-69', '\\x52c3a967656e63652028313731352d3137323329', NULL, 3145, 3146, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1574, 1, 'T4-41', '\\x43656e74206a6f75727320283138313529', NULL, 3147, 3148, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1575, 1, 'T4-43', '\\x67756572726573206e61706f6cc3a96f6e69656e6e65732028313830302d3138313529', NULL, 3149, 3150, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1596, 1, 'T4-21', '\\x313365207369c3a8636c65', NULL, 3191, 3192, 3, 1, 1, '2011-05-25 10:09:31', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1597, 1, 'T4-16', '\\x3865207369c3a8636c65', NULL, 3193, 3194, 3, 1, 1, '2011-05-25 10:09:31', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1598, 1, 'T4-23', '\\x313565207369c3a8636c65', NULL, 3195, 3196, 3, 1, 1, '2011-05-25 10:09:31', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1599, 1, 'T4-50', '\\x63726f697361646520636f6e747265206c657320416c626967656f69732028313230382d3132323929', NULL, 3197, 3198, 3, 1, 1, '2011-05-25 10:09:31', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1623, 1, 'T3-26', '\\x62696c616e20636f6d707461626c65', NULL, 3245, 3246, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1624, 1, 'T3-96', '\\x646f73736965722064652070656e73696f6e', NULL, 3247, 3248, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1625, 1, 'T3-197', '\\x726567697374726520647520636f6d6d657263652065742064657320736f6369c3a974c3a973', NULL, 3249, 3250, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1626, 1, 'T3-94', '\\x646f7373696572206465206361727269c3a87265', NULL, 3251, 3252, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1627, 1, 'T3-168', '\\x706f727472616974', NULL, 3253, 3254, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1628, 1, 'T3-180', '\\x70726f74c3aa74', NULL, 3255, 3256, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1629, 1, 'T3-146', '\\x6e6f7469636520696e646976696475656c6c65', NULL, 3257, 3258, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1631, 1, 'T3-13', '\\x617272c3aa74c3a9206427616c69676e656d656e74', NULL, 3261, 3262, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1633, 1, 'T3-158', '\\x70c3a9746974696f6e', NULL, 3265, 3266, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1634, 1, 'T3-112', '\\x696e76656e7461697265', NULL, 3267, 3268, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1540, 1, 'T4-100', '\\x46726f6e7420706f70756c616972652028313933362d3139333829', NULL, 3079, 3080, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1541, 1, 'T4-93', '\\x636f6e66c3a972656e63652064652042616e64756e6720283139353529', NULL, 3081, 3082, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1610, 1, 'T3-199', '\\x7265676973747265206d6174726963756c65', NULL, 3219, 3220, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1611, 1, 'T3-196', '\\x72656769737472652064657320656e7472c3a965732065742064657320736f7274696573', NULL, 3221, 3222, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1612, 1, 'T3-76', '\\x64c3a9636973696f6e2062756467c3a97461697265206d6f64696669636174697665', NULL, 3223, 3224, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1710, 1, 'T3-128', '\\x6d61717565747465', NULL, 3419, 3420, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1712, 1, 'T3-142', '\\x6e6f6d656e636c6174757265', NULL, 3423, 3424, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1614, 1, 'T3-217', '\\x736368c3a96d6120646972656374657572206427616dc3a96e6167656d656e74206574206427757262616e69736d65', NULL, 3227, 3228, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1711, 1, 'T3-191', '\\x7265676973747265', NULL, 3421, 3422, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1615, 1, 'T3-81', '\\x64c3a9636c61726174696f6e2064652073756363657373696f6e', NULL, 3229, 3230, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1617, 1, 'T3-77', '\\x64c3a9636973696f6e20646520706169656d656e74', NULL, 3233, 3234, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1302, 1, 'T1-953', '\\x636f6d6dc3a96d6f726174696f6e', NULL, 2603, 2604, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1303, 1, 'T1-957', '\\x72656c6174696f6e73206575726f70c3a9656e6e6573', NULL, 2605, 2606, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1560, 1, 'T4-130', '\\x7472616974c3a920737572206c27756e696f6e206575726f70c3a9656e6e6520283139393229', NULL, 3119, 3120, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1561, 1, 'T4-129', '\\x7472616974c3a920646520526f6d6520283139353729', NULL, 3121, 3122, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1562, 1, 'T4-31', '\\x6d6f6e617263686965206427416e6369656e2052c3a967696d65', NULL, 3123, 3124, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1564, 1, 'T4-12', '\\x67756572726520646573204761756c6573', NULL, 3127, 3128, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1565, 1, 'T4-64', '\\x72c3a9766f636174696f6e206465206c27c3a9646974206465204e616e74657320283136383529', NULL, 3129, 3130, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1618, 1, 'T3-64', '\\x636f6d7074652064276578706c6f69746174696f6e', NULL, 3235, 3236, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1619, 1, 'T3-99', '\\x646f737369657220646573206f75767261676573206578c3a9637574c3a973', NULL, 3237, 3238, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1620, 1, 'T3-4', '\\x6163746520646520636f6e6772c3a873', NULL, 3239, 3240, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1621, 1, 'T3-116', '\\x6c6973746520c3a96c6563746f72616c65', NULL, 3241, 3242, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1649, 1, 'T3-19', '\\x61746c61732074657272696572', NULL, 3297, 3298, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1650, 1, 'T3-111', '\\x696e736372697074696f6e206879706f7468c3a96361697265', NULL, 3299, 3300, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1651, 1, 'T3-25', '\\x6269626c696f67726170686965', NULL, 3301, 3302, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1652, 1, 'T3-122', '\\x6c69767265206c697475726769717565', NULL, 3303, 3304, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1653, 1, 'T3-127', '\\x6d616e696665737465206465206e6176697265', NULL, 3305, 3306, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1663, 1, 'T3-9', '\\x616c62756d', NULL, 3325, 3326, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1636, 1, 'T3-151', '\\x70616e6e6561752064276578706f736974696f6e', NULL, 3271, 3272, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1637, 1, 'T3-183', '\\x7175657374696f6e6e61697265206427656e7175c3aa7465', NULL, 3273, 3274, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1638, 1, 'T3-159', '\\x70686f746f67726170686965', NULL, 3275, 3276, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1639, 1, 'T3-60', '\\x63697263756c61697265', NULL, 3277, 3278, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1640, 1, 'T3-33', '\\x62756467657420737570706cc3a96d656e7461697265', NULL, 3279, 3280, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1641, 1, 'T3-115', '\\x6c69737465206427c3a96d617267656d656e74', NULL, 3281, 3282, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1643, 1, 'T3-204', '\\x72c3a8676c656d656e7420696e74657269657572', NULL, 3285, 3286, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1644, 1, 'T3-57', '\\x636861727465', NULL, 3287, 3288, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1600, 1, 'T3-34', '\\x62756c6c6520706f6e7469666963616c65', NULL, 3199, 3200, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1602, 1, 'T3-198', '\\x726567697374726520647520636f757272696572', NULL, 3203, 3204, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1603, 1, 'T3-83', '\\x64c3a9636c61726174696f6e206427696e74656e74696f6e206427616c69c3a96e6572', NULL, 3205, 3206, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1604, 1, 'T3-157', '\\x7065726d697320646520726563686572636865', NULL, 3207, 3208, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1606, 1, 'T3-118', '\\x6c6976726520636f6d707461626c65', NULL, 3211, 3212, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1607, 1, 'T3-185', '\\x726170706f727420642761637469766974c3a9', NULL, 3213, 3214, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1609, 1, 'T3-32', '\\x627564676574207072696d69746966', NULL, 3217, 3218, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1684, 1, 'T3-145', '\\x6e6f74652064652073657276696365', NULL, 3367, 3368, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1685, 1, 'T3-22', '\\x61766973', NULL, 3369, 3370, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1724, 1, 'T3-161', '\\x706c616e', NULL, 3447, 3448, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1595, 1, 'T4-20', '\\x313265207369c3a8636c65', NULL, 3189, 3190, 3, 1, 1, '2011-05-25 10:09:31', '2012-03-07 09:41:20');
INSERT INTO keywords VALUES (1570, 1, 'T4-67', '\\x696e64c3a970656e64616e63652064657320c389746174732d556e69732028313737352d3137383229', NULL, 3139, 3140, 3, 1, 1, '2011-05-25 10:09:30', '2012-03-07 09:41:19');
INSERT INTO keywords VALUES (1675, 1, 'T3-140', '\\x6d6f64c3a86c65206465206661627269717565', NULL, 3349, 3350, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1677, 1, 'T3-61', '\\x636f6d6d756e697175c3a920646520707265737365', NULL, 3353, 3354, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1680, 1, 'T3-1', '\\x6163636f7264206465207472617661696c', NULL, 3359, 3360, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1670, 1, 'T3-17', '\\x617272c3aa74c3a9207072c3a9666563746f72616c', NULL, 3339, 3340, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1703, 1, 'T3-164', '\\x706c616e206465207365636f757273', NULL, 3405, 3406, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1704, 1, 'T3-153', '\\x7061737365706f7274', NULL, 3407, 3408, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1706, 1, 'T3-232', '\\x7472616e736372697074696f6e206879706f7468c3a96361697265', NULL, 3411, 3412, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1707, 1, 'T3-205', '\\x72c3a9706572746f697265', NULL, 3413, 3414, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1708, 1, 'T3-233', '\\x77617272616e74', NULL, 3415, 3416, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1709, 1, 'T3-5', '\\x6163746520646520736f6369c3a974c3a9', NULL, 3417, 3418, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1713, 1, 'T3-174', '\\x70726f63c3a8732d76657262616c2064652072c3a9756e696f6e', NULL, 3425, 3426, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1714, 1, 'T3-38', '\\x636168696572206465732063686172676573', NULL, 3427, 3428, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1715, 1, 'T3-139', '\\x6d696e757465206e6f74617269616c65', NULL, 3429, 3430, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1716, 1, 'T3-222', '\\x73756a65742064276578616d656e', NULL, 3431, 3432, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1717, 1, 'T3-20', '\\x6175746f7269736174696f6e206427757262616e69736d65', NULL, 3433, 3434, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1718, 1, 'T3-113', '\\x6c61697373657a2d706173736572', NULL, 3435, 3436, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1720, 1, 'T3-227', '\\x74696d627265', NULL, 3439, 3440, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1721, 1, 'T3-39', '\\x63616c656e6472696572', NULL, 3441, 3442, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1722, 1, 'T3-28', '\\x62696c616e20736f6369616c', NULL, 3443, 3444, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1723, 1, 'T3-182', '\\x7075626c69636174696f6e2070c3a972696f6469717565', NULL, 3445, 3446, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1725, 1, 'T3-62', '\\x636f6d7074652061646d696e6973747261746966', NULL, 3449, 3450, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1726, 1, 'T3-186', '\\x726170706f72742064652067656e6461726d65726965', NULL, 3451, 3452, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1727, 1, 'T3-63', '\\x636f6d7074652064652067657374696f6e', NULL, 3453, 3454, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1729, 1, 'T3-143', '\\x6e6f726d65', NULL, 3457, 3458, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1730, 1, 'T3-171', '\\x70726f63c3a8732d76657262616c206427616dc3a96e6167656d656e7420666f72657374696572', NULL, 3459, 3460, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1731, 1, 'T3-225', '\\x74c3a96cc3a96772616d6d65', NULL, 3461, 3462, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1732, 1, 'T3-69', '\\x636f6e76656e74696f6e', NULL, 3463, 3464, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1733, 1, 'T3-121', '\\x6c6976726520666f6e63696572', NULL, 3465, 3466, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1734, 1, 'T3-48', '\\x63617274652064276964656e746974c3a9', NULL, 3467, 3468, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1735, 1, 'T3-165', '\\x706c616e2064276f636375706174696f6e2064657320736f6c73', NULL, 3469, 3470, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1736, 1, 'T3-85', '\\x64c3a9636c61726174696f6e2066697363616c65', NULL, 3471, 3472, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1738, 1, 'T3-72', '\\x636f7069652064276578616d656e', NULL, 3475, 3476, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1743, 1, 'T3-169', '\\x70726573736520696e737469747574696f6e6e656c6c65', NULL, 3485, 3486, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1744, 1, 'T3-230', '\\x74697472652064652073c3a96a6f7572', NULL, 3487, 3488, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1747, 1, 'T3-226', '\\x7465787465206f6666696369656c', NULL, 3493, 3494, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1748, 1, 'T3-35', '\\x62756c6c6574696e2064652073616c61697265', NULL, 3495, 3496, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1749, 1, 'T3-114', '\\x6c69737465', NULL, 3497, 3498, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1750, 1, 'T3-73', '\\x636f72726573706f6e64616e6365', NULL, 3499, 3500, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1751, 1, 'T3-47', '\\x636172746520646520766f6575', NULL, 3501, 3502, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1753, 1, 'T3-166', '\\x706c616e206427757262616e69736d65', NULL, 3505, 3506, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1754, 1, 'T3-10', '\\x616e6e6f6e6365206f6666696369656c6c65206574206cc3a967616c65', NULL, 3507, 3508, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1755, 1, 'T3-218', '\\x736f6d6d69657220646573206d61726368616e6469736573', NULL, 3509, 3510, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1646, 1, 'T3-84', '\\x64c3a9636c61726174696f6e2064277574696c6974c3a9207075626c69717565', NULL, 3291, 3292, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1647, 1, 'T3-3', '\\x6163746520636976696c207075626c6963', NULL, 3293, 3294, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1648, 1, 'T3-89', '\\x646f63756d656e7420617564696f76697375656c', NULL, 3295, 3296, 4, 1, 1, '2011-05-25 10:13:14', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1671, 1, 'T3-162', '\\x706c616e2063616461737472616c', NULL, 3341, 3342, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1674, 1, 'T3-126', '\\x6d616e69666573746520642761c3a9726f6e6566', NULL, 3347, 3348, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1681, 1, 'T3-103', '\\xc3a9746174206475206d6f6e74616e742064657320726f6c6573', NULL, 3361, 3362, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1682, 1, 'T3-131', '\\x6d6174726963652063616461737472616c65', NULL, 3363, 3364, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1683, 1, 'T3-45', '\\x6361727465206465206d656d627265', NULL, 3365, 3366, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1686, 1, 'T3-136', '\\x6d657263757269616c65', NULL, 3371, 3372, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1687, 1, 'T3-29', '\\x627265766574206427696e76656e74696f6e', NULL, 3373, 3374, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1776, 1, 'T3-31', '\\x627564676574', NULL, 3551, 3552, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1688, 1, 'T3-55', '\\x63657274696669636174206427757262616e69736d65', NULL, 3375, 3376, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1689, 1, 'T3-2', '\\x616374652061757468656e7469717565', NULL, 3377, 3378, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:21');
INSERT INTO keywords VALUES (1690, 1, 'T3-137', '\\x6d6963726f666f726d65', NULL, 3379, 3380, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1693, 1, 'T3-167', '\\x706fc3a86d65', NULL, 3385, 3386, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1694, 1, 'T3-14', '\\x617272c3aa74c3a9206475206d61697265', NULL, 3387, 3388, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1696, 1, 'T3-207', '\\x72c3a9706572746f69726520646573206dc3a97469657273', NULL, 3391, 3392, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1697, 1, 'T3-71', '\\x636f6e76656e74696f6e20646520666f726d6174696f6e', NULL, 3393, 3394, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1700, 1, 'T3-194', '\\x726567697374726520646520636174686f6c69636974c3a9', NULL, 3399, 3400, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1701, 1, 'T3-130', '\\x6d6172717565206465206661627269717565', NULL, 3401, 3402, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1702, 1, 'T3-43', '\\x636172746520c3a0206a6f756572', NULL, 3403, 3404, 4, 1, 1, '2011-05-25 10:13:15', '2012-03-07 09:41:22');
INSERT INTO keywords VALUES (1756, 1, 'T3-54', '\\x63657274696669636174', NULL, 3511, 3512, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1757, 1, 'T3-88', '\\x646973636f757273', NULL, 3513, 3514, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1758, 1, 'T3-210', '\\x72c3a9706572746f6972652067c3a96ec3a972616c2064657320616666616972657320636976696c6573', NULL, 3515, 3516, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1759, 1, 'T3-97', '\\x646f737369657220646520707265737365', NULL, 3517, 3518, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1760, 1, 'T3-15', '\\x617272c3aa74c3a9206475207072c3a9736964656e7420647520636f6e7365696c2067c3a96ec3a972616c', NULL, 3519, 3520, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1761, 1, 'T3-149', '\\x6f7267616e696772616d6d65', NULL, 3521, 3522, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1762, 1, 'T3-181', '\\x7075626c69636174696f6e206f6666696369656c6c65', NULL, 3523, 3524, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1763, 1, 'T3-229', '\\x74697472652064652070726f707269c3a974c3a9', NULL, 3525, 3526, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1764, 1, 'T3-80', '\\x64c3a9636c61726174696f6e20646520726576656e7573', NULL, 3527, 3528, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1765, 1, 'T3-91', '\\x646f63756d656e74206669677572c3a9', NULL, 3529, 3530, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1766, 1, 'T3-120', '\\x6c697672652064276f72', NULL, 3531, 3532, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1767, 1, 'T3-184', '\\x726170706f7274', NULL, 3533, 3534, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1768, 1, 'T3-107', '\\x66616972652d70617274', NULL, 3535, 3536, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1769, 1, 'T3-119', '\\x6c6976726520646520726169736f6e', NULL, 3537, 3538, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1770, 1, 'T3-223', '\\x7461626c65', NULL, 3539, 3540, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1771, 1, 'T3-100', '\\x646f737369657220696e646976696475656c', NULL, 3541, 3542, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1772, 1, 'T3-6', '\\x6163746520736f7573207365696e672070726976c3a9', NULL, 3543, 3544, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1773, 1, 'T3-104', '\\xc3a97461742070617263656c6c61697265', NULL, 3545, 3546, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1774, 1, 'T3-219', '\\x7374617469737469717565', NULL, 3547, 3548, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1778, 1, 'T3-172', '\\x70726f63c3a8732d76657262616c206427c3a96c656374696f6e', NULL, 3555, 3556, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1779, 1, 'T3-187', '\\x726170706f7274206465206d6572', NULL, 3557, 3558, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1780, 1, 'T3-98', '\\x646f73736965722064652070726f63c3a964757265', NULL, 3559, 3560, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1781, 1, 'T3-79', '\\x64c3a9636c61726174696f6e20646520646f75616e65', NULL, 3561, 3562, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1782, 1, 'T3-82', '\\x64c3a9636c61726174696f6e2064652074726176617578', NULL, 3563, 3564, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1783, 1, 'T3-193', '\\x726567697374726520642761756469656e6365', NULL, 3565, 3566, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1785, 1, 'T3-109', '\\x6669636869657220696d6d6f62696c696572', NULL, 3569, 3570, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1786, 1, 'T3-70', '\\x636f6e76656e74696f6e20636f6c6c656374697665', NULL, 3571, 3572, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1787, 1, 'T3-160', '\\x7069c3a8636520636f6d707461626c65', NULL, 3573, 3574, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1790, 1, 'T3-200', '\\x7265676973747265207061726f69737369616c', NULL, 3579, 3580, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1178, 1, 'T1-956', '\\x64c3a973696e66656374696f6e', NULL, 2355, 2356, 1, 1, 1, '2011-05-25 10:03:11', '2012-03-07 09:41:12');
INSERT INTO keywords VALUES (1791, 1, 'T3-163', '\\x706c616e206427616c69676e656d656e74', NULL, 3581, 3582, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1792, 1, 'T3-86', '\\x64c3a96c6962c3a9726174696f6e', NULL, 3583, 3584, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1793, 1, 'T3-190', '\\x726170706f7274206427657870657274697365', NULL, 3585, 3586, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1795, 1, 'T3-65', '\\x636f6d7074652066696e616e63696572', NULL, 3589, 3590, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1796, 1, 'T3-175', '\\x70726f63c3a8732d76657262616c206465207363656c6cc3a973', NULL, 3591, 3592, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1797, 1, 'T3-201', '\\x72656769737472652070726f74657374616e74', NULL, 3593, 3594, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1798, 1, 'T3-188', '\\x726170706f727420646520706f6c696365', NULL, 3595, 3596, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1802, 1, 'T3-24', '\\x6261736520646520646f6e6ec3a96573', NULL, 3603, 3604, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1830, 1, 'T3-106', '\\x6578706c6f69742064276875697373696572', NULL, 3659, 3660, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1831, 1, 'T3-117', '\\x6c69737465206e6f6d696e6174697665', NULL, 3661, 3662, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1, 1, 'T1-503', '\\x636f6d6d756e69636174696f6e73', NULL, 1, 2, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (60, 1, 'T1-495', '\\x6269656e732064c3a970617274656d656e74617578', NULL, 119, 120, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:53');
INSERT INTO keywords VALUES (81, 1, 'T1-891', '\\x72c3a9756e696f6e207075626c69717565', NULL, 161, 162, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (92, 1, 'T1-1310', '\\x707269c3a87265207075626c69717565', NULL, 183, 184, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (114, 1, 'T1-1300', '\\x7265636865726368652064616e73206c27696e74c3a972c3aa74206465732066616d696c6c6573', NULL, 227, 228, 1, 1, 1, '2011-05-25 10:02:56', '2012-03-07 09:40:54');
INSERT INTO keywords VALUES (148, 1, 'T1-41', '\\x61696465207075626c69717565206175206c6f67656d656e74', NULL, 295, 296, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (1803, 1, 'T3-123', '\\x6c6976726574206d696c697461697265', NULL, 3605, 3606, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1804, 1, 'T3-173', '\\x70726f63c3a8732d76657262616c206427656e7175c3aa7465', NULL, 3607, 3608, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1805, 1, 'T3-78', '\\x64c3a9636c61726174696f6e', NULL, 3609, 3610, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1806, 1, 'T3-95', '\\x646f737369657220646520636f6e73756c746174696f6e2064657320656e747265707269736573', NULL, 3611, 3612, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (1807, 1, 'T3-195', '\\x7265676973747265206427c3a963726f75', NULL, 3613, 3614, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1808, 1, 'T3-87', '\\x6469706cc3b46d65', NULL, 3615, 3616, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1809, 1, 'T3-46', '\\x6361727465206465207472617661696c', NULL, 3617, 3618, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1810, 1, 'T3-215', '\\x7363656175', NULL, 3619, 3620, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1811, 1, 'T3-213', '\\x72c3b46c65206427c3a971756970616765', NULL, 3621, 3622, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (703, 1, 'T1-1256', '\\x626174656175206465206e617669676174696f6e20696e74c3a9726965757265', NULL, 1405, 1406, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:04');
INSERT INTO keywords VALUES (724, 1, 'T1-577', '\\x616e6e6578696f6e206465207465727269746f697265', NULL, 1447, 1448, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (748, 1, 'T1-355', '\\xc3a97461626c697373656d656e74206dc3a96469636f20736f6369616c', NULL, 1495, 1496, 1, 1, 1, '2011-05-25 10:03:05', '2012-03-07 09:41:05');
INSERT INTO keywords VALUES (781, 1, 'T1-1138', '\\x66696e616e6365732064c3a970617274656d656e74616c6573', NULL, 1561, 1562, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (793, 1, 'T1-1371', '\\x6173736f63696174696f6e20646520636861737365', NULL, 1585, 1586, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (404, 1, 'T1-138', '\\x6173736f63696174696f6e20666f6e6369c3a872652075726261696e65', NULL, 807, 808, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (435, 1, 'T1-1319', '\\x616374696f6e2073616e697461697265', NULL, 869, 870, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (458, 1, 'T1-457', '\\x70726f64756374696f6e20c3a96c6563747269717565', NULL, 915, 916, 1, 1, 1, '2011-05-25 10:03:01', '2012-03-07 09:41:00');
INSERT INTO keywords VALUES (803, 1, 'T1-914', '\\x70617274696369706174696f6e2064657320656d706c6f7965757273', NULL, 1605, 1606, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:06');
INSERT INTO keywords VALUES (833, 1, 'T1-707', '\\x61747465696e746520c3a0206c276f72647265207075626c6963', NULL, 1665, 1666, 1, 1, 1, '2011-05-25 10:03:06', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (859, 1, 'T1-1435', '\\x61646d696e697374726174696f6e20636f6d6d756e616c65', NULL, 1717, 1718, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (871, 1, 'T1-911', '\\x64726f697473206369766971756573', NULL, 1741, 1742, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (872, 1, 'T1-675', '\\x636f6c6cc3a8676520c3a96c6563746f72616c', NULL, 1743, 1744, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:07');
INSERT INTO keywords VALUES (895, 1, 'T1-1226', '\\x6f6666696369657220646520706f6c696365206a756469636961697265', NULL, 1789, 1790, 1, 1, 1, '2011-05-25 10:03:07', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (923, 1, 'T1-851', '\\x766f7961676575722072657072c3a973656e74616e7420706c6163696572', NULL, 1845, 1846, 1, 1, 1, '2011-05-25 10:03:08', '2012-03-07 09:41:08');
INSERT INTO keywords VALUES (1324, 1, 'T1-1007', '\\x6f7267616e69736d652067c3a96ec3a974697175656d656e74206d6f64696669c3a9', NULL, 2647, 2648, 1, 1, 1, '2011-05-25 10:03:13', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1347, 1, 'T1-1332', '\\x61726d656d656e74206d61726974696d65', NULL, 2693, 2694, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1349, 1, 'T1-1261', '\\x736f6369c3a974c3a9206427696e74c3a972c3aa7420636f6c6c65637469662061677269636f6c65', NULL, 2697, 2698, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:15');
INSERT INTO keywords VALUES (1385, 1, 'T1-1289', '\\x68c3b474656c206465206c61207072c3a966656374757265', NULL, 2769, 2770, 1, 1, 1, '2011-05-25 10:03:14', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1413, 1, 'T2-23', '\\x636f6e7472c3b46c652062756467c3a97461697265', NULL, 2825, 2826, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:16');
INSERT INTO keywords VALUES (1428, 1, 'T2-71', '\\x747574656c6c652066696e616e6369c3a87265', NULL, 2855, 2856, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1438, 1, 'T2-57', '\\x7072c3a97061726174696f6e2062756467c3a97461697265', NULL, 2875, 2876, 2, 1, 1, '2011-05-25 10:06:26', '2012-03-07 09:41:17');
INSERT INTO keywords VALUES (1801, 1, 'T3-68', '\\x636f6e7472617420642761707072656e74697373616765', NULL, 3601, 3602, 4, 1, 1, '2011-05-25 10:13:16', '2012-03-07 09:41:23');
INSERT INTO keywords VALUES (6, 1, 'T1-1359', '\\x74656d7073206c6962726520657420736f63696162696c6974c3a9', NULL, 11, 12, 1, 1, 1, '2011-05-25 10:02:54', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (32, 1, 'T1-861', '\\x636f6e646974696f6e73206475207472617661696c', NULL, 63, 64, 1, 1, 1, '2011-05-25 10:02:55', '2012-03-07 09:40:52');
INSERT INTO keywords VALUES (1817, 1, 'T3-177', '\\x70726f63c3a8732d76657262616c206427696e6672616374696f6e', NULL, 3633, 3634, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1818, 1, 'T3-133', '\\x6dc3a96461696c6c65', NULL, 3635, 3636, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1819, 1, 'T3-101', '\\x646f7373696572206dc3a9646963616c', NULL, 3637, 3638, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1820, 1, 'T3-110', '\\x6879706f7468c3a8717565206d61726974696d65', NULL, 3639, 3640, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1823, 1, 'T3-66', '\\x636f6d7074652072656e6475', NULL, 3645, 3646, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1824, 1, 'T3-90', '\\x646f63756d656e742064652073c3a9616e6365', NULL, 3647, 3648, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (1825, 1, 'T3-37', '\\x63616869657220646520646f6cc3a9616e6365', NULL, 3649, 3650, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (319, 1, 'T1-878', '\\x677265666669657220646520636f6d6d65726365', NULL, 637, 638, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (338, 1, 'T1-870', '\\xc3a96c656374696f6e206575726f70c3a9656e6e65', NULL, 675, 676, 1, 1, 1, '2011-05-25 10:02:59', '2012-03-07 09:40:58');
INSERT INTO keywords VALUES (362, 1, 'T1-478', '\\xc3a97461626c697373656d656e74207265636576616e74206475207075626c6963', NULL, 723, 724, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (393, 1, 'T1-131', '\\x6f7267616e69736d6520636f6e73756c7461746966', NULL, 785, 786, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (398, 1, 'T1-1094', '\\x70726f74656374696f6e206d617465726e656c6c6520657420696e66616e74696c65', NULL, 795, 796, 1, 1, 1, '2011-05-25 10:03:00', '2012-03-07 09:40:59');
INSERT INTO keywords VALUES (1815, 1, 'T3-93', '\\x646f73736965722064652063616e6469646174757265', NULL, 3629, 3630, 4, 1, 1, '2011-05-25 10:13:17', '2012-03-07 09:41:24');
INSERT INTO keywords VALUES (161, 1, 'T1-46', '\\x636f6e74656e7469657578206465206c612073c3a96375726974c3a920736f6369616c65', NULL, 321, 322, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:55');
INSERT INTO keywords VALUES (190, 1, 'T1-1167', '\\x70726f74656374696f6e2073616e697461697265206475206368657074656c', NULL, 379, 380, 1, 1, 1, '2011-05-25 10:02:57', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (220, 1, 'T1-513', '\\x636f6d7061676e69652064276173737572616e636573', NULL, 439, 440, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:56');
INSERT INTO keywords VALUES (238, 1, 'T1-917', '\\x74696572732d6f72647265', NULL, 475, 476, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');
INSERT INTO keywords VALUES (239, 1, 'T1-633', '\\x636f6e6772c3a9676174696f6e', NULL, 477, 478, 1, 1, 1, '2011-05-25 10:02:58', '2012-03-07 09:40:57');


--
-- Name: keywords_id_seq; Type: SEQUENCE SET; Schema: public; Owner: webgfc
--

SELECT pg_catalog.setval('keywords_id_seq', 1833, false);


--
-- Name: keywordtypes_id_seq; Type: SEQUENCE SET; Schema: public; Owner: webgfc
--

SELECT pg_catalog.setval('keywordtypes_id_seq', 10, true);


--
-- Data for Name: rights; Type: TABLE DATA; Schema: public; Owner: webgfc
--
INSERT INTO rights VALUES (1, true, true, true, false, false, true, true, true, false, false, true, false, false, false, false, '2013-04-02 11:37:43.523222', '2013-04-02 11:37:43.523222', false, 7, false, false, true, true, true, true, true, true, true, false, true, true, true, 'Profil', 7, true, true, true, true);
INSERT INTO rights VALUES (2, true, true, true, true, false, false, true, true, true, false, false, false, false, false, true, '2013-04-02 11:37:45.624083', '2013-04-02 11:37:45.624083', false, 6, true, true, true, true, true, true, true, true, true, true, true, true, true, 'Profil', 6, true, true, true, true);
INSERT INTO rights VALUES (3, true, true, true, true, false, true, true, true, false, false, false, false, true, true, false, '2013-04-02 11:37:48.130788', '2013-04-02 11:37:48.130788', false, 5, true, true, true, true, true, true, true, true, true, true, true, true, true, 'Profil', 5, true, true, true, true);
INSERT INTO rights VALUES (4, true, true, true, true, false, false, true, true, false, false, false, false, true, false, false, '2013-04-02 11:37:50.466315', '2013-04-02 11:37:50.466315', false, 4, true, true, true, true, true, true, true, true, true, true, true, true, true, 'Profil', 4, true, true, true, true);
INSERT INTO rights VALUES (5, true, true, true, false, true, true, true, true, false, false, false, true, false, false, false, '2013-04-02 11:37:52.88356', '2013-04-02 11:37:52.88356', false, 3, true, false, true, true, true, true, true, true, true, false, true, true, true, 'Profil', 3, true, true, true, true);
INSERT INTO rights VALUES (6, true, true, true, true, true, true, true, true, true, true, false, false, false, false, false, '2013-04-02 11:37:55.024301', '2013-04-02 11:37:55.024301', false, 2, true, false, true, true, true, true, true, true, true, false, true, false, true, 'Profil', 2, true, true, true, true);


--
-- Name: rights_id_seq; Type: SEQUENCE SET; Schema: public; Owner: webgfc
--

SELECT pg_catalog.setval('rights_id_seq', 7, true);


--
-- Data for Name: typemetadonnees; Type: TABLE DATA; Schema: public; Owner: webgfc
--

INSERT INTO typemetadonnees VALUES (1, 'date', '2012-02-17 14:55:14.779645', '2012-02-17 14:55:14.779645');
INSERT INTO typemetadonnees VALUES (2, 'texte', '2012-02-17 14:55:14.811863', '2012-02-17 14:55:14.811863');
INSERT INTO typemetadonnees VALUES (3, 'booléen', '2012-02-17 14:55:14.825279', '2012-02-17 14:55:14.825279');
INSERT INTO typemetadonnees VALUES (4, 'select', '2014-05-16 10:06:15.491712', '2014-05-16 10:06:15.491712');


--
-- Name: typemetadonnees_id_seq; Type: SEQUENCE SET; Schema: public; Owner: webgfc
--

SELECT pg_catalog.setval('typemetadonnees_id_seq', 5, true);


--
-- Data for Name: typenotifs; Type: TABLE DATA; Schema: public; Owner: webgfc
--

INSERT INTO typenotifs VALUES (1, 'wkf_new');
INSERT INTO typenotifs VALUES (2, 'wkf_wait');
INSERT INTO typenotifs VALUES (3, 'wkf_done');
INSERT INTO typenotifs VALUES (4, 'ban_xnew');
INSERT INTO typenotifs VALUES (5, 'ban_cancel');
INSERT INTO typenotifs VALUES (6, 'ban_response');
INSERT INTO typenotifs VALUES (7, 'ban_info');
INSERT INTO typenotifs VALUES (8, 'sys_rights');
INSERT INTO typenotifs VALUES (9, 'dkt_delegto');
INSERT INTO typenotifs VALUES (10, 'wkf_jmp');
INSERT INTO typenotifs VALUES (11, 'wkf_jbk');
INSERT INTO typenotifs VALUES (12, 'ban_copy');


--
-- Name: typenotifs_id_seq; Type: SEQUENCE SET; Schema: public; Owner: webgfc
--

SELECT pg_catalog.setval('typenotifs_id_seq', 13, false);


--
-- PostgreSQL database dump complete
--

INSERT INTO addressbooks VALUES (1, 'Carnet principal');

SELECT pg_catalog.setval('addressbooks_id_seq', 2, true);

INSERT INTO organismes VALUES (1, 'Sans organisme', 1);

SELECT pg_catalog.setval('organismes_id_seq', 2, true);

INSERT INTO contacts (id, nom, name, organisme_id, addressbook_id) VALUES (1, 'Sans contact', 'Sans contact', 1, 1);

SELECT pg_catalog.setval('contacts_id_seq', 2, true);



INSERT INTO desktops (name, profil_id, created, modified, id, active) VALUES ( 'Parapheur', '5', NOW(), NOW(), '-1',TRUE);
INSERT INTO desktops (name, profil_id, created, modified, id, active) VALUES ( 'Pastell', '5', NOW(), NOW(), '-3',TRUE);
SELECT pg_catalog.setval('desktops_id_seq', 3, true);

INSERT INTO desktopsmanagers (name, created, modified, id, active, isautocreated) VALUES ( 'Bureau Parapheur', NOW(), NOW(), '-1', TRUE, true);
INSERT INTO desktopsmanagers (name, created, modified, id, active, isautocreated) VALUES ( 'Bureau administrateur', NOW(), NOW(), '-2', TRUE, true);
INSERT INTO desktopsmanagers (name, created, modified, id, active, isautocreated) VALUES ( 'Bureau Pastell', NOW(), NOW(), '-3', TRUE, true);
SELECT pg_catalog.setval('desktopsmanagers_id_seq', 1, true);

INSERT INTO desktops_desktopsmanagers (id, desktop_id, desktopmanager_id, created, modified) VALUES ( '1', '-1', '-1', NOW(), NOW());
INSERT INTO desktops_desktopsmanagers (id, desktop_id, desktopmanager_id, created, modified) VALUES ( '3', '-3', '-3', NOW(), NOW());
SELECT pg_catalog.setval('desktops_desktopsmanagers_id_seq', 3, true);
--

-- PostgreSQL database dump complete
--



INSERT INTO templatenotifications VALUES (1, 'aiguillage', 'Un flux vient de vous être aiguillé', 'Bonjour #PRENOM# #NOM#,

  Un flux vient de vous être aiguillé.

  Objet : #OBJET_FLUX#
  Référence : #REFERENCE_FLUX#
  Commentaire: #COMMENTAIRE_FLUX#

  Le flux peut être consulté à cette adresse : #ADRESSE_A_MODIFIER#

  Très cordialement.

  web-GFC

  Ce mail de notification a été envoyé par web-GFC pour votre information. Merci de ne pas répondre.', true, NOW(), NOW() );


INSERT INTO templatenotifications VALUES (2, 'copy', 'Une copie vient de vous être adressée', 'Bonjour #PRENOM# #NOM#,

  Un flux vous a été envoyé pour copie.

  Objet : #OBJET_FLUX#
  Référence : #REFERENCE_FLUX#
  Circuit : #LIBELLE_CIRCUIT#
  Commentaire: #COMMENTAIRE_FLUX#
  Affaire suivie par: #AFFAIRESUIVIE_PAR#

  Le flux peut être consulté à cette adresse : #ADRESSE_A_MODIFIER#

  Très cordialement.

  web-GFC

  Ce mail de notification a été envoyé par web-GFC pour votre information. Merci de ne pas répondre.', true, NOW(), NOW() );

INSERT INTO templatenotifications VALUES (3, 'delegation', 'Un bureau vien de vous être délégué', 'Bonjour #PRENOM# #NOM#,

  Un bureau vient de vous être délégué.

  Bureau : #NOM_BUREAU#
  Date de début: #DEBUT_DELEGATION#
  Date de fin: #FIN_DELEGATION#

  Très cordialement.

  web-GFC

  Ce mail de notification a été envoyé par web-GFC pour votre information. Merci de ne pas répondre.', true, NOW(), NOW() );

INSERT INTO templatenotifications VALUES (4, 'insertion', 'Vous allez recevoir un nouveau flux', 'Bonjour #PRENOM# #NOM#,

  Un flux vient d''être inséré dans un circuit dont vous êtes valideur.

  Id : #IDENTIFIANT_FLUX#
  Référence : #REFERENCE_FLUX#
  Objet : #OBJET_FLUX#
  Circuit : #LIBELLE_CIRCUIT#

  Vous pouvez le consulter à l''adresse : #ADRESSE_A_VISUALISER#

  Très cordialement.

  web-GFC

  Ce mail de notification a été envoyé par web-GFC pour votre information. Merci de ne pas répondre.', true, NOW(), NOW() );

INSERT INTO templatenotifications VALUES (5, 'insertion_reponse', 'Réception d''un flux réponse', 'Bonjour #PRENOM# #NOM#,

  Un flux en réponse au flux #FLUX_ORIGINE# vient d''être ajouté dans votre bannette "Mes flux réponse".

  Id : #IDENTIFIANT_FLUX#
  Référence : #REFERENCE_FLUX#
  Objet : #OBJET_FLUX#
  Circuit : #LIBELLE_CIRCUIT#

  Vous pouvez le consulter à l''adresse : #ADRESSE_A_VISUALISER#

  Très cordialement.

  web-GFC

  Ce mail de notification a été envoyé par web-GFC pour votre information. Merci de ne pas répondre.', true, NOW(), NOW() );


INSERT INTO templatenotifications VALUES (6, 'refus', 'Un flux a été refusé', 'Bonjour #PRENOM# #NOM#,

  Un flux a été refusé.

  Objet : #OBJET_FLUX#
  Référence : #REFERENCE_FLUX#
  Nouvel identifiant : #IDENTIFIANT_FLUX#

  Le flux peut être consulté à cette adresse : #ADRESSE_A_MODIFIER#

  Très cordialement.

  web-GFC

  Ce mail de notification a été envoyé par web-GFC pour votre information. Merci de ne pas répondre.', true, NOW(), NOW() );

INSERT INTO templatenotifications VALUES (7, 'retard_validation', 'Un flux est en retard', 'Bonjour #PRENOM# #NOM#,

  Un flux dans votre bannette "Mes flux à traiter" est en retard et attend votre validation.

  Id : #IDENTIFIANT_FLUX#
  Référence : #REFERENCE_FLUX#
  Objet : #OBJET_FLUX#
  Circuit : #LIBELLE_CIRCUIT#

  Vous pouvez consulter ce flux à l''adresse : #ADRESSE_A_TRAITER#

  Très cordialement.

  web-GFC

  Ce mail de notification a été envoyé par web-GFC pour votre information. Merci de ne pas répondre.', true, NOW(), NOW() );

INSERT INTO templatenotifications VALUES (8, 'traitement', 'Vous avez un flux à traiter', 'Bonjour #PRENOM# #NOM#,

  Un flux est arrivé dans votre bannette "Mes flux à traiter".

  Id : #IDENTIFIANT_FLUX#
  Référence : #REFERENCE_FLUX#
  Objet : #OBJET_FLUX#
  Circuit : #LIBELLE_CIRCUIT#

  Vous pouvez le consulter à l''adresse : #ADRESSE_A_TRAITER#

  Très cordialement.

  web-GFC

  Ce mail de notification a été envoyé par web-GFC pour votre information. Merci de ne pas répondre.', true, NOW(), NOW() );

INSERT INTO templatenotifications VALUES (9, 'quotidien', '[web-GFC]  Récapitulatif journalier de vos flux', 'Bonjour #PRENOM# #NOM#,

  Voici la liste des flux vous concernant pour la date du #DATERECEPTION_FLUX#.

  Flux :
  #REFERENCE_FLUX#

  Très cordialement.

  web-GFC

  Ce mail de notification a été envoyé par web-GFC pour votre information. Merci de ne pas répondre.', true, NOW(), NOW() );


INSERT INTO templatenotifications VALUES (10, 'clos', '[web-GFC]  Un flux dont vous êtes l''initiateur a été clos', 'Bonjour #PRENOM# #NOM#,

Le flux #REFERENCE_FLUX#, dont vous êtes l''initiateur vient d''être clos

Très cordialement.

web-GFC

Ce mail de notification a été envoyé par web-GFC pour votre information. Merci de ne pas répondre.', true, NOW(), NOW() );


SELECT pg_catalog.setval('templatenotifications_id_seq', 11, true);


INSERT INTO types VALUES ('-1', 'Flux sans traitement', '2020-12-01 14:55:14.779645', '2020-12-01 14:55:14.779645');

INSERT INTO soustypes VALUES ('-1', 'Clôture automatique',	 '2020-12-01 14:55:14.779645', '2020-12-01 14:55:14.779645', '-1');

INSERT INTO originesflux VALUES ( 1, 'Courrier papier', TRUE, '2021-12-01 14:55:14.779645', '2021-12-01 14:55:14.779645');
INSERT INTO originesflux VALUES ( 2, 'Mail', TRUE, '2021-12-01 14:55:14.779645', '2021-12-01 14:55:14.779645');
SELECT pg_catalog.setval('originesflux_id_seq', 3, true);

INSERT INTO services (name, created, modified, id, active) VALUES ( 'MAIRE', NOW(), NOW(), 1, TRUE);
SELECT pg_catalog.setval('services_id_seq', 2, true);

COMMIT;
